import kinds;
import system;
import opengl;
import images = images.src.images;
import std.math;
import std.stdio;
import util;
import base;
import ship;
import glutils;
import sounds;
import rand;

template DefineEnemy(int s)
{
  int score(){ return s; }
}

void fireBulletToPlayer(Bullet)(Registry reg,
                                int n, Point from,
                                double v)
{
  void fireToPlayerNormal()
    {
      auto pl = Player.search(reg);
      if (!pl)
        return;
      
      Point diff = pl.pos - from;
      reg.add(new Bullet(from, (v/diff.abs)*diff));
    }
  
  static const Point[] reference8
    = [ {1, 0},
        {1, 1},
        {0, 1}
    ];
  static const Point[] reference16
    = [ {1, 0},
        {92, 38},
        {1, 1},
        {38, 92},
        {0, 1}
    ];

  static const Point[] reference32
    = [ 
        {1.0,0.0},
        {0.98078528040323,0.195090322016128},
        {0.923879532511287,0.38268343236509},
        {0.831469612302545,0.555570233019602},
        {0.707106781186548,0.707106781186547},
        {0.555570233019602,0.831469612302545},
        {0.38268343236509,0.923879532511287},
        {0.195090322016128,0.98078528040323},
        {0.0, 1.0},
        ];

  static const Vector[] reference64
    = [
        {0.0, 1.0},
        {0.0980171403295606, 0.995184726672197},
        {0.195090322016128, 0.98078528040323},
        {0.290284677254462, 0.956940335732209},
        {0.38268343236509, 0.923879532511287},
        {0.471396736825998, 0.881921264348355},
        {0.555570233019602, 0.831469612302545},
        {0.634393284163645, 0.773010453362737},
        {0.707106781186547, 0.707106781186548},
        {0.773010453362737, 0.634393284163645},
        {0.831469612302545, 0.555570233019602},
        {0.881921264348355, 0.471396736825998},
        {0.923879532511287, 0.38268343236509},
        {0.956940335732209, 0.290284677254462},
        {0.98078528040323, 0.195090322016128},
        {0.995184726672197, 0.0980171403295608},
        {1.0, 0},
        ];
  auto pl = Player.search(reg);

  Point[] reference = null;
  
  switch(n) {
  case 0:
    fireToPlayerNormal();
    return;
  case 8:
    reference = reference8;
    break;
  case 16:
    reference = reference16;
    break;
  case 32:
    reference = reference32;
    break;
  case 64:
    reference = reference64;
    break;
  }
  
  if (!pl)
    return;
  
  Point diff = pl.pos - from;
  Point diff_a = Point(abs(diff.x), abs(diff.y));
  Point refe = reference[0];
  foreach (u; reference) {
    if (Point.angle(diff_a,u) > Point.angle(diff_a,refe))
      refe = u;
  }
  refe.x *= sign(diff.x);
  refe.y *= sign(diff.y);
  
  reg.add(new Bullet(from, (v/refe.abs)*refe));
}

abstract class LBullet: Animatable, Bullet
{
private:
  Vector v_;
  bool scroll_;
  
public:
  this(Point pos, Vector v, Animation anim, bool scroll) {
    super(SubKind.NORMAL_BULLET, pos, anim, Color.bullet);
    v_ = v;
    scroll_ = scroll;
  }

  void run(Registry reg) {
    move(v_);
    if (scroll_)
      scroll(reg);
    die_if_out_of_window();
  }

  void collide_with_wall(Registry, Wall) {
    die();
  }
  
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
    die();
  }
}

class NormalBullet: LBullet
{
private:
  static void function()[] image = [ &images.enemies.nbullet.draw ];
  
public:
  this(Point pos, Point vect, bool scroll=false) {
    super(pos, vect, new Animation(image), scroll);
  }

  mixin DefineRect!(pos, 8, 8, true);

  alias fireBulletToPlayer!(NormalBullet) fireToPlayer;
}

class StripBullet: LBullet
{
  this(Point pos, Point vect, bool scroll = false) {
    super(pos, vect,
          new Animation(&images.enemies.strip.draw),
          scroll);
    degree = atan2(vect.y, vect.x)*180/PI;
  }

  mixin DefineRect!(pos, 8, 8, true);
  alias fireBulletToPlayer!(StripBullet) fireToPlayer;
}

class SharpBullet: LBullet
{
  this(Point pos, Point vect, bool scroll=false) {
    super(pos, vect, new Animation(&images.enemies.tri.draw),
          scroll);
    degree = atan2(vect.y, vect.x)*180/PI;
  }

  mixin DefineRect!(pos, 8, 8, true);
  alias fireBulletToPlayer!(SharpBullet) fireToPlayer;
}


alias fireBulletToPlayer!(NormalBullet) fireToPlayer;

class BaseWall: BaseWorker, Wall
{
private:
  Rect rect_;

public:
  this(Rect rect) {
    super(SubKind.WALL);
    rect_ = rect;
  }

  void run(Registry reg) {
    Scroll scr = Scroll.search(reg);
    rect_.p += scr.v;
    rect_.q += scr.v;

    if(!rect_.is_intersected(LARGE_WINDOW))
      die();
  }

  Rect rect() { return rect_; }
}

class NormalWall: BaseWall
{
  this(Rect rect) { super(rect); }
  
  void draw(Registry reg) {
    glColor3da(Color.wall);
    rect.draw();
  }
}

class MovableWall: BaseWorker, Wall
{
private:
  Rect rect_;

public:
  this(Point pos, double width, double height) {
    super(SubKind.WALL);
    rect_ = Rect(pos.x, pos.y-height, width, height);
  }

  void draw(Registry reg) {
    glColor3da(Color.wall);
    rect_.draw();
  }
 
  void run(Registry reg) {
    Scroll scr = Scroll.search(reg);
    rect_.p += scr.v;
    rect_.q += scr.v;
  }

  void move(Vector v) {
    rect_.p += v;
    rect_.q += v;
  }
  //pがqよりも左上に来るようにしてください
  void pmove(Vector v) {
    rect_.p += v;
    check();
  }
  void qmove(Vector v) {
    rect_.q += v;
    check();
  }
  void check() {
    double temp;
    if (rect_.p.x > rect.q.x) {
      temp = rect_.p.x;
      rect_.p.x = rect_.q.x;
      rect_.q.x = temp;
    }
    if (rect_.p.y > rect.q.y) {
      temp = rect_.p.y;
      rect_.p.y = rect_.q.y;
      rect_.q.y = temp;
    }
  }

  Rect rect() { return rect_; }
}

class OneMovableWall : MovableWall
{
private:
  Vector v_;
  int ct_;

public:
  this(Point pos, double width, double height, Vector v, int ct = 0) {
    super(pos, width, height);
    v_ = v;
    ct_ = ct;
  }
  
  void run(Registry reg) {
    move(v_);
    if(!rect.is_intersected(LARGE_WINDOW) && count > ct_)
      die();
    super.run(reg);
  }
}

class InvisibleWall: BaseWall
{
  this(Rect rect) { super(rect); }
  void draw(Registry reg) {
    if (reg.root.game_config.draw_bounding_box) {
      glColor3d(0.5, 0.5, 0.5);
      rect.draw();
    }
  }
}

abstract class BaseEnemy: Animatable, Enemy
{
private:
  int life_;
  GLdouble[] base_color_;
  GLdouble[3] real_color_;
  int count_damage_;
  int fire_count_ = -1;
  int fire_rate_ = -1;
  int bullet_speed_;
  int fire_resolution_;
  int fire_probability_;
  void function(Registry, Enemy) destroyed_hook_ = null;
  
public:
  this(SubKind subkind, Point pos, Animation anim, int life,
       GLdouble[] col = Color.enemy,
       double deg=0, double sc=1.0, Movable parent=null) {
    base_color_ = col;
    real_color_[] = col[];
    super(subkind, pos, anim, real_color_, deg, sc, parent);
    count_damage_ = 0;
    life_ = life;
  }

  void set_repeating_fire(int start, int rate, int speed, int resolution=32, int probablity = 100) {
    fire_count_ = start;
    fire_rate_ = rate;
    bullet_speed_ = speed;
    fire_resolution_ = resolution;
    fire_probability_ = probablity;  //値は0～100%の間
  }
  
  void set_destroyed_hook(void function(Registry, Enemy) hook) {
    destroyed_hook_ = hook;
  }

  void set_capsule_drop(Capsule)() {
    static void add_capsule(Registry reg, Enemy e){
      reg.add(new Capsule(e.pos));
    }
    set_destroyed_hook(&add_capsule);
  }

  void move_vector_when(Vector v, int start, int end) {
    if (start <= count && count <= end) 
      move(v);
  }
  
  void move_forward(double speed, bool using_degree = true, double deg = 0) {
    move(degree_to_vector(speed, using_degree, deg));
  }
  
  Vector degree_to_vector(double length = 1, bool using_degree = true, double deg = 0) {
    if (using_degree)
      return length * Vector(cos((degree-90)/180*PI), sin((degree-90)/180*PI));
    else
      return length * Vector(cos(deg/180*PI), sin(deg/180*PI));
  }

  double homing(double pre_deg, double change_degree, Registry reg) {
    auto pl = Player.search(reg);
    if (pl) {
    // 自機誘導部分
      int deg_loop;
      double deg;
      Point diff = pl.pos - pos;
      deg = atan2(diff.y, diff.x) * 180 / PI + 90;

      deg_loop = cast(int)floor((pre_deg + 90) / 360);
      if (pre_deg - deg - 360 * deg_loop > 180)
        pre_deg += change_degree;
      else if (pre_deg - deg - 360 * deg_loop < -180)
        pre_deg -= change_degree;
      else if (pre_deg - deg - 360 * deg_loop > change_degree+1)
        pre_deg -= change_degree;
      else if (pre_deg - deg - 360 * deg_loop < -(change_degree+1))
        pre_deg += change_degree;
    }
    return pre_deg;
  }

  void _run(Registry reg) {
    if (count_damage_ > 0) {
      count_damage_--;
      if ((count / 3) % 2)
        color = Color.damaged_enemy;
      else
        color = base_color_;
    } else {
      color = base_color_;
    }
    
    if (fire_rate_ > 0) {
      fire_count_--;
      if (fire_count_ == 0) {
        fire_count_ = fire_rate_;
        if (fire_probability_>= 100)
          fireToPlayer(reg, fire_resolution_, pos, bullet_speed_);
        else {
          RNG rng = reg.root.rng;
          if (fire_probability_ >= rng.irand(1, 100))
            fireToPlayer(reg, fire_resolution_, pos, bullet_speed_);
        }
      }
    }
    super._run(reg);
  }
  
  void collide_with_shot(Registry reg, Shot shot) {
    if (shot.power == 0)
      return;
    
    life_ -= shot.power();
    count_damage_ = 30;

    Wave.enemy_damaged.play();
    
    if (life_ <= 0 && !dead()) {
      if (subkind == SubKind.BOSS_ENEMY || subkind == SubKind.BOSS3_A)
        Wave.boss_destroyed.play();
      else
        Wave.enemy_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().w));
      reg.root.add_score(score());
      if (destroyed_hook_)
        destroyed_hook_(reg, this);
      die();
    }
  }

  class EnemyWall: BaseMovable, Wall {
    double width_;
    double height_;
    this(Enemy parent) {
      super(SubKind.WALL, Point(0,0), parent);
      width_ = parent.rect.w;
      height_ = parent.rect.h;
    }
    this(Enemy parent, Point pos, double width, double height) {
      super(SubKind.WALL, pos, parent);
      width_ = width;
      height_ = height;
    }
    void run(Registry reg) {
      if (parent.dead)
        die();
    }

    Rect rect() {
      return Rect(pos.x-(width_/2), pos.y-(height_/2), width_, height_);
    }
  };

  bool dead() {
    return super.dead();
  }
  
  int life() { return life_;}

  void collide_with_wall(Registry, Wall) {
  }
}  

abstract class ChildEnemy: BaseEnemy {
  this(SubKind subkind, Point pos, Animation anim,
       GLdouble[] col = Color.enemy,
       double deg=0, double sc=1.0, Movable parent=null) {
    super(subkind, pos, anim, 0, col, deg, sc, parent);
  }

  void _run(Registry reg) {
    super._run(reg);
    color = (cast(Animatable)parent).color;
    if (parent.dead)
      die();
  }

  void collide_with_shot(Registry reg, Shot shot) {
    (cast(Enemy)parent).collide_with_shot(reg, shot);
  }
  
  int life() { return (cast(Enemy)parent).life;}
}

class EnemyCreator: BaseMovable {
private:
  int head_count_;
  int interval_;
public:
  this(Point pos, int head_count, int interval) {
    super(SubKind.NORMAL_ENEMY_CREATOR, pos);
    head_count_ = head_count;
    interval_ = interval;
  }
  
  void run(Registry reg) {
    if (count > interval_ * (head_count_ -1))
      die();
    scroll(reg);
  }
  
  int head_count() { return head_count; }
  int interval() { return interval_; }
}

class Flak: BaseEnemy
{
public:
  this(Point pos, int deg) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.flak.draw),
          10, Color.enemy, deg, 1.5);
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);

  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
  }
}

class FisFlak: Flak
{
  this(Point pos, int deg, int start, int rate, int speed) {
    super(pos, deg);
    set_repeating_fire(start, rate, speed);
  }
}

class Flyer: BaseEnemy {
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.flyer.draw),
          10, Color.enemy);
    scale = 1.4;
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);

  void _run(Registry reg) {
    ydegree += 10;
    super._run(reg);
  }
}

class LFlyer: Flyer {
private:
  Vector v_;
  
public:
  this(Point pos, int deg, double speed, int firerate, int fire_first) {
    super(pos);
    degree = deg+90;
    v_ = Point(speed*cos(PI*deg/180), speed*sin(PI*deg/180));
    set_repeating_fire(fire_first, firerate, 4);
  }

    
  void run(Registry reg) {
    move(v_);
    die_if_out_of_window();
  }
}

class BitLoader: BaseEnemy {
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.bitloader.draw),
          80, Color.enemy);
    scale = 2.0;
  }

  mixin DefineEnemy!(500);
  mixin DefineRect!(pos, 60, 80, true);
}

class CapsuleCarrier: BaseEnemy {
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.capsule_carrier.draw),
          10, Color.enemy);
    scale = 1.5;
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 20, 20, true);
}

class Wiper: BaseEnemy {
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.wiper.draw),
          20, Color.enemy);
  }

  mixin DefineEnemy!(200);
  mixin DefineRect!(pos, 30, 30, true);
  void _run(Registry reg) {
    degree += 10;
    super._run(reg);
  }
}

class Helicopter: BaseEnemy {
  this(Point pos, Registry reg) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.helicopter.draw),
          10, Color.enemy);
          scale = 2.0;
    reg.add(new Helicopter.Wing(this));
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);

  static class Wing : Animatable {
    this(Helicopter heli) {
      super(SubKind.NORMAL_EFFECT, Point(0, 0),
            new Animation(&images.enemies.helicopter_wing.draw),
            Color.enemy, 0, 2.0, heli);
    }
    
    
    void run(Registry reg) {
      if (parent.dead)
        die();
      color = (cast(Animatable)parent).color;
      degree += 10;
      visible(!visible());    //点滅部分
      super.run(reg);
    }
  };
}

class Patapata: BaseEnemy {
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.patapata.draw),
          10, Color.enemy);
    scale = 1.4;
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);

  void _run(Registry reg) {
    super._run(reg);
  }
}

class Shutter: BaseEnemy {
private:
  bool shutout_ = false;
  double target_x_;
  double dx_;
  
public:
  static void add(Point pos, double target_x, double deg,
             Registry reg) {
    Shutter shutter = new Shutter(pos, target_x, deg);
    reg.add(shutter);
    reg.add(shutter.new ShutterWall());
  }
  
  this(Point pos, double target_x, double deg) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.shutter.draw),
          600);
    target_x_ = target_x;
    scale = 3.0;
    degree = deg;
    dx_ = (target_x_ > pos.x) ? 60 : -60;
  }
  
  mixin DefineEnemy!(1000);
  mixin DefineRect!(pos, 300, 20, true);

  void run(Registry reg) {
    Player pl = Player.search(reg);
    scroll(reg);

    if (pos.y > LARGE_WINDOW.q.y)
      die();

    if (!pl)
      return;
    
    if (shutout_) {
      if (abs(target_x_-pos.x) < 60)
        pos = Point(target_x_, pos.y);
      else {
        move(Vector(dx_, 0));
      }
    } else 
      if (abs(pl.pos.y - pos.y) < 120)
        shutout_ = true;
  }
  
  class ShutterWall: BaseWorker, Wall {
    this() {
      super(SubKind.WALL);
    }
    void run(Registry reg) {
      if (this.outer.dead)
        die();
    }

    Rect rect() {
      Point p = this.outer.pos;
      return Rect(p.x-150, p.y-10, 300, 20);
    }
    
  }
}

class Top: BaseEnemy {
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.top.draw),
          20, Color.enemy);
    scale = 2.0;
  }

  mixin DefineEnemy!(300);
  mixin DefineRect!(pos, 20, 18, true);
  void _run(Registry reg) {
    ydegree += 20;
    super._run(reg);
    die_if_out_of_window();
  }
}

class Viper: BaseEnemy {
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.viper.draw),
          10, Color.enemy);
    scale = 1.5;
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);
}

class Bud: BaseEnemy {
public:
  this(Point pos, double deg, Movable parent = null) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.bud.draw),
          20, Color.enemy, deg, 3.0, parent);
  }

  mixin DefineEnemy!(200);
  mixin DefineRect!(pos, 20, 20, true);
}


class DestroyedEnemy: BaseMovable {
private:
  bool scroll_;
  double[] color_;
  double size_;
  
public:
  this(Point pos, bool scroll, double size=20, double[] color=Color.enemy) {
    super(SubKind.DESTROYED_ENEMY, pos);
    scroll_ = scroll;
    color_ = color;
    size_ = size;
  }

  void run(Registry reg) {
    if (scroll_)
      scroll(reg);

    if (count >= 10)
      die();
  }

  void draw(Registry reg) {
    glPushMatrix();
    glColor3da(color_);
    glTranslated(pos.x, pos.y, 0);
    glRotated(count*24, 0, 0, 1);
    glBegin(GL_LINES);
    glVertex2d(0, size_/2);
    glVertex2d(0, -size_/2);
    glEnd();
    glPopMatrix();
  }
}
