import std.stdio;
import std.string;
import varray;

enum OptType{
  NO_PARAM,
  NEED_PARAM,
  OPT_PARAM,
}
  

void optparse(char[][] args,
              char[] opt_noparam, char[] opt_needparam, char[] opt_optparam,
              void delegate(char, char[]) handler)
{
  OptType[char] opts;
  
  foreach (char ch; opt_noparam)
    opts[ch] = OptType.NO_PARAM;
  foreach (char ch; opt_needparam)
    opts[ch] = OptType.NEED_PARAM;
  foreach (char ch; opt_optparam)
    opts[ch] = OptType.OPT_PARAM;

   void delegate(int i, int j) parse;
  
  void parse_opt(int i, int j) {
    char ch;
    if (args.length <= i)
      return;
    if (args[i].length <= j) {
      parse(i+1, 0);
      return;
    }
    
    ch = args[i][j];
    if (!(ch in opts))
      throw new Error(format("unknown option -%s", ch));
    switch(opts[ch]) {
    case OptType.NO_PARAM:
      handler(ch, null);
      parse_opt(i, j+1);
      return;
    case OptType.NEED_PARAM:
      if (args[i].length == j+1) {
        if (args.length == i+1)
          throw new Error(format("-%s needs parameter", ch));
        else
          handler(ch, args[i+1]);
        parse(i+2, 0);
        return;
      } else {
        handler(ch, args[i][(j+1)..$]);
        parse(i+1, 0);
        return;
      }
    case OptType.OPT_PARAM:
      /* todo: */
      return;
    default:
      throw new Error(format("unknown option -%s", ch));
    }
  }
  
  void parse_(int i, int j) {
    if (args.length <= i)
      return;
    if (args[i].length <= j) {
      parse(i+1, 0);
      return;
    }
    if (args[i][j] == '-' ) {
      parse_opt(i, j+1);
      return;
    }
  }

  parse = &parse_;
  parse(1, 0);
}

debug (TestOpt)
{
  int main(char[][] args)
    {

      void opt(char ch, char[] optstr) {
        switch (ch) {
        case 'a':
          puts("a");
          break;
        case 'b':
          writefln("b %s", optstr);
        }
      }
      
      optparse(args, "a", "b", "", &opt);
      
      return 0;
    }
}

