/*
  Pencil-Bullet: A shump game

  Copyright (C) 2007-2008 OHBAYASHI Ippei, Mori Singo, HARA Yutaka
  Copyright (C) 2007-2008 Kyoto univ. Microcomputer Club

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 */
import SDL;
import std.string;
import util;
import kinds;
import std.stdio;
import base;
import images = images.src.images;
import opengl;
import glutils;
import std.typetuple;
import system;
import std.ctype;
import std.conv;
import recorder;
static import stage0dia;
import opt;
import std.cstream;
static import version_str;
static import std.c.stdlib;

class Tester : Animatable
{
  static void function()[] image = [&images.tester.draw];

  this() {
    super(SubKind.TESTER,
          Point(200, 200),
          new Animation(image));
    scale = 4.0;
  }

  void run(Registry){
    degree += 1.4;
  }
}

class TextTester : BaseMovable
{
  this() {
    super(SubKind.TEXTTESTER, Point(0, 0));
  }

  void draw(Registry) {
    glPushMatrix();
    glTranslated(600, 20, 0);
    glRotated(90, 0, 0, 1);
    glColor3da(Color.white);
    draw_text("ABCDEFGHIJKLMNOPQRSTUVWXYZ.0123456789");
    glPopMatrix();
  }
}

private void show_version()
{
  
}

private void show_license()
{
  char[] license = "Pencil-Bullet: A shump game

Copyright (C) 2007-2008 OHBAYASHI Ippei, Mori Singo, HARA Yutaka
Copyright (C) 2007-2008 Kyoto univ. Microcomputer Club

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.";
  
  dout.writefln("%s", license);
  dout.writefln("\n%s", version_str.version_str);
}

int main(char[][] args)
{
  
  static KeyConfig parse_keyconfig(char[] str) {
    if (str.length != 4 ) {
      throw new Error("keyconfig parse error");
    }
    foreach (char c; str) {
      if (!isdigit(c))
        throw new Error("keyconfig parse error");
    }
    
    return new KeyConfig(toInt(str[0..1]), toInt(str[1..2]),
                         toInt(str[2..3]), toInt(str[3..4]));
  };
    
  bool fullscreen = false;
  GameConfig.GameMode mode = GameConfig.GameMode.NORMAL;
  bool sound = true;
  int stage = 1;
  int rnum = 0;
  int life = 2;
  bool invincible = false;
  bool draw_bounding_box = false;
  int noptions = 0;
  int magazine = 0;
  KeyConfig keyconfig = KeyConfig.default_config;
  char[] replayfile = null;
  char[] exec_replayfile = null;
  
  void opt(char ch, char[] optstr) {
    switch (ch) {
    case 'f':
      fullscreen = true;
      break;
    case 's':
      stage = atoi(optstr);
      break;
    case 'r':
      rnum = atoi(optstr);
      break;
    case 'd':
      mode = GameConfig.GameMode.DEBUG;
      break;
    case 'l':
      life = atoi(optstr);
      break;
    case 'n':
      sound = false;
      break;
    case 'i':
      invincible = true;
      break;
    case 'b':
      draw_bounding_box = true;
      break;
    case 'm':
      magazine = atoi(optstr);
      break;
    case 'o':
      noptions = atoi(optstr);
      break;
    case 'k':
      keyconfig = parse_keyconfig(optstr);
      break;
    case 'p':
      replayfile = optstr.dup;
      break;
    case 'e':
      exec_replayfile = optstr;
      break;
    case 'v':
      show_license();
      std.c.stdlib.exit(0);
      break;
    case 't':
      if (mode == GameConfig.GameMode.NORMAL)
        mode = GameConfig.GameMode.TRAINING;
      break;
    }
  }

  optparse(args, "fdnibvt", "srlmokpe", "",  &opt);

  GameConfig config = new GameConfig(mode,
                                     sound, fullscreen,
                                     invincible,
                                     draw_bounding_box,
                                     life, magazine, noptions,
                                     stage, rnum, replayfile);

  Game game;
  if (exec_replayfile)
    game =  new ReplayGame(keyconfig, config,
                           AllRecords.load_from_file(exec_replayfile));
  else
    game = new NormalGame(keyconfig, config);

  game.start();
  return 0;
}
