import kinds;
import std.string;
import util;
import std.stdio;
import base;
import rand;
import varray;
import std.stream;
import std.socket;
import system;

static const uint REPLAY_FILE_VERSION = 0;

ushort encode_input_to_ushort(bool l, bool r, bool u, bool d,
                              bool shot, bool speed_change,
                              bool option_change, int charge)
{
  static ushort bit(bool p, int n) {
    return p ? (1<<n) : 0;
  }

  return bit(l,0)|bit(r,1)|bit(u,2)|bit(d,3)|bit(shot,4)|
    bit(speed_change,5)|bit(option_change,6)|
    (((charge > 127)?127:charge)<<8);
}


class Recorder: BaseWorker {
private:
  int stage_;
  int restart_num_;
  int score_;
  int rest_players_;
  int extend_count_;
  int speed_;
  int num_options_;
  int magazine_index_;
  
  uint mt_[RNG.N];
  int mti_;
  
  VArray!(ushort) record_;
  
public:
  this(Game root) {
    super(SubKind.RECORDER);
    stage_ = root.stage;
    restart_num_ = root.restart_num;
    score_ = root.score;
    rest_players_ = root.rest_players;
    extend_count_ = root.extend_count;
    speed_ = root.speed;
    num_options_ = root.num_options;
    magazine_index_ = root.magazine_index;
    
    mt_[] = root.rng.internal_vector[];
    mti_ = root.rng.internal_index;
    record_ = new VArray!(ushort)();
  }

  void run(Registry reg) {
    Input input = Input.search(reg);

    record_.add(input.encode_to_ushort());
  }

  /* return value: record data size */
  int datasize() {
    return 10*4+4*RNG.N+record_.length*2;
  }
  
  void save_to_file(Stream output) {
    void write_uint(uint u) {
      for (int i=0; i<4; i++) 
        output.write(cast(ubyte)((u>>i*8)&0xff));
    }
    void write_ushort(ushort u) {
      for (int i=0; i<2; i++) 
        output.write(cast(ubyte)((u>>i*8)&0xff));
    }
    write_uint(stage_);
    write_uint(restart_num_);
    write_uint(score_); 
    write_uint(rest_players_); 
    write_uint(extend_count_); 
    write_uint(speed_); 
    write_uint(num_options_); 
    write_uint(magazine_index_); 
    write_uint(mti_); 
    foreach (uint u; mt_)
      write_uint(u);
    write_uint(record_.length);
    foreach (ushort us; record_)
      write_ushort(us);
  }
}

static uint read_uint(InputStream input) {
  uint n=0; ubyte u;
  for (int i=0; i<4; i++) {
    input.read(u); n += u << (i*8);
  }
  return n;
}

static ushort read_ushort(InputStream input) {
  ushort n=0; ubyte u;
  for (int i=0; i<2; i++) {
    input.read(u); n += u << (i*8);
  }
  return n;
}

class AllRecords {
private:
  StageRecord[] records_;
  int score_;
  uint mode_;
public:
  this(StageRecord[] records, int score, uint mode) {
    records_ = records;
    score_ = score;
    mode_ = mode;
  }

  int length(){ return records_.length; }
  int score(){ return score_; }
  uint mode(){ return mode_; }
  
  StageRecord opIndex(size_t i) {
    return records_[i];
  }

  static AllRecords load_from_file(char[] fname) {
    char[] header;
    StageRecord[] records;
    
    Stream input = new File(fname);
    header = input.readString(12);
    if (header != "-PB_REPLAY-\0")
      throw new Error("This file is not PencilBullet save file");

    uint nstage = read_uint(input);
    uint stage_data_hd = read_uint(input);
    input.seekCur(nstage*8);
    uint score = read_uint(input);
    uint mode = read_uint(input);
    uint replay_ver = read_uint(input);

    if (replay_ver != REPLAY_FILE_VERSION) {
      throw new Error("Replay file version is different");
    }
    
    input.seekSet(stage_data_hd);

    records = new StageRecord[nstage];
    
    for (int i=0; i<nstage; i++)
      records[i] = new StageRecord(input);

    input.close();
    
    return new AllRecords(records, score, mode);
  }
}

class StageRecord {
private:
  int stage_;
  int restart_num_;
  int score_;
  int rest_players_;
  int extend_count_;
  int speed_;
  int num_options_;
  int magazine_index_;
  
  uint mt_[RNG.N];
  int mti_;
  
  VArray!(ushort) record_;
public:
  this(Stream input) {
    stage_ = read_uint(input);
    restart_num_ = read_uint(input);
    score_ = read_uint(input);
    rest_players_ = read_uint(input);
    extend_count_ = read_uint(input);
    speed_ = read_uint(input);
    num_options_ = read_uint(input);
    magazine_index_ = read_uint(input);
    mti_ = read_uint(input);
    for (int i=0; i<RNG.N; i++)
      mt_[i] = read_uint(input);
    uint len = read_uint(input);
    record_ = new VArray!(ushort);
    for (int i=0; i<len; i++)
      record_.add(read_ushort(input));
  }

  int stage(){ return stage_; }
  int restart_num(){ return restart_num_; }
  int score(){ return score_; }
  int rest_players(){ return rest_players_; }
  int extend_count(){ return extend_count_; }
  int speed(){ return speed_; }
  int num_options(){ return num_options_; }
  int magazine_index(){ return magazine_index_; }
  uint[]  mt(){ return mt_; }
  int mti(){ return mti_; }
  VArray!(ushort) record(){ return record_; }
}

class RecordPlayer: Input {
private:
  VArray!(ushort) record_;
  
  bool left_, right_, up_, down_;
  bool shot_, speed_change_, option_change_;
  int charge_;
  
public:
  this(VArray!(ushort) record) {
    super(SubKind.INPUT);
    record_ = record;
  }

  bool left(){ return left_; }
  bool right(){ return right_; }
  bool up(){ return up_; }
  bool down(){ return down_; }
  bool shot(){ return shot_; }
  bool speed_change(){ return speed_change_; }
  bool option_change(){ return option_change_; }
  int charge(){ return charge_; }
  
  ushort encode_to_ushort(){ return 0; };

  void decode_from_ushort(ushort dat) {
    bool bitcheck(int n) {
      return (dat&(1<<n)) != 0;
    }

    left_ = bitcheck(0);
    right_ = bitcheck(1);
    up_ = bitcheck(2);
    down_ = bitcheck(3);
    shot_ = bitcheck(4);
    speed_change_ = bitcheck(5);
    option_change_ = bitcheck(6);
    charge_ = dat>>8;
  }

  void run(Registry reg) {
    decode_from_ushort(record_[count]);
    if (speed_change_)
      reg.root.speed_change();
  }
}


