import SDL;
import kinds;
import std.string;
import util;
import std.stdio;
import base;
import system;
import images = images.src.images;
import opengl;
import std.math;
import glutils;
import sounds;

class Player: Animatable
{
  private static int[] dv = [2,3,4];
  private int num_capsules=0;
  
  this() {
    super(SubKind.PLAYER, Point(240, 240),
          new Animation(&images.player.pl.draw));
    color = Color.player;
    scale = 32.0/20.0;
  }

  void run(Registry reg) {
    Input input = Input.search(reg);
    Point p = pos;
    int speed = reg.root.speed;
    
    if (input.left)
      p.x -= dv[speed];
    if (input.right)
      p.x += dv[speed];
    if (input.up)
      p.y -= dv[speed];
    if (input.down)
      p.y += dv[speed];

    
    if (input.shot && !input.charging) {
      Wave.shot.play();
      reg.root.magazine()(reg, pos);
    }

    if (input.charge == 90) {
      reg.add(new ChargeBlocker(this));
    }

    if (input.charge > 20 && input.charge < 90 && count % 3 == 0) {
      Wave.charge1.play();
    }
    if (input.charge >= 90 && count % 3 == 0) {
      Wave.charge2.play();
    }


    if (p.x < WINDOW.p.x || WINDOW.q.x < p.x)
      p.x = pos.x;
    if (p.y < WINDOW.p.y || WINDOW.q.y < p.y)
      p.y = pos.y;

    pos = p;
  }

  mixin DefineRect!(pos, 20, 20, true);
  
  void destroy(Registry reg) {
    if (!reg.root.game_config.invincible && !dead()) {
      Wave.down.play();
      reg.add(new DestroyedPlayer(pos));
      die();
    }
  }

  static Player search(Registry reg) {
    return reg.search!(Player)(SubKind.PLAYER);
  }
}

class PlayerReseter: BaseWorker
{
private:
  int last_num_options;
  
public:
  this() {
    super(SubKind.PLAYER_RESETER);
    last_num_options = 0;
  }

  void run(Registry reg) {
    if (last_num_options != reg.root.num_options) {
      Player pl = Player.search(reg);
      if (!pl)
        return;

      int direction = 0;
      Option opt = reg.search!(Option)(SubKind.FIXED_OPTION);
      if (opt) {
        if (opt.degree > -90 && opt.degree <= 0)
          direction = 0;
        else if (opt.degree > 0 && opt.degree <= 180)
          direction = 1;
        else
          direction = 2;
      }
      reg.clear(Kind.OPTION);
      if (reg.root.num_options >= 1)
        reg.add(new Option(pl, Point(-32, 5), true, direction));
      if (reg.root.num_options == 2)
        reg.add(new Option(pl, Point(32, 5), false, direction));
      last_num_options = reg.root.num_options;
    }
  }
}

class Extend: BaseWorker
{
private:
  static const int max_extend_count_ = 4;
  static const int[4] extend_score_ = [40000, 100000, 200000, 300000];
public:
  this() {
    super(SubKind.EXTEND);
  }

  void run(Registry reg) {
    if (reg.root.extend_count == max_extend_count_)
      return;
    int score = reg.root.score;
    if (score >= extend_score_[reg.root.extend_count]) {
      reg.root.extend();
      Wave.extend.play();
    }
  }
}



class ChargeEffect: Animatable, Shot
{
public:
  this(Player player) {
    super(SubKind.CHARGE_EFFECT, Point(0, -40),
          new Animation(&images.player.chargeeffect.draw),
          Color.shot,
          0, 2.0, player);
    this.visible = false;
  }

  void run(Registry reg) {
    if (!Player.search(reg)) {
      this.visible = false;
      return;
    }

    if (Input.search(reg).charging)
      this.visible = true;
    else
      this.visible = false;
    degree += 10;
  }

  mixin DefineRect!(pos, 46, 46, true);
  int power(){
    return visible ? 2 : 0;
  }

  void collide_with_wall(Registry, Wall){}
  void collide_with_enemy(Registry, Enemy){}
}

class ChargeBlocker: Animatable, Wall
{
private:
  int death_count_;
  
public:
  this(Player player) {
    super(SubKind.CHARGE_BLOCKER, Point(0, -40),
          new Animation(&images.player.octagon.draw),
          Color.shot,
          0, 0.0, player);
    death_count_ = -1;
  }

  mixin DefineRect!(pos, 46, 46, true);

  void run(Registry reg) {
    if (!Player.search(reg)) {
      die();
      return;
    }

    if (death_count_ > 0) {
      death_count_--;
      if (death_count_ <= 0)
        die();
      return;
    }
    
    if (!Input.search(reg).charging) {
      Wave.charge_shot.play();
      death_count_ = 9;
      visible = false;
      reg.add(new ChargeShot(pos()+Vector(0, 40)));
    }
    
    if (count <= 10)
      scale = count/5.0;
    else
      scale = 2.0;
    degree -= 5;
  }
}

class ChargeShot: Animatable, Shot
{
private:
  int power_ = 400;
  int power_of_next_frame_ = 400;
  
public:
  this(Point pos) {
    super(SubKind.CHARGE_SHOT, pos,
          new Animation(&images.player.chargeshot.draw),
          Color.shot, 0, 2.0);
  }

  mixin DefineRect!(pos, 30, 60, true);
  int power(){ return power_; }
  
  void run(Registry reg) {
    move(Vector(0, -6));
    die_if_out_of_window();
    power_ = power_of_next_frame_;
  }

  void collide_with_wall(Registry, Wall){ die();}
  void collide_with_enemy(Registry, Enemy enemy){
    if (enemy.life <= 0)
      return;
    
    power_of_next_frame_ -= enemy.life;
    if (power_of_next_frame_ <= 0)
      die();
  }
}
  
class DestroyedPlayer: BaseMovable
{
private:

public:
  this(Point pos) {
    super(SubKind.DESTROYED_PLAYER, pos);
  }

  void run(Registry reg) {
    if (count >= 120) {
      die();
      reg.root.next_player();
    }
  }

  void draw(Registry reg) {
    void draw_octagon(int radius) {
      glPushMatrix();
      glTranslated(pos.x, pos.y, 0);
      glScaled(radius, radius, 1);
      glColor3da(Color.player);
      images.player.octagon.draw();
      glPopMatrix();
    }

    draw_octagon(count);
    draw_octagon(120 - count);
  }
}

template NonPenetrationShot() {
  void collide_with_wall(Registry, Wall) {
    die();
  }

  void collide_with_enemy(Registry, Enemy) {
    die();
  }
}

class NormalShot: Animatable, Shot
{
  static const void function()[] image = [ &images.player.nshot.draw ];
  static const Vector v0 = {0, -6};
  private Vector vec_;
  
  mixin DefineRect!(pos, 8, 16);
  
  this(Point pos, Vector v = v0, double deg=0) {
    super(SubKind.NORMAL_SHOT, pos, new Animation(image), Color.shot);
    vec_ = v;
    degree = deg;
  }

  int power(){ return 10; }

  void run(Registry) {
    move(vec_);
    die_if_out_of_window();
  }
  
  mixin NonPenetrationShot;

  static void create(Registry reg, Point pos) {
    reg.add(new NormalShot(pos));
  }
}

class OptionsShot: Animatable, Shot {
  private Vector vec_;

  this(Point pos, Vector v, double deg=0) {
    super(SubKind.OPTIONS_SHOT, pos,
          new Animation(&images.player.optshot.draw),
          Color.shot);
    vec_ = v;
    degree = deg;
  }

  int power(){ return 20; }

  mixin DefineRect!(pos, 14, 14, true);
  
  void run(Registry){ move(vec_); die_if_out_of_window(); }

  mixin NonPenetrationShot;
}

class FiveWay {
  static void create(Registry reg, Point pos) {
    static double deg2rad(double d) { return d*PI/180; }
      
    for (int i=-2; i<=2; i++) {
      auto deg = -90+15*i;
      reg.add(new NormalShot(pos,
                             6*Point(cos(deg2rad(deg)), sin(deg2rad(deg))),
                             deg-90));
    }
  }
}

class ShortRange: Animatable, Shot {
  static const void function()[] image = [&images.player.sshot.draw];
  static Animation anm;
  static this() { anm = new Animation(image); }
  
  this(Point pos) {
    super(SubKind.SHORTRANGE, pos, anm, Color.shot);
  }

  mixin DefineRect!(pos, 6, 6);

  int power(){ return 80; }

  void run(Registry reg) {
    move(Vector(0, -6));
    if (count > 25)
      collide(reg);

    die_if_out_of_window();
  }
  
  void collide(Registry reg) {
    reg.add(new Explosion(pos));
    die();
  }
  
  void collide_with_wall(Registry reg, Wall) { collide(reg); }
  void collide_with_enemy(Registry reg, Enemy) { collide(reg); }

  static void create(Registry reg, Point pos) {
      reg.add(new ShortRange(pos));
  }
  
  static class Explosion: BaseMovable, Shot {
  public:
    this(Point pos) {
      super(SubKind.SHORTRANGE_EXPLOSION, pos);
    }

    Rect rect() {
      int l = count*4;
      return Rect(pos.x-l, pos.y-l, 2*l, 2*l);
    }
    
    void run(Registry) {
      if (count >= 5)
        die();
    }

    int power(){ return 10; }
    
    void draw(Registry reg) {
      glPushMatrix();
      glTranslated(pos.x, pos.y, 0);
      glColor3da(Color.shot);
      for (int i=1; i<=3; i++) {
        glPushMatrix();
        glScaled(count*i/3.0, count*i/3.0, 1);
        images.player.sshot_exp.draw();
        glPopMatrix();
      }
      glPopMatrix();

      if (reg.root.game_config.draw_bounding_box) {
        rect().draw();
      }
    }

    void collide_with_wall(Registry reg, Wall) {}
    void collide_with_enemy(Registry reg, Enemy) {}
  }
}

class Blade: Animatable, Shot {
private:
  static const void function()[] image = [ &images.player.blade.draw ];
  Vector vec_;
  
public:
  this(Point pos, Vector v) {
    super(SubKind.BLADE_SHOT, pos, new Animation(image), Color.shot);
    vec_ = v;
  }

  mixin DefineRect!(pos, 8, 8);

  int power(){ return 40; }
  
  void run(Registry) {
    move(vec_);
    degree += 20;
    die_if_out_of_window();
  }

  mixin NonPenetrationShot;

  static void create(Registry reg, Point pos) {
    int n = reg.root.blade_count;
    
    Vector v = 6*Vector(sin(PI/11*(n-1)), -cos(PI/11*(n-1)));
    reg.add(new Blade(Point(pos.x-10,pos.y), v));
    reg.add(new Blade(Point(pos.x+10,pos.y), Vector(-v.x, v.y)));

    reg.root.blade_count_next();
  }
}

class Reflective: Animatable, Shot {
private:
  Vector vec_;
  int count_reflect_ = 0;
  static const void function()[] image = [&images.player.octagon.draw];
  static Animation anm_;
  
public:
  static this() { anm_ = new Animation(image); }
  
  this(Point pos, Vector v) {
    super(SubKind.REFLECTIVE_SHOT, pos, anm_, Color.shot);
    vec_ = v;
    scale = 0.5;
  }

  mixin DefineRect!(pos, 8, 8);

  int power(){ return 10; }
  
  void run(Registry) {
    move(vec_);
    die_if_out_of_window();
  }

  void collide_with_enemy(Registry reg, Enemy) {die();}
  
  void collide_with_wall(Registry reg, Wall wall) {
    static const Point invalid = {-1000, -1000};
    
    Point cross_point(Point p1, Point d1, Point p2, Point d2, bool x_axis) {
      double d = Vector.det(d1, d2);
      Point cp;
      
      if (abs(d) < 1e-10)
        return invalid;

      assert(wall.rect.p.y <= wall.rect.q.y);
      
      cp = p1 + (Vector.det(p2-p1, d2)/d)*d1;
      if (x_axis && wall.rect.x <= cp.x && cp.x <= wall.rect.q.x)
        return cp;
      if ((!x_axis) && wall.rect.p.y <= cp.y && cp.y <= wall.rect.q.y)
        return cp;
      
      return invalid;
    }

    static Vector reflect_x(Vector v) { return Point(-v.x, v.y); }
    static Vector reflect_y(Vector v) { return Point(v.x, -v.y); }

    count_reflect_++;
    if (count_reflect_>=4)
      die();
    
    Point upper_cp = cross_point(pos, vec_, wall.rect.p, Vector(1, 0), true);
    Point lower_cp = cross_point(pos, vec_, wall.rect.q, Vector(-1, 0), true);
    Point left_cp = cross_point(pos, vec_, wall.rect.p, Vector(0, 1), false);
    Point right_cp = cross_point(pos, vec_, wall.rect.q, Vector(0, -1), false);

    Point cp = upper_cp;
    Vector function(Vector) f = &reflect_y;
    
    Point q = pos-vec_;
    
    if ((q - lower_cp).abs() < (q - cp).abs()) {
      cp = lower_cp; f = &reflect_y;
    }
    if ((q - left_cp).abs() < (q - cp).abs()) {
      cp = left_cp; f = &reflect_x;
    }
    if ((q - right_cp).abs() < (q - cp).abs()) {
      cp = right_cp; f = &reflect_x;
    }

    vec_ = f(vec_);
    move(vec_);
  }
  
  static void create(Registry reg, Point pos) {
    reg.add(new Reflective(pos, Vector(-4.2, -4.2)));
    reg.add(new Reflective(pos, Vector(0, -6.0)));
    reg.add(new Reflective(pos, Vector(4.2, -4.2)));
  }

  static class Marker: BaseMovable {
    private bool scroll_;
    
    this(Point pos, bool scroll=false) {
      super(SubKind.TEST_MARKER, pos);
      scroll_ = scroll;
    }

    void run(Registry reg) {
      if (count() > 30)
        die();
      if (scroll_)
        scroll(reg);
    }
    
    void draw(Registry reg) {
      glPushMatrix();
      glColor3da(Color.shot);
      glTranslated(pos.x, pos.y, 0);
      images.player.octagon.draw();
      glPopMatrix();
    }
    
  }
}

class Option: Animatable
{
  private double ddeg=0;
  private int rotate_count=0;
  private int shot_wait=0;
  private int change_count_=0;
  bool left_;
  int sign_; 
  void delegate(Registry reg) fire_;
  void delegate() switch_;
  
  mixin FixedAnimation!(images.player.option_fixed);
  this(Player player, Point rpos, bool left, int direction = 0) {
    super(SubKind.FIXED_OPTION, rpos, anim_, Color.player, 0, 1.5, player);
    left_ = left;
    sign_ = left_ ? 1 : -1;
    if (direction == 0) {
      degree = 0;
      fire_ = &fire_forward;
      switch_ = &switch_to_fb;
    } else if (direction == 1) {
      degree = left_ ? 180: -180;
      fire_ = &fire_backward;
      switch_ = &switch_to_side;
    } else {
      degree = left_ ? 270: 90;
      fire_ = &fire_side;
      switch_ = &switch_to_forward;
    }
  }
  
  void run(Registry reg) {
    Input input = Input.search(reg);
    
    if (rotate_count > 0) {
      rotate_count--;
      degree += ddeg;
    }

    if (shot_wait > 0)
      shot_wait--;

    if (input.shot && !input.charging && shot_wait == 0) 
      fire_(reg);

    if (input.option_change)
      change_count_++;
    
    if (change_count_ > 0 && shot_wait == 0) {
      change_count_--;
      shot_wait = 20;
      rotate_count = 10;
      switch_();
    }
  }
  void fire_forward(Registry reg) {
    degree = 0;
    reg.add(new OptionsShot(pos(), Vector(0, -6)));
  }
  void fire_backward(Registry reg) {
    degree = 180;
    reg.add(new OptionsShot(pos(), Vector(0, 6), degree));
  }
  void fire_side(Registry reg) {
    degree = -90*sign_;
    reg.add(new OptionsShot(pos(), Vector(-sign_*6, 0), degree));
  }
  void switch_to_forward() {
    fire_ = &fire_forward;
    switch_ = &switch_to_fb;
    ddeg = left_ ? 90/10: -90/10;
  }
  void switch_to_fb() {
    sign_ = left_ ? 1 : -1;
    fire_ = &fire_backward;
    switch_ = &switch_to_side;
    ddeg = left_? 180/10 : -180/10;
  }
  void switch_to_side() {
    double target = left_ ? 270: 90;
    sign_ = left_ ? 1 : -1;
    fire_ = &fire_side;
    switch_ = &switch_to_forward;
    degree = degree - floor(degree/360)*360;
    
    ddeg = (target - degree)/10;
    
  }
}

class WeaponCapsule: Animatable, Capsule
{
  alias void function(Registry reg, Point pos) ShotCreator;
  ShotCreator creator_;
  
  this(SubKind subkind, ShotCreator shot_creator,
       Animation anim, Point pos)
  {
    super(subkind, pos,
          anim,
          Color.capsule);
    scale = 1.7;
    creator_ = shot_creator;
  }

  mixin DefineRect!(pos, 30, 30, true);
  
  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
  }
  
  void collide_with_player(Registry reg, Player player) {
    Wave.get_item.play();
    reg.root.new_weapon(creator_);
    die();
  }
}

class ShortRangeCapsule: WeaponCapsule
{
  this(Point pos) {
    super(SubKind.SHORTRANGE_CAPSULE, &ShortRange.create,
          new Animation(&images.capsules.shot_shortrange.draw), pos);
  }
}

class ReflectiveCapsule: WeaponCapsule
{
  this(Point pos) {
    super(SubKind.REFLECTIVE_CAPSULE,
          &Reflective.create,
          new Animation(&images.capsules.shot_reflective.draw),
          pos);
  }
}

class FiveWayCapsule: WeaponCapsule
{
  this(Point pos) {
    super(SubKind.FIVEWAY_CAPSULE,
          &FiveWay.create,
          new Animation(&images.capsules.shot_fiveway.draw),
          pos);
  }
}

class BladeCapsule: WeaponCapsule
{
  this(Point pos) {
    super(SubKind.BLADE_CAPSULE,
          &Blade.create,
          new Animation(&images.capsules.shot_blade.draw),
          pos);
  }
}

class OptionCapsule: Animatable, Capsule
{
  this(Point pos) {
    super(SubKind.OPTION_CAPSULE, pos, 
          new Animation(&images.capsules.opt_f.draw), Color.capsule);
    scale = 2.0;
  }

  mixin DefineRect!(pos, 30, 30, true);
  
  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
  }
  void collide_with_player(Registry reg, Player player) {
    Wave.get_item.play();
    reg.root.add_option();
    die();
  }
}


