import SDL_mixer;
import std.string;
import util;
import std.file;
import base;

static bool enable_sound_=true;

void enable_sounds(bool enable)
{
  enable_sound_ = enable;
}

abstract class Music
{
public:
  static const int N_STAGES = 7;

  void play_real();
  final void play() {
    if (enable_sound_)
      play_real();
  }

  static void load_musics() {
    gameover = load_if_exist("sounds/dummy.ogg");
    stages[0] = new NullMusic();

    Music stage1 = load_if_exist("sounds/stage1.ogg");
    Music boss1 = load_if_exist("sounds/boss1.ogg");
    
    for (int i=1; i<=N_STAGES; i++) {
      stages[i] = load_if_exist(format("sounds/stage%d.ogg", i));
      boss[i] = load_if_exist(format("sounds/boss%d.ogg", i));
      if (stages[i].is_null)
        stages[i] = stage1;
      if (boss[i].is_null)
        boss[i] = boss1;
    }
  }

  private static Music load_if_exist(char[] fname, int repeat=-1) {
    if (std.file.exists(fname) && enable_sound_)
      return new NormalMusic(fname, repeat);
    else
      return new NullMusic();
  }

  static void halt() {
    if (enable_sound_)
      Mix_HaltMusic();
  }

  abstract bool is_null();
  
  static Music gameover;
  static Music stages[N_STAGES+1];
  static Music boss[N_STAGES+1];
}

class NormalMusic : Music
{
private:
  Mix_Music* music_;
  int repeat_;

public:
  this(char[] fname, int repeat=-1) {
    if (!enable_sound_)
      return;
    
    music_ = Mix_LoadMUS(toStringz(fname));
    if (!music_)
      raiseSDLError();
    
    repeat_ = repeat;
  }

  ~this() {
    Mix_FreeMusic(music_);
  }
  
  void play_real() {
    Mix_PlayMusic(music_, repeat_);
  }

  bool is_null(){ return false; }
}

class NullMusic: Music
{
public:
  this() {}

  void play_real() {
    Mix_HaltMusic();
  }

  bool is_null(){ return true; }
}

abstract class Wave
{
public:
  void play_real();
  final void play() {
    if (enable_sound_)
      play_real();
  }
  
  static void load_waves() {
    shot = new NormalWave("waves/w5.wav", 0);
    charge1 = new NormalWave("waves/charge1.wav", 0);
    charge2 = new NormalWave("waves/charge2.wav", 0);
    charge_shot = new NormalWave("waves/charge_shot.wav", 1);
    down = new NormalWave("waves/down.wav", 0);
    spread = new NormalWave("waves/w9.wav", 2);
    get_item = new NormalWave("waves/get_item.wav", 3);
    extend = new NormalWave("waves/extend.wav", 3);
    enemy_damaged = new NormalWave("waves/w10.wav", 4);
    enemy_destroyed = new NormalWave("waves/explosion.wav", 5);
    boss_destroyed = new NormalWave("waves/boss_explosion.wav", 6);
    bomb3 = new NormalWave("waves/bomb3.wav", 6);
    zigzag = new NormalWave("waves/zigzag.wav", 6);
    beam = new NormalWave("waves/beam.wav", 6);
    beam_charge = new NormalWave("waves/beam_charge.wav", 6);
  }

  static Wave shot;
  static Wave charge1;
  static Wave charge2;
  static Wave charge_shot;
  static Wave down;
  static Wave spread;
  static Wave get_item;
  static Wave extend;
  static Wave enemy_damaged;
  static Wave enemy_destroyed;
  static Wave boss_destroyed;
  static Wave bomb3;
  static Wave zigzag;
  static Wave beam;
  static Wave beam_charge;
}

class NormalWave: Wave
{
public:  
  this(char[] fname, int channel, int repeat=0) {
    if (!enable_sound_)
      return;
    
    chunk_ = Mix_LoadWAV(toStringz(fname));
    if (!chunk_)
      raiseSDLError();
    
    repeat_ = repeat;
    channel_ = channel;
  }
  
  void play_real() {
    Mix_PlayChannel(channel_, chunk_, repeat_);
  }
private:
  Mix_Chunk* chunk_;
  int channel_, repeat_;
}

class NullWave: Wave {
  void play_real() {}
}
