import SDL;
import std.string;
import kinds;
import util;
import std.stdio;
import base;
import images = images.src.images;
import opengl;
import glutils;
import std.typetuple;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;

void f01(double speed, double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 120, speed, 10000, 40));
}

void f02(double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 70, 2, 10000, 20));
}

void f03(double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 90, 2, 10000, 30));
}

void f05(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) BitLoader {
    this(Point p){
      super(p);
      set_repeating_fire(40, 60, 4);
    }
    void run(Registry reg) {
      if (count < 100) {
        move(Vector(0, 2));
      } 
    }
  };

  reg.add(enemy);
}

void f06(double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 90, 2, -1, 30));
}

void f07(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) BitLoader {
    this(Point p){
      super(p);
      set_repeating_fire(40, 40, 5);
    }
    
    void run(Registry reg) {
      if (count < 120) {
        move(Vector(0, 2));
      } 
    }
  };

  reg.add(enemy);
}

void f08(int sign, double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y), sign) Wiper {
    int sign_;
    this(Point p, int sign){
      sign_ = sign;
      super(p);
      set_repeating_fire(60, 10000, 5);
    }
    void run(Registry reg) {
      if (count < 100) {
        move(Vector(0, 3));
      } else {
        move(Vector(sign_*3, 0));
      }
      die_if_out_of_window();
    }
  };

  reg.add(enemy);
}

void f09(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(BladeCapsule)();
    }

    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f10(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(FiveWayCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f20(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 180, 60, 200, 4));
}
  
void f21(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 180, 60, 100, 4));
}

void f22(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 90, 60, 100, 4));
}

void f24(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) Wiper{
    int offset_;
    this(Point p) {
      super(p);
      offset_ = cast(int)(p.x-560);
      set_repeating_fire(120+offset_/6, 110, 4);
    }

    void run(Registry reg) {
      scroll(reg);
      if (count < 150) {
        move(Vector(-3, 0));
      } else {
        move(Vector(0, -3));
        die_if_out_of_window();
      }
    }
  };
  reg.add(enemy);
}
  
void f25(int offset, double x, double y, Registry reg)
{
}

void f26(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) BitLoader {
    this(Point p){
      super(p);
      set_repeating_fire(40, 60, 4);
    }
    void run(Registry reg) {
      if (count < 60) {
        move(Vector(0, 4));
      } else {
        scroll(reg);
      }
    }
  };

  reg.add(enemy);
}

void f27(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flyer {
    double x_, dx_;
    this(Point p) {
      super(p);
      degree = 180;
      x_ = p.x;
      dx_ = -5;
      set_repeating_fire(30, 100, 4);
    }
    void run(Registry reg){
      dx_ += -0.02*(pos.x - x_);
      move(Vector(dx_, 3));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f30(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p); set_capsule_drop!(OptionCapsule)();
      degree = 90;
    }
    void run(Registry reg) {
      move(Vector(-2, 0)); scroll(reg);
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f31(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p); set_capsule_drop!(OptionCapsule)();
      degree = -90;
    }
    void run(Registry reg) {
      move(Vector(2, 0)); scroll(reg);
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

class Boss1: BaseEnemy {
  this(Point pos) {
    super(SubKind.BOSS_ENEMY, pos,
          new Animation(&images.enemies.boss1.draw),
          1500);
    scale = 10;
    set_repeating_fire(400, 80, 4);
    set_destroyed_hook(&destroyed);
  }

  mixin DefineRect!(pos, 400, 100, true);
  mixin DefineEnemy!(10000);

  static const LOOP = 600;

  static void destroyed(Registry reg, Enemy e) {
    reg.add(new StageEnd(1, 2));
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
    if (count < 250) {
      move(Vector(0, 1));
      return;
    }

    if (count == 60*60 + 280 + 280) {
      destroyed(reg, this);
      die();
      return;
    }
    
    if (count > 60*60 + 280) {
      move(Vector(0, -1));
      return;
    }
    
    int ct = (count - 280)%600;
    if (ct % 3 == 0) {
      if (ct <= 150 || 450 < ct) 
        move(Vector(1, 0));
      else
        move(Vector(-1,0));
    }

    circ_8_fire(reg, Point(pos.x-120, pos.y), 40, (ct-100) % 200, 1, 0);
    circ_8_fire(reg, Point(pos.x+120, pos.y), 40, (ct-100) % 200, -1, 4);

    
  }

  void circ_8_fire(Registry reg, Point center, int limit, int count,
                   int sign, int offset) {
    if (limit < count || count % (limit/8) != 0)
      return;
    int n = (count / (limit/8) + offset)*sign;
    reg.add(new NormalBullet(center, 4*Vector(cos(PI_4*n), sin(PI_4*n))));
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 280) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

  
  
  static class Child: ChildEnemy {
    this(Point pos, Boss1 boss) {
      super(SubKind.BOSS_ENEMY, pos, Animation.empty(),
            Color.enemy, 0, 0, boss); 
    }
    
    mixin DefineRect!(pos, 30, 100, true);
    mixin DefineEnemy!(0);

    void run(Registry reg){
      if (count < 280)
        return;
      
      int ct = (count - 280)%120;
      
      if (ct <= 20 && ct % 5 == 0) {
        reg.add(new Sharp(pos));
      }
    }
  }
}

static class Sharp: BaseMovable, Bullet {
  this(Point pos) {
    super(SubKind.NORMAL_BULLET, pos);
  }

  mixin DefineRect!(pos, 6, 20, true);
    
  void draw(Registry){
    glPushMatrix();
    glColor3da(Color.bullet);
    glTranslated(pos.x, pos.y, 0);
    glRotated(90, 0, 0, 1);
    glScaled(6, 1, 1);
    images.triangle.draw();
    glPopMatrix();
  }

  void run(Registry) {
    move(Vector(0, 8));
    die_if_out_of_window();
  }
  
  void collide_with_wall(Registry, Wall) {
    die();
  }
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
    die();
  }
}

  
void f40(double x, double y, Registry reg)
{
  Boss1 boss = new Boss1(Point(x, y-100));
  reg.add(boss);
  reg.add(new Boss1.Child(Point(120, 105), boss));
  reg.add(new Boss1.Child(Point(-120, 105), boss));
  Music.boss[1].play();
}
class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun2},
  {160, &fun10},
  {160, &fun7},
  {160, &fun8},
  {160, &fun9},
  {200, &fun3},
  {200, &fun5},
  {220, &fun6},
  {220, &fun4},
  {400, &fun11},
  {400, &fun13},
  {420, &fun14},
  {420, &fun12},
  {520, &fun20},
  {520, &fun16},
  {540, &fun15},
  {540, &fun22},
  {540, &fun18},
  {560, &fun21},
  {560, &fun17},
  {580, &fun19},
  {580, &fun23},
  {760, &fun39},
  {760, &fun38},
  {940, &fun24},
  {960, &fun26},
  {960, &fun25},
  {1020, &fun28},
  {1020, &fun29},
  {1080, &fun30},
  {1080, &fun27},
  {1100, &fun31},
  {1120, &fun32},
  {1120, &fun33},
  {1180, &fun35},
  {1180, &fun36},
  {1240, &fun37},
  {1240, &fun34},
  {1380, &fun65},
  {1400, &fun64},
  {1420, &fun41},
  {1420, &fun40},
  {1600, &fun68},
  {1600, &fun67},
  {1625, &fun45},
  {1625, &fun46},
  {1640, &fun42},
  {1640, &fun43},
  {1670, &fun47},
  {1700, &fun44},
  {1880, &fun53},
  {1880, &fun52},
  {1880, &fun51},
  {1900, &fun50},
  {1980, &fun54},
  {2000, &fun55},
  {2020, &fun56},
  {2080, &fun48},
  {2080, &fun60},
  {2100, &fun58},
  {2100, &fun59},
  {2100, &fun57},
  {2120, &fun62},
  {2170, &fun49},
  {2180, &fun61},
  {2220, &fun66},
  {2260, &fun63},
  {2400, &fun70},
  {2420, &fun69},
  {2700, &fun71},
  {2740, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun2(Registry reg){
  set_scroll(reg, 1);
}
static void fun10(Registry reg){
  f02(140.000000, -20.000000, reg);
}
static void fun7(Registry reg){
  f02(60.000000, -20.000000, reg);
}
static void fun8(Registry reg){
  f02(80.000000, -20.000000, reg);
}
static void fun9(Registry reg){
  f02(120.000000, -20.000000, reg);
}
static void fun3(Registry reg){
  f01(2,460.000000, -20.000000, reg);
}
static void fun5(Registry reg){
  f01(2,380.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  f01(2,340.000000, -20.000000, reg);
}
static void fun4(Registry reg){
  f01(2,420.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  f03(100.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  f03(420.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  f03(320.000000, -20.000000, reg);
}
static void fun12(Registry reg){
  f03(200.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  f06(360.000000, -20.000000, reg);
}
static void fun16(Registry reg){
  f06(200.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  f05(280.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f06(360.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  f06(200.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  f06(360.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  f06(200.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  f06(200.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f06(360.000000, -20.000000, reg);
}
static void fun39(Registry reg){
  f10(420.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  f09(100.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  f07(80.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  f08(-1,140.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  f08(+1,20.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  f08(+1,20.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  f08(-1,140.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  f08(-1,140.000000, -20.000000, reg);
}
static void fun27(Registry reg){
  f08(+1,20.000000, -20.000000, reg);
}
static void fun31(Registry reg){
  f07(440.000000, -20.000000, reg);
}
static void fun32(Registry reg){
  f08(+1,380.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  f08(-1,500.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  f08(+1,380.000000, -20.000000, reg);
}
static void fun36(Registry reg){
  f08(-1,500.000000, -20.000000, reg);
}
static void fun37(Registry reg){
  f08(-1,500.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  f08(+1,380.000000, -20.000000, reg);
}
static void fun65(Registry reg){
  set_scroll(reg, 1);
}
static void fun64(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(280, -80, 280, 60)));
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 160, 60)));
}
static void fun68(Registry reg){
  f31(-20.000000, -20.000000, reg);
}
static void fun67(Registry reg){
  f30(580.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f20(40.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f21(300.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 380, 60)));
}
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(540, -80, 20, 60)));
}
static void fun47(Registry reg){
  f22(395.000000, -20.000000, reg);
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, -560, 40, 540)));
}
static void fun53(Registry reg){
  f24(800.000000, -20.000000, reg);
}
static void fun52(Registry reg){
  f24(740.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f24(680.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f24(660.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f24(720.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f24(780.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f26(180.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  f25(20,800.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f25(40,820.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f25(0,780.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f27(280.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f27(200.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f27(520.000000, -20.000000, reg);
}
static void fun63(Registry reg){
  f27(400.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  set_scroll(reg, 1);
}
static void fun69(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun71(Registry reg){
  f40(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun64},
  {1, &fun40},
  {1, &fun41},
  {1, &fun65},
  {180, &fun68},
  {180, &fun67},
  {205, &fun45},
  {205, &fun46},
  {220, &fun42},
  {220, &fun43},
  {250, &fun47},
  {280, &fun44},
  {460, &fun52},
  {460, &fun51},
  {460, &fun53},
  {480, &fun50},
  {560, &fun54},
  {580, &fun55},
  {600, &fun56},
  {660, &fun48},
  {660, &fun60},
  {680, &fun59},
  {680, &fun58},
  {680, &fun57},
  {700, &fun62},
  {750, &fun49},
  {760, &fun61},
  {800, &fun66},
  {840, &fun63},
  {980, &fun70},
  {1000, &fun69},
  {1280, &fun71},
  {1320, &fun1},
];
static void fun64(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 160, 60)));
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(280, -80, 280, 60)));
}
static void fun65(Registry reg){
  set_scroll(reg, 1);
}
static void fun68(Registry reg){
  f31(-20.000000, -20.000000, reg);
}
static void fun67(Registry reg){
  f30(580.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f20(40.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f21(300.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 380, 60)));
}
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(540, -80, 20, 60)));
}
static void fun47(Registry reg){
  f22(395.000000, -20.000000, reg);
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, -560, 40, 540)));
}
static void fun52(Registry reg){
  f24(740.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f24(680.000000, -20.000000, reg);
}
static void fun53(Registry reg){
  f24(800.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f24(660.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f24(720.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f24(780.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f26(180.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f25(40,820.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  f25(20,800.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f25(0,780.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f27(280.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  f22(55.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f27(200.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f27(520.000000, -20.000000, reg);
}
static void fun63(Registry reg){
  f27(400.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  set_scroll(reg, 1);
}
static void fun69(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun71(Registry reg){
  f40(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events2{
const static Event[] events = [
  {1, &fun43},
  {1, &fun40},
  {1, &fun41},
  {1, &fun42},
  {1, &fun70},
  {1, &fun44},
  {1, &fun69},
  {260, &fun71},
  {300, &fun1},
];
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(540, 720, 20, 60)));
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(0, 940, 160, 60)));
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(280, 940, 280, 60)));
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(0, 720, 380, 60)));
}
static void fun70(Registry reg){
  set_scroll(reg, 1);
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, 180, 40, 540)));
}
static void fun69(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun71(Registry reg){
  f40(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
  Events2.events,
];
