import SDL;
import std.string;
import kinds;
import util;
import std.stdio;
import base;
import images = images.src.images;
import opengl;
import glutils;
import std.typetuple;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;

void f01(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(OptionCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f02(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ReflectiveCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f03(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(BladeCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f04(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ShortRangeCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f11(double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 90, 2, 100, 20));
}

void f12(double x, double y, Registry reg)
{
  reg.add(new LFlyer(Point(x, y), 90, 2, 1000, 1000));
}

void f21(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) Wiper{
    int offset_;
    this(Point p) {
      super(p);
      offset_ = cast(int)(-p.x);
      set_repeating_fire(offset_/2, 40, 4);
    }

    void run(Registry reg) {
      scroll(reg);
      if (pos.x < 150) {
        move(Vector(3, 0));
      } else {
        move(Vector(0, -3));
        die_if_out_of_window();
      }
    }
  };
  reg.add(enemy);
}

void f22(double x, double y, Registry reg)
{
  double x_;
  Enemy enemy = new class(Point(x,y)) Wiper{
    int offset_;
    this(Point p) {
      super(p);
      x_ = p.x;
      offset_ = cast(int)(-p.x);
      set_repeating_fire(120+offset_/2, 110, 4);
    }

    void run(Registry reg) {
      scroll(reg);
      if (pos.x < 370) {
        move(Vector(3, 0));
      } else {
        move(Vector(0, -3));
        die_if_out_of_window();
      }
    }
  };
  reg.add(enemy);
}

void f23(double x, double y, Registry reg)
{
  double x_;
  Enemy enemy = new class(Point(x,y)) Wiper{
    int offset_;
    this(Point p) {
      super(p);
      offset_ = cast(int)(p.x-600);
      set_repeating_fire(120+offset_/2, 110, 4);
    }

    void run(Registry reg) {
      scroll(reg);
      if (count < 180) {
        move(Vector(-3, 0));
      } else {
        move(Vector(0, 3));
        die_if_out_of_window();
      }
    }
  };
  reg.add(enemy);
}

void f31(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 180, 1000, 1000, 4));
}

void f32(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 90, 60, 50, 4));
}

void f33(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 0, 1000, 500, 4));
}

void f34(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 270, 60, 50, 4));
}

void f41(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Patapata {
    double x_, dx_;
    this(Point p) {
      super(p);
      degree = 180;
      x_ = p.x;
      dx_ = -2;
      set_repeating_fire(50, 1000, 4);
    }
    void run(Registry reg){
      dx_ += -0.01*(pos.x - x_);
      ydegree += (dx_*3);
      move(Vector(dx_, 2));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f51(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) BitLoader {
    this(Point p){
      super(p);
      set_repeating_fire(40, 40, 5);
    }
    
    void run(Registry reg) {
      if (count < 60) {
        move(Vector(0, 3));
      } else {
        scroll(reg);
      }
    }
  };

  reg.add(enemy);
}

class Boss2: BaseEnemy {

  double dx_ = 0;
  double dy_ = 0;

  this(Point pos) {
    super(SubKind.BOSS_ENEMY, pos,
          new Animation(&images.enemies.boss2.draw),
          2500);
    scale = 10;
    set_destroyed_hook(&destroyed);
  }

  mixin DefineRect!(pos, 180, 130, true);
  mixin DefineEnemy!(15000);

  static const LOOP = 300;
  static const RETURN_SPEED = -3.0;
  static const DASH = 200;  //突撃するタイミング


  static void destroyed(Registry reg, Enemy e) {
    if (reg.root.game_config.mode == GameConfig.GameMode.TRAINING) 
      reg.add(new Reserve(SubKind.GAMEEND, 60, &reg.root.show_ending));
    else
      reg.add(new StageEnd(2, 3));
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
    //登場部分
    if (count < 250) {
      move(Vector(0, 1));
      return;
    }
    //時間切れ
    if (count >= 280 + LOOP*10) {
      move(Vector(0, RETURN_SPEED));
      if (count >= 380 + LOOP*10) {
        destroyed(reg, this);
        die();
      }
      return;
    }
    
    
    //ループ部分
    int ct = (count - 280) % LOOP;
    
    if (ct % 50 == 0 && ct < 200)
      fireToPlayer(reg, 32, pos + Vector(0,10) , 4);

    if (ct % 20 == 0 && ct < DASH) {
      dy_ = 0.0;
      auto pl = Player.search(reg);
      if (pl) {
        Point diff = pl.pos;
        if ((diff.x > pos.x  && pos.x < 450) || pos.x < 150)
          dx_ = 3.0;
        else
          dx_ = -3.0;
        }
      else {
        dx_ = 0.0;
        dy_ = RETURN_SPEED;
        }
    }
    
    if (ct % LOOP >= DASH && dy_ != RETURN_SPEED) {
      dx_ = 0.0;
      if (ct == DASH + 25)
        dy_ = 11.0;
      if (ct == DASH + 50)
        dy_ = 0.0;
      if (ct == DASH + 75)
        dy_ = -11.0;
    }
    
    move(Vector(dx_, dy_));

    if (ct < 200) {
      circ_5_fire(reg, Point(pos.x, pos.y + 20), 20, ct % 100, 1, 0);
      circ_5_fire(reg, Point(pos.x, pos.y + 20), 20, ct % 100, -1, 0);
    }

    repeat(600, 2400, {
    });
  }

  void circ_5_fire(Registry reg, Point center, int limit, int count,
                   int sign, int offset) {
    if (count < 0 || limit < count || count % (limit/4) != 0)
      return;
    int n = (count / (limit/4) + offset);
    reg.add(new NormalBullet(center, 4*Vector(sign*cos(PI_4*n), sin(PI_4*n))));
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 280) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

  
  
  static class Child: ChildEnemy {
    this(Point pos, Boss2 boss) {
      super(SubKind.BOSS_ENEMY, pos, Animation.empty(),
            Color.enemy, 0, 0, boss); 
    }
    
    mixin DefineRect!(pos, 30, 190, true);
    mixin DefineEnemy!(0);

    void run(Registry reg){
      if (count < 280)
        return;
      
      int ct = (count - 280) % LOOP;
      
      if (ct < 150)
        if (ct % 50 == 0) {
          three_way_fire(reg, Point (pos.x, pos.y + 80));
        }

    }

    void three_way_fire(Registry reg, Point center) {
      reg.add(new NormalBullet(center, degree_to_vector(4, false, 60)));
      reg.add(new NormalBullet(center, degree_to_vector(4, false, 90)));
      reg.add(new NormalBullet(center, degree_to_vector(4, false, 120)));
    }

  }
}

  
void f61(double x, double y, Registry reg)
{
  Boss2 boss = new Boss2(Point(x, y-100));
  reg.add(boss);
  reg.add(new Boss2.Child(Point(60, -10), boss));
  reg.add(new Boss2.Child(Point(-60, -10), boss));
  Music.boss[2].play();
}

class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun2},
  {80, &fun10},
  {105, &fun107},
  {105, &fun127},
  {105, &fun105},
  {105, &fun106},
  {105, &fun11},
  {120, &fun9},
  {150, &fun8},
  {305, &fun111},
  {305, &fun110},
  {305, &fun108},
  {305, &fun128},
  {305, &fun109},
  {320, &fun13},
  {350, &fun15},
  {440, &fun12},
  {505, &fun114},
  {505, &fun113},
  {505, &fun129},
  {505, &fun115},
  {505, &fun112},
  {520, &fun14},
  {550, &fun116},
  {640, &fun45},
  {640, &fun61},
  {640, &fun46},
  {640, &fun20},
  {640, &fun44},
  {640, &fun39},
  {640, &fun42},
  {640, &fun41},
  {640, &fun40},
  {640, &fun43},
  {660, &fun62},
  {670, &fun117},
  {680, &fun16},
  {680, &fun63},
  {700, &fun64},
  {710, &fun123},
  {710, &fun118},
  {720, &fun65},
  {740, &fun66},
  {750, &fun124},
  {750, &fun119},
  {760, &fun69},
  {760, &fun67},
  {780, &fun68},
  {780, &fun70},
  {800, &fun52},
  {800, &fun51},
  {800, &fun49},
  {800, &fun53},
  {800, &fun54},
  {800, &fun71},
  {800, &fun47},
  {800, &fun48},
  {800, &fun50},
  {820, &fun72},
  {840, &fun73},
  {840, &fun19},
  {860, &fun74},
  {870, &fun120},
  {880, &fun75},
  {880, &fun31},
  {880, &fun17},
  {900, &fun76},
  {900, &fun32},
  {910, &fun125},
  {910, &fun121},
  {920, &fun33},
  {940, &fun34},
  {950, &fun122},
  {950, &fun126},
  {960, &fun35},
  {980, &fun36},
  {1000, &fun85},
  {1000, &fun37},
  {1020, &fun38},
  {1020, &fun86},
  {1040, &fun23},
  {1040, &fun87},
  {1040, &fun77},
  {1040, &fun21},
  {1060, &fun88},
  {1060, &fun24},
  {1060, &fun78},
  {1080, &fun25},
  {1080, &fun18},
  {1080, &fun79},
  {1080, &fun89},
  {1100, &fun90},
  {1100, &fun80},
  {1100, &fun26},
  {1120, &fun81},
  {1120, &fun91},
  {1120, &fun27},
  {1140, &fun28},
  {1140, &fun92},
  {1140, &fun82},
  {1160, &fun83},
  {1160, &fun29},
  {1180, &fun84},
  {1180, &fun30},
  {1220, &fun58},
  {1220, &fun55},
  {1220, &fun60},
  {1220, &fun59},
  {1220, &fun57},
  {1220, &fun56},
  {1240, &fun97},
  {1240, &fun93},
  {1240, &fun94},
  {1240, &fun95},
  {1240, &fun98},
  {1240, &fun96},
  {1260, &fun103},
  {1260, &fun100},
  {1260, &fun101},
  {1260, &fun102},
  {1260, &fun99},
  {1260, &fun104},
  {1320, &fun22},
  {1460, &fun131},
  {1540, &fun130},
  {1600, &fun4},
  {1647, &fun3},
  {1700, &fun170},
  {1740, &fun150},
  {1740, &fun161},
  {1760, &fun132},
  {1780, &fun136},
  {1780, &fun160},
  {1800, &fun133},
  {1820, &fun162},
  {1840, &fun134},
  {1880, &fun135},
  {1900, &fun138},
  {1920, &fun151},
  {1940, &fun139},
  {1940, &fun137},
  {1980, &fun163},
  {2000, &fun159},
  {2020, &fun164},
  {2040, &fun158},
  {2040, &fun145},
  {2060, &fun165},
  {2080, &fun166},
  {2100, &fun142},
  {2120, &fun157},
  {2140, &fun153},
  {2140, &fun144},
  {2160, &fun167},
  {2180, &fun146},
  {2200, &fun143},
  {2200, &fun141},
  {2240, &fun168},
  {2280, &fun140},
  {2300, &fun147},
  {2320, &fun152},
  {2320, &fun148},
  {2360, &fun169},
  {2380, &fun149},
  {2600, &fun6},
  {2620, &fun5},
  {2640, &fun156},
  {2700, &fun172},
  {2700, &fun171},
  {2740, &fun154},
  {2740, &fun155},
  {3220, &fun7},
  {3280, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun2(Registry reg){
  set_scroll(reg, 1);
}
static void fun10(Registry reg){
  f01(460.000000, -20.000000, reg);
}
static void fun107(Registry reg){
  f31(280.000000, -20.000000, reg);
}
static void fun127(Registry reg){
  f31(360.000000, -20.000000, reg);
}
static void fun105(Registry reg){
  f31(120.000000, -20.000000, reg);
}
static void fun106(Registry reg){
  f31(200.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  f31(40.000000, -20.000000, reg);
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 400, 60)));
}
static void fun8(Registry reg){
  f32(415.000000, -20.000000, reg);
}
static void fun111(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun110(Registry reg){
  f31(340.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  f31(180.000000, -20.000000, reg);
}
static void fun128(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun109(Registry reg){
  f31(260.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(160, -80, 400, 60)));
}
static void fun15(Registry reg){
  f34(144.000000, -20.000000, reg);
}
static void fun12(Registry reg){
  f01(60.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  f31(200.000000, -20.000000, reg);
}
static void fun113(Registry reg){
  f31(120.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f31(360.000000, -20.000000, reg);
}
static void fun115(Registry reg){
  f31(280.000000, -20.000000, reg);
}
static void fun112(Registry reg){
  f31(40.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 400, 60)));
}
static void fun116(Registry reg){
  f32(415.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f21(-383.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f21(-400.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(240, -160, 60, 140)));
}
static void fun44(Registry reg){
  f21(-360.000000, -20.000000, reg);
}
static void fun39(Registry reg){
  f21(-260.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  f21(-320.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  f21(-300.000000, -20.000000, reg);
}
static void fun40(Registry reg){
  f21(-280.000000, -20.000000, reg);
}
static void fun43(Registry reg){
  f21(-340.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun117(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(0, -120, 60, 100)));
}
static void fun63(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun64(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun123(Registry reg){
  f32(75.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun65(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun124(Registry reg){
  f32(75.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun69(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun67(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun68(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun52(Registry reg){
  f22(-280.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f22(-260.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  f22(-220.000000, -20.000000, reg);
}
static void fun53(Registry reg){
  f22(-300.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f22(-320.000000, -20.000000, reg);
}
static void fun71(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun47(Registry reg){
  f22(-200.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f22(-180.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f22(-240.000000, -20.000000, reg);
}
static void fun72(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun73(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(240, -160, 60, 140)));
}
static void fun74(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun75(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun31(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(0, -120, 60, 100)));
}
static void fun76(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun32(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun125(Registry reg){
  f32(75.000000, -20.000000, reg);
}
static void fun121(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun122(Registry reg){
  f34(224.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  f32(75.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun36(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun85(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun37(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun86(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun77(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(240, -160, 60, 140)));
}
static void fun88(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, -120, 60, 100)));
}
static void fun79(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun89(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun90(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun91(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun27(Registry reg){
  f02(140.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun92(Registry reg){
  f12(500.000000, -20.000000, reg);
}
static void fun82(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun83(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun84(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  f23(660.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f23(720.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f23(620.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f23(640.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f23(680.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f23(700.000000, -20.000000, reg);
}
static void fun97(Registry reg){
  f23(660.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  f23(740.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  f23(720.000000, -20.000000, reg);
}
static void fun95(Registry reg){
  f23(700.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f23(640.000000, -20.000000, reg);
}
static void fun96(Registry reg){
  f23(680.000000, -20.000000, reg);
}
static void fun103(Registry reg){
  f23(680.000000, -20.000000, reg);
}
static void fun100(Registry reg){
  f23(740.000000, -20.000000, reg);
}
static void fun101(Registry reg){
  f23(720.000000, -20.000000, reg);
}
static void fun102(Registry reg){
  f23(700.000000, -20.000000, reg);
}
static void fun99(Registry reg){
  f23(760.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f23(660.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 420, 60)));
}
static void fun131(Registry reg){
  reg.add(new NormalWall(Rect(540, -1240, 20, 1220)));
}
static void fun130(Registry reg){
  reg.add(new NormalWall(Rect(0, -1160, 20, 1140)));
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun170(Registry reg){
  f03(340.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(320, -40, 200, 20)));
}
static void fun161(Registry reg){
  f41(80.000000, -20.000000, reg);
}
static void fun132(Registry reg){
  f41(100.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f41(160.000000, -20.000000, reg);
}
static void fun160(Registry reg){
  f41(240.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun162(Registry reg){
  f41(140.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f41(200.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f41(100.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f41(160.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  f51(420.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f41(220.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f41(80.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f41(240.000000, -20.000000, reg);
}
static void fun164(Registry reg){
  f41(180.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  f41(120.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f41(220.000000, -20.000000, reg);
}
static void fun165(Registry reg){
  f41(320.000000, -20.000000, reg);
}
static void fun166(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f41(380.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(40, -40, 200, 20)));
}
static void fun144(Registry reg){
  f41(280.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  f41(400.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f41(380.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f41(320.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f41(480.000000, -20.000000, reg);
}
static void fun168(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f41(360.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f41(420.000000, -20.000000, reg);
}
static void fun152(Registry reg){
  f51(140.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f41(300.000000, -20.000000, reg);
}
static void fun169(Registry reg){
  f41(340.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  set_scroll(reg, 1);
}
static void fun5(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun156(Registry reg){
  reg.add(new NormalWall(Rect(100, -40, 360, 20)));
}
static void fun172(Registry reg){
  f01(60.000000, -20.000000, reg);
}
static void fun171(Registry reg){
  f04(490.000000, -20.000000, reg);
}
static void fun154(Registry reg){
  reg.add(new NormalWall(Rect(60, -40, 100, 20)));
}
static void fun155(Registry reg){
  reg.add(new NormalWall(Rect(400, -40, 100, 20)));
}
static void fun7(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun18},
  {1, &fun3},
  {1, &fun4},
  {1, &fun131},
  {1, &fun130},
  {1, &fun22},
  {1, &fun9},
  {1, &fun13},
  {1, &fun14},
  {1, &fun16},
  {1, &fun17},
  {1, &fun21},
  {1, &fun19},
  {1, &fun20},
  {33, &fun170},
  {73, &fun161},
  {73, &fun150},
  {93, &fun132},
  {113, &fun160},
  {113, &fun136},
  {133, &fun133},
  {153, &fun162},
  {173, &fun134},
  {213, &fun135},
  {233, &fun138},
  {253, &fun151},
  {273, &fun137},
  {273, &fun139},
  {313, &fun163},
  {333, &fun159},
  {353, &fun164},
  {373, &fun158},
  {373, &fun145},
  {393, &fun165},
  {413, &fun166},
  {433, &fun142},
  {453, &fun157},
  {473, &fun153},
  {473, &fun144},
  {493, &fun167},
  {513, &fun146},
  {533, &fun141},
  {533, &fun143},
  {573, &fun168},
  {613, &fun140},
  {633, &fun147},
  {653, &fun152},
  {653, &fun148},
  {693, &fun169},
  {713, &fun149},
  {933, &fun6},
  {953, &fun5},
  {973, &fun156},
  {1033, &fun171},
  {1033, &fun172},
  {1073, &fun155},
  {1073, &fun154},
  {1553, &fun7},
  {1613, &fun1},
];
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, 467, 60, 100)));
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun131(Registry reg){
  reg.add(new NormalWall(Rect(540, -1033, 20, 1220)));
}
static void fun130(Registry reg){
  reg.add(new NormalWall(Rect(0, -1033, 20, 1140)));
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(140, 267, 420, 60)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, 1467, 400, 60)));
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(160, 1267, 400, 60)));
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, 1067, 400, 60)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(0, 867, 60, 100)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(0, 667, 60, 100)));
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(240, 467, 60, 140)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(240, 667, 60, 140)));
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(240, 867, 60, 140)));
}
static void fun170(Registry reg){
  f03(340.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f41(80.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(320, -40, 200, 20)));
}
static void fun132(Registry reg){
  f41(100.000000, -20.000000, reg);
}
static void fun160(Registry reg){
  f41(240.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f41(160.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun162(Registry reg){
  f41(140.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f41(200.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f41(100.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f41(160.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  f51(420.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f41(80.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f41(220.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f41(240.000000, -20.000000, reg);
}
static void fun164(Registry reg){
  f41(180.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  f41(120.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f41(220.000000, -20.000000, reg);
}
static void fun165(Registry reg){
  f41(320.000000, -20.000000, reg);
}
static void fun166(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f41(380.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  f41(60.000000, -20.000000, reg);
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(40, -40, 200, 20)));
}
static void fun144(Registry reg){
  f41(280.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  f41(400.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f41(380.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f41(480.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f41(320.000000, -20.000000, reg);
}
static void fun168(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f41(360.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f41(420.000000, -20.000000, reg);
}
static void fun152(Registry reg){
  f51(140.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f41(300.000000, -20.000000, reg);
}
static void fun169(Registry reg){
  f41(340.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  f41(440.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  set_scroll(reg, 1);
}
static void fun5(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun156(Registry reg){
  reg.add(new NormalWall(Rect(100, -40, 360, 20)));
}
static void fun171(Registry reg){
  f04(490.000000, -20.000000, reg);
}
static void fun172(Registry reg){
  f01(60.000000, -20.000000, reg);
}
static void fun155(Registry reg){
  reg.add(new NormalWall(Rect(400, -40, 100, 20)));
}
static void fun154(Registry reg){
  reg.add(new NormalWall(Rect(60, -40, 100, 20)));
}
static void fun7(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events2{
const static Event[] events = [
  {1, &fun20},
  {1, &fun5},
  {1, &fun6},
  {1, &fun153},
  {1, &fun9},
  {1, &fun13},
  {1, &fun14},
  {1, &fun16},
  {1, &fun17},
  {1, &fun150},
  {1, &fun19},
  {1, &fun156},
  {1, &fun21},
  {1, &fun22},
  {1, &fun130},
  {1, &fun131},
  {1, &fun18},
  {60, &fun171},
  {60, &fun172},
  {100, &fun155},
  {100, &fun154},
  {580, &fun7},
  {640, &fun1},
];
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(240, 1840, 60, 140)));
}
static void fun5(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun6(Registry reg){
  set_scroll(reg, 1);
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(40, 460, 200, 20)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, 2440, 400, 60)));
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(160, 2240, 400, 60)));
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, 2040, 400, 60)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(0, 1840, 60, 100)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(0, 1640, 60, 100)));
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(320, 860, 200, 20)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(240, 1640, 60, 140)));
}
static void fun156(Registry reg){
  reg.add(new NormalWall(Rect(100, -40, 360, 20)));
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(240, 1440, 60, 140)));
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(140, 1240, 420, 60)));
}
static void fun130(Registry reg){
  reg.add(new NormalWall(Rect(0, -60, 20, 1140)));
}
static void fun131(Registry reg){
  reg.add(new NormalWall(Rect(540, -60, 20, 1220)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, 1440, 60, 100)));
}
static void fun171(Registry reg){
  f04(490.000000, -20.000000, reg);
}
static void fun172(Registry reg){
  f01(60.000000, -20.000000, reg);
}
static void fun155(Registry reg){
  reg.add(new NormalWall(Rect(400, -40, 100, 20)));
}
static void fun154(Registry reg){
  reg.add(new NormalWall(Rect(60, -40, 100, 20)));
}
static void fun7(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
  Events2.events,
];
