import kinds;
import util;
import base;
import images = images.src.images;
import opengl;
import glutils;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;


private class DotWallFront: BaseMovable, Wall
{
  this(Point pos, DotOuter parent) {
    super(SubKind.WALL, pos, parent);
  }
  mixin DefineRect!(pos, 60, 20, true);
  void run(Registry reg){
    if (parent.dead) die();
  }
}

private class DotWallSide: BaseMovable, Wall
{
  this(Point pos, DotOuter parent) {
    super(SubKind.WALL, pos, parent);
  }
  mixin DefineRect!(pos, 20, 40, true);
  void run(Registry reg){
    if (parent.dead) die();
  }
}

private class DotOuterTail: ChildEnemy
{
  this(Point pos, DotOuter parent) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.circle.draw),
          Color.enemy, 0, 2.0, parent);
  }

  mixin DefineEnemy!(0);
  mixin DefineRect!(pos, 20, 20, true);
  
  void run(Registry reg) {
    ydegree += 10;
  }
}

private class DotOuterFrame: ChildEnemy
{
  this(DotOuter parent, int ydeg) {
    super(SubKind.NORMAL_EFFECT, Point(0, 0),
          new Animation(&images.enemies.dotouter.draw),
          Color.enemy, 0, 2.0, parent);
    ydegree = ydeg;
  }
  
  mixin DefineEnemy!(0);
  mixin DefineRect!(pos, 2, 2, true);

  void run(Registry) {
    ydegree -= 5;
  }
}

private class DotOuter: BaseEnemy
{
private:
  Vector v_;
public:
  this(Point pos, Vector v) {
    super(SubKind.NORMAL_ENEMY, pos, Animation.empty, 80);
    scale = 2.5;
    v_ = v;
  }

  mixin DefineEnemy!(500);
  mixin DefineRect!(pos, 30, 30, true);

  void run(Registry reg) {
    move(v_);
    die_if_out_of_window();
    if (count % 60 == 30) {
      reg.add(new DotBlade(pos));
      reg.add(new DotBlade(pos+Vector(24, -35)));
      reg.add(new DotBlade(pos+Vector(18, -25)));
      reg.add(new DotBlade(pos+Vector(12, -10)));
      reg.add(new DotBlade(pos+Vector(6, -20)));
      reg.add(new DotBlade(pos+Vector(-6, -20)));
      reg.add(new DotBlade(pos+Vector(-12, -10)));
      reg.add(new DotBlade(pos+Vector(-18, -25)));
      reg.add(new DotBlade(pos+Vector(-24, -35)));
      
    }
  }
}

private class DotBlade: LBullet
{
  this(Point pos) {
    super(pos, Vector(0, 5),
          new Animation(&images.enemies.dotblade.draw),
          false);
    scale = 1;
  }

  mixin DefineRect!(pos, 2, 40, true);
  
  void collide_with_wall(Registry, Wall) {
  }
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
}

private class SlowChaser: BaseEnemy
{
private:
  int deg_;
  Vector v_;
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos, Animation.empty, 10);
    deg_ = 0;
    v_ = Vector(0, 0);
    set_repeating_fire(80, 120, 6);
  }

  void run(Registry reg) {
    static double d(double x) {
      if (fabs(x) < 22) 
        return 0;
      else
        return sign(x);
    }
    
    Player pl = Player.search(reg);
    deg_ -= 5;
    if (!pl) {
      scroll(reg);
      return;
    }
    if (count % 20 == 3) {
      v_.x = d(pl.pos.x - pos.x);
      v_.y = d(pl.pos.y - pos.y);
    }
    move(v_);
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 30, 30, true);
  
  void draw(Registry reg) {
    glColor3da(Color.enemy);
    glPushMatrix();
      glTranslated(pos.x, pos.y, 0);
      glPushMatrix();
        glRotated(deg_, 0, 0, 1);
        for (int i=0; i<4; i++) {
          glRotated(45, 0, 0, 1);
          glPushMatrix();
            glScaled(1.1, 1.1, 1);
            images.enemies.dotblade.draw();
          glPopMatrix();
        }
      glPopMatrix();
      glScaled(1.5, 1.5, 1);
      images.player.octagon.draw();
    glPopMatrix();
    if (reg.root.game_config.draw_bounding_box) {
      glColor3d(0.5, 0.5, 0.5);
      rect().draw();
    }
    super.draw(reg);
    
  }
}

void dotouter(double x, double y, Registry reg)
{
  DotOuter dotouter = new DotOuter(Point(x,y), Vector(0, 2));
  reg.add(dotouter);
  reg.add(new DotOuterFrame(dotouter, 0));
  reg.add(new DotOuterFrame(dotouter, 90));
  reg.add(new DotWallFront(Vector(0, 30), dotouter));
  reg.add(new DotWallSide(Vector(-30, 0), dotouter));
  reg.add(new DotWallSide(Vector(30, 0), dotouter));
  reg.add(new DotOuterTail(Vector(0, -30), dotouter));
}

void slowchaser(double dy, double x, double y, Registry reg)
{
  reg.add(new SlowChaser(Point(x,y+dy)));
}

private void capsule(CapsuleClass)(double x, double y, Registry reg)
{
  Enemy enemy = new class() CapsuleCarrier {
    this() {
      super(Point(x,y));
      set_capsule_drop!(CapsuleClass)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

private class WiperCreator: BaseMovable {
private:
  Vector v_;
  int start_, interval_, end_;
  int fire_start_, fire_rate_, fire_speed_;
  bool scroll_;
public:
  this(Point p, Vector v, int start, int interval, int end, bool scroll = true) {
    super(SubKind.NORMAL_ENEMY_CREATOR, p);
    v_ = v; start_ = start; interval_ = interval; end_ = end;
    scroll_ = scroll;
  }

  void run(Registry reg) {
    if (scroll_)
      scroll(reg);
    if (count < start_)
      return;
    if (count >= end_) {
      die();
      return;
    }

    if ((count - start_) % interval_ == 0) {
      Enemy enemy = new class(pos, v_) Wiper {
        Vector u_;
        this(Point p, Vector u){
          super(p); u_ = u;
        };
        void run(Registry reg) {
          move(u_);
          die_if_out_of_window();
        }
      };
      reg.add(enemy);
    }
  }
}

void f01(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(-3, 1), 0, 20, 100));
}

void f02(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(4, 1), 0, 10, 100));
}
 
void f03(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(-4, -4), 0, 10, 120, false));
}

void f04(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(-4, 4), 0, 10, 120, false));
}

void f05(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(0, -4), 0, 10, 70, false));
}

void f06(int dy, double x, double y, Registry reg)
{
  reg.add(new WiperCreator(Point(x, y+dy), Vector(0, 4), 0, 10, 70, false));
}

private class LineBox: BaseWorker, Bullet {
private:
  Rect rect_;
  double rat_, w_, h_;
  Movable p1_, p2_;
  int timeout_;
public:
  this(Movable p1, Movable p2, double rat, double w, double h, int timeout = -1) {
    super(SubKind.NORMAL_BULLET);
    p1_ = p1; p2_ = p2; rat_ = rat; w_ = w; h_ = h;
    set_rect();
    timeout_ = timeout;
  }

  void run(Registry reg) {
    if (p1_.dead || p2_.dead || timeout_ == count) 
      die();

    set_rect();
  }

  void set_rect() {
    Point center = rat_*p1_.pos + (1-rat_)*p2_.pos;
    rect_.p = center - Vector(w_/2, h_/2);
    rect_.q = center + Vector(w_/2, h_/2);
  }
  
  Rect rect(){ return rect_; }
  void collide_with_wall(Registry, Wall){};
  void collide_with_charge_blocker(Registry, ChargeBlocker){};

  static void create(Registry reg, Movable p1, Movable p2, int n, int w, int h, int timeout) {
    for (int i=0; i<=n ;i++) 
      reg.add(new LineBox(p1, p2, cast(double)i/n, w, h, timeout));
  }

  void draw(Registry reg) {
    if (reg.root.game_config.draw_bounding_box) {
        glColor3d(0.5, 0.5, 0.5);
        rect().draw();
      }
  }
}

private void draw_zigzag(Point p, Point q)
{
  int len = cast(int)((p-q).abs/20);
  for (int i=0; i<2; i++) {
    glBegin(GL_LINE_STRIP);
    for (int j=0; j<=len; j++) {
      double r = cast(double)j/len;
      (r*p + (1-r)*q + Vector(drand(-20, 20), drand(-20, 20))).glVertex();
    }
    glEnd();
  }
}

private class ZLazer: BaseWorker {
private:
  Movable p1_, p2_;
  int timeout_;
public:
  this(Movable p1, Movable p2, int timeout = -1) {
    super(SubKind.NORMAL_EFFECT);
    p1_ = p1; p2_ = p2; timeout_ = timeout;
  }

  void run(Registry reg) {
    if (p1_.dead || p2_.dead || count == timeout_)
      die();
  }
  
  void draw(Registry reg) {
    Point p = p1_.pos;
    Point q = p2_.pos;

    glColor3da(Color.bullet);
    draw_zigzag(p, q);
  }
  
  static void create(Registry reg, Movable p1, Movable p2, int timeout=-1) {
    reg.add(new ZLazer(p1, p2, timeout));
    LineBox.create(reg, p1, p2, 30, 20, 20, timeout);
  }
}

private void test(double x, double y, Registry reg)
{
  NormalBullet p1 = new NormalBullet(Point(-20, 400), Vector(3, 0));
  NormalBullet p2 = new NormalBullet(Point(580, 100), Vector(-3, 0));
  reg.add(p1); reg.add(p2);
  ZLazer.create(reg, p1, p2);
  //LineBox.create(reg, p1, p2, 30, 20, 20);
}

private void boss(double x, double y, Registry reg)
{
  Boss3BCenter ctr = new Boss3BCenter();
  Boss3B bb1, bb2;
  Boss3A boss3a;
  reg.add(boss3a = new Boss3A());
  reg.add(ctr);
  reg.add(bb1 = new Boss3B(1, ctr));
  reg.add(bb2 = new Boss3B(-1, ctr));
  bb1.brother = bb2;
  bb2.brother = bb1;
  reg.add(new Boss3Checker());
  Movable pl, pbl, pr, pbr;
  reg.add(pl = new Placer(Point(-700, 0), boss3a));
  reg.add(pbl = new Placer(Point(-30, 0), boss3a));
  reg.add(pr = new Placer(Point(700, 0), boss3a));
  reg.add(pbr = new Placer(Point(30, 0), boss3a));
  ZLazer.create(reg, pl, pbl);
  ZLazer.create(reg, pr, pbr);
  Music.boss[3].play();
}

private class Placer: BaseMovable {
  this(Point pos, Movable parent) {
    super(SubKind.NORMAL_EFFECT, pos, parent);
  }

  void run(Registry) {
    if (parent.dead)
      die();
  }
}

class Boss3Bomb: Animatable {
private:
  Vector v_;
public:
  this(Point p) {
    super(SubKind.NORMAL_EFFECT, p,
          new Animation(&images.enemies.boss3_bomb.draw),
          Color.bullet);
    scale = 2.5;
    v_ = Vector(0, 2);
  }

  void run(Registry reg) {
    ydegree += 4;
    move(v_);

    if (count > 90)
      v_.y -= 2.0/30;
    if (count == 120) {
      reg.add(new Boss3BombExplosion(pos));
      die();
    }
  }
}

private class Boss3BombExplosion: BaseMovable, Bullet {
private:
  Rect rect_;
public:
  this(Point p) {
    super(SubKind.NORMAL_BULLET, p);
    rect_.p = rect_.q = p;
  }

  void run(Registry reg) {
    if (count == 0)
      Wave.bomb3.play();

    if (count < 60) {
      rect_ = Rect(pos.x, pos.y - count*6, 0, count*12);
      return;
    }
    
    if (80 < count && count < 90) {
      int d = (count - 80)*12;
      rect_ = Rect(pos.x - d, -20, 2*d, 600);
    }

    if (count == 90)
      Wave.zigzag.play();

    if (count == 130) 
      die();
  }

  void draw(Registry reg) {
    glColor3da(Color.bullet);
    rect_.draw();
    if (count > 90) {
      for (int i=-2; i<=2; i++) {
        double x = i*50 + pos.x;
        draw_zigzag(Point(x, -20), Point(x, 500));
      }
    }
  }
  
  Rect rect(){ return rect_; }
  void collide_with_wall(Registry, Wall) {
  }
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
}

private class Boss3Checker: BaseWorker {
  this() {
    super(SubKind.BOSS3CHEKER);
  }

  void run(Registry reg) {
    bool die_boss_a_and_b = reg.search(SubKind.BOSS3_A) == null &&
      reg.search(SubKind.BOSS3_B) == null;
    bool timeout = count > 60 + 710*20 + 60;
      
    if (die_boss_a_and_b || timeout) {
      reg.add(new StageEnd(3, 4));
    }
  }
}

private class Boss3A: BaseEnemy {
private:
  int bomb_;
  void delegate(Registry, int) fire_;
  
public:
  this() {
    super(SubKind.BOSS3_A, Point(280, -80),
          new Animation(&images.enemies.boss3_a.draw),
          1500);
    scale = 6.0;
    bomb_ = 3;
  }

  void run(Registry reg) {
    if (count < 40) {
      move(Vector(0, 3));
      return;
    }

    if (count > 60 + 710*20) {
      move(Vector(0, -3));
      return;
    }
    
    
    int t = (count-60)%710;

    if (t == 1) {
      if (reg.search(SubKind.BOSS3_B) == null)
        fire_ = &fire_both;
      else
        fire_ = reg.root.rng.brand() ? &fire_blade : &fire_bomb;
      bomb_ = reg.root.rng.irand(0, 15);
    }
    
    if (t < 70) return;
    if ((t-70)%40 < 20) {
      switch ((t-70)/40) {
      case 0, 1, 6, 7, 8, 9, 14, 15:
        move(Vector(-6, 0));
        break;
      case 2, 3, 4, 5, 10, 11, 12, 13:
        move(Vector(6, 0));
        break;
      }
    }

    
    if ((t-70)%40 == 30) {
      fire_(reg, (t-70)/40);
      
    }
    if (t == 450 || (fire_ == &fire_both && t%50 == 0)) {
      NormalBullet.fireToPlayer(reg, 64, pos + Vector(20, -10), 4);
      NormalBullet.fireToPlayer(reg, 64, pos + Vector(-20, -10), 4);
    }
  }

  void fire_blade(Registry reg, int n) {
    reg.add(new DotBlade(pos));
    reg.add(new DotBlade(pos+Vector(24, -35)));
    reg.add(new DotBlade(pos+Vector(18, -25)));
    reg.add(new DotBlade(pos+Vector(12, -10)));
    reg.add(new DotBlade(pos+Vector(6, -20)));
    reg.add(new DotBlade(pos+Vector(-6, -20)));
    reg.add(new DotBlade(pos+Vector(-12, -10)));
    reg.add(new DotBlade(pos+Vector(-18, -25)));
    reg.add(new DotBlade(pos+Vector(-24, -35)));
  }

  void fire_bomb(Registry reg, int n) {
    if (n == bomb_)
      reg.add(new Boss3Bomb(pos));
  }

  void fire_both(Registry reg, int n) {
    fire_bomb(reg, n);
    fire_blade(reg, n);
  }
  
  mixin DefineEnemy!(1000);
  mixin DefineRect!(pos, 70, 50, true);
  
  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 80) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }
}

private class Boss3BCenter: BaseEnemy {
public:
  int rot_sign;
  bool type0;
  
  this() {
    super(SubKind.BOSS3_B, Point(0, 0),
          Animation.empty(), 1800);
  }
  mixin DefineEnemy!(2000);
  mixin DefineRect!(pos, 0, 0, true);

  void run(Registry reg) {
    if ((count - 60)%710 == 200 || count==1)
      update_next_pattern(reg);
  }

  void update_next_pattern(Registry reg) {
    rot_sign = reg.root.rng.brand() ? 1 : -1;
    type0 = reg.root.rng.brand();
  }
}

private class Boss3B: ChildEnemy {
private:
  int sign_;
  int rot_sign_;
  int start_;
  void delegate(Registry reg) run_;
  Boss3B brother_;
  
public:
  this(int sign, Movable center) {
    super(SubKind.BOSS_ENEMY, Point(280+sign*320, 400),
          new Animation(&images.enemies.boss3_b.draw),
          Color.enemy, 0, 3.0, center);
    sign_ = sign;
    degree = sign_ * 120;
    reset(&wait_1sec);
  }
  
  mixin DefineEnemy!(0);
  mixin DefineRect!(pos, 40, 40, true);

  void reset(void delegate(Registry reg) run) {
    start_ = count;
    run_ = run;
  }

  void brother(Boss3B br) { brother_ = br; }
  
  void run(Registry reg) {
    if (count < 60) {
      move(Vector(-2*sign_, 0));
      return;
    }
    if (count > 60 + 710*20) {
      move(Vector(2*sign_, 0));
      return;
    }

    run_(reg);
    
    if (parent.dead)
      reg.add(new DestroyedEnemy(pos, true, 40));
  }

  int ct(){ return count-start_; }

  void wait(Registry reg) {}

  void wait_1sec(Registry reg) {
    if (ct == 60) reset(&next_pattern);
  }
  
  void run_0_0(Registry reg) {
    double r = ct/40.0;
    Point p = Point(280+sign_*200, 400);
    Point q = Point(280+sign_*240, 300);
    degree = (1-r)*sign_*120+r*sign_*90;
    pos = (1-r)*p + r*q;
    if (ct == 40) reset(&run_0_1);
  }

  void run_0_1(Registry reg) {
    if (ct == 1 && sign_ > 0) {
      ZLazer.create(reg, this, brother_, 620);
    }
    if (ct == 20) reset(&run_0_2);
  }

  void rotate(Registry reg) {
    double d = rot_sign_*360*ct/600 + (sign_>0?0:180);
    pos = Point(280, 300) + Point.from_polar_coord(240, PI*d/180);
    degree = d+90;
  }
    
  void run_0_2(Registry reg) {
    rotate(reg);
    int t = ct % 60;
    if (t==5 || t==7 || t==9) {
      for (int i=0; i<4; i++) 
        reg.add(new StripBullet(pos, Vector.from_polar_coord(4.5, -(ct/20.0+PI*i/2)*sign_)));
    }
    if (ct == 600) reset(&run_0_3);
  }

  void run_0_3(Registry reg) {
    if (ct == 9) reset(&run_0_4);
  }
  
  void run_0_4(Registry reg) {
    double r = (ct)/40.0;
    Point p = Point(280+sign_*200, 400);
    Point q = Point(280+sign_*240, 300);
    degree = r*sign_*120+(1-r)*sign_*90;
    pos = r*p + (1-r)*q;
    if (ct == 40) reset(&next_pattern);
  }
  
  void rot_strip(Registry reg) {
    if ((ct % 10) == 0 && ((ct % 120)<70)) {
      for (int i=0; i<2; i++) 
        reg.add(new StripBullet(pos, Vector.from_polar_coord(4.5, -(ct/100.0+PI*i)*sign_)));
    }
  }
  
  void run_1_0(Registry reg) {
    rot_strip(reg);

    if (ct == 60 && sign_ > 0) {
      ZLazer.create(reg, this, brother_, 600);
    }
    if (ct == 709) reset(&next_pattern);
  }

  void next_pattern(Registry reg) {
    Boss3BCenter ctr = cast(Boss3BCenter)parent;
    rot_sign_ = ctr.rot_sign;

    if (reg.search(SubKind.BOSS3_A) == null)
      reset(&run_2_0);
    else
      reset(ctr.type0 ? &run_0_0 : &run_1_0);
  }

  void run_2_0(Registry reg) {
    if (sign_ > 0)
      ZLazer.create(reg, this, brother_, -1);
    reset(&run_2_1);
  }

  void run_2_1(Registry reg) {
    double r = ct/40.0;
    Point p = Point(280+sign_*200, 400);
    Point q = Point(280+sign_*240, 300);
    degree = (1-r)*sign_*120+r*sign_*90;
    pos = (1-r)*p + r*q;
    if (ct == 40) reset(&run_2_2);
  }
  
  void run_2_2(Registry reg) {
    rotate(reg);
    rot_strip(reg);
  }
  
  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 80) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }
}
class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun2},
  {60, &fun8},
  {80, &fun10},
  {80, &fun9},
  {180, &fun7},
  {360, &fun3},
  {400, &fun11},
  {400, &fun4},
  {440, &fun5},
  {460, &fun12},
  {480, &fun6},
  {640, &fun28},
  {640, &fun13},
  {660, &fun14},
  {740, &fun17},
  {760, &fun16},
  {780, &fun24},
  {800, &fun15},
  {820, &fun25},
  {1080, &fun19},
  {1080, &fun20},
  {1100, &fun22},
  {1100, &fun18},
  {1120, &fun21},
  {1260, &fun23},
  {1400, &fun27},
  {1420, &fun26},
  {1440, &fun29},
  {1820, &fun30},
  {1840, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun2(Registry reg){
  set_scroll(reg, 1);
}
static void fun8(Registry reg){
  capsule!(OptionCapsule)(300.000000, -20.000000, reg);
}
static void fun10(Registry reg){
  f02(0,-20.000000, -20.000000, reg);
}
static void fun9(Registry reg){
  f01(300,580.000000, -20.000000, reg);
}
static void fun7(Registry reg){
  slowchaser(0,220.000000, -20.000000, reg);
}
static void fun3(Registry reg){
  dotouter(40.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  slowchaser(280,-20.000000, -20.000000, reg);
}
static void fun4(Registry reg){
  dotouter(200.000000, -20.000000, reg);
}
static void fun5(Registry reg){
  dotouter(360.000000, -20.000000, reg);
}
static void fun12(Registry reg){
  slowchaser(550,440.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  dotouter(520.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  capsule!(ShortRangeCapsule)(200.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  f03(550, 520.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  f04(0,500.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  f05(550,400.000000, -20.000000, reg);
}
static void fun16(Registry reg){
  slowchaser(280,580.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  slowchaser(550,80.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  dotouter(120.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  capsule!(ReflectiveCapsule)(440.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  f02(300,-20.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  slowchaser(0,580.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f06(0,120.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  dotouter(440.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  slowchaser(0,240.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  slowchaser(0,0.000000, -20.000000, reg);
}
static void fun27(Registry reg){
  set_scroll(reg, 1);
}
static void fun26(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun29(Registry reg){
  capsule!(OptionCapsule)(260.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  boss(160.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun27},
  {1, &fun26},
  {1, &fun29},
  {380, &fun30},
  {400, &fun1},
];
static void fun27(Registry reg){
  set_scroll(reg, 1);
}
static void fun26(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun29(Registry reg){
  capsule!(OptionCapsule)(260.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  boss(160.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
];
