import kinds;
import util;
import base;
import images = images.src.images;
import opengl;
import glutils;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;

class HomingHeli : Helicopter {
private:
  double speed_;
  double change_degree_;
  int ct_;

public:
  this(Point p, Registry reg, double speed, double deg, double change_degree, int ct, int firerate, int fire_first, int fire_speed = 4) {
    super(p, reg);
    set_repeating_fire(fire_first, firerate, fire_speed);
    speed_ = speed;
    degree = deg;
    change_degree_ = change_degree;
    ct_ = ct;
  }

  void run(Registry reg) {
    degree = homing(degree, change_degree_, reg);

    move_forward(speed_);

    if(count > ct_) {
      die_if_out_of_window();
    }
  }

  void collide_with_wall(Registry reg, Wall wall) {
    if (!dead()) {
      Wave.enemy_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().w));
      die();
    }
  }
};

class NormalTop: Top {
private:
  double dx_;
  double dy_ = 1;
public:
  this(Point p, double dx, int fire_first = 10000, int fire_rate = 10000, int fire_speed = 3) {
    super(p);
    dx_ = dx;
    set_repeating_fire(fire_first, fire_rate, fire_speed);
  }
  void run(Registry reg){
    move(Vector(dx_, dy_));
    dy_ += 0.2;
    scroll(reg);
    if (count > 100)
      die_if_out_of_window();
  }
  void collide_with_wall(Registry reg, Wall) {
    move(Vector(0, -dy_));  //横の壁に当たらないようにすること
    dy_ = 0;
  }
}

void f01(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ShortRangeCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 3)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f02(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ReflectiveCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 3)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f03(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(FiveWayCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 3)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f04(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(BladeCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 3)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f05(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(OptionCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 3)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f11(double x, double y, Registry reg)
{
  reg.add(new HomingHeli(Point(x, y), reg, 7, 180, 1, 0, 100, 40, 8));
}

void f12(double x, double y, Registry reg)
{
  reg.add(new HomingHeli(Point(x, y), reg, 7, 180, 1, 0, 10000, 10000));
}

void f13(double x, double y, Registry reg)
{
  reg.add(new HomingHeli(Point(x, y), reg, 5, 180, 2, 30, 50, 30, 6));
}

void f21(double x, double y, Registry reg)
{
  reg.add(new Shutter(Point(x, y), 420+x-550, 0));
}

void f22(double x, double y, Registry reg)
{
  reg.add(new Shutter(Point(x, y), 180+x, 180));
}

void f31(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flyer {
    this(Point p) {
      super(p);
      degree = 270;
    }
    void run(Registry reg){
      move(Vector(-5, 0));
      scroll(reg);
      if (count > 100)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f32(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flyer {
    this(Point p) {
      super(p);
      degree = 90;
    }
    void run(Registry reg){
      move(Vector(5, 0));
      scroll(reg);
      if (count > 100)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f41(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flak {
    this(Point p) {
      super(p, 90);
    }
    void run(Registry reg){
      move(Vector(3, 0));
      scroll(reg);
      if (count > 200)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f42(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flak {
    this(Point p) {
      super(p, 270);
    }
    void run(Registry reg){
      move(Vector(3, 0));
      scroll(reg);
      if (count > 200)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f43(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flak {
    this(Point p) {
      super(p, 90);
    }
    void run(Registry reg){
      move(Vector(-3, 0));
      scroll(reg);
      if (count > 200)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f44(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flak {
    this(Point p) {
      super(p, 270);
    }
    void run(Registry reg){
      move(Vector(-3, 0));
      scroll(reg);
      if (count > 200)
        die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f45(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y - 1)) Flak {
    this(Point p) {
      super(p, 0);
    }
    void run(Registry reg){
      scroll(reg);
      die_if_out_of_window();
    }
    void collide_with_wall(Registry reg, Wall wall) {
      if (!dead()) {
        Wave.enemy_destroyed.play();
        reg.add(new DestroyedEnemy(pos, true, rect().w));
        die();
      }
    }
  };
  reg.add(enemy);
}

void f46(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Flak {
    this(Point p) {
      super(p, 180);
    }
    void run(Registry reg){
      scroll(reg);
      die_if_out_of_window();
    }
    void collide_with_wall(Registry reg, Wall wall) {
      if (!dead()) {
        Wave.enemy_destroyed.play();
        reg.add(new DestroyedEnemy(pos, true, rect().w));
        die();
      }
    }
  };
  reg.add(enemy);
}

void f51(double x, double y, Registry reg)
{
  reg.add(new NormalTop(Point(x, y), -2, 50, 10000, 3));
}

void f52(double x, double y, Registry reg)
{
  reg.add(new NormalTop(Point(x, y), 2, 50, 10000, 3));
}

void f53(double x, double y, Registry reg)
{
  reg.add(new NormalTop(Point(x, y), -2));
}

void f54(double x, double y, Registry reg)
{
  reg.add(new NormalTop(Point(x, y), 2));
}

class Boss4: BaseEnemy {
private:
  double dx_ = 0;
  double ax_ = 0;
  double pos_deg_ = -90;  //現在の位置(極座標、ループの前半で使用)
  double wall_speed_ = 5;
  double omega_ = 2;
  double radius_ = 150;
  int sign_ = 1;
  
public:
  this(Point pos) {
    super(SubKind.BOSS_ENEMY, pos,
          new Animation(&images.enemies.boss4.draw),
          4000);
    scale = 6;
    set_repeating_fire(280, 30, 8);
    set_destroyed_hook(&destroyed);
  }

  mixin DefineRect!(pos, 100, 100, true);
  mixin DefineEnemy!(14000);

  static const LOOP = 360;

  static void destroyed(Registry reg, Enemy e) {
    reg.add(new StageEnd(4, 5));
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
    //左右の壁
    if (count % 10 == 1) {
      wall_speed_ += 0.05;
      reg.add(new OneMovableWall(Point(10, -20), 30, 30, Vector(0, wall_speed_), 0));
      reg.add(new OneMovableWall(Point(520, -20), 30, 30, Vector(0, wall_speed_), 0));
    }
    //登場部分
    if (count < 250) {
      move(Vector(0, 1));
      return;
    }
    //時間切れ
    if (count >= 250 + LOOP*10) {
      move(Vector(0, 10));
      if (count >= 300 + LOOP*10) {
        destroyed(reg, this);
        die();
      }
      return;
    }
    
    int ct = (count - 250) % LOOP;

    //ループ部分
    if (ct < 180) {
      pos_deg_ += omega_;
      pos = degree_to_vector(radius_, false, pos_deg_) + Point(280, 280);
    } else if (ct < 225) {
      if (ct == 180)
        dx_ = 4.5;
      ax_ = -0.1;
      dx_ += ax_;
      if (ct % 10 == 0)
        reg.add(new HomingHeli(Point(280 - 180*sign_, -20), reg, 7, 180, 2, 0, 10000, 10000, 8));
    } else if (ct < 315) {
      if (ct == 225)
        dx_ = -4.5;
      ax_ = 0.45 / 9.0;
      dx_ += ax_;
      if (ct % 10 == 0 && ct > 260)
        reg.add(new HomingHeli(Point(280 + 180*sign_, -20), reg, 7, 180, 2, 0, 10000, 10000, 8));
    } else {
      if (ct == 315)
        dx_ = 4.5;
      ax_ = -0.1;
      dx_ += ax_;
    }

    move(Vector(sign_ * dx_, 0));

    //ループ後の処理
    if (ct == LOOP-1) {
      sign_ *= -1;
      omega_ *= -1;
      dx_ = 0;
      ax_ = 0;
    }

    repeat(600, 2400, {
    });
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 250) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }


  //ボスのビット
  static class Bit: ChildEnemy {
    double radius_ = 100;
    double pos_deg_;
    double omega_;
    this(double pos_deg, Boss4 boss, double omega = 3) {
      super(SubKind.BOSS_ENEMY, degree_to_vector(radius_, false, pos_deg),
            new Animation(&images.enemies.circle.draw),
            Color.enemy, 0, 4.0, boss);
      pos_deg_ = pos_deg;
      omega_ = omega;
      set_repeating_fire(340, 90, 4);
    }
    
    mixin DefineRect!(pos, 60, 60, true);
    mixin DefineEnemy!(0);

    void run(Registry reg){
      pos_deg_ += omega_;
      pos = degree_to_vector(radius_, false, pos_deg_);

      if (count < 250)
        return;
      
      int ct = (count - 250) % LOOP;

      if (((ct >= 200 && ct <= 220) || (ct >= 290 && ct <= 310)) && ct % 5 == 0) {
        reg.add(new Sharp(pos));
      }
    }
  }
  
  //回転する円盤
  static class Effect: Animatable {
    double pos_deg_;
    double omega_;
    double domega_ = 0;  //未使用
    double radius_;
    double dxdegree_ = 0;
    double dydegree_ = 0;
    static const MAXDEG = 5;  //x,y軸回転の角速度の上限
    bool spin_;  //x,y軸で回転するかどうか
    this(double pos_deg, Enemy enemy, double size, double omega = 0, double radius = 0, bool spin = true) {
      super(SubKind.NORMAL_EFFECT, degree_to_vector(radius_, false, pos_deg),
            new Animation(&images.enemies.circle.draw),
            Color.enemy, 0, size, enemy);
      pos_deg_ = pos_deg;
      omega_ = omega;
      radius_ = radius;
      spin_ = spin;
    }
    Vector degree_to_vector(double length = 1, bool using_degree = true, double deg = 0) {
      if (using_degree)
        return length * Vector(cos((degree-90)/180*PI), sin((degree-90)/180*PI));
      else
        return length * Vector(cos(deg/180*PI), sin(deg/180*PI));
    }
    void run(Registry reg){
      if (parent.dead)
        die();
      color = (cast(Animatable)parent).color;
      pos_deg_ += omega_;
      omega_ += domega_;
      pos = degree_to_vector(radius_, false, pos_deg_);
      if (spin_) {
        //x,y軸回転の部分
        dxdegree_ += drand(-1, 1);
        dydegree_ += drand(-1, 1);
        if (dxdegree_ >  MAXDEG) dxdegree_ =  MAXDEG;
        if (dxdegree_ < -MAXDEG) dxdegree_ = -MAXDEG;
        if (dydegree_ >  MAXDEG) dydegree_ =  MAXDEG;
        if (dydegree_ < -MAXDEG) dydegree_ = -MAXDEG;
      
        xdegree += dxdegree_;
        ydegree += dydegree_;
      }
    }
  }

}


static class Sharp: BaseMovable, Bullet {
  this(Point pos) {
    super(SubKind.NORMAL_BULLET, pos);
  }

  mixin DefineRect!(pos, 6, 20, true);
    
  void draw(Registry){
    glPushMatrix();
    glColor3da(Color.bullet);
    glTranslated(pos.x, pos.y, 0);
    glRotated(90, 0, 0, 1);
    glScaled(6, 1, 1);
    images.triangle.draw();
    glPopMatrix();
  }

  void run(Registry) {
    move(Vector(0, 8));
    die_if_out_of_window();
  }
  
  void collide_with_wall(Registry, Wall) {
    die();
  }
  
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
    die();
  }
}

void f61(double x, double y, Registry reg)
{
  Boss4 boss = new Boss4(Point(x, y-100));
  reg.add(boss);
  reg.add(new Boss4.Bit(0,   boss));
  reg.add(new Boss4.Bit(90,  boss));
  reg.add(new Boss4.Bit(180, boss));
  reg.add(new Boss4.Bit(270, boss));
  reg.add(new Boss4.Effect(0,   boss, 1, -3, 50, false));
  reg.add(new Boss4.Effect(90,  boss, 1, -3, 50, false));
  reg.add(new Boss4.Effect(180, boss, 1, -3, 50, false));
  reg.add(new Boss4.Effect(270, boss, 1, -3, 50, false));
  reg.add(new Boss4.Effect(0, boss, 0.1, 0, 0, false));
  reg.add(new Boss4.Effect(0, boss, 1));
  reg.add(new Boss4.Effect(0, boss, 3));
  reg.add(new Boss4.Effect(0, boss, 5));
  Music.boss[4].play();
}






//以下MovableWall

void f101(double x, double y, Registry reg)
{
  reg.add(new OneMovableWall(Point(x, y), 30, 30, Vector(0, 2), 0));
}

void f102(double x, double y, Registry reg)
{
  reg.add(new OneMovableWall(Point(x, y), 60, 60, Vector(3, 0), 200));
}

void f103(double x, double y, Registry reg)
{
  reg.add(new OneMovableWall(Point(x, y), 60, 60, Vector(-3, 0), 200));
}

void f111(double x, double y, Registry reg)
{
  static const width = 30;
  static const height = 30;

  Wall wall = new class() MovableWall {
    double dy_ = -4.0;
    double axis = 30.0;
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      dy_ -= 0.03*(axis - (rect.q.y - rect.p.y));
      pmove(Vector(0, dy_));
      if(!rect.is_intersected(LARGE_WINDOW))
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}

void f112(double x, double y, Registry reg)
{
  static const width = 30;
  static const height = 29;

  Wall wall = new class() MovableWall {
    double dy_ = 4.0;
    double axis = 30.0;
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      dy_ += 0.03*(axis - (rect.q.y - rect.p.y));
      qmove(Vector(0, dy_));
      if(!rect.is_intersected(LARGE_WINDOW))
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}


void f113(double x, double y, Registry reg)
{
  static const width = 10;
  static const height = 10;

  Wall wall = new class() MovableWall {
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      if (count < 29)
        pmove(Vector(-20, 0));
      if(!rect.is_intersected(LARGE_WINDOW) && count > 100)
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}

void f114(double x, double y, Registry reg)
{
  static const width = 10;
  static const height = 10;

  Wall wall = new class() MovableWall {
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      if (count < 29)
        qmove(Vector(20, 0));
      if(!rect.is_intersected(LARGE_WINDOW) && count > 100)
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}

void f115(double x, double y, Registry reg)
{
  static const width = 400;
  static const height = 20;

  Wall wall = new class() MovableWall {
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      if (count > 60)
        move(Vector(20, 0));
      if(!rect.is_intersected(LARGE_WINDOW))
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}

void f116(double x, double y, Registry reg)
{
  static const width = 400;
  static const height = 20;

  Wall wall = new class() MovableWall {
    this() {
      super(Point(x, y), width, height);
    }
    void run(Registry reg) {
      if (count > 60)
        move(Vector(-20, 0));
      if(!rect.is_intersected(LARGE_WINDOW))
        die();
      super.run(reg);
    }
  };
  reg.add(wall);
}

class QuickBar : MovableWall
{
private:
  int length_;
  int timing_;
  int sign_;
  double speed_;

public:
  this(Point pos, int length, int sign, int timing = 60, double height = 30, double speed = 5) {
    super(pos, 1, height);
    length_ = (length + 2) * 20 / 5;  //使用する場合は画面端から2メモリ空けること
    sign_ = sign;
    timing_ = timing;
    speed_ = speed;
  }
  
  void run(Registry reg) {
    if (count > timing_ && count < length_ + timing_ + 1) {
      if (sign_ < 0)
        pmove(Vector(-speed_, 0));
      else
        qmove(Vector(speed_, 0));
    }
    if(!rect.is_intersected(LARGE_WINDOW) && count > 100)
      die();
    super.run(reg);
  }
}

void f117(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 16, -1));
}

void f118(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 10, 1));
}

void f119(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 18, -1));
}

void f120(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 8, 1));
}

void f121(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, 1));
}

void f122(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 20, -1));
}

void f123(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, -1));
}

void f124(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 12, 1));
}

void f125(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, 1));
}

void f126(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 12, -1));
}

void f127(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 5, -1));
}

void f128(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, 1));
}

void f129(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 21, 1));
}

void f130(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, 1));
}

void f131(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 28, -1));
}

void f132(double x, double y, Registry reg)
{
  reg.add(new QuickBar(Point(x, y), 6, 1));
}



class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun2},
  {6, &fun5},
  {6, &fun6},
  {13, &fun36},
  {13, &fun85},
  {20, &fun60},
  {20, &fun37},
  {26, &fun61},
  {26, &fun38},
  {33, &fun62},
  {33, &fun39},
  {40, &fun63},
  {40, &fun40},
  {46, &fun64},
  {46, &fun41},
  {53, &fun42},
  {53, &fun65},
  {60, &fun88},
  {60, &fun43},
  {60, &fun66},
  {60, &fun86},
  {66, &fun44},
  {66, &fun7},
  {66, &fun11},
  {66, &fun10},
  {66, &fun9},
  {66, &fun8},
  {66, &fun67},
  {73, &fun45},
  {73, &fun68},
  {80, &fun46},
  {80, &fun69},
  {86, &fun47},
  {86, &fun70},
  {90, &fun89},
  {90, &fun87},
  {93, &fun71},
  {93, &fun48},
  {100, &fun72},
  {100, &fun18},
  {100, &fun49},
  {100, &fun12},
  {106, &fun73},
  {106, &fun50},
  {113, &fun19},
  {113, &fun20},
  {113, &fun21},
  {113, &fun23},
  {113, &fun74},
  {113, &fun22},
  {113, &fun51},
  {120, &fun52},
  {120, &fun75},
  {126, &fun53},
  {126, &fun76},
  {133, &fun54},
  {133, &fun77},
  {140, &fun78},
  {140, &fun55},
  {146, &fun56},
  {146, &fun79},
  {153, &fun13},
  {153, &fun14},
  {153, &fun15},
  {153, &fun57},
  {153, &fun17},
  {153, &fun16},
  {153, &fun80},
  {160, &fun81},
  {160, &fun58},
  {166, &fun82},
  {166, &fun59},
  {173, &fun84},
  {173, &fun83},
  {193, &fun24},
  {200, &fun213},
  {206, &fun25},
  {206, &fun214},
  {206, &fun215},
  {220, &fun26},
  {226, &fun216},
  {226, &fun217},
  {233, &fun27},
  {246, &fun28},
  {260, &fun29},
  {273, &fun30},
  {280, &fun219},
  {280, &fun218},
  {286, &fun31},
  {293, &fun163},
  {293, &fun164},
  {300, &fun32},
  {313, &fun33},
  {326, &fun212},
  {326, &fun211},
  {333, &fun96},
  {333, &fun34},
  {346, &fun112},
  {346, &fun109},
  {346, &fun111},
  {346, &fun107},
  {346, &fun108},
  {346, &fun103},
  {346, &fun104},
  {346, &fun105},
  {346, &fun106},
  {346, &fun110},
  {360, &fun551},
  {360, &fun550},
  {360, &fun558},
  {360, &fun557},
  {360, &fun556},
  {360, &fun549},
  {360, &fun553},
  {360, &fun554},
  {360, &fun552},
  {360, &fun555},
  {373, &fun97},
  {373, &fun90},
  {386, &fun141},
  {386, &fun139},
  {386, &fun140},
  {386, &fun142},
  {386, &fun137},
  {386, &fun133},
  {386, &fun134},
  {386, &fun135},
  {386, &fun136},
  {386, &fun138},
  {400, &fun577},
  {400, &fun578},
  {400, &fun576},
  {400, &fun574},
  {400, &fun573},
  {400, &fun570},
  {400, &fun569},
  {400, &fun572},
  {400, &fun575},
  {400, &fun571},
  {413, &fun91},
  {413, &fun98},
  {426, &fun116},
  {426, &fun118},
  {426, &fun117},
  {426, &fun119},
  {426, &fun122},
  {426, &fun114},
  {426, &fun113},
  {426, &fun120},
  {426, &fun121},
  {426, &fun115},
  {440, &fun628},
  {440, &fun619},
  {440, &fun620},
  {440, &fun621},
  {440, &fun622},
  {440, &fun623},
  {440, &fun624},
  {440, &fun625},
  {440, &fun626},
  {440, &fun627},
  {453, &fun92},
  {453, &fun99},
  {466, &fun146},
  {466, &fun149},
  {466, &fun147},
  {466, &fun150},
  {466, &fun152},
  {466, &fun143},
  {466, &fun144},
  {466, &fun151},
  {466, &fun148},
  {466, &fun145},
  {480, &fun584},
  {480, &fun582},
  {480, &fun581},
  {480, &fun580},
  {480, &fun579},
  {480, &fun585},
  {480, &fun586},
  {480, &fun587},
  {480, &fun588},
  {480, &fun583},
  {493, &fun93},
  {493, &fun100},
  {506, &fun123},
  {506, &fun129},
  {506, &fun128},
  {506, &fun127},
  {506, &fun124},
  {506, &fun125},
  {506, &fun132},
  {506, &fun130},
  {506, &fun131},
  {506, &fun126},
  {520, &fun568},
  {520, &fun567},
  {520, &fun566},
  {520, &fun560},
  {520, &fun559},
  {520, &fun565},
  {520, &fun564},
  {520, &fun563},
  {520, &fun561},
  {520, &fun562},
  {533, &fun101},
  {533, &fun94},
  {546, &fun162},
  {546, &fun161},
  {546, &fun159},
  {546, &fun160},
  {546, &fun156},
  {546, &fun157},
  {546, &fun158},
  {546, &fun153},
  {546, &fun155},
  {546, &fun154},
  {560, &fun593},
  {560, &fun592},
  {560, &fun591},
  {560, &fun594},
  {560, &fun589},
  {560, &fun596},
  {560, &fun597},
  {560, &fun598},
  {560, &fun595},
  {560, &fun590},
  {573, &fun95},
  {573, &fun102},
  {586, &fun176},
  {586, &fun177},
  {586, &fun178},
  {586, &fun179},
  {586, &fun175},
  {586, &fun181},
  {586, &fun182},
  {586, &fun183},
  {586, &fun184},
  {586, &fun180},
  {600, &fun601},
  {600, &fun608},
  {600, &fun599},
  {600, &fun600},
  {600, &fun607},
  {600, &fun602},
  {600, &fun603},
  {600, &fun604},
  {600, &fun605},
  {600, &fun606},
  {613, &fun170},
  {613, &fun165},
  {626, &fun195},
  {626, &fun198},
  {626, &fun199},
  {626, &fun197},
  {626, &fun196},
  {626, &fun201},
  {626, &fun202},
  {626, &fun203},
  {626, &fun204},
  {626, &fun200},
  {640, &fun636},
  {640, &fun638},
  {640, &fun637},
  {640, &fun629},
  {640, &fun635},
  {640, &fun634},
  {640, &fun633},
  {640, &fun631},
  {640, &fun632},
  {640, &fun630},
  {653, &fun171},
  {653, &fun166},
  {666, &fun186},
  {666, &fun187},
  {666, &fun188},
  {666, &fun185},
  {666, &fun190},
  {666, &fun191},
  {666, &fun192},
  {666, &fun193},
  {666, &fun194},
  {666, &fun189},
  {680, &fun609},
  {680, &fun615},
  {680, &fun611},
  {680, &fun612},
  {680, &fun613},
  {680, &fun614},
  {680, &fun618},
  {680, &fun617},
  {680, &fun610},
  {680, &fun616},
  {693, &fun172},
  {693, &fun167},
  {713, &fun173},
  {713, &fun168},
  {733, &fun174},
  {733, &fun169},
  {746, &fun205},
  {746, &fun208},
  {760, &fun206},
  {760, &fun209},
  {766, &fun210},
  {766, &fun207},
  {873, &fun35},
  {873, &fun221},
  {873, &fun220},
  {873, &fun223},
  {873, &fun222},
  {893, &fun224},
  {893, &fun227},
  {893, &fun226},
  {893, &fun228},
  {893, &fun225},
  {913, &fun232},
  {913, &fun231},
  {913, &fun230},
  {913, &fun229},
  {913, &fun233},
  {933, &fun238},
  {933, &fun234},
  {933, &fun235},
  {933, &fun236},
  {933, &fun237},
  {953, &fun240},
  {953, &fun239},
  {953, &fun243},
  {953, &fun241},
  {953, &fun242},
  {973, &fun247},
  {973, &fun246},
  {973, &fun248},
  {973, &fun244},
  {973, &fun245},
  {993, &fun252},
  {993, &fun249},
  {993, &fun250},
  {993, &fun251},
  {993, &fun253},
  {1013, &fun256},
  {1013, &fun255},
  {1013, &fun254},
  {1013, &fun257},
  {1013, &fun258},
  {1033, &fun262},
  {1033, &fun259},
  {1033, &fun260},
  {1033, &fun261},
  {1033, &fun263},
  {1053, &fun264},
  {1053, &fun265},
  {1053, &fun268},
  {1053, &fun267},
  {1053, &fun266},
  {1073, &fun269},
  {1073, &fun270},
  {1073, &fun271},
  {1073, &fun272},
  {1073, &fun273},
  {1093, &fun275},
  {1093, &fun277},
  {1093, &fun274},
  {1093, &fun278},
  {1093, &fun276},
  {1113, &fun282},
  {1113, &fun280},
  {1113, &fun279},
  {1113, &fun281},
  {1113, &fun283},
  {1180, &fun4},
  {1186, &fun3},
  {1226, &fun285},
  {1246, &fun287},
  {1266, &fun288},
  {1286, &fun290},
  {1306, &fun292},
  {1326, &fun284},
  {1360, &fun291},
  {1360, &fun293},
  {1360, &fun289},
  {1360, &fun286},
  {1360, &fun294},
  {1453, &fun486},
  {1453, &fun295},
  {1453, &fun480},
  {1453, &fun477},
  {1453, &fun483},
  {1453, &fun489},
  {1463, &fun296},
  {1463, &fun487},
  {1463, &fun488},
  {1463, &fun485},
  {1463, &fun484},
  {1463, &fun490},
  {1463, &fun482},
  {1463, &fun491},
  {1463, &fun297},
  {1463, &fun479},
  {1463, &fun478},
  {1463, &fun481},
  {1506, &fun504},
  {1506, &fun298},
  {1506, &fun498},
  {1506, &fun495},
  {1506, &fun492},
  {1506, &fun501},
  {1516, &fun300},
  {1516, &fun502},
  {1516, &fun500},
  {1516, &fun299},
  {1516, &fun506},
  {1516, &fun496},
  {1516, &fun497},
  {1516, &fun503},
  {1516, &fun499},
  {1516, &fun494},
  {1516, &fun505},
  {1516, &fun493},
  {1580, &fun507},
  {1580, &fun522},
  {1580, &fun519},
  {1580, &fun525},
  {1580, &fun510},
  {1580, &fun516},
  {1580, &fun513},
  {1590, &fun527},
  {1590, &fun514},
  {1590, &fun515},
  {1590, &fun512},
  {1590, &fun511},
  {1590, &fun509},
  {1590, &fun517},
  {1590, &fun518},
  {1590, &fun508},
  {1590, &fun524},
  {1590, &fun523},
  {1590, &fun526},
  {1590, &fun520},
  {1590, &fun521},
  {1620, &fun537},
  {1620, &fun535},
  {1620, &fun533},
  {1620, &fun532},
  {1620, &fun530},
  {1620, &fun528},
  {1620, &fun539},
  {1630, &fun531},
  {1630, &fun529},
  {1630, &fun541},
  {1630, &fun540},
  {1630, &fun542},
  {1630, &fun543},
  {1630, &fun534},
  {1630, &fun545},
  {1630, &fun546},
  {1630, &fun538},
  {1630, &fun547},
  {1630, &fun548},
  {1630, &fun544},
  {1630, &fun536},
  {1660, &fun302},
  {1680, &fun383},
  {1686, &fun385},
  {1700, &fun301},
  {1706, &fun384},
  {1753, &fun303},
  {1766, &fun469},
  {1766, &fun468},
  {1766, &fun467},
  {1766, &fun412},
  {1773, &fun413},
  {1780, &fun414},
  {1786, &fun415},
  {1793, &fun416},
  {1793, &fun304},
  {1806, &fun441},
  {1813, &fun442},
  {1820, &fun443},
  {1826, &fun444},
  {1833, &fun388},
  {1853, &fun386},
  {1873, &fun389},
  {1893, &fun387},
  {1913, &fun382},
  {1933, &fun390},
  {1941, &fun457},
  {1941, &fun452},
  {1941, &fun458},
  {1941, &fun455},
  {1941, &fun453},
  {1941, &fun454},
  {1946, &fun305},
  {1946, &fun306},
  {1966, &fun307},
  {1986, &fun391},
  {1993, &fun404},
  {2000, &fun405},
  {2006, &fun411},
  {2013, &fun440},
  {2040, &fun392},
  {2053, &fun308},
  {2066, &fun309},
  {2086, &fun393},
  {2133, &fun394},
  {2140, &fun445},
  {2146, &fun446},
  {2153, &fun447},
  {2160, &fun448},
  {2166, &fun310},
  {2186, &fun395},
  {2220, &fun398},
  {2226, &fun417},
  {2226, &fun422},
  {2226, &fun423},
  {2226, &fun470},
  {2226, &fun420},
  {2226, &fun418},
  {2226, &fun419},
  {2226, &fun421},
  {2240, &fun313},
  {2240, &fun311},
  {2260, &fun396},
  {2273, &fun399},
  {2280, &fun471},
  {2280, &fun424},
  {2280, &fun430},
  {2280, &fun429},
  {2280, &fun428},
  {2280, &fun427},
  {2280, &fun425},
  {2280, &fun426},
  {2293, &fun312},
  {2300, &fun397},
  {2306, &fun381},
  {2313, &fun400},
  {2318, &fun407},
  {2318, &fun406},
  {2318, &fun408},
  {2318, &fun409},
  {2318, &fun410},
  {2326, &fun401},
  {2346, &fun449},
  {2353, &fun450},
  {2360, &fun380},
  {2360, &fun451},
  {2366, &fun314},
  {2400, &fun463},
  {2400, &fun436},
  {2406, &fun437},
  {2406, &fun464},
  {2413, &fun402},
  {2413, &fun438},
  {2413, &fun465},
  {2420, &fun439},
  {2420, &fun466},
  {2426, &fun403},
  {2435, &fun475},
  {2435, &fun473},
  {2435, &fun476},
  {2435, &fun639},
  {2435, &fun640},
  {2435, &fun474},
  {2440, &fun315},
  {2440, &fun316},
  {2446, &fun317},
  {2446, &fun318},
  {2500, &fun323},
  {2513, &fun435},
  {2513, &fun431},
  {2520, &fun460},
  {2520, &fun432},
  {2526, &fun461},
  {2526, &fun433},
  {2533, &fun462},
  {2533, &fun434},
  {2540, &fun321},
  {2540, &fun322},
  {2581, &fun642},
  {2581, &fun459},
  {2581, &fun456},
  {2581, &fun641},
  {2586, &fun319},
  {2586, &fun320},
  {2593, &fun325},
  {2593, &fun324},
  {2660, &fun331},
  {2660, &fun332},
  {2686, &fun328},
  {2686, &fun329},
  {2686, &fun327},
  {2686, &fun326},
  {2686, &fun330},
  {2733, &fun374},
  {2733, &fun365},
  {2746, &fun373},
  {2746, &fun364},
  {2753, &fun351},
  {2753, &fun349},
  {2760, &fun379},
  {2760, &fun377},
  {2760, &fun363},
  {2760, &fun376},
  {2760, &fun372},
  {2760, &fun378},
  {2760, &fun375},
  {2766, &fun350},
  {2766, &fun348},
  {2773, &fun371},
  {2773, &fun362},
  {2780, &fun370},
  {2780, &fun347},
  {2780, &fun361},
  {2780, &fun344},
  {2786, &fun366},
  {2786, &fun360},
  {2793, &fun343},
  {2793, &fun368},
  {2793, &fun359},
  {2793, &fun367},
  {2793, &fun358},
  {2793, &fun346},
  {2800, &fun342},
  {2800, &fun345},
  {2800, &fun357},
  {2800, &fun369},
  {2806, &fun337},
  {2806, &fun338},
  {2813, &fun339},
  {2813, &fun335},
  {2813, &fun336},
  {2813, &fun340},
  {2820, &fun341},
  {2820, &fun334},
  {2820, &fun333},
  {2933, &fun356},
  {2946, &fun355},
  {2973, &fun352},
  {2980, &fun354},
  {2986, &fun353},
  {3153, &fun472},
  {3166, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun2(Registry reg){
  set_scroll(reg, 3);
}
static void fun5(Registry reg){
  reg.add(new NormalWall(Rect(0, -180, 40, 160)));
}
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(520, -180, 40, 160)));
}
static void fun36(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun85(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun37(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun39(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun63(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun40(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun64(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun65(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun88(Registry reg){
  f111(525.000000, -20.000000, reg);
}
static void fun43(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun86(Registry reg){
  f111(5.000000, -20.000000, reg);
}
static void fun44(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun7(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  f11(380.000000, -20.000000, reg);
}
static void fun10(Registry reg){
  f11(320.000000, -20.000000, reg);
}
static void fun9(Registry reg){
  f11(260.000000, -20.000000, reg);
}
static void fun8(Registry reg){
  f11(200.000000, -20.000000, reg);
}
static void fun67(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun68(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun69(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun47(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun89(Registry reg){
  f112(525.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  f112(5.000000, -20.000000, reg);
}
static void fun71(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun72(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(520, -180, 40, 160)));
}
static void fun49(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(0, -180, 40, 160)));
}
static void fun73(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  f11(200.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  f11(260.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f11(380.000000, -20.000000, reg);
}
static void fun74(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f11(320.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun52(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun75(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun53(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun76(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun77(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun79(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  f11(140.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  f11(200.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  f11(260.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  f11(380.000000, -20.000000, reg);
}
static void fun16(Registry reg){
  f11(320.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun82(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun84(Registry reg){
  f101(460.000000, -20.000000, reg);
}
static void fun83(Registry reg){
  f101(60.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  f21(640.000000, -20.000000, reg);
}
static void fun213(Registry reg){
  f02(100.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  f21(620.000000, -20.000000, reg);
}
static void fun214(Registry reg){
  f02(20.000000, -20.000000, reg);
}
static void fun215(Registry reg){
  f02(180.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  f21(600.000000, -20.000000, reg);
}
static void fun216(Registry reg){
  f02(60.000000, -20.000000, reg);
}
static void fun217(Registry reg){
  f02(140.000000, -20.000000, reg);
}
static void fun27(Registry reg){
  f21(580.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  f21(560.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  f22(-120.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  f22(-100.000000, -20.000000, reg);
}
static void fun219(Registry reg){
  f02(480.000000, -20.000000, reg);
}
static void fun218(Registry reg){
  f02(440.000000, -20.000000, reg);
}
static void fun31(Registry reg){
  f22(-80.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  f02(440.000000, -20.000000, reg);
}
static void fun164(Registry reg){
  f02(480.000000, -20.000000, reg);
}
static void fun32(Registry reg){
  f22(-60.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  f22(-40.000000, -20.000000, reg);
}
static void fun212(Registry reg){
  f116(-120.000000, -20.000000, reg);
}
static void fun211(Registry reg){
  f115(280.000000, -20.000000, reg);
}
static void fun96(Registry reg){
  f114(-240.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  f113(1040.000000, -20.000000, reg);
}
static void fun112(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun109(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun111(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun107(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun103(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun105(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun106(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun110(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun551(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun550(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun558(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun557(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun556(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun549(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun553(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun554(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun552(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun555(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun97(Registry reg){
  f114(-280.000000, -20.000000, reg);
}
static void fun90(Registry reg){
  f113(1000.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun577(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun578(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun576(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun574(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun573(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun570(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun569(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun572(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun575(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun571(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun91(Registry reg){
  f113(940.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f114(-340.000000, -20.000000, reg);
}
static void fun116(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun117(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun122(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun113(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun121(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun115(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun628(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun619(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun620(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun621(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun622(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun623(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun624(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun625(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun626(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun627(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun92(Registry reg){
  f113(820.000000, -20.000000, reg);
}
static void fun99(Registry reg){
  f114(-460.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun152(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun144(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun584(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun582(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun581(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun580(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun579(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun585(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun586(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun587(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun588(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun583(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  f113(760.000000, -20.000000, reg);
}
static void fun100(Registry reg){
  f114(-520.000000, -20.000000, reg);
}
static void fun123(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun128(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun127(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun124(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun125(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun132(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun568(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun567(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun566(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun560(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun559(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun565(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun564(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun563(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun561(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun562(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun101(Registry reg){
  f114(-500.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  f113(780.000000, -20.000000, reg);
}
static void fun162(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun160(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun156(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun153(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun155(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun154(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun593(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun592(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun591(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun594(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun589(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun596(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun597(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun598(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun595(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun590(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun95(Registry reg){
  f113(880.000000, -20.000000, reg);
}
static void fun102(Registry reg){
  f114(-400.000000, -20.000000, reg);
}
static void fun176(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun177(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun178(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun179(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun175(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun181(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun182(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun183(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun184(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun180(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun601(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun608(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun599(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun600(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun607(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun602(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun603(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun604(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun605(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun606(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun170(Registry reg){
  f114(-300.000000, -20.000000, reg);
}
static void fun165(Registry reg){
  f113(980.000000, -20.000000, reg);
}
static void fun195(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun198(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun199(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun197(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun196(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun201(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun202(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun203(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun204(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun200(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun636(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun638(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun637(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun629(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun635(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun634(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun633(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun631(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun632(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun630(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun171(Registry reg){
  f114(-280.000000, -20.000000, reg);
}
static void fun166(Registry reg){
  f113(1000.000000, -20.000000, reg);
}
static void fun186(Registry reg){
  f31(420.000000, -20.000000, reg);
}
static void fun187(Registry reg){
  f31(460.000000, -20.000000, reg);
}
static void fun188(Registry reg){
  f31(500.000000, -20.000000, reg);
}
static void fun185(Registry reg){
  f31(380.000000, -20.000000, reg);
}
static void fun190(Registry reg){
  f31(580.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  f31(620.000000, -20.000000, reg);
}
static void fun192(Registry reg){
  f31(660.000000, -20.000000, reg);
}
static void fun193(Registry reg){
  f31(700.000000, -20.000000, reg);
}
static void fun194(Registry reg){
  f31(740.000000, -20.000000, reg);
}
static void fun189(Registry reg){
  f31(540.000000, -20.000000, reg);
}
static void fun609(Registry reg){
  f32(-60.000000, -20.000000, reg);
}
static void fun615(Registry reg){
  f32(100.000000, -20.000000, reg);
}
static void fun611(Registry reg){
  f32(-140.000000, -20.000000, reg);
}
static void fun612(Registry reg){
  f32(-180.000000, -20.000000, reg);
}
static void fun613(Registry reg){
  f32(20.000000, -20.000000, reg);
}
static void fun614(Registry reg){
  f32(-20.000000, -20.000000, reg);
}
static void fun618(Registry reg){
  f32(140.000000, -20.000000, reg);
}
static void fun617(Registry reg){
  f32(180.000000, -20.000000, reg);
}
static void fun610(Registry reg){
  f32(-100.000000, -20.000000, reg);
}
static void fun616(Registry reg){
  f32(60.000000, -20.000000, reg);
}
static void fun172(Registry reg){
  f114(-300.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  f113(980.000000, -20.000000, reg);
}
static void fun173(Registry reg){
  f114(-340.000000, -20.000000, reg);
}
static void fun168(Registry reg){
  f113(940.000000, -20.000000, reg);
}
static void fun174(Registry reg){
  f114(-360.000000, -20.000000, reg);
}
static void fun169(Registry reg){
  f113(920.000000, -20.000000, reg);
}
static void fun205(Registry reg){
  f113(920.000000, -20.000000, reg);
}
static void fun208(Registry reg){
  f114(-360.000000, -20.000000, reg);
}
static void fun206(Registry reg){
  f113(920.000000, -20.000000, reg);
}
static void fun209(Registry reg){
  f114(-360.000000, -20.000000, reg);
}
static void fun210(Registry reg){
  f114(-360.000000, -20.000000, reg);
}
static void fun207(Registry reg){
  f113(920.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun221(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun220(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun223(Registry reg){
  f12(40.000000, -20.000000, reg);
}
static void fun222(Registry reg){
  f12(100.000000, -20.000000, reg);
}
static void fun224(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun227(Registry reg){
  f12(140.000000, -20.000000, reg);
}
static void fun226(Registry reg){
  f12(200.000000, -20.000000, reg);
}
static void fun228(Registry reg){
  f12(80.000000, -20.000000, reg);
}
static void fun225(Registry reg){
  f12(260.000000, -20.000000, reg);
}
static void fun232(Registry reg){
  f12(180.000000, -20.000000, reg);
}
static void fun231(Registry reg){
  f12(240.000000, -20.000000, reg);
}
static void fun230(Registry reg){
  f12(300.000000, -20.000000, reg);
}
static void fun229(Registry reg){
  f12(360.000000, -20.000000, reg);
}
static void fun233(Registry reg){
  f12(120.000000, -20.000000, reg);
}
static void fun238(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun234(Registry reg){
  f12(520.000000, -20.000000, reg);
}
static void fun235(Registry reg){
  f12(460.000000, -20.000000, reg);
}
static void fun236(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun237(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun240(Registry reg){
  f12(420.000000, -20.000000, reg);
}
static void fun239(Registry reg){
  f12(480.000000, -20.000000, reg);
}
static void fun243(Registry reg){
  f12(240.000000, -20.000000, reg);
}
static void fun241(Registry reg){
  f12(360.000000, -20.000000, reg);
}
static void fun242(Registry reg){
  f12(300.000000, -20.000000, reg);
}
static void fun247(Registry reg){
  f12(260.000000, -20.000000, reg);
}
static void fun246(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun248(Registry reg){
  f12(200.000000, -20.000000, reg);
}
static void fun244(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun245(Registry reg){
  f12(380.000000, -20.000000, reg);
}
static void fun252(Registry reg){
  f12(100.000000, -20.000000, reg);
}
static void fun249(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun250(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun251(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun253(Registry reg){
  f12(40.000000, -20.000000, reg);
}
static void fun256(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun255(Registry reg){
  f12(460.000000, -20.000000, reg);
}
static void fun254(Registry reg){
  f12(520.000000, -20.000000, reg);
}
static void fun257(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun258(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun262(Registry reg){
  f12(140.000000, -20.000000, reg);
}
static void fun259(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun260(Registry reg){
  f12(260.000000, -20.000000, reg);
}
static void fun261(Registry reg){
  f12(200.000000, -20.000000, reg);
}
static void fun263(Registry reg){
  f12(80.000000, -20.000000, reg);
}
static void fun264(Registry reg){
  f12(480.000000, -20.000000, reg);
}
static void fun265(Registry reg){
  f12(420.000000, -20.000000, reg);
}
static void fun268(Registry reg){
  f12(240.000000, -20.000000, reg);
}
static void fun267(Registry reg){
  f12(300.000000, -20.000000, reg);
}
static void fun266(Registry reg){
  f12(360.000000, -20.000000, reg);
}
static void fun269(Registry reg){
  f12(360.000000, -20.000000, reg);
}
static void fun270(Registry reg){
  f12(300.000000, -20.000000, reg);
}
static void fun271(Registry reg){
  f12(240.000000, -20.000000, reg);
}
static void fun272(Registry reg){
  f12(180.000000, -20.000000, reg);
}
static void fun273(Registry reg){
  f12(120.000000, -20.000000, reg);
}
static void fun275(Registry reg){
  f12(380.000000, -20.000000, reg);
}
static void fun277(Registry reg){
  f12(260.000000, -20.000000, reg);
}
static void fun274(Registry reg){
  f12(440.000000, -20.000000, reg);
}
static void fun278(Registry reg){
  f12(200.000000, -20.000000, reg);
}
static void fun276(Registry reg){
  f12(320.000000, -20.000000, reg);
}
static void fun282(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun280(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun279(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun281(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun283(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun4(Registry reg){
  set_scroll(reg, 3);
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun285(Registry reg){
  reg.add(new NormalWall(Rect(520, -520, 40, 500)));
}
static void fun287(Registry reg){
  reg.add(new NormalWall(Rect(420, -460, 20, 440)));
}
static void fun288(Registry reg){
  reg.add(new NormalWall(Rect(320, -400, 20, 380)));
}
static void fun290(Registry reg){
  reg.add(new NormalWall(Rect(220, -340, 20, 320)));
}
static void fun292(Registry reg){
  reg.add(new NormalWall(Rect(120, -280, 20, 260)));
}
static void fun284(Registry reg){
  reg.add(new NormalWall(Rect(0, -220, 40, 200)));
}
static void fun291(Registry reg){
  f03(280.000000, -20.000000, reg);
}
static void fun293(Registry reg){
  f01(180.000000, -20.000000, reg);
}
static void fun289(Registry reg){
  f04(380.000000, -20.000000, reg);
}
static void fun286(Registry reg){
  f05(480.000000, -20.000000, reg);
}
static void fun294(Registry reg){
  f05(80.000000, -20.000000, reg);
}
static void fun486(Registry reg){
  f102(-260.000000, -20.000000, reg);
}
static void fun295(Registry reg){
  f102(540.000000, -20.000000, reg);
}
static void fun480(Registry reg){
  f102(140.000000, -20.000000, reg);
}
static void fun477(Registry reg){
  f102(340.000000, -20.000000, reg);
}
static void fun483(Registry reg){
  f102(-60.000000, -20.000000, reg);
}
static void fun489(Registry reg){
  f102(-460.000000, -20.000000, reg);
}
static void fun296(Registry reg){
  f41(645.000000, -20.000000, reg);
}
static void fun487(Registry reg){
  f41(-155.000000, -20.000000, reg);
}
static void fun488(Registry reg){
  f42(-246.000000, -20.000000, reg);
}
static void fun485(Registry reg){
  f42(-46.000000, -20.000000, reg);
}
static void fun484(Registry reg){
  f41(45.000000, -20.000000, reg);
}
static void fun490(Registry reg){
  f41(-355.000000, -20.000000, reg);
}
static void fun482(Registry reg){
  f42(154.000000, -20.000000, reg);
}
static void fun491(Registry reg){
  f42(-446.000000, -20.000000, reg);
}
static void fun297(Registry reg){
  f42(554.000000, -20.000000, reg);
}
static void fun479(Registry reg){
  f42(354.000000, -20.000000, reg);
}
static void fun478(Registry reg){
  f41(445.000000, -20.000000, reg);
}
static void fun481(Registry reg){
  f41(245.000000, -20.000000, reg);
}
static void fun504(Registry reg){
  f103(980.000000, -20.000000, reg);
}
static void fun298(Registry reg){
  f103(-20.000000, -20.000000, reg);
}
static void fun498(Registry reg){
  f103(580.000000, -20.000000, reg);
}
static void fun495(Registry reg){
  f103(380.000000, -20.000000, reg);
}
static void fun492(Registry reg){
  f103(180.000000, -20.000000, reg);
}
static void fun501(Registry reg){
  f103(780.000000, -20.000000, reg);
}
static void fun300(Registry reg){
  f44(-66.000000, -20.000000, reg);
}
static void fun502(Registry reg){
  f43(825.000000, -20.000000, reg);
}
static void fun500(Registry reg){
  f44(534.000000, -20.000000, reg);
}
static void fun299(Registry reg){
  f43(25.000000, -20.000000, reg);
}
static void fun506(Registry reg){
  f44(934.000000, -20.000000, reg);
}
static void fun496(Registry reg){
  f43(425.000000, -20.000000, reg);
}
static void fun497(Registry reg){
  f44(334.000000, -20.000000, reg);
}
static void fun503(Registry reg){
  f44(734.000000, -20.000000, reg);
}
static void fun499(Registry reg){
  f43(625.000000, -20.000000, reg);
}
static void fun494(Registry reg){
  f44(134.000000, -20.000000, reg);
}
static void fun505(Registry reg){
  f43(1025.000000, -20.000000, reg);
}
static void fun493(Registry reg){
  f43(225.000000, -20.000000, reg);
}
static void fun507(Registry reg){
  f102(540.000000, -20.000000, reg);
}
static void fun522(Registry reg){
  f102(-360.000000, -20.000000, reg);
}
static void fun519(Registry reg){
  f102(-180.000000, -20.000000, reg);
}
static void fun525(Registry reg){
  f102(-540.000000, -20.000000, reg);
}
static void fun510(Registry reg){
  f102(360.000000, -20.000000, reg);
}
static void fun516(Registry reg){
  f102(0.000000, -20.000000, reg);
}
static void fun513(Registry reg){
  f102(180.000000, -20.000000, reg);
}
static void fun527(Registry reg){
  f42(-526.000000, -20.000000, reg);
}
static void fun514(Registry reg){
  f41(285.000000, -20.000000, reg);
}
static void fun515(Registry reg){
  f42(194.000000, -20.000000, reg);
}
static void fun512(Registry reg){
  f42(374.000000, -20.000000, reg);
}
static void fun511(Registry reg){
  f41(465.000000, -20.000000, reg);
}
static void fun509(Registry reg){
  f42(554.000000, -20.000000, reg);
}
static void fun517(Registry reg){
  f41(105.000000, -20.000000, reg);
}
static void fun518(Registry reg){
  f42(14.000000, -20.000000, reg);
}
static void fun508(Registry reg){
  f41(645.000000, -20.000000, reg);
}
static void fun524(Registry reg){
  f42(-346.000000, -20.000000, reg);
}
static void fun523(Registry reg){
  f41(-255.000000, -20.000000, reg);
}
static void fun526(Registry reg){
  f41(-435.000000, -20.000000, reg);
}
static void fun520(Registry reg){
  f41(-75.000000, -20.000000, reg);
}
static void fun521(Registry reg){
  f42(-166.000000, -20.000000, reg);
}
static void fun537(Registry reg){
  f103(880.000000, -20.000000, reg);
}
static void fun535(Registry reg){
  f103(700.000000, -20.000000, reg);
}
static void fun533(Registry reg){
  f103(520.000000, -20.000000, reg);
}
static void fun532(Registry reg){
  f103(340.000000, -20.000000, reg);
}
static void fun530(Registry reg){
  f103(160.000000, -20.000000, reg);
}
static void fun528(Registry reg){
  f103(-20.000000, -20.000000, reg);
}
static void fun539(Registry reg){
  f103(1060.000000, -20.000000, reg);
}
static void fun531(Registry reg){
  f44(114.000000, -20.000000, reg);
}
static void fun529(Registry reg){
  f44(-66.000000, -20.000000, reg);
}
static void fun541(Registry reg){
  f44(294.000000, -20.000000, reg);
}
static void fun540(Registry reg){
  f44(1014.000000, -20.000000, reg);
}
static void fun542(Registry reg){
  f43(385.000000, -20.000000, reg);
}
static void fun543(Registry reg){
  f43(205.000000, -20.000000, reg);
}
static void fun534(Registry reg){
  f44(474.000000, -20.000000, reg);
}
static void fun545(Registry reg){
  f43(565.000000, -20.000000, reg);
}
static void fun546(Registry reg){
  f43(745.000000, -20.000000, reg);
}
static void fun538(Registry reg){
  f44(834.000000, -20.000000, reg);
}
static void fun547(Registry reg){
  f43(925.000000, -20.000000, reg);
}
static void fun548(Registry reg){
  f43(1105.000000, -20.000000, reg);
}
static void fun544(Registry reg){
  f43(25.000000, -20.000000, reg);
}
static void fun536(Registry reg){
  f44(654.000000, -20.000000, reg);
}
static void fun302(Registry reg){
  reg.add(new NormalWall(Rect(200, -220, 40, 200)));
}
static void fun383(Registry reg){
  f117(600.000000, -20.000000, reg);
}
static void fun385(Registry reg){
  f118(-41.000000, -20.000000, reg);
}
static void fun301(Registry reg){
  reg.add(new NormalWall(Rect(0, -360, 40, 340)));
}
static void fun384(Registry reg){
  f117(600.000000, -20.000000, reg);
}
static void fun303(Registry reg){
  reg.add(new NormalWall(Rect(520, -220, 40, 200)));
}
static void fun469(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun468(Registry reg){
  f52(240.000000, -20.000000, reg);
}
static void fun467(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun412(Registry reg){
  f52(200.000000, -20.000000, reg);
}
static void fun413(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun414(Registry reg){
  f52(360.000000, -20.000000, reg);
}
static void fun415(Registry reg){
  f52(340.000000, -20.000000, reg);
}
static void fun416(Registry reg){
  f52(320.000000, -20.000000, reg);
}
static void fun304(Registry reg){
  reg.add(new NormalWall(Rect(160, -360, 40, 340)));
}
static void fun441(Registry reg){
  f51(440.000000, -20.000000, reg);
}
static void fun442(Registry reg){
  f51(460.000000, -20.000000, reg);
}
static void fun443(Registry reg){
  f51(480.000000, -20.000000, reg);
}
static void fun444(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun388(Registry reg){
  f120(-41.000000, -20.000000, reg);
}
static void fun386(Registry reg){
  f119(600.000000, -20.000000, reg);
}
static void fun389(Registry reg){
  f120(-41.000000, -20.000000, reg);
}
static void fun387(Registry reg){
  f119(600.000000, -20.000000, reg);
}
static void fun382(Registry reg){
  f02(60.000000, -20.000000, reg);
}
static void fun390(Registry reg){
  f121(-41.000000, -20.000000, reg);
}
static void fun457(Registry reg){
  f46(479.000000, -20.000000, reg);
}
static void fun452(Registry reg){
  f46(80.000000, -20.000000, reg);
}
static void fun458(Registry reg){
  f46(539.000000, -20.000000, reg);
}
static void fun455(Registry reg){
  f46(399.000000, -20.000000, reg);
}
static void fun453(Registry reg){
  f46(20.000000, -20.000000, reg);
}
static void fun454(Registry reg){
  f46(160.000000, -20.000000, reg);
}
static void fun305(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 220, 60)));
}
static void fun306(Registry reg){
  reg.add(new NormalWall(Rect(340, -80, 220, 60)));
}
static void fun307(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 160, 140)));
}
static void fun391(Registry reg){
  f122(600.000000, -20.000000, reg);
}
static void fun404(Registry reg){
  f51(480.000000, -20.000000, reg);
}
static void fun405(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun411(Registry reg){
  f51(520.000000, -20.000000, reg);
}
static void fun440(Registry reg){
  f51(540.000000, -20.000000, reg);
}
static void fun392(Registry reg){
  f123(600.000000, -20.000000, reg);
}
static void fun308(Registry reg){
  reg.add(new NormalWall(Rect(140, -60, 420, 40)));
}
static void fun309(Registry reg){
  reg.add(new NormalWall(Rect(240, -140, 320, 120)));
}
static void fun393(Registry reg){
  f124(-41.000000, -20.000000, reg);
}
static void fun394(Registry reg){
  f125(-41.000000, -20.000000, reg);
}
static void fun445(Registry reg){
  f52(440.000000, -20.000000, reg);
}
static void fun446(Registry reg){
  f52(420.000000, -20.000000, reg);
}
static void fun447(Registry reg){
  f52(400.000000, -20.000000, reg);
}
static void fun448(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun310(Registry reg){
  reg.add(new NormalWall(Rect(0, -140, 320, 120)));
}
static void fun395(Registry reg){
  f126(600.000000, -20.000000, reg);
}
static void fun398(Registry reg){
  f128(-41.000000, -20.000000, reg);
}
static void fun417(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun422(Registry reg){
  f52(60.000000, -20.000000, reg);
}
static void fun423(Registry reg){
  f52(20.000000, -20.000000, reg);
}
static void fun470(Registry reg){
  f52(-20.000000, -20.000000, reg);
}
static void fun420(Registry reg){
  f52(140.000000, -20.000000, reg);
}
static void fun418(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun419(Registry reg){
  f52(180.000000, -20.000000, reg);
}
static void fun421(Registry reg){
  f52(100.000000, -20.000000, reg);
}
static void fun313(Registry reg){
  reg.add(new NormalWall(Rect(420, -240, 40, 220)));
}
static void fun311(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 320, 60)));
}
static void fun396(Registry reg){
  f127(600.000000, -20.000000, reg);
}
static void fun399(Registry reg){
  f129(-41.000000, -20.000000, reg);
}
static void fun471(Registry reg){
  f52(-20.000000, -20.000000, reg);
}
static void fun424(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun430(Registry reg){
  f52(20.000000, -20.000000, reg);
}
static void fun429(Registry reg){
  f52(60.000000, -20.000000, reg);
}
static void fun428(Registry reg){
  f52(100.000000, -20.000000, reg);
}
static void fun427(Registry reg){
  f52(140.000000, -20.000000, reg);
}
static void fun425(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun426(Registry reg){
  f52(180.000000, -20.000000, reg);
}
static void fun312(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 320, 60)));
}
static void fun397(Registry reg){
  f127(600.000000, -20.000000, reg);
}
static void fun381(Registry reg){
  f03(370.000000, -20.000000, reg);
}
static void fun400(Registry reg){
  f130(0.000000, -20.000000, reg);
}
static void fun407(Registry reg){
  f45(80.000000, -20.000000, reg);
}
static void fun406(Registry reg){
  f45(20.000000, -20.000000, reg);
}
static void fun408(Registry reg){
  f45(140.000000, -20.000000, reg);
}
static void fun409(Registry reg){
  f45(200.000000, -20.000000, reg);
}
static void fun410(Registry reg){
  f45(260.000000, -20.000000, reg);
}
static void fun401(Registry reg){
  f131(600.000000, -20.000000, reg);
}
static void fun449(Registry reg){
  f52(400.000000, -20.000000, reg);
}
static void fun450(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun380(Registry reg){
  f05(40.000000, -20.000000, reg);
}
static void fun451(Registry reg){
  f52(360.000000, -20.000000, reg);
}
static void fun314(Registry reg){
  reg.add(new NormalWall(Rect(120, -80, 320, 60)));
}
static void fun463(Registry reg){
  f54(178.000000, -20.000000, reg);
}
static void fun436(Registry reg){
  f53(380.000000, -20.000000, reg);
}
static void fun437(Registry reg){
  f53(400.000000, -20.000000, reg);
}
static void fun464(Registry reg){
  f54(158.000000, -20.000000, reg);
}
static void fun402(Registry reg){
  f130(-41.000000, -20.000000, reg);
}
static void fun438(Registry reg){
  f53(420.000000, -20.000000, reg);
}
static void fun465(Registry reg){
  f54(138.000000, -20.000000, reg);
}
static void fun439(Registry reg){
  f53(440.000000, -20.000000, reg);
}
static void fun466(Registry reg){
  f54(118.000000, -20.000000, reg);
}
static void fun403(Registry reg){
  f131(600.000000, -20.000000, reg);
}
static void fun475(Registry reg){
  f46(539.000000, -20.000000, reg);
}
static void fun473(Registry reg){
  f46(20.000000, -20.000000, reg);
}
static void fun476(Registry reg){
  f46(499.000000, -20.000000, reg);
}
static void fun639(Registry reg){
  f46(140.000000, -20.000000, reg);
}
static void fun640(Registry reg){
  f46(419.000000, -20.000000, reg);
}
static void fun474(Registry reg){
  f46(60.000000, -20.000000, reg);
}
static void fun315(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun316(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun317(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun318(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun323(Registry reg){
  reg.add(new NormalWall(Rect(240, -120, 80, 100)));
}
static void fun435(Registry reg){
  f52(58.000000, -20.000000, reg);
}
static void fun431(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun460(Registry reg){
  f52(38.000000, -20.000000, reg);
}
static void fun432(Registry reg){
  f51(520.000000, -20.000000, reg);
}
static void fun461(Registry reg){
  f52(18.000000, -20.000000, reg);
}
static void fun433(Registry reg){
  f51(540.000000, -20.000000, reg);
}
static void fun462(Registry reg){
  f52(-2.000000, -20.000000, reg);
}
static void fun434(Registry reg){
  f51(560.000000, -20.000000, reg);
}
static void fun321(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun322(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun642(Registry reg){
  f46(379.000000, -20.000000, reg);
}
static void fun459(Registry reg){
  f46(419.000000, -20.000000, reg);
}
static void fun456(Registry reg){
  f46(140.000000, -20.000000, reg);
}
static void fun641(Registry reg){
  f46(180.000000, -20.000000, reg);
}
static void fun319(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun320(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun325(Registry reg){
  reg.add(new NormalWall(Rect(380, -220, 180, 200)));
}
static void fun324(Registry reg){
  reg.add(new NormalWall(Rect(0, -220, 180, 200)));
}
static void fun331(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun332(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun328(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun329(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun327(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun326(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun330(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun374(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun365(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun373(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun364(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun351(Registry reg){
  f13(-40.000000, -20.000000, reg);
}
static void fun349(Registry reg){
  f13(600.000000, -20.000000, reg);
}
static void fun379(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun377(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun363(Registry reg){
  f13(40.000000, -20.000000, reg);
}
static void fun376(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun372(Registry reg){
  f13(520.000000, -20.000000, reg);
}
static void fun378(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun375(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun350(Registry reg){
  f13(-40.000000, -20.000000, reg);
}
static void fun348(Registry reg){
  f13(600.000000, -20.000000, reg);
}
static void fun371(Registry reg){
  f13(520.000000, -20.000000, reg);
}
static void fun362(Registry reg){
  f13(40.000000, -20.000000, reg);
}
static void fun370(Registry reg){
  f13(480.000000, -20.000000, reg);
}
static void fun347(Registry reg){
  f13(580.000000, -20.000000, reg);
}
static void fun361(Registry reg){
  f13(80.000000, -20.000000, reg);
}
static void fun344(Registry reg){
  f13(-20.000000, -20.000000, reg);
}
static void fun366(Registry reg){
  f13(440.000000, -20.000000, reg);
}
static void fun360(Registry reg){
  f13(120.000000, -20.000000, reg);
}
static void fun343(Registry reg){
  f13(-20.000000, -20.000000, reg);
}
static void fun368(Registry reg){
  f13(340.000000, -20.000000, reg);
}
static void fun359(Registry reg){
  f13(160.000000, -20.000000, reg);
}
static void fun367(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun358(Registry reg){
  f13(220.000000, -20.000000, reg);
}
static void fun346(Registry reg){
  f13(580.000000, -20.000000, reg);
}
static void fun342(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun345(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun357(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun369(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun337(Registry reg){
  f13(60.000000, -20.000000, reg);
}
static void fun338(Registry reg){
  f13(500.000000, -20.000000, reg);
}
static void fun339(Registry reg){
  f13(460.000000, -20.000000, reg);
}
static void fun335(Registry reg){
  f13(160.000000, -20.000000, reg);
}
static void fun336(Registry reg){
  f13(100.000000, -20.000000, reg);
}
static void fun340(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun341(Registry reg){
  f13(340.000000, -20.000000, reg);
}
static void fun334(Registry reg){
  f13(220.000000, -20.000000, reg);
}
static void fun333(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun356(Registry reg){
  set_scroll(reg, 3);
}
static void fun355(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun352(Registry reg){
  f05(300.000000, -20.000000, reg);
}
static void fun354(Registry reg){
  f04(280.000000, -20.000000, reg);
}
static void fun353(Registry reg){
  f05(260.000000, -20.000000, reg);
}
static void fun472(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun6},
  {1, &fun3},
  {1, &fun4},
  {1, &fun5},
  {1, &fun18},
  {1, &fun12},
  {33, &fun285},
  {53, &fun287},
  {73, &fun288},
  {93, &fun290},
  {113, &fun292},
  {133, &fun284},
  {166, &fun286},
  {166, &fun289},
  {166, &fun291},
  {166, &fun294},
  {166, &fun293},
  {260, &fun477},
  {260, &fun295},
  {260, &fun480},
  {260, &fun483},
  {260, &fun486},
  {260, &fun489},
  {270, &fun297},
  {270, &fun491},
  {270, &fun478},
  {270, &fun490},
  {270, &fun479},
  {270, &fun488},
  {270, &fun487},
  {270, &fun296},
  {270, &fun485},
  {270, &fun482},
  {270, &fun481},
  {270, &fun484},
  {313, &fun504},
  {313, &fun298},
  {313, &fun501},
  {313, &fun495},
  {313, &fun492},
  {313, &fun498},
  {323, &fun493},
  {323, &fun499},
  {323, &fun497},
  {323, &fun500},
  {323, &fun494},
  {323, &fun299},
  {323, &fun496},
  {323, &fun502},
  {323, &fun503},
  {323, &fun506},
  {323, &fun505},
  {323, &fun300},
  {386, &fun507},
  {386, &fun525},
  {386, &fun522},
  {386, &fun510},
  {386, &fun519},
  {386, &fun516},
  {386, &fun513},
  {396, &fun511},
  {396, &fun515},
  {396, &fun512},
  {396, &fun517},
  {396, &fun518},
  {396, &fun514},
  {396, &fun520},
  {396, &fun521},
  {396, &fun509},
  {396, &fun527},
  {396, &fun523},
  {396, &fun508},
  {396, &fun524},
  {396, &fun526},
  {426, &fun528},
  {426, &fun539},
  {426, &fun530},
  {426, &fun537},
  {426, &fun532},
  {426, &fun533},
  {426, &fun535},
  {436, &fun540},
  {436, &fun536},
  {436, &fun531},
  {436, &fun538},
  {436, &fun529},
  {436, &fun534},
  {436, &fun541},
  {436, &fun542},
  {436, &fun543},
  {436, &fun548},
  {436, &fun545},
  {436, &fun546},
  {436, &fun547},
  {436, &fun544},
  {466, &fun302},
  {486, &fun383},
  {493, &fun385},
  {506, &fun301},
  {513, &fun384},
  {560, &fun303},
  {573, &fun467},
  {573, &fun469},
  {573, &fun412},
  {573, &fun468},
  {580, &fun413},
  {586, &fun414},
  {593, &fun415},
  {600, &fun304},
  {600, &fun416},
  {613, &fun441},
  {620, &fun442},
  {626, &fun443},
  {633, &fun444},
  {640, &fun388},
  {660, &fun386},
  {680, &fun389},
  {700, &fun387},
  {720, &fun382},
  {740, &fun390},
  {748, &fun458},
  {748, &fun457},
  {748, &fun455},
  {748, &fun454},
  {748, &fun452},
  {748, &fun453},
  {753, &fun306},
  {753, &fun305},
  {773, &fun307},
  {793, &fun391},
  {800, &fun404},
  {806, &fun405},
  {813, &fun411},
  {820, &fun440},
  {846, &fun392},
  {860, &fun308},
  {873, &fun309},
  {893, &fun393},
  {940, &fun394},
  {946, &fun445},
  {953, &fun446},
  {960, &fun447},
  {966, &fun448},
  {973, &fun310},
  {993, &fun395},
  {1026, &fun398},
  {1033, &fun419},
  {1033, &fun420},
  {1033, &fun470},
  {1033, &fun423},
  {1033, &fun418},
  {1033, &fun417},
  {1033, &fun421},
  {1033, &fun422},
  {1046, &fun311},
  {1046, &fun313},
  {1066, &fun396},
  {1080, &fun399},
  {1086, &fun424},
  {1086, &fun425},
  {1086, &fun430},
  {1086, &fun426},
  {1086, &fun429},
  {1086, &fun471},
  {1086, &fun428},
  {1086, &fun427},
  {1100, &fun312},
  {1106, &fun397},
  {1113, &fun381},
  {1120, &fun400},
  {1125, &fun409},
  {1125, &fun407},
  {1125, &fun410},
  {1125, &fun406},
  {1125, &fun408},
  {1133, &fun401},
  {1153, &fun449},
  {1160, &fun450},
  {1166, &fun451},
  {1166, &fun380},
  {1173, &fun314},
  {1206, &fun436},
  {1206, &fun463},
  {1213, &fun464},
  {1213, &fun437},
  {1220, &fun465},
  {1220, &fun402},
  {1220, &fun438},
  {1226, &fun439},
  {1226, &fun466},
  {1233, &fun403},
  {1241, &fun476},
  {1241, &fun473},
  {1241, &fun640},
  {1241, &fun475},
  {1241, &fun474},
  {1241, &fun639},
  {1246, &fun316},
  {1246, &fun315},
  {1253, &fun318},
  {1253, &fun317},
  {1306, &fun323},
  {1320, &fun435},
  {1320, &fun431},
  {1326, &fun460},
  {1326, &fun432},
  {1333, &fun461},
  {1333, &fun433},
  {1340, &fun434},
  {1340, &fun462},
  {1346, &fun321},
  {1346, &fun322},
  {1388, &fun456},
  {1388, &fun459},
  {1388, &fun641},
  {1388, &fun642},
  {1393, &fun320},
  {1393, &fun319},
  {1400, &fun325},
  {1400, &fun324},
  {1466, &fun332},
  {1466, &fun331},
  {1493, &fun328},
  {1493, &fun327},
  {1493, &fun326},
  {1493, &fun330},
  {1493, &fun329},
  {1540, &fun365},
  {1540, &fun374},
  {1553, &fun373},
  {1553, &fun364},
  {1560, &fun349},
  {1560, &fun351},
  {1566, &fun363},
  {1566, &fun379},
  {1566, &fun378},
  {1566, &fun377},
  {1566, &fun372},
  {1566, &fun375},
  {1566, &fun376},
  {1573, &fun348},
  {1573, &fun350},
  {1580, &fun371},
  {1580, &fun362},
  {1586, &fun361},
  {1586, &fun344},
  {1586, &fun347},
  {1586, &fun370},
  {1593, &fun360},
  {1593, &fun366},
  {1600, &fun358},
  {1600, &fun346},
  {1600, &fun367},
  {1600, &fun359},
  {1600, &fun368},
  {1600, &fun343},
  {1606, &fun357},
  {1606, &fun342},
  {1606, &fun345},
  {1606, &fun369},
  {1613, &fun337},
  {1613, &fun338},
  {1620, &fun340},
  {1620, &fun339},
  {1620, &fun335},
  {1620, &fun336},
  {1626, &fun333},
  {1626, &fun334},
  {1626, &fun341},
  {1740, &fun356},
  {1753, &fun355},
  {1780, &fun352},
  {1786, &fun354},
  {1793, &fun353},
  {1960, &fun472},
  {1973, &fun1},
];
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(520, 3380, 40, 160)));
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun4(Registry reg){
  set_scroll(reg, 3);
}
static void fun5(Registry reg){
  reg.add(new NormalWall(Rect(0, 3380, 40, 160)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(520, 3100, 40, 160)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(0, 3100, 40, 160)));
}
static void fun285(Registry reg){
  reg.add(new NormalWall(Rect(520, -520, 40, 500)));
}
static void fun287(Registry reg){
  reg.add(new NormalWall(Rect(420, -460, 20, 440)));
}
static void fun288(Registry reg){
  reg.add(new NormalWall(Rect(320, -400, 20, 380)));
}
static void fun290(Registry reg){
  reg.add(new NormalWall(Rect(220, -340, 20, 320)));
}
static void fun292(Registry reg){
  reg.add(new NormalWall(Rect(120, -280, 20, 260)));
}
static void fun284(Registry reg){
  reg.add(new NormalWall(Rect(0, -220, 40, 200)));
}
static void fun286(Registry reg){
  f05(480.000000, -20.000000, reg);
}
static void fun289(Registry reg){
  f04(380.000000, -20.000000, reg);
}
static void fun291(Registry reg){
  f03(280.000000, -20.000000, reg);
}
static void fun294(Registry reg){
  f05(80.000000, -20.000000, reg);
}
static void fun293(Registry reg){
  f01(180.000000, -20.000000, reg);
}
static void fun477(Registry reg){
  f102(340.000000, -20.000000, reg);
}
static void fun295(Registry reg){
  f102(540.000000, -20.000000, reg);
}
static void fun480(Registry reg){
  f102(140.000000, -20.000000, reg);
}
static void fun483(Registry reg){
  f102(-60.000000, -20.000000, reg);
}
static void fun486(Registry reg){
  f102(-260.000000, -20.000000, reg);
}
static void fun489(Registry reg){
  f102(-460.000000, -20.000000, reg);
}
static void fun297(Registry reg){
  f42(554.000000, -20.000000, reg);
}
static void fun491(Registry reg){
  f42(-446.000000, -20.000000, reg);
}
static void fun478(Registry reg){
  f41(445.000000, -20.000000, reg);
}
static void fun490(Registry reg){
  f41(-355.000000, -20.000000, reg);
}
static void fun479(Registry reg){
  f42(354.000000, -20.000000, reg);
}
static void fun488(Registry reg){
  f42(-246.000000, -20.000000, reg);
}
static void fun487(Registry reg){
  f41(-155.000000, -20.000000, reg);
}
static void fun296(Registry reg){
  f41(645.000000, -20.000000, reg);
}
static void fun485(Registry reg){
  f42(-46.000000, -20.000000, reg);
}
static void fun482(Registry reg){
  f42(154.000000, -20.000000, reg);
}
static void fun481(Registry reg){
  f41(245.000000, -20.000000, reg);
}
static void fun484(Registry reg){
  f41(45.000000, -20.000000, reg);
}
static void fun504(Registry reg){
  f103(980.000000, -20.000000, reg);
}
static void fun298(Registry reg){
  f103(-20.000000, -20.000000, reg);
}
static void fun501(Registry reg){
  f103(780.000000, -20.000000, reg);
}
static void fun495(Registry reg){
  f103(380.000000, -20.000000, reg);
}
static void fun492(Registry reg){
  f103(180.000000, -20.000000, reg);
}
static void fun498(Registry reg){
  f103(580.000000, -20.000000, reg);
}
static void fun493(Registry reg){
  f43(225.000000, -20.000000, reg);
}
static void fun499(Registry reg){
  f43(625.000000, -20.000000, reg);
}
static void fun497(Registry reg){
  f44(334.000000, -20.000000, reg);
}
static void fun500(Registry reg){
  f44(534.000000, -20.000000, reg);
}
static void fun494(Registry reg){
  f44(134.000000, -20.000000, reg);
}
static void fun299(Registry reg){
  f43(25.000000, -20.000000, reg);
}
static void fun496(Registry reg){
  f43(425.000000, -20.000000, reg);
}
static void fun502(Registry reg){
  f43(825.000000, -20.000000, reg);
}
static void fun503(Registry reg){
  f44(734.000000, -20.000000, reg);
}
static void fun506(Registry reg){
  f44(934.000000, -20.000000, reg);
}
static void fun505(Registry reg){
  f43(1025.000000, -20.000000, reg);
}
static void fun300(Registry reg){
  f44(-66.000000, -20.000000, reg);
}
static void fun507(Registry reg){
  f102(540.000000, -20.000000, reg);
}
static void fun525(Registry reg){
  f102(-540.000000, -20.000000, reg);
}
static void fun522(Registry reg){
  f102(-360.000000, -20.000000, reg);
}
static void fun510(Registry reg){
  f102(360.000000, -20.000000, reg);
}
static void fun519(Registry reg){
  f102(-180.000000, -20.000000, reg);
}
static void fun516(Registry reg){
  f102(0.000000, -20.000000, reg);
}
static void fun513(Registry reg){
  f102(180.000000, -20.000000, reg);
}
static void fun511(Registry reg){
  f41(465.000000, -20.000000, reg);
}
static void fun515(Registry reg){
  f42(194.000000, -20.000000, reg);
}
static void fun512(Registry reg){
  f42(374.000000, -20.000000, reg);
}
static void fun517(Registry reg){
  f41(105.000000, -20.000000, reg);
}
static void fun518(Registry reg){
  f42(14.000000, -20.000000, reg);
}
static void fun514(Registry reg){
  f41(285.000000, -20.000000, reg);
}
static void fun520(Registry reg){
  f41(-75.000000, -20.000000, reg);
}
static void fun521(Registry reg){
  f42(-166.000000, -20.000000, reg);
}
static void fun509(Registry reg){
  f42(554.000000, -20.000000, reg);
}
static void fun527(Registry reg){
  f42(-526.000000, -20.000000, reg);
}
static void fun523(Registry reg){
  f41(-255.000000, -20.000000, reg);
}
static void fun508(Registry reg){
  f41(645.000000, -20.000000, reg);
}
static void fun524(Registry reg){
  f42(-346.000000, -20.000000, reg);
}
static void fun526(Registry reg){
  f41(-435.000000, -20.000000, reg);
}
static void fun528(Registry reg){
  f103(-20.000000, -20.000000, reg);
}
static void fun539(Registry reg){
  f103(1060.000000, -20.000000, reg);
}
static void fun530(Registry reg){
  f103(160.000000, -20.000000, reg);
}
static void fun537(Registry reg){
  f103(880.000000, -20.000000, reg);
}
static void fun532(Registry reg){
  f103(340.000000, -20.000000, reg);
}
static void fun533(Registry reg){
  f103(520.000000, -20.000000, reg);
}
static void fun535(Registry reg){
  f103(700.000000, -20.000000, reg);
}
static void fun540(Registry reg){
  f44(1014.000000, -20.000000, reg);
}
static void fun536(Registry reg){
  f44(654.000000, -20.000000, reg);
}
static void fun531(Registry reg){
  f44(114.000000, -20.000000, reg);
}
static void fun538(Registry reg){
  f44(834.000000, -20.000000, reg);
}
static void fun529(Registry reg){
  f44(-66.000000, -20.000000, reg);
}
static void fun534(Registry reg){
  f44(474.000000, -20.000000, reg);
}
static void fun541(Registry reg){
  f44(294.000000, -20.000000, reg);
}
static void fun542(Registry reg){
  f43(385.000000, -20.000000, reg);
}
static void fun543(Registry reg){
  f43(205.000000, -20.000000, reg);
}
static void fun548(Registry reg){
  f43(1105.000000, -20.000000, reg);
}
static void fun545(Registry reg){
  f43(565.000000, -20.000000, reg);
}
static void fun546(Registry reg){
  f43(745.000000, -20.000000, reg);
}
static void fun547(Registry reg){
  f43(925.000000, -20.000000, reg);
}
static void fun544(Registry reg){
  f43(25.000000, -20.000000, reg);
}
static void fun302(Registry reg){
  reg.add(new NormalWall(Rect(200, -220, 40, 200)));
}
static void fun383(Registry reg){
  f117(600.000000, -20.000000, reg);
}
static void fun385(Registry reg){
  f118(-41.000000, -20.000000, reg);
}
static void fun301(Registry reg){
  reg.add(new NormalWall(Rect(0, -360, 40, 340)));
}
static void fun384(Registry reg){
  f117(600.000000, -20.000000, reg);
}
static void fun303(Registry reg){
  reg.add(new NormalWall(Rect(520, -220, 40, 200)));
}
static void fun467(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun469(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun412(Registry reg){
  f52(200.000000, -20.000000, reg);
}
static void fun468(Registry reg){
  f52(240.000000, -20.000000, reg);
}
static void fun413(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun414(Registry reg){
  f52(360.000000, -20.000000, reg);
}
static void fun415(Registry reg){
  f52(340.000000, -20.000000, reg);
}
static void fun304(Registry reg){
  reg.add(new NormalWall(Rect(160, -360, 40, 340)));
}
static void fun416(Registry reg){
  f52(320.000000, -20.000000, reg);
}
static void fun441(Registry reg){
  f51(440.000000, -20.000000, reg);
}
static void fun442(Registry reg){
  f51(460.000000, -20.000000, reg);
}
static void fun443(Registry reg){
  f51(480.000000, -20.000000, reg);
}
static void fun444(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun388(Registry reg){
  f120(-41.000000, -20.000000, reg);
}
static void fun386(Registry reg){
  f119(600.000000, -20.000000, reg);
}
static void fun389(Registry reg){
  f120(-41.000000, -20.000000, reg);
}
static void fun387(Registry reg){
  f119(600.000000, -20.000000, reg);
}
static void fun382(Registry reg){
  f02(60.000000, -20.000000, reg);
}
static void fun390(Registry reg){
  f121(-41.000000, -20.000000, reg);
}
static void fun458(Registry reg){
  f46(539.000000, -20.000000, reg);
}
static void fun457(Registry reg){
  f46(479.000000, -20.000000, reg);
}
static void fun455(Registry reg){
  f46(399.000000, -20.000000, reg);
}
static void fun454(Registry reg){
  f46(160.000000, -20.000000, reg);
}
static void fun452(Registry reg){
  f46(80.000000, -20.000000, reg);
}
static void fun453(Registry reg){
  f46(20.000000, -20.000000, reg);
}
static void fun306(Registry reg){
  reg.add(new NormalWall(Rect(340, -80, 220, 60)));
}
static void fun305(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 220, 60)));
}
static void fun307(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 160, 140)));
}
static void fun391(Registry reg){
  f122(600.000000, -20.000000, reg);
}
static void fun404(Registry reg){
  f51(480.000000, -20.000000, reg);
}
static void fun405(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun411(Registry reg){
  f51(520.000000, -20.000000, reg);
}
static void fun440(Registry reg){
  f51(540.000000, -20.000000, reg);
}
static void fun392(Registry reg){
  f123(600.000000, -20.000000, reg);
}
static void fun308(Registry reg){
  reg.add(new NormalWall(Rect(140, -60, 420, 40)));
}
static void fun309(Registry reg){
  reg.add(new NormalWall(Rect(240, -140, 320, 120)));
}
static void fun393(Registry reg){
  f124(-41.000000, -20.000000, reg);
}
static void fun394(Registry reg){
  f125(-41.000000, -20.000000, reg);
}
static void fun445(Registry reg){
  f52(440.000000, -20.000000, reg);
}
static void fun446(Registry reg){
  f52(420.000000, -20.000000, reg);
}
static void fun447(Registry reg){
  f52(400.000000, -20.000000, reg);
}
static void fun448(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun310(Registry reg){
  reg.add(new NormalWall(Rect(0, -140, 320, 120)));
}
static void fun395(Registry reg){
  f126(600.000000, -20.000000, reg);
}
static void fun398(Registry reg){
  f128(-41.000000, -20.000000, reg);
}
static void fun419(Registry reg){
  f52(180.000000, -20.000000, reg);
}
static void fun420(Registry reg){
  f52(140.000000, -20.000000, reg);
}
static void fun470(Registry reg){
  f52(-20.000000, -20.000000, reg);
}
static void fun423(Registry reg){
  f52(20.000000, -20.000000, reg);
}
static void fun418(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun417(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun421(Registry reg){
  f52(100.000000, -20.000000, reg);
}
static void fun422(Registry reg){
  f52(60.000000, -20.000000, reg);
}
static void fun311(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 320, 60)));
}
static void fun313(Registry reg){
  reg.add(new NormalWall(Rect(420, -240, 40, 220)));
}
static void fun396(Registry reg){
  f127(600.000000, -20.000000, reg);
}
static void fun399(Registry reg){
  f129(-41.000000, -20.000000, reg);
}
static void fun424(Registry reg){
  f52(260.000000, -20.000000, reg);
}
static void fun425(Registry reg){
  f52(220.000000, -20.000000, reg);
}
static void fun430(Registry reg){
  f52(20.000000, -20.000000, reg);
}
static void fun426(Registry reg){
  f52(180.000000, -20.000000, reg);
}
static void fun429(Registry reg){
  f52(60.000000, -20.000000, reg);
}
static void fun471(Registry reg){
  f52(-20.000000, -20.000000, reg);
}
static void fun428(Registry reg){
  f52(100.000000, -20.000000, reg);
}
static void fun427(Registry reg){
  f52(140.000000, -20.000000, reg);
}
static void fun312(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 320, 60)));
}
static void fun397(Registry reg){
  f127(600.000000, -20.000000, reg);
}
static void fun381(Registry reg){
  f03(370.000000, -20.000000, reg);
}
static void fun400(Registry reg){
  f130(0.000000, -20.000000, reg);
}
static void fun409(Registry reg){
  f45(200.000000, -20.000000, reg);
}
static void fun407(Registry reg){
  f45(80.000000, -20.000000, reg);
}
static void fun410(Registry reg){
  f45(260.000000, -20.000000, reg);
}
static void fun406(Registry reg){
  f45(20.000000, -20.000000, reg);
}
static void fun408(Registry reg){
  f45(140.000000, -20.000000, reg);
}
static void fun401(Registry reg){
  f131(600.000000, -20.000000, reg);
}
static void fun449(Registry reg){
  f52(400.000000, -20.000000, reg);
}
static void fun450(Registry reg){
  f52(380.000000, -20.000000, reg);
}
static void fun451(Registry reg){
  f52(360.000000, -20.000000, reg);
}
static void fun380(Registry reg){
  f05(40.000000, -20.000000, reg);
}
static void fun314(Registry reg){
  reg.add(new NormalWall(Rect(120, -80, 320, 60)));
}
static void fun436(Registry reg){
  f53(380.000000, -20.000000, reg);
}
static void fun463(Registry reg){
  f54(178.000000, -20.000000, reg);
}
static void fun464(Registry reg){
  f54(158.000000, -20.000000, reg);
}
static void fun437(Registry reg){
  f53(400.000000, -20.000000, reg);
}
static void fun465(Registry reg){
  f54(138.000000, -20.000000, reg);
}
static void fun402(Registry reg){
  f130(-41.000000, -20.000000, reg);
}
static void fun438(Registry reg){
  f53(420.000000, -20.000000, reg);
}
static void fun439(Registry reg){
  f53(440.000000, -20.000000, reg);
}
static void fun466(Registry reg){
  f54(118.000000, -20.000000, reg);
}
static void fun403(Registry reg){
  f131(600.000000, -20.000000, reg);
}
static void fun476(Registry reg){
  f46(499.000000, -20.000000, reg);
}
static void fun473(Registry reg){
  f46(20.000000, -20.000000, reg);
}
static void fun640(Registry reg){
  f46(419.000000, -20.000000, reg);
}
static void fun475(Registry reg){
  f46(539.000000, -20.000000, reg);
}
static void fun474(Registry reg){
  f46(60.000000, -20.000000, reg);
}
static void fun639(Registry reg){
  f46(140.000000, -20.000000, reg);
}
static void fun316(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun315(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun318(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun317(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun323(Registry reg){
  reg.add(new NormalWall(Rect(240, -120, 80, 100)));
}
static void fun435(Registry reg){
  f52(58.000000, -20.000000, reg);
}
static void fun431(Registry reg){
  f51(500.000000, -20.000000, reg);
}
static void fun460(Registry reg){
  f52(38.000000, -20.000000, reg);
}
static void fun432(Registry reg){
  f51(520.000000, -20.000000, reg);
}
static void fun461(Registry reg){
  f52(18.000000, -20.000000, reg);
}
static void fun433(Registry reg){
  f51(540.000000, -20.000000, reg);
}
static void fun434(Registry reg){
  f51(560.000000, -20.000000, reg);
}
static void fun462(Registry reg){
  f52(-2.000000, -20.000000, reg);
}
static void fun321(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun322(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun456(Registry reg){
  f46(140.000000, -20.000000, reg);
}
static void fun459(Registry reg){
  f46(419.000000, -20.000000, reg);
}
static void fun641(Registry reg){
  f46(180.000000, -20.000000, reg);
}
static void fun642(Registry reg){
  f46(379.000000, -20.000000, reg);
}
static void fun320(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun319(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun325(Registry reg){
  reg.add(new NormalWall(Rect(380, -220, 180, 200)));
}
static void fun324(Registry reg){
  reg.add(new NormalWall(Rect(0, -220, 180, 200)));
}
static void fun332(Registry reg){
  reg.add(new NormalWall(Rect(460, -160, 100, 140)));
}
static void fun331(Registry reg){
  reg.add(new NormalWall(Rect(0, -160, 100, 140)));
}
static void fun328(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun327(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun326(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun330(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun329(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun365(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun374(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun373(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun364(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun349(Registry reg){
  f13(600.000000, -20.000000, reg);
}
static void fun351(Registry reg){
  f13(-40.000000, -20.000000, reg);
}
static void fun363(Registry reg){
  f13(40.000000, -20.000000, reg);
}
static void fun379(Registry reg){
  f12(160.000000, -20.000000, reg);
}
static void fun378(Registry reg){
  f12(220.000000, -20.000000, reg);
}
static void fun377(Registry reg){
  f12(280.000000, -20.000000, reg);
}
static void fun372(Registry reg){
  f13(520.000000, -20.000000, reg);
}
static void fun375(Registry reg){
  f12(400.000000, -20.000000, reg);
}
static void fun376(Registry reg){
  f12(340.000000, -20.000000, reg);
}
static void fun348(Registry reg){
  f13(600.000000, -20.000000, reg);
}
static void fun350(Registry reg){
  f13(-40.000000, -20.000000, reg);
}
static void fun371(Registry reg){
  f13(520.000000, -20.000000, reg);
}
static void fun362(Registry reg){
  f13(40.000000, -20.000000, reg);
}
static void fun361(Registry reg){
  f13(80.000000, -20.000000, reg);
}
static void fun344(Registry reg){
  f13(-20.000000, -20.000000, reg);
}
static void fun347(Registry reg){
  f13(580.000000, -20.000000, reg);
}
static void fun370(Registry reg){
  f13(480.000000, -20.000000, reg);
}
static void fun360(Registry reg){
  f13(120.000000, -20.000000, reg);
}
static void fun366(Registry reg){
  f13(440.000000, -20.000000, reg);
}
static void fun358(Registry reg){
  f13(220.000000, -20.000000, reg);
}
static void fun346(Registry reg){
  f13(580.000000, -20.000000, reg);
}
static void fun367(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun359(Registry reg){
  f13(160.000000, -20.000000, reg);
}
static void fun368(Registry reg){
  f13(340.000000, -20.000000, reg);
}
static void fun343(Registry reg){
  f13(-20.000000, -20.000000, reg);
}
static void fun357(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun342(Registry reg){
  f13(20.000000, -20.000000, reg);
}
static void fun345(Registry reg){
  f13(540.000000, -20.000000, reg);
}
static void fun369(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun337(Registry reg){
  f13(60.000000, -20.000000, reg);
}
static void fun338(Registry reg){
  f13(500.000000, -20.000000, reg);
}
static void fun340(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun339(Registry reg){
  f13(460.000000, -20.000000, reg);
}
static void fun335(Registry reg){
  f13(160.000000, -20.000000, reg);
}
static void fun336(Registry reg){
  f13(100.000000, -20.000000, reg);
}
static void fun333(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun334(Registry reg){
  f13(220.000000, -20.000000, reg);
}
static void fun341(Registry reg){
  f13(340.000000, -20.000000, reg);
}
static void fun356(Registry reg){
  set_scroll(reg, 3);
}
static void fun355(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun352(Registry reg){
  f05(300.000000, -20.000000, reg);
}
static void fun354(Registry reg){
  f04(280.000000, -20.000000, reg);
}
static void fun353(Registry reg){
  f05(260.000000, -20.000000, reg);
}
static void fun472(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events2{
const static Event[] events = [
  {1, &fun312},
  {1, &fun5},
  {1, &fun6},
  {1, &fun12},
  {1, &fun18},
  {1, &fun284},
  {1, &fun285},
  {1, &fun287},
  {1, &fun288},
  {1, &fun290},
  {1, &fun292},
  {1, &fun301},
  {1, &fun302},
  {1, &fun303},
  {1, &fun304},
  {1, &fun305},
  {1, &fun306},
  {1, &fun307},
  {1, &fun308},
  {1, &fun309},
  {1, &fun310},
  {1, &fun355},
  {1, &fun356},
  {1, &fun313},
  {1, &fun314},
  {1, &fun315},
  {1, &fun316},
  {1, &fun317},
  {1, &fun318},
  {1, &fun319},
  {1, &fun320},
  {1, &fun321},
  {1, &fun322},
  {1, &fun323},
  {1, &fun324},
  {1, &fun325},
  {1, &fun331},
  {1, &fun332},
  {1, &fun311},
  {20, &fun352},
  {26, &fun354},
  {33, &fun353},
  {200, &fun472},
  {213, &fun1},
];
static void fun312(Registry reg){
  reg.add(new NormalWall(Rect(0, 1900, 320, 60)));
}
static void fun5(Registry reg){
  reg.add(new NormalWall(Rect(0, 8660, 40, 160)));
}
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(520, 8660, 40, 160)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(0, 8380, 40, 160)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(520, 8380, 40, 160)));
}
static void fun284(Registry reg){
  reg.add(new NormalWall(Rect(0, 4660, 40, 200)));
}
static void fun285(Registry reg){
  reg.add(new NormalWall(Rect(520, 4660, 40, 500)));
}
static void fun287(Registry reg){
  reg.add(new NormalWall(Rect(420, 4660, 20, 440)));
}
static void fun288(Registry reg){
  reg.add(new NormalWall(Rect(320, 4660, 20, 380)));
}
static void fun290(Registry reg){
  reg.add(new NormalWall(Rect(220, 4660, 20, 320)));
}
static void fun292(Registry reg){
  reg.add(new NormalWall(Rect(120, 4660, 20, 260)));
}
static void fun301(Registry reg){
  reg.add(new NormalWall(Rect(0, 3400, 40, 340)));
}
static void fun302(Registry reg){
  reg.add(new NormalWall(Rect(200, 3660, 40, 200)));
}
static void fun303(Registry reg){
  reg.add(new NormalWall(Rect(520, 3380, 40, 200)));
}
static void fun304(Registry reg){
  reg.add(new NormalWall(Rect(160, 3120, 40, 340)));
}
static void fun305(Registry reg){
  reg.add(new NormalWall(Rect(0, 2940, 220, 60)));
}
static void fun306(Registry reg){
  reg.add(new NormalWall(Rect(340, 2940, 220, 60)));
}
static void fun307(Registry reg){
  reg.add(new NormalWall(Rect(0, 2800, 160, 140)));
}
static void fun308(Registry reg){
  reg.add(new NormalWall(Rect(140, 2640, 420, 40)));
}
static void fun309(Registry reg){
  reg.add(new NormalWall(Rect(240, 2520, 320, 120)));
}
static void fun310(Registry reg){
  reg.add(new NormalWall(Rect(0, 2220, 320, 120)));
}
static void fun355(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun356(Registry reg){
  set_scroll(reg, 3);
}
static void fun313(Registry reg){
  reg.add(new NormalWall(Rect(420, 1900, 40, 220)));
}
static void fun314(Registry reg){
  reg.add(new NormalWall(Rect(120, 1680, 320, 60)));
}
static void fun315(Registry reg){
  reg.add(new NormalWall(Rect(0, 1500, 220, 20)));
}
static void fun316(Registry reg){
  reg.add(new NormalWall(Rect(340, 1500, 220, 20)));
}
static void fun317(Registry reg){
  reg.add(new NormalWall(Rect(0, 1360, 100, 140)));
}
static void fun318(Registry reg){
  reg.add(new NormalWall(Rect(460, 1360, 100, 140)));
}
static void fun319(Registry reg){
  reg.add(new NormalWall(Rect(0, 1060, 220, 20)));
}
static void fun320(Registry reg){
  reg.add(new NormalWall(Rect(340, 1060, 220, 20)));
}
static void fun321(Registry reg){
  reg.add(new NormalWall(Rect(460, 1080, 100, 140)));
}
static void fun322(Registry reg){
  reg.add(new NormalWall(Rect(0, 1080, 100, 140)));
}
static void fun323(Registry reg){
  reg.add(new NormalWall(Rect(240, 1240, 80, 100)));
}
static void fun324(Registry reg){
  reg.add(new NormalWall(Rect(0, 860, 180, 200)));
}
static void fun325(Registry reg){
  reg.add(new NormalWall(Rect(380, 860, 180, 200)));
}
static void fun331(Registry reg){
  reg.add(new NormalWall(Rect(0, 720, 100, 140)));
}
static void fun332(Registry reg){
  reg.add(new NormalWall(Rect(460, 720, 100, 140)));
}
static void fun311(Registry reg){
  reg.add(new NormalWall(Rect(0, 2060, 320, 60)));
}
static void fun352(Registry reg){
  f05(300.000000, -20.000000, reg);
}
static void fun354(Registry reg){
  f04(280.000000, -20.000000, reg);
}
static void fun353(Registry reg){
  f05(260.000000, -20.000000, reg);
}
static void fun472(Registry reg){
  f61(270.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
  Events2.events,
];
