import kinds;
import util;
import base;
import images = images.src.images;
import opengl;
import glutils;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;
import rand;

class SmallFlak: BaseEnemy
{
public:
  this(Point pos, int deg, int start, int rate, int speed) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.flak.draw),
          10, Color.enemy, deg, 1);
    set_repeating_fire(start, rate, speed);
  }

  mixin DefineEnemy!(100);
  mixin DefineRect!(pos, 20, 20, true);

  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
  }
}

class Chaser: BaseEnemy
{
private:
  bool collide_with_wall_ = false;
  bool move_x_ = true;
  Point old_pos;
  static double d = 4;
  
public:
  this(Point pos) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.chaser.draw),
          160);
    scale = 1.5;
  }
  mixin DefineEnemy!(400);
  mixin DefineRect!(pos, 20, 20, true);

  void run(Registry reg) {
    Player pl = Player.search(reg);

    die_if_out_of_window();

    if (collide_with_wall_) {
      this.pos = old_pos;
      scroll(reg);
    }

    collide_with_wall_ = false;
    old_pos = pos;

    degree += 2;

    if (!pl)
      return;
    
    Vector dp = pl.pos - this.pos;
    
    if (count % 100 < 40)
      chase(dp);

    if (count % 100 == 90)
      fireToPlayer(reg, 16, pos, 5);
    
  }

  
  void chase(Vector dp) {
    move_x_ = !move_x_;
    
    if (move_x_) {
      if (abs(dp.x) > d)
        move(Vector(sign(dp.x)*d, 0));
    }else{
      if (abs(dp.y) > d)
        move(Vector(0, sign(dp.y)*d));
    }
  }

  
  void collide_with_wall(Registry reg, Wall wall) {
    collide_with_wall_ = true;
  }
}

class XYChaser: BaseEnemy
{
private:
  bool collide_with_wall_ = false;
  Point old_pos_;
  double deg_;
  Vector v_;
  int walkcount_ = 0;
  int dt_;
  static double d = 2;
  
public:
  this(Point pos, int dt) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.chaser.draw),
          400);
    scale = 1.5;
    deg_ = drand(-180, 180);
    dt_ = dt;
  }
  
  mixin DefineEnemy!(1200);
  mixin DefineRect!(pos, 40, 40, true);
  
  void run(Registry reg) {
    Player pl = Player.search(reg);
    deg_ += 6;
    degree += 6;
    die_if_out_of_window();
    

    if (collide_with_wall_) {
      this.pos = old_pos_;
      scroll(reg);
    }

    collide_with_wall_ = false;
    old_pos_ = pos;

    if (!pl)
      return;
    
    int time = (count+dt_) % 120;
    
    if (time == 0) {
      walkcount_ = cast(int)(abs(pl.pos.x - pos.x)/d);
      v_ = Vector(sign(pl.pos.x - pos.x)*d, 0);
    }
    if (time == 40)
      walkcount_ = 0;
    if (time == 60) {
      walkcount_ = cast(int)(abs(pl.pos.y - pos.y)/d);
      v_ = Vector(0, sign(pl.pos.y - pos.y)*d);
    }
    if (time == 100)
      walkcount_ = 0;
    if (walkcount_ > 0) {
      move(v_);
      walkcount_--;
    }
  }

  void collide_with_wall(Registry reg, Wall wall) {
    collide_with_wall_ = true;
  }
  void draw(Registry reg) {
    glColor3da(Color.enemy);
    glPushMatrix();
    glTranslated(pos.x, pos.y, 0);
    glPushMatrix();
    glRotated(deg_, 1, 0, 0);
    glScaled(2.2, 2.2, 1);
    images.player.octagon.draw();
    glPopMatrix();
    glRotated(deg_, 0, 1, 0);
    glScaled(2.2, 2.2, 1);
    images.player.octagon.draw();
    glPopMatrix();
    super.draw(reg);
  }
  
}

class FixedFlow: BaseEnemy {
  this(Point p, int start, int rate, int speed) {
    super(SubKind.NORMAL_ENEMY, p,
          new Animation(&images.enemies.fixedflow.draw),
          450, Color.enemy);
    scale = 3;
    set_repeating_fire(start, rate, speed);
  }
  mixin DefineRect!(pos, 60, 60, true);
  mixin DefineEnemy!(800);

  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
  }
}

class TimeLimitWall: BaseWall {
  int limit_;
  void function(TimeLimitWall,Registry) destroyed_hook_;
  
  this(Rect rect, int limit,
       void function(TimeLimitWall,Registry) destroyed) {
    super(rect);
    limit_ = limit;
    destroyed_hook_ = destroyed;
  }

  void run(Registry reg) {
    if (count == limit_ && !dead()) {
      if (destroyed_hook_)
        destroyed_hook_(this, reg);
      reg.add(new DestroyedEnemy(rect().center, true,
                                 rect().w));
      die();
    }
    super.run(reg);
  }
  void draw(Registry reg) {
    glColor3da(Color.wall);
    rect.draw();
  }
}

void f01(double x, double y, Registry reg)
{
  Shutter.add(Point(x,y), 420, 0, reg);
}

void f02(double x, double y, Registry reg)
{
  Shutter.add(Point(x,y), 140, 180, reg);
}

void f03(double dy, double sy, double x, double y, Registry reg)
{
  Enemy enemy = new class (Point(x,y+sy), dy) Patapata {
    double x_, dx_, dy_;
    this(Point p, double dy) {
      super(p);
      if (dy < 0)
        degree = 0;
      else
        degree = 180;
      x_ = p.x;
      dx_ = -2;
      dy_ = dy;
      set_repeating_fire(50, 40, 6);
    }
    void run(Registry reg){
      dx_ += -0.01*(pos.x - x_);
      ydegree += (dx_*3);
      move(Vector(dx_, dy_));
      die_if_out_of_window();
    }
    void collide_with_wall(Registry reg, Wall wall) {
      if (!dead()) {
        Wave.enemy_destroyed.play();
        reg.add(new DestroyedEnemy(pos, true, rect().w));
        die();
      }
    }
  };
  reg.add(enemy);
}

void f04(double x, double y, Registry reg)
{
  Shutter.add(Point(x, y), 520, 0, reg);
}

void f05(double x, double y, Registry reg)
{
  RNG rng = reg.root.rng;
  reg.add(new SmallFlak(Point(x, y), 90, rng.irand(50, 60),
                        80, 4));
}

void f06(double x, double y, Registry reg)
{
  RNG rng = reg.root.rng;
  reg.add(new SmallFlak(Point(x, y), 0, rng.irand(50, 60),
                        80, 4));
}

void f07(double x, double y, Registry reg)
{
  reg.add(new Chaser(Point(x,y)));
}
  
void f08(double x, double y, Registry reg)
{
  Enemy carrier = new class(Point(x ,y)) CapsuleCarrier {
    this(Point pos) {
      super(pos);
      set_capsule_drop!(OptionCapsule)();
    }
    
    void run(Registry reg) {
      move(Vector(0, 2));
      die_if_out_of_window();
    }
  };

  reg.add(carrier);
}

void f09(int dt, double x, double y, Registry reg)
{
  reg.add(new XYChaser(Point(x,y), dt));
}

void f10(double x, double y, Registry reg)
{
  reg.add(new SmallFlak(Point(x, y), -90, 50, 80, 4));
}

void f11(int dt, double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y), dt, reg.root.rng) Wiper {
    int dt_;
    
    this(Point p, int dt, RNG rng) {
      super(p);
      dt_ = dt;
      set_repeating_fire(rng.irand(10, 70), 60, 5);
    }
    void run(Registry reg) {
      scroll(reg);
      move_vector_when(Vector(2, 0), 0, dt_+49);
      move_vector_when(Vector(0, 2), dt_+50, dt_+99);
      move_vector_when(Vector(2, 0), dt_+100, dt_+169);
      move_vector_when(Vector(0, -2), dt_+170, dt_+439);
      move_vector_when(Vector(2, 0), dt_+440, 999);
      if (count == 1000) {
        die();
      }
    }

  };
  reg.add(enemy);
}

void f12(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p); set_capsule_drop!(BladeCapsule)();
    }
    void run(Registry reg) {
      move(Vector(0, 2));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f13(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y), reg.root.rng) XYChaser {
    this(Point p, RNG rng) {
      super(p, rng.irand(-150, -90));
    }
    void run(Registry reg) {
      if (count < 150) {
        scroll(reg);
        deg_ += 6;
      } else {
        XYChaser.run(reg);
      }
    }
  };
  reg.add(enemy);
}

void f14(int limit, double x, double y, Registry reg)
{
  static void add_fixedflow(TimeLimitWall wall,
                            Registry reg) {
    reg.add(new FixedFlow(wall.rect.center, 120, 80, 4));
  }

  
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_fixedflow));
  
}

class KpFlyer: LFlyer {
public:
  this(Point pos, int deg, double speed, int fire_first) {
    super(pos, deg, speed, 8000, fire_first);
  }
  void collide_with_wall(Registry reg, Wall) {
    if (!dead()) {
      reg.add(new DestroyedEnemy(pos, true, rect().w));
      die();
    }
  }
}

void f15(int limit, double x, double y, Registry reg)
{
  static void add_4flyer(TimeLimitWall wall, Registry reg) {
    RNG rng = reg.root.rng;
    int i;
    Point p = wall.rect.center;
    for (i=0; i<4; i++) {
      reg.add(new KpFlyer(p, 45+i*90, 2, rng.irand(35, 45)));
    }
  }
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_4flyer));
}

void f16(int limit, double x, double y, Registry reg)
{
  static void add_xychaser(TimeLimitWall wall, Registry reg) {
    reg.add(new XYChaser(wall.rect.center,
                         reg.root.rng.irand(0, 20)));
  }
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_xychaser));
}

void f17(int limit, double x, double y, Registry reg)
{
  static void add_2flak_vertically(TimeLimitWall wall,
                                   Registry reg) {
    Point p1 = wall.rect.center + Vector(0, -19);
    Point p2 = wall.rect.center + Vector(0, 21);
    int n = reg.root.rng.irand(35, 45);
    reg.add(new SmallFlak(p1, 180, n, 65, 4));
    reg.add(new SmallFlak(p2, 0, n, 65, 4));
  }
  
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_2flak_vertically));
}

void f18(int limit, double x, double y, Registry reg)
{
  static void add_2flak_horizontally(TimeLimitWall wall,
                                     Registry reg) {
    Point p1 = wall.rect.center + Vector(-19, 0);
    Point p2 = wall.rect.center + Vector(21, 0);
    int n = reg.root.rng.irand(35, 45);
    reg.add(new SmallFlak(p1, 90, n, 65, 4));
    reg.add(new SmallFlak(p2, -90, n, 65, 4));
  }
  
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_2flak_horizontally));
}


template add_flak_f(int deg, int dx, int  dy, int vx, int vy)
{
  void add_flak_f(TimeLimitWall wall, Registry reg) {
    Point p = wall.rect.center + Vector(dx, dy);
    Enemy enemy = new class(p) Flak {
      this(Point p) {
        super(p, deg);
      }
      void run(Registry reg) {
        scroll(reg);
        if (count % 40 == 10) {
          reg.add(new StripBullet(pos, Vector(vx, vy), true));
        }
      }
    };
    reg.add(enemy);
  }
}

void f19(int limit, double x, double y, Registry reg)
{
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_flak_f!(180, 0, -14, 0, 6)));
}

void f20(int limit, double x, double y, Registry reg)
{
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_flak_f!(0, 0, 15, 0, -6)));
}

void f21(int limit, double x, double y, Registry reg)
{
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_flak_f!(90, -15, 0, 6, 0)));
}

void f22(int limit, double x, double y, Registry reg)
{
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_flak_f!(-90, 14, 0, -6, 0)));
}

void f23(int limit, double x, double y, Registry reg)
{
  static void add_3flyer(TimeLimitWall wall, Registry reg) {
    RNG rng = reg.root.rng;
    Point p = wall.rect.center;
    Point[3] ps = [p+Vector(0, -16),
                   p+Vector(12 ,8),
                   p+Vector(-12 ,8)];
    for (int i=0; i<3; i++) {
      reg.add(new KpFlyer(ps[i], -90, 2, rng.irand(35, 45)));
    }
  }

  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_3flyer));
}

void f24(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p); set_capsule_drop!(FiveWayCapsule)();
    }
    void run(Registry reg) {
      move(Vector(0, 1));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f25(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p); set_capsule_drop!(OptionCapsule)();
    }
    void run(Registry reg) {
      move(Vector(0, 1));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f26(int limit, double x, double y, Registry reg)
{
  static void add_flak(TimeLimitWall wall, Registry reg) {
    Point p = wall.rect.center + Vector(21, 0);
    RNG rng = reg.root.rng;
    reg.add(new SmallFlak(p, -90, rng.irand(35, 45), 65, 4));
  }
  
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit, &add_flak));
}

void f27(int limit, double x, double y, Registry reg)
{
  static void add_flak(TimeLimitWall wall, Registry reg) {
    Point p = wall.rect.center + Vector(0, 21);
    RNG rng = reg.root.rng;
    reg.add(new SmallFlak(p, 0, rng.irand(35, 45), 65, 4));
  }
  
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit, &add_flak));
}

void f28(int limit, double x, double y, Registry reg)
{
  static void add_3flyer(TimeLimitWall wall, Registry reg) {
    RNG rng = reg.root.rng;
    Point p = wall.rect.center;
    Point[3] ps = [p+Vector(0, -16),
                   p+Vector(12 ,8),
                   p+Vector(-12 ,8)];
    for (int i=0; i<3; i++) {
      reg.add(new KpFlyer(ps[i], -90, 2, 8000));
    }
  }

  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit,
                            &add_3flyer));
}

void nilwall(int limit, double x, double y, Registry reg)
{
  reg.add(new TimeLimitWall(Rect(x, y-60, 60, 60), limit, null));
}

void create_boss5(double x, double y, Registry reg)
{
  Boss5 boss = new Boss5(Point(x, y-160));
  reg.add(boss);
  reg.add(new Boss5.RingA(Point(200, 30), -1, boss));
  reg.add(new Boss5.RingA(Point(-200, 30), 1, boss));
  reg.add(boss.new Boss5.RingB(Point(50, 80), -1, boss));
  reg.add(boss.new Boss5.RingB(Point(-50, 80), 1, boss));
  reg.add(boss.new Boss5.RingC(Point(100, -60), -1, boss));
  reg.add(boss.new Boss5.RingC(Point(-100, -60), 1, boss));
  reg.add(boss.new Boss5.RollRingCreator(1));
  reg.add(boss.new Boss5.RollRingCreator(-1));
  Music.boss[5].play();
}

template DefineDefFire(int r1, int r2,
                       int start, int end,
                       bool upper) {
  private int firecount_;
  void deffire(Registry reg, int t) {
    if (start-1 == t) {
      firecount_ = reg.root.rng.irand(r1, r2);
    }
    if (start <= t && t <= end) {
      if (firecount_ == 0) {
        Vector v;
        if (upper) {
          v = Vector(0, -6);
        } else {
          v = Vector(0, 6);
        }
        reg.add(new DefBulletCreator(pos, v, 10, 2));
        firecount_ = reg.root.rng.irand(r1, r2);
      } else {
        --firecount_;
      }
    }
  }
}

class Boss5: BaseEnemy {
private:
  int start_sign_;
  
public:
  static const int WAIT = 300;
  static const int LOOP = 1500;

  
  this(Point pos) {
    super(SubKind.BOSS_ENEMY, pos,
          new Animation(&images.enemies.boss5.draw),
          5000);
    scale = 4;
    set_destroyed_hook(&destroyed);
  }

  mixin DefineRect!(pos, 300, 60, true);
  mixin DefineEnemy!(30000);
  
  static void destroyed(Registry reg, Enemy) {
    reg.add(new StageEnd(5, 6));
  }

  void run(Registry reg) {
    if (count < WAIT-30) {
      move(Vector(0, 1));
      return;
    } 
    if (count > 60*60*3) {
      move(Vector(0, -1));
    }
    if (count == 60*60*3 + WAIT) {
      destroyed(reg, this);
      die();
    }
    
    int t = (count - WAIT) % LOOP;
    if (t==100)
      start_sign_ = reg.root.rng.brand() ? 1 : -1;
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < WAIT) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }
  
  static class Ring: ChildEnemy {
  protected:
    int sign_;
  public:
    this(Point pos, int sign, Boss5 boss) {
      super(SubKind.BOSS_ENEMY, pos,
            new Animation(&images.enemies.boss5ring.draw),
            Color.enemy, 0, 0, boss);
      scale = 3;
      sign_ = sign;
      ymirror = -sign;
    }
    mixin DefineRect!(pos, 80, 80, true);
    mixin DefineEnemy!(0);
    void _run(Registry reg) {
      degree += sign_*6;
      super._run(reg);
    }
  }

  static class RingA: Ring{
  public:
    this(Point pos, int sign, Boss5 boss) {
      super(pos, sign, boss);
    }

    void run(Registry reg) {
      if (count < WAIT)
        return;
      
      int t = (count - WAIT) % LOOP;
      if (t==300) {
        reg.add(new DefBulletCreator(pos, Vector(0, 5), 10, 2));
      }

      int tu = sign_ > 0 ? 0 : 10;
      if (400 <= t && t <= 800 && t % 20 == tu) {
        reg.add(new RingA_F(pos, sign_));
      }

      if (850 <= t && t <= 881)
        move(Vector(0, 10));

      if (900 <= t && t <= 1300 && t % 60 == 0) {
        StripBullet.fireToPlayer(reg, 32, pos, 3);
      }
      if (1400 <= t && t <= 1431)
        move(Vector(0, -10));
      
      
    }
  }
  class RingB: Ring{
  private:
    double fire_rad_;
    double fire_drad_;

  public:
    
    this(Point pos, int sign, Boss5 boss) {
      super(pos, sign, boss);
    }

    void charge_and_return(int t, Registry reg) {
      
      
      if (400 <= t && t < 420) {
        move(Vector(0, -2));
      } else if (435 <= t && t < 470) {
        move(Vector(0, 10));
      } else if (t==519) {
        Player pl = Player.search(reg);
        if (pl) {
          Vector v = pl.pos - pos;
          fire_drad_ = (reg.root.rng.brand() ? 1: -1)*0.01;
          fire_rad_ = atan2(v.y, v.x) - fire_drad_*5;
        }
      } else if (520 <= t && t < 600 && t % 4 == 0) {
        double sp=(t-500)*0.1+3;
        fire_rad_ += fire_drad_;

        if (t==560)
          fire_drad_ *= -1;

        for (int i=-3; i<=3; i++) {
          Vector v = Vector.from_polar_coord(sp,
                                             fire_rad_+0.4*i);
          reg.add(new StripBullet(pos, v));
        }
      } else if (850 <= t && t < 881) {
        move(Vector(0, -10));
      }
      
    }

    mixin DefineDefFire!(40, 80, 1000, 1380, false);
    
    void run(Registry reg) {
      int t = (count - WAIT) % LOOP;
      if (200 <= t  && t <= 220 && t % 2 == 0) {
        StripBullet.fireToPlayer(reg, 16, pos, 5);
      }
      if (start_sign_ == -sign_) 
        charge_and_return(t, reg);

      // 70, 50, 120
      if ((1000 <= t && t < 1070) ||
          (1190 <= t && t < 1310)) {
        move(Vector(4*sign_, 0));
      }
      if ((1070 <= t && t < 1190) ||
          (1310 <= t && t < 1380)) {
        move(Vector(-4*sign_, 0));
      }
      deffire(reg, t);
    }
  }            

  static class RingA_F: BaseEnemy{
  private:
    int sign_;
  public:
    import base;
    this(Point p, int sign) {
      super(SubKind.NORMAL_ENEMY, p, 
            new Animation(&images.enemies.boss5ring.draw),
            0xffffff);
      scale = 3;
      sign_ = sign;
      ymirror = -sign;
    }
    
    mixin DefineRect!(pos, 80, 80, true);
    mixin DefineEnemy!(0);
    
    void run(Registry reg) {
      degree += sign_*6;
      move(Vector(0, 6));
      die_if_out_of_window();
    }
    
  }
  
  class RingC: Ring{
    this(Point pos, int sign, Boss5 boss) {
      super(pos, sign, boss);
      scale = 1.5;
    }
    mixin DefineRect!(pos, 40, 40, true);
    
    mixin DefineDefFire!(120, 120, 1000, 1380, false);
    void run(Registry reg) {
      if (count < WAIT)
        return;
      int t = (count-WAIT) % LOOP;
      switch(t) {
      
      case 30,50,70:
        reg.add(new FlyRing(pos, sign_, -3*sign_));
        break;
      case 35,45,55,65,75:
        reg.add(new FlyRing(pos, sign_, 0));
        break;
      case 40,60,80:
        reg.add(new FlyRing(pos, sign_, 3*sign_));
        break;
        
      case 330,350,370:
        if (start_sign_ == sign_)
          reg.add(new FlyRing(pos, sign_, -3*sign_));
        break;
      case 335,345,355,365,375:
        if (start_sign_ == sign_)
          reg.add(new FlyRing(pos, sign_, 0));
        break;
      case 340,360,380:
        if (start_sign_ == sign_)
          reg.add(new FlyRing(pos, sign_, 3*sign_));
        break;

      default:
        break;
      }

      deffire(reg, t);
    }
    
    static class FlyRing: BaseEnemy {
    private:
      int sign_;
      int dx_;
    public:
      this(Point pos, int sign, int dx) {
        super(SubKind.NORMAL_ENEMY, pos,
              new Animation(&images.enemies.boss5ring.draw),
              100);
        scale = 1.5;
        sign_ = sign;
        dx_ = dx;
      }
      mixin DefineRect!(pos, 40, 40, true);
      mixin DefineEnemy!(0);
      
      void run(Registry reg) {
        degree = sign_*6;
        move(Vector(dx_, -5));
        die_if_out_of_window();
      }
    }
  }

  static class RollRing: BaseEnemy {
  private:
    int sign_;
    int dt_;
  public:
    this(Point pos, int sign, int dt) {
      super(SubKind.NORMAL_ENEMY, pos,
            new Animation(&images.enemies.boss5ring.draw),
            100);
      scale = 1.5;
      sign_ = sign;
      dt_ = dt;
    }
    mixin DefineRect!(pos, 40, 40, true);
    mixin DefineEnemy!(50);
    
    void run(Registry reg) {
      degree += sign_*6;
      move(Vector(0, 4));

      if (count < 12)
        return;
      
      if ((count-dt_) % 30 == 0) {
        reg.add(new StripBullet(pos, Vector(sign_*5, 0)));
      }
      
      die_if_out_of_window();
    }
  }

  class RollRingCreator: BaseWorker {
  private:
    int sign_;
    int x_,dt_;
  public:
    this(int sign) {
      super(SubKind.BOSS5_RINGCREATOR);
      sign_ = sign;
      x_ = (sign_ == 1) ? 40 : 520;
    }

    void run(Registry reg) {
      int t = (count - WAIT) % LOOP;
      if (t==0)
        dt_ = reg.root.rng.irand(0, 30);
      
      if ((120 <= t && t <= 200) ||
          (400 <= t && t <= 800 && sign_ == start_sign_)) {
        if (t % 20 == 0)
          reg.add(new RollRing(Point(x_, -30), sign_, dt_));
        
      }
    }
  }
    
}

class DefBulletLeader: BaseMovable {
  static struct OneFrameMem{
    Point p;
    double degree;
  }
  
private:
  const static int MEMSIZE = 30;
  int seek_count_;
  bool seek_x_;
  double speed_;
  Vector v_;
  OneFrameMem[MEMSIZE] memory_;
  double degree_;
  int death_count_;
  
public:
  this(Point pos, Vector v, double speed, int seek_count=1,
       int death_count=240) {
    super(SubKind.DEF_BULLET_LEADER, pos);
    speed_ = speed;
    turn(v);
    seek_count_ = seek_count;
    death_count_ = death_count;
  }
  mixin DefineRect!(pos, 4, 4, true);

  void turn(Vector v) {
    seek_count_--;
    v_ = v;
    degree_ = atan2(v.y, v.x)*180/PI;
  }
  
  void run(Registry reg) {
    Player pl = Player.search(reg);
    move(v_);
    memory_[count % MEMSIZE].degree = degree_;
    memory_[count % MEMSIZE].p = pos;
    
    if (seek_count_ <= 0 || !pl) {
      return;
    }
    Vector rel = pl.pos - pos;
    double xsign = sign(rel.x);
    double ysign = sign(rel.y);

    if (v_.x >= 0.1 && abs(rel.x) < 12) {
      turn(Vector(0, speed_*ysign));
    } else if (v_.x <= -0.1 && abs(rel.x) < 12) {
      turn(Vector(0, speed_*ysign));
    } else if (v_.y >= 0.1 && abs(rel.y) < 12) {
      turn(Vector(speed_*xsign, 0));
    } else if (v_.y <= -0.1 && abs(rel.y) < 12) {
      turn(Vector(speed_*xsign, 0));
    }

    if (count >= death_count_)
      die();
  }

  OneFrameMem get_past(int dt) {
    return memory_[(count - dt) % MEMSIZE];
  }

  Vector v(){ return v_; }
}

class DefBulletBase: Animatable, Bullet {
private:
  DefBulletLeader leader_;
  int dt_;
public:
  this(DefBulletLeader leader, int dt, Animation anim) {
    super(SubKind.DEF_BULLET_CHILD, Point.out_of_window, 
          anim, Color.bullet);
    leader_ = leader;
    dt_ = dt;
  }
  
  mixin DefineRect!(pos, 4, 4, true);
  
  void run(Registry reg) {
    if (leader_.dead) {
      move(leader_.v);
    } else {
      DefBulletLeader.OneFrameMem mem = leader_.get_past(dt_);
      pos(mem.p);
      degree = mem.degree;
    }
    die_if_out_of_window();
  }
  
  void collide_with_wall(Registry, Wall) {
    die();
  }
  
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
    die();
  } 
}

class DefBulletHeader: DefBulletBase {
  this(DefBulletLeader leader, int dt) {
    super(leader, dt, new Animation(&images.enemies.tri.draw));
  }
}

class DefBulletChild: DefBulletBase {
  this(DefBulletLeader leader, int dt) {
    super(leader, dt, new Animation(&images.enemies.strip.draw));
  }
}

class DefBulletCreator: BaseWorker {
private:
  Point p_;
  Vector v_;
  int times_, interval_;
  double speed_;
  DefBulletLeader leader_;
public:
  this(Point p, Vector v, int times, int interval) {
    super(SubKind.DEF_BULLET_CREATOR);
    p_ = p; v_ = v;
    times_ = times;
    interval_ = interval;
    speed_ = v_.abs;
  }

  void run(Registry reg) {
    if (count == 0) {
      leader_ = new DefBulletLeader(p_, v_, speed_);
      reg.add(leader_);
    } else if (count == interval_) {
        reg.add(new DefBulletHeader(leader_, count));
    } else if (count < times_*interval_) {
      if (count % interval_ == 0)
        reg.add(new DefBulletChild(leader_, count));
    } else {
      die();
    }
  }
}
class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun7},
  {1, &fun2},
  {1, &fun8},
  {160, &fun26},
  {200, &fun10},
  {200, &fun9},
  {220, &fun11},
  {240, &fun12},
  {300, &fun15},
  {320, &fun14},
  {340, &fun13},
  {380, &fun5},
  {380, &fun6},
  {420, &fun24},
  {420, &fun17},
  {420, &fun16},
  {440, &fun18},
  {440, &fun19},
  {480, &fun32},
  {480, &fun33},
  {580, &fun21},
  {580, &fun20},
  {640, &fun36},
  {640, &fun38},
  {640, &fun37},
  {640, &fun40},
  {640, &fun34},
  {640, &fun35},
  {640, &fun39},
  {660, &fun31},
  {700, &fun23},
  {720, &fun29},
  {760, &fun22},
  {780, &fun43},
  {800, &fun30},
  {820, &fun27},
  {840, &fun47},
  {860, &fun25},
  {860, &fun28},
  {880, &fun46},
  {880, &fun41},
  {900, &fun42},
  {940, &fun45},
  {960, &fun44},
  {1060, &fun48},
  {1180, &fun51},
  {1180, &fun49},
  {1180, &fun50},
  {1200, &fun52},
  {1200, &fun54},
  {1220, &fun56},
  {1220, &fun53},
  {1240, &fun55},
  {1360, &fun58},
  {1360, &fun57},
  {1380, &fun4},
  {1400, &fun3},
  {1480, &fun59},
  {1480, &fun62},
  {1560, &fun69},
  {1560, &fun70},
  {1620, &fun61},
  {1620, &fun60},
  {1659, &fun65},
  {1659, &fun63},
  {1660, &fun66},
  {1660, &fun67},
  {1660, &fun79},
  {1660, &fun64},
  {1700, &fun99},
  {1700, &fun103},
  {1700, &fun72},
  {1700, &fun68},
  {1700, &fun75},
  {1720, &fun71},
  {1760, &fun194},
  {1760, &fun120},
  {1760, &fun80},
  {1760, &fun100},
  {1820, &fun78},
  {1820, &fun98},
  {1820, &fun104},
  {1820, &fun116},
  {1820, &fun97},
  {1840, &fun81},
  {1880, &fun101},
  {1880, &fun117},
  {1900, &fun105},
  {1940, &fun96},
  {1940, &fun73},
  {1940, &fun77},
  {1940, &fun195},
  {1940, &fun113},
  {1940, &fun74},
  {1960, &fun106},
  {2000, &fun102},
  {2000, &fun114},
  {2020, &fun95},
  {2040, &fun76},
  {2060, &fun121},
  {2080, &fun82},
  {2080, &fun125},
  {2080, &fun84},
  {2080, &fun162},
  {2100, &fun115},
  {2100, &fun83},
  {2120, &fun122},
  {2120, &fun87},
  {2120, &fun135},
  {2140, &fun91},
  {2160, &fun124},
  {2180, &fun108},
  {2180, &fun85},
  {2180, &fun88},
  {2200, &fun92},
  {2200, &fun199},
  {2200, &fun107},
  {2200, &fun134},
  {2220, &fun90},
  {2220, &fun112},
  {2240, &fun89},
  {2240, &fun86},
  {2240, &fun123},
  {2260, &fun93},
  {2260, &fun109},
  {2260, &fun119},
  {2260, &fun200},
  {2280, &fun130},
  {2300, &fun94},
  {2300, &fun136},
  {2320, &fun118},
  {2340, &fun131},
  {2360, &fun196},
  {2360, &fun110},
  {2360, &fun128},
  {2380, &fun126},
  {2380, &fun129},
  {2380, &fun137},
  {2400, &fun132},
  {2400, &fun127},
  {2400, &fun111},
  {2420, &fun139},
  {2420, &fun133},
  {2420, &fun144},
  {2440, &fun148},
  {2440, &fun157},
  {2440, &fun140},
  {2460, &fun143},
  {2480, &fun145},
  {2480, &fun167},
  {2480, &fun141},
  {2500, &fun147},
  {2500, &fun158},
  {2500, &fun160},
  {2520, &fun149},
  {2540, &fun142},
  {2540, &fun159},
  {2540, &fun138},
  {2540, &fun146},
  {2560, &fun154},
  {2560, &fun161},
  {2560, &fun182},
  {2580, &fun156},
  {2600, &fun198},
  {2620, &fun150},
  {2620, &fun183},
  {2640, &fun155},
  {2640, &fun152},
  {2640, &fun151},
  {2640, &fun168},
  {2660, &fun186},
  {2660, &fun153},
  {2680, &fun165},
  {2700, &fun163},
  {2700, &fun175},
  {2700, &fun172},
  {2700, &fun187},
  {2700, &fun169},
  {2720, &fun166},
  {2720, &fun180},
  {2740, &fun164},
  {2760, &fun173},
  {2760, &fun170},
  {2760, &fun184},
  {2760, &fun177},
  {2780, &fun178},
  {2780, &fun181},
  {2800, &fun179},
  {2800, &fun188},
  {2820, &fun174},
  {2820, &fun176},
  {2820, &fun185},
  {2820, &fun171},
  {3060, &fun192},
  {3060, &fun193},
  {3280, &fun190},
  {3300, &fun189},
  {3440, &fun197},
  {3700, &fun191},
  {3740, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun7(Registry reg){
  reg.add(new NormalWall(Rect(0, -220, 220, 220)));
}
static void fun2(Registry reg){
  set_scroll(reg, 1);
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(340, -220, 220, 220)));
}
static void fun26(Registry reg){
  f08(260.000000, -20.000000, reg);
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(520, -180, 40, 160)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, -180, 40, 160)));
}
static void fun11(Registry reg){
  f03(2,0,260.000000, -20.000000, reg);
}
static void fun12(Registry reg){
  f03(2,0,300.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  f03(-2,500,300.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  f03(-2,500,280.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  f03(-2,500,260.000000, -20.000000, reg);
}
static void fun5(Registry reg){
  f01(600.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  f02(-40.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(100, -100, 80, 80)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 60, 80)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(180, -40, 60, 20)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, -180, 20, 160)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(540, -320, 20, 300)));
}
static void fun32(Registry reg){
  f10(330.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  f05(190.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(340, -340, 40, 320)));
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(140, -340, 40, 320)));
}
static void fun36(Registry reg){
  f11(20,-60.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  f11(40,-100.000000, -20.000000, reg);
}
static void fun37(Registry reg){
  f11(30,-80.000000, -20.000000, reg);
}
static void fun40(Registry reg){
  f11(60,-140.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  f11(0,-20.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  f11(10,-40.000000, -20.000000, reg);
}
static void fun39(Registry reg){
  f11(50,-120.000000, -20.000000, reg);
}
static void fun31(Registry reg){
  f10(130.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f05(190.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  f10(530.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f04(680.000000, -20.000000, reg);
}
static void fun43(Registry reg){
  f12(300.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  f09(0,80.000000, -20.000000, reg);
}
static void fun27(Registry reg){
  f05(390.000000, -20.000000, reg);
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(120, -40, 20, 20)));
}
static void fun25(Registry reg){
  f07(-20.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  f05(390.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f10(130.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  f03(2,0,420.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  f03(2,0,460.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f03(-2,500,460.000000, -20.000000, reg);
}
static void fun44(Registry reg){
  f03(-2,500,400.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f03(-2,500,440.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f13(240.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  f13(80.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun52(Registry reg){
  f03(-2,500,20.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f03(-2,500,80.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f03(-2,500,100.000000, -20.000000, reg);
}
static void fun53(Registry reg){
  f03(-2,500,40.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f03(-2,500,60.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  reg.add(new NormalWall(Rect(380, -140, 180, 120)));
}
static void fun57(Registry reg){
  reg.add(new NormalWall(Rect(0, -140, 180, 120)));
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun59(Registry reg){
  reg.add(new NormalWall(Rect(140, -160, 40, 140)));
}
static void fun62(Registry reg){
  reg.add(new NormalWall(Rect(380, -160, 40, 140)));
}
static void fun69(Registry reg){
  f14(150,180.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f14(160,320.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  reg.add(new NormalWall(Rect(380, -60, 180, 40)));
}
static void fun60(Registry reg){
  reg.add(new NormalWall(Rect(0, -60, 180, 40)));
}
static void fun65(Registry reg){
  reg.add(new BGLine(Point(380, -21), Point(500, -81)));
}
static void fun63(Registry reg){
  reg.add(new BGLine(Point(80, -61), Point(180, -21)));
}
static void fun66(Registry reg){
  reg.add(new InvisibleWall(Rect(480, -60, 80, 40)));
}
static void fun67(Registry reg){
  reg.add(new InvisibleWall(Rect(440, -40, 40, 20)));
}
static void fun79(Registry reg){
  reg.add(new NormalWall(Rect(0, -60, 80, 40)));
}
static void fun64(Registry reg){
  reg.add(new InvisibleWall(Rect(80, -40, 40, 20)));
}
static void fun99(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun103(Registry reg){
  f21(20,20.000000, -20.000000, reg);
}
static void fun72(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun68(Registry reg){
  reg.add(new InvisibleWall(Rect(520, -40, 40, 20)));
}
static void fun75(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 20, 20)));
}
static void fun71(Registry reg){
  reg.add(new NormalWall(Rect(0, -420, 20, 400)));
}
static void fun194(Registry reg){
  f08(220.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  nilwall(400,500.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  reg.add(new NormalWall(Rect(20, -100, 60, 80)));
}
static void fun100(Registry reg){
  f16(95,260.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f15(125,260.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f15(130,320.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f23(450,80.000000, -20.000000, reg);
}
static void fun116(Registry reg){
  f22(290,500.000000, -20.000000, reg);
}
static void fun97(Registry reg){
  f15(120,200.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun101(Registry reg){
  f19(125,380.000000, -20.000000, reg);
}
static void fun117(Registry reg){
  f22(200,500.000000, -20.000000, reg);
}
static void fun105(Registry reg){
  f27(45,20.000000, -20.000000, reg);
}
static void fun96(Registry reg){
  f26(200,320.000000, -20.000000, reg);
}
static void fun73(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun77(Registry reg){
  f14(60,200.000000, -20.000000, reg);
}
static void fun195(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun113(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun74(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun106(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun102(Registry reg){
  f20(125,380.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun95(Registry reg){
  f21(45,20.000000, -20.000000, reg);
}
static void fun76(Registry reg){
  f12(480.000000, -20.000000, reg);
}
static void fun121(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun82(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun125(Registry reg){
  f23(320,140.000000, -20.000000, reg);
}
static void fun84(Registry reg){
  reg.add(new NormalWall(Rect(280, -80, 60, 60)));
}
static void fun162(Registry reg){
  f25(220.000000, -20.000000, reg);
}
static void fun115(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun83(Registry reg){
  reg.add(new NormalWall(Rect(80, -40, 20, 20)));
}
static void fun122(Registry reg){
  f21(260,80.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun135(Registry reg){
  reg.add(new NormalWall(Rect(0, -760, 20, 740)));
}
static void fun91(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun124(Registry reg){
  f17(260,440.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun85(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun88(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun92(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun199(Registry reg){
  reg.add(new NormalWall(Rect(320, -80, 60, 60)));
}
static void fun107(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun134(Registry reg){
  f28(535,380.000000, -20.000000, reg);
}
static void fun90(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun112(Registry reg){
  nilwall(120,140.000000, -20.000000, reg);
}
static void fun89(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun86(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun123(Registry reg){
  f20(60,260.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun109(Registry reg){
  f20(300,200.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  nilwall(220,380.000000, -20.000000, reg);
}
static void fun200(Registry reg){
  f20(60,320.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  nilwall(220,440.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun136(Registry reg){
  nilwall(400,80.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  nilwall(220,380.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  nilwall(180,440.000000, -20.000000, reg);
}
static void fun196(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun110(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun128(Registry reg){
  f15(450,260.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun129(Registry reg){
  f15(400,320.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun132(Registry reg){
  nilwall(220,440.000000, -20.000000, reg);
}
static void fun127(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun111(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun139(Registry reg){
  nilwall(200,260.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun144(Registry reg){
  f14(80,80.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f21(80,20.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  nilwall(80,200.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  nilwall(200,320.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f14(80,140.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  nilwall(260,80.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun141(Registry reg){
  f19(125,380.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun158(Registry reg){
  nilwall(125,200.000000, -20.000000, reg);
}
static void fun160(Registry reg){
  nilwall(120,320.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun142(Registry reg){
  f23(450,440.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f14(180,260.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun146(Registry reg){
  nilwall(200,80.000000, -20.000000, reg);
}
static void fun154(Registry reg){
  nilwall(200,200.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f15(400,320.000000, -20.000000, reg);
}
static void fun182(Registry reg){
  f23(540,20.000000, -20.000000, reg);
}
static void fun156(Registry reg){
  nilwall(200,500.000000, -20.000000, reg);
}
static void fun198(Registry reg){
  f27(120,380.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun183(Registry reg){
  nilwall(60,300.000000, -20.000000, reg);
}
static void fun155(Registry reg){
  f24(160.000000, -20.000000, reg);
}
static void fun152(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun151(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun168(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun186(Registry reg){
  f18(190,20.000000, -20.000000, reg);
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun165(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  nilwall(120,260.000000, -20.000000, reg);
}
static void fun175(Registry reg){
  nilwall(160,380.000000, -20.000000, reg);
}
static void fun172(Registry reg){
  f22(60,440.000000, -20.000000, reg);
}
static void fun187(Registry reg){
  nilwall(260,80.000000, -20.000000, reg);
}
static void fun169(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun166(Registry reg){
  nilwall(60,320.000000, -20.000000, reg);
}
static void fun180(Registry reg){
  f15(125,140.000000, -20.000000, reg);
}
static void fun164(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun173(Registry reg){
  f22(80,440.000000, -20.000000, reg);
}
static void fun170(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun184(Registry reg){
  f14(60,260.000000, -20.000000, reg);
}
static void fun177(Registry reg){
  nilwall(330,380.000000, -20.000000, reg);
}
static void fun178(Registry reg){
  f15(320,20.000000, -20.000000, reg);
}
static void fun181(Registry reg){
  f15(325,140.000000, -20.000000, reg);
}
static void fun179(Registry reg){
  f15(300,80.000000, -20.000000, reg);
}
static void fun188(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun174(Registry reg){
  f22(40,440.000000, -20.000000, reg);
}
static void fun176(Registry reg){
  nilwall(240,380.000000, -20.000000, reg);
}
static void fun185(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun171(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun192(Registry reg){
  reg.add(new NormalWall(Rect(0, -760, 20, 740)));
}
static void fun193(Registry reg){
  reg.add(new NormalWall(Rect(540, -760, 20, 740)));
}
static void fun190(Registry reg){
  set_scroll(reg, 1);
}
static void fun189(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun197(Registry reg){
  f08(240.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  create_boss5(280.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun58},
  {1, &fun3},
  {1, &fun4},
  {1, &fun7},
  {1, &fun8},
  {1, &fun9},
  {1, &fun10},
  {1, &fun16},
  {1, &fun17},
  {1, &fun18},
  {1, &fun19},
  {1, &fun20},
  {1, &fun21},
  {1, &fun57},
  {1, &fun47},
  {1, &fun24},
  {60, &fun62},
  {60, &fun59},
  {140, &fun69},
  {140, &fun70},
  {200, &fun61},
  {200, &fun60},
  {239, &fun65},
  {239, &fun63},
  {240, &fun67},
  {240, &fun66},
  {240, &fun64},
  {240, &fun79},
  {280, &fun68},
  {280, &fun75},
  {280, &fun72},
  {280, &fun103},
  {280, &fun99},
  {300, &fun71},
  {340, &fun100},
  {340, &fun194},
  {340, &fun120},
  {340, &fun80},
  {400, &fun97},
  {400, &fun116},
  {400, &fun78},
  {400, &fun104},
  {400, &fun98},
  {420, &fun81},
  {460, &fun117},
  {460, &fun101},
  {480, &fun105},
  {520, &fun73},
  {520, &fun77},
  {520, &fun74},
  {520, &fun96},
  {520, &fun113},
  {520, &fun195},
  {540, &fun106},
  {580, &fun102},
  {580, &fun114},
  {600, &fun95},
  {620, &fun76},
  {640, &fun121},
  {660, &fun162},
  {660, &fun125},
  {660, &fun82},
  {660, &fun84},
  {680, &fun83},
  {680, &fun115},
  {700, &fun122},
  {700, &fun87},
  {700, &fun135},
  {720, &fun91},
  {740, &fun124},
  {760, &fun108},
  {760, &fun88},
  {760, &fun85},
  {780, &fun134},
  {780, &fun199},
  {780, &fun92},
  {780, &fun107},
  {800, &fun90},
  {800, &fun112},
  {820, &fun86},
  {820, &fun89},
  {820, &fun123},
  {840, &fun119},
  {840, &fun109},
  {840, &fun93},
  {840, &fun200},
  {860, &fun130},
  {880, &fun94},
  {880, &fun136},
  {900, &fun118},
  {920, &fun131},
  {940, &fun196},
  {940, &fun110},
  {940, &fun128},
  {960, &fun126},
  {960, &fun137},
  {960, &fun129},
  {980, &fun127},
  {980, &fun132},
  {980, &fun111},
  {1000, &fun133},
  {1000, &fun144},
  {1000, &fun139},
  {1020, &fun148},
  {1020, &fun157},
  {1020, &fun140},
  {1040, &fun143},
  {1060, &fun141},
  {1060, &fun167},
  {1060, &fun145},
  {1080, &fun158},
  {1080, &fun147},
  {1080, &fun160},
  {1100, &fun149},
  {1120, &fun142},
  {1120, &fun159},
  {1120, &fun146},
  {1120, &fun138},
  {1140, &fun154},
  {1140, &fun161},
  {1140, &fun182},
  {1160, &fun156},
  {1180, &fun198},
  {1200, &fun150},
  {1200, &fun183},
  {1220, &fun168},
  {1220, &fun155},
  {1220, &fun151},
  {1220, &fun152},
  {1240, &fun186},
  {1240, &fun153},
  {1260, &fun165},
  {1280, &fun163},
  {1280, &fun175},
  {1280, &fun187},
  {1280, &fun169},
  {1280, &fun172},
  {1300, &fun180},
  {1300, &fun166},
  {1320, &fun164},
  {1340, &fun173},
  {1340, &fun170},
  {1340, &fun184},
  {1340, &fun177},
  {1360, &fun178},
  {1360, &fun181},
  {1380, &fun179},
  {1380, &fun188},
  {1400, &fun185},
  {1400, &fun176},
  {1400, &fun174},
  {1400, &fun171},
  {1640, &fun192},
  {1640, &fun193},
  {1860, &fun190},
  {1880, &fun189},
  {2020, &fun197},
  {2280, &fun191},
  {2320, &fun1},
];
static void fun58(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 180, 120)));
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun7(Registry reg){
  reg.add(new NormalWall(Rect(0, 1200, 220, 220)));
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(340, 1200, 220, 220)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, 1040, 40, 160)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(520, 1040, 40, 160)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(180, 960, 60, 20)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(340, 900, 60, 80)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, 800, 20, 160)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(540, 660, 20, 300)));
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(140, 500, 40, 320)));
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(340, 500, 40, 320)));
}
static void fun57(Registry reg){
  reg.add(new NormalWall(Rect(0, -80, 180, 120)));
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(120, 540, 20, 20)));
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(100, 900, 80, 80)));
}
static void fun62(Registry reg){
  reg.add(new NormalWall(Rect(380, -160, 40, 140)));
}
static void fun59(Registry reg){
  reg.add(new NormalWall(Rect(140, -160, 40, 140)));
}
static void fun69(Registry reg){
  f14(150,180.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f14(160,320.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  reg.add(new NormalWall(Rect(380, -60, 180, 40)));
}
static void fun60(Registry reg){
  reg.add(new NormalWall(Rect(0, -60, 180, 40)));
}
static void fun65(Registry reg){
  reg.add(new BGLine(Point(380, -21), Point(500, -81)));
}
static void fun63(Registry reg){
  reg.add(new BGLine(Point(80, -61), Point(180, -21)));
}
static void fun67(Registry reg){
  reg.add(new InvisibleWall(Rect(440, -40, 40, 20)));
}
static void fun66(Registry reg){
  reg.add(new InvisibleWall(Rect(480, -60, 80, 40)));
}
static void fun64(Registry reg){
  reg.add(new InvisibleWall(Rect(80, -40, 40, 20)));
}
static void fun79(Registry reg){
  reg.add(new NormalWall(Rect(0, -60, 80, 40)));
}
static void fun68(Registry reg){
  reg.add(new InvisibleWall(Rect(520, -40, 40, 20)));
}
static void fun75(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 20, 20)));
}
static void fun72(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun103(Registry reg){
  f21(20,20.000000, -20.000000, reg);
}
static void fun99(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun71(Registry reg){
  reg.add(new NormalWall(Rect(0, -420, 20, 400)));
}
static void fun100(Registry reg){
  f16(95,260.000000, -20.000000, reg);
}
static void fun194(Registry reg){
  f08(220.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  nilwall(400,500.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  reg.add(new NormalWall(Rect(20, -100, 60, 80)));
}
static void fun97(Registry reg){
  f15(120,200.000000, -20.000000, reg);
}
static void fun116(Registry reg){
  f22(290,500.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f15(125,260.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f23(450,80.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f15(130,320.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun117(Registry reg){
  f22(200,500.000000, -20.000000, reg);
}
static void fun101(Registry reg){
  f19(125,380.000000, -20.000000, reg);
}
static void fun105(Registry reg){
  f27(45,20.000000, -20.000000, reg);
}
static void fun73(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun77(Registry reg){
  f14(60,200.000000, -20.000000, reg);
}
static void fun74(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun96(Registry reg){
  f26(200,320.000000, -20.000000, reg);
}
static void fun113(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun195(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun106(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun102(Registry reg){
  f20(125,380.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun95(Registry reg){
  f21(45,20.000000, -20.000000, reg);
}
static void fun76(Registry reg){
  f12(480.000000, -20.000000, reg);
}
static void fun121(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun162(Registry reg){
  f25(220.000000, -20.000000, reg);
}
static void fun125(Registry reg){
  f23(320,140.000000, -20.000000, reg);
}
static void fun82(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun84(Registry reg){
  reg.add(new NormalWall(Rect(280, -80, 60, 60)));
}
static void fun83(Registry reg){
  reg.add(new NormalWall(Rect(80, -40, 20, 20)));
}
static void fun115(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun122(Registry reg){
  f21(260,80.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun135(Registry reg){
  reg.add(new NormalWall(Rect(0, -760, 20, 740)));
}
static void fun91(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun124(Registry reg){
  f17(260,440.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun88(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun85(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun134(Registry reg){
  f28(535,380.000000, -20.000000, reg);
}
static void fun199(Registry reg){
  reg.add(new NormalWall(Rect(320, -80, 60, 60)));
}
static void fun92(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun107(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun90(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun112(Registry reg){
  nilwall(120,140.000000, -20.000000, reg);
}
static void fun86(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun89(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun123(Registry reg){
  f20(60,260.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  nilwall(220,380.000000, -20.000000, reg);
}
static void fun109(Registry reg){
  f20(300,200.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun200(Registry reg){
  f20(60,320.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  nilwall(220,440.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun136(Registry reg){
  nilwall(400,80.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  nilwall(220,380.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  nilwall(180,440.000000, -20.000000, reg);
}
static void fun196(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun110(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun128(Registry reg){
  f15(450,260.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun137(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun129(Registry reg){
  f15(400,320.000000, -20.000000, reg);
}
static void fun127(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun132(Registry reg){
  nilwall(220,440.000000, -20.000000, reg);
}
static void fun111(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun133(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun144(Registry reg){
  f14(80,80.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  nilwall(200,260.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f21(80,20.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  nilwall(80,200.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  nilwall(200,320.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f14(80,140.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f19(125,380.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun145(Registry reg){
  nilwall(260,80.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  nilwall(125,200.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  reg.add(new NormalWall(Rect(20, -80, 60, 60)));
}
static void fun160(Registry reg){
  nilwall(120,320.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun142(Registry reg){
  f23(450,440.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f14(180,260.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  nilwall(200,80.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  reg.add(new NormalWall(Rect(380, -80, 60, 60)));
}
static void fun154(Registry reg){
  nilwall(200,200.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f15(400,320.000000, -20.000000, reg);
}
static void fun182(Registry reg){
  f23(540,20.000000, -20.000000, reg);
}
static void fun156(Registry reg){
  nilwall(200,500.000000, -20.000000, reg);
}
static void fun198(Registry reg){
  f27(120,380.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(200, -80, 60, 60)));
}
static void fun183(Registry reg){
  nilwall(60,300.000000, -20.000000, reg);
}
static void fun168(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun155(Registry reg){
  f24(160.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  reg.add(new NormalWall(Rect(440, -80, 60, 60)));
}
static void fun152(Registry reg){
  reg.add(new NormalWall(Rect(80, -80, 60, 60)));
}
static void fun186(Registry reg){
  f18(190,20.000000, -20.000000, reg);
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(140, -80, 60, 60)));
}
static void fun165(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  nilwall(120,260.000000, -20.000000, reg);
}
static void fun175(Registry reg){
  nilwall(160,380.000000, -20.000000, reg);
}
static void fun187(Registry reg){
  nilwall(260,80.000000, -20.000000, reg);
}
static void fun169(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun172(Registry reg){
  f22(60,440.000000, -20.000000, reg);
}
static void fun180(Registry reg){
  f15(125,140.000000, -20.000000, reg);
}
static void fun166(Registry reg){
  nilwall(60,320.000000, -20.000000, reg);
}
static void fun164(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun173(Registry reg){
  f22(80,440.000000, -20.000000, reg);
}
static void fun170(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun184(Registry reg){
  f14(60,260.000000, -20.000000, reg);
}
static void fun177(Registry reg){
  nilwall(330,380.000000, -20.000000, reg);
}
static void fun178(Registry reg){
  f15(320,20.000000, -20.000000, reg);
}
static void fun181(Registry reg){
  f15(325,140.000000, -20.000000, reg);
}
static void fun179(Registry reg){
  f15(300,80.000000, -20.000000, reg);
}
static void fun188(Registry reg){
  nilwall(120,200.000000, -20.000000, reg);
}
static void fun185(Registry reg){
  reg.add(new NormalWall(Rect(260, -80, 60, 60)));
}
static void fun176(Registry reg){
  nilwall(240,380.000000, -20.000000, reg);
}
static void fun174(Registry reg){
  f22(40,440.000000, -20.000000, reg);
}
static void fun171(Registry reg){
  reg.add(new NormalWall(Rect(500, -80, 60, 60)));
}
static void fun192(Registry reg){
  reg.add(new NormalWall(Rect(0, -760, 20, 740)));
}
static void fun193(Registry reg){
  reg.add(new NormalWall(Rect(540, -760, 20, 740)));
}
static void fun190(Registry reg){
  set_scroll(reg, 1);
}
static void fun189(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun197(Registry reg){
  f08(240.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  create_boss5(280.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events2{
const static Event[] events = [
  {1, &fun199},
  {1, &fun7},
  {1, &fun8},
  {1, &fun9},
  {1, &fun10},
  {1, &fun16},
  {1, &fun17},
  {1, &fun18},
  {1, &fun19},
  {1, &fun20},
  {1, &fun21},
  {1, &fun24},
  {1, &fun47},
  {1, &fun57},
  {1, &fun58},
  {1, &fun59},
  {1, &fun60},
  {1, &fun61},
  {1, &fun62},
  {1, &fun63},
  {1, &fun64},
  {1, &fun65},
  {1, &fun66},
  {1, &fun67},
  {1, &fun68},
  {1, &fun71},
  {1, &fun72},
  {1, &fun73},
  {1, &fun74},
  {1, &fun75},
  {1, &fun79},
  {1, &fun80},
  {1, &fun81},
  {1, &fun82},
  {1, &fun83},
  {1, &fun84},
  {1, &fun85},
  {1, &fun86},
  {1, &fun87},
  {1, &fun88},
  {1, &fun89},
  {1, &fun90},
  {1, &fun91},
  {1, &fun92},
  {1, &fun93},
  {1, &fun94},
  {1, &fun99},
  {1, &fun106},
  {1, &fun107},
  {1, &fun108},
  {1, &fun110},
  {1, &fun111},
  {1, &fun113},
  {1, &fun114},
  {1, &fun115},
  {1, &fun121},
  {1, &fun126},
  {1, &fun127},
  {1, &fun133},
  {1, &fun135},
  {1, &fun137},
  {1, &fun195},
  {1, &fun147},
  {1, &fun149},
  {1, &fun150},
  {1, &fun151},
  {1, &fun152},
  {1, &fun153},
  {1, &fun167},
  {1, &fun168},
  {1, &fun169},
  {1, &fun170},
  {1, &fun171},
  {1, &fun185},
  {1, &fun189},
  {1, &fun190},
  {1, &fun196},
  {1, &fun192},
  {1, &fun193},
  {1, &fun138},
  {120, &fun197},
  {380, &fun191},
  {420, &fun1},
];
static void fun199(Registry reg){
  reg.add(new NormalWall(Rect(320, 1040, 60, 60)));
}
static void fun7(Registry reg){
  reg.add(new NormalWall(Rect(0, 3100, 220, 220)));
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(340, 3100, 220, 220)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(0, 2940, 40, 160)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(520, 2940, 40, 160)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(180, 2860, 60, 20)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(340, 2800, 60, 80)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(0, 2700, 20, 160)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(540, 2560, 20, 300)));
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(140, 2400, 40, 320)));
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(340, 2400, 40, 320)));
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(100, 2800, 80, 80)));
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(120, 2440, 20, 20)));
}
static void fun57(Registry reg){
  reg.add(new NormalWall(Rect(0, 1820, 180, 120)));
}
static void fun58(Registry reg){
  reg.add(new NormalWall(Rect(380, 1820, 180, 120)));
}
static void fun59(Registry reg){
  reg.add(new NormalWall(Rect(140, 1680, 40, 140)));
}
static void fun60(Registry reg){
  reg.add(new NormalWall(Rect(0, 1640, 180, 40)));
}
static void fun61(Registry reg){
  reg.add(new NormalWall(Rect(380, 1640, 180, 40)));
}
static void fun62(Registry reg){
  reg.add(new NormalWall(Rect(380, 1680, 40, 140)));
}
static void fun63(Registry reg){
  reg.add(new BGLine(Point(80, 1600), Point(180, 1640)));
}
static void fun64(Registry reg){
  reg.add(new InvisibleWall(Rect(80, 1620, 40, 20)));
}
static void fun65(Registry reg){
  reg.add(new BGLine(Point(380, 1640), Point(500, 1580)));
}
static void fun66(Registry reg){
  reg.add(new InvisibleWall(Rect(480, 1600, 80, 40)));
}
static void fun67(Registry reg){
  reg.add(new InvisibleWall(Rect(440, 1620, 40, 20)));
}
static void fun68(Registry reg){
  reg.add(new InvisibleWall(Rect(520, 1580, 40, 20)));
}
static void fun71(Registry reg){
  reg.add(new NormalWall(Rect(0, 1180, 20, 400)));
}
static void fun72(Registry reg){
  reg.add(new NormalWall(Rect(500, 1540, 60, 60)));
}
static void fun73(Registry reg){
  reg.add(new NormalWall(Rect(140, 1300, 60, 60)));
}
static void fun74(Registry reg){
  reg.add(new NormalWall(Rect(380, 1300, 60, 60)));
}
static void fun75(Registry reg){
  reg.add(new NormalWall(Rect(0, 1580, 20, 20)));
}
static void fun79(Registry reg){
  reg.add(new NormalWall(Rect(0, 1600, 80, 40)));
}
static void fun80(Registry reg){
  reg.add(new NormalWall(Rect(20, 1460, 60, 80)));
}
static void fun81(Registry reg){
  reg.add(new NormalWall(Rect(20, 1400, 60, 60)));
}
static void fun82(Registry reg){
  reg.add(new NormalWall(Rect(20, 1160, 60, 60)));
}
static void fun83(Registry reg){
  reg.add(new NormalWall(Rect(80, 1180, 20, 20)));
}
static void fun84(Registry reg){
  reg.add(new NormalWall(Rect(280, 1160, 60, 60)));
}
static void fun85(Registry reg){
  reg.add(new NormalWall(Rect(80, 1060, 60, 60)));
}
static void fun86(Registry reg){
  reg.add(new NormalWall(Rect(80, 1000, 60, 60)));
}
static void fun87(Registry reg){
  reg.add(new NormalWall(Rect(500, 1120, 60, 60)));
}
static void fun88(Registry reg){
  reg.add(new NormalWall(Rect(500, 1060, 60, 60)));
}
static void fun89(Registry reg){
  reg.add(new NormalWall(Rect(500, 1000, 60, 60)));
}
static void fun90(Registry reg){
  reg.add(new NormalWall(Rect(440, 1020, 60, 60)));
}
static void fun91(Registry reg){
  reg.add(new NormalWall(Rect(20, 1100, 60, 60)));
}
static void fun92(Registry reg){
  reg.add(new NormalWall(Rect(20, 1040, 60, 60)));
}
static void fun93(Registry reg){
  reg.add(new NormalWall(Rect(20, 980, 60, 60)));
}
static void fun94(Registry reg){
  reg.add(new NormalWall(Rect(500, 940, 60, 60)));
}
static void fun99(Registry reg){
  reg.add(new NormalWall(Rect(260, 1540, 60, 60)));
}
static void fun106(Registry reg){
  reg.add(new NormalWall(Rect(20, 1280, 60, 60)));
}
static void fun107(Registry reg){
  reg.add(new NormalWall(Rect(200, 1040, 60, 60)));
}
static void fun108(Registry reg){
  reg.add(new NormalWall(Rect(260, 1060, 60, 60)));
}
static void fun110(Registry reg){
  reg.add(new NormalWall(Rect(80, 880, 60, 60)));
}
static void fun111(Registry reg){
  reg.add(new NormalWall(Rect(140, 840, 60, 60)));
}
static void fun113(Registry reg){
  reg.add(new NormalWall(Rect(500, 1300, 60, 60)));
}
static void fun114(Registry reg){
  reg.add(new NormalWall(Rect(500, 1240, 60, 60)));
}
static void fun115(Registry reg){
  reg.add(new NormalWall(Rect(440, 1140, 60, 60)));
}
static void fun121(Registry reg){
  reg.add(new NormalWall(Rect(500, 1180, 60, 60)));
}
static void fun126(Registry reg){
  reg.add(new NormalWall(Rect(200, 860, 60, 60)));
}
static void fun127(Registry reg){
  reg.add(new NormalWall(Rect(380, 840, 60, 60)));
}
static void fun133(Registry reg){
  reg.add(new NormalWall(Rect(500, 820, 60, 60)));
}
static void fun135(Registry reg){
  reg.add(new NormalWall(Rect(0, 440, 20, 740)));
}
static void fun137(Registry reg){
  reg.add(new NormalWall(Rect(20, 860, 60, 60)));
}
static void fun195(Registry reg){
  reg.add(new NormalWall(Rect(260, 1300, 60, 60)));
}
static void fun147(Registry reg){
  reg.add(new NormalWall(Rect(20, 740, 60, 60)));
}
static void fun149(Registry reg){
  reg.add(new NormalWall(Rect(140, 720, 60, 60)));
}
static void fun150(Registry reg){
  reg.add(new NormalWall(Rect(200, 620, 60, 60)));
}
static void fun151(Registry reg){
  reg.add(new NormalWall(Rect(440, 600, 60, 60)));
}
static void fun152(Registry reg){
  reg.add(new NormalWall(Rect(80, 600, 60, 60)));
}
static void fun153(Registry reg){
  reg.add(new NormalWall(Rect(140, 580, 60, 60)));
}
static void fun167(Registry reg){
  reg.add(new NormalWall(Rect(500, 760, 60, 60)));
}
static void fun168(Registry reg){
  reg.add(new NormalWall(Rect(500, 600, 60, 60)));
}
static void fun169(Registry reg){
  reg.add(new NormalWall(Rect(500, 540, 60, 60)));
}
static void fun170(Registry reg){
  reg.add(new NormalWall(Rect(500, 480, 60, 60)));
}
static void fun171(Registry reg){
  reg.add(new NormalWall(Rect(500, 420, 60, 60)));
}
static void fun185(Registry reg){
  reg.add(new NormalWall(Rect(260, 420, 60, 60)));
}
static void fun189(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun190(Registry reg){
  set_scroll(reg, 1);
}
static void fun196(Registry reg){
  reg.add(new NormalWall(Rect(500, 880, 60, 60)));
}
static void fun192(Registry reg){
  reg.add(new NormalWall(Rect(0, -500, 20, 740)));
}
static void fun193(Registry reg){
  reg.add(new NormalWall(Rect(540, -500, 20, 740)));
}
static void fun138(Registry reg){
  reg.add(new NormalWall(Rect(380, 700, 60, 60)));
}
static void fun197(Registry reg){
  f08(240.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  create_boss5(280.000000, -20.000000, reg);
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
  Events2.events,
];
