import kinds;
import util;
import base;
import images = images.src.images;
import opengl;
import glutils;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;

class Artillery: BaseEnemy
{
public:
  this(Point pos, int deg, Registry reg, int fire_fast, int fire_rate, int fire_speed, double change_deg = 1, int left_deg = 90, int right_deg = 90, Movable parent = null, double omega = 0) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.artillery.draw),
          30, Color.enemy, deg, 1.0, parent);
    reg.add(new Cannon(pos, deg, this, fire_fast, fire_rate, fire_speed, change_deg, left_deg, right_deg, omega));
  }

  mixin DefineEnemy!(200);
  mixin DefineRect!(pos, 15, 15, true);

  void run(Registry reg) {
    scroll(reg);
    die_if_out_of_window();
    super.run(reg);
  }
  
  static class Cannon: BaseEnemy {
    double change_deg_;
    int left_deg_;
    int right_deg_;
    int cannon_fire_first_;
    int cannon_fire_rate_;
    int cannon_fire_speed_;
    int kick_back_ = 0;
    double omega_;
    this(Point pos, int deg, Artillery parent, int fire_first, int fire_rate, int fire_speed, double change_deg, int left_deg, int right_deg, double omega) {
      super(SubKind.NORMAL_ENEMY, Point(0, 0),
            new Animation(&images.enemies.cannon.draw),
            10, Color.enemy, deg, 2, parent);
      change_deg_ = change_deg;
      left_deg_ = deg - left_deg;
      right_deg_ = deg + right_deg;
      cannon_fire_first_ = fire_first;
      cannon_fire_rate_ = fire_rate;
      cannon_fire_speed_ = fire_speed;
      omega_ = omega;
    }

    mixin DefineEnemy!(100);
    mixin DefineRect!(pos, 25, 25, true);

    void run(Registry reg) {
      if (parent.dead())
        die();
      left_deg_ += omega_;
      right_deg_ += omega_;
      degree = homing(degree, change_deg_, reg);
      if (degree < left_deg_)
        degree = left_deg_;
      if (degree > right_deg_)
        degree = right_deg_;
      if (count - cannon_fire_first_ >= 0 && (count - cannon_fire_first_) % cannon_fire_rate_ == 0) {
        reg.add(new NormalBullet(pos, cannon_fire_speed_ * degree_to_vector()));
        kick_back_ = 10;
      }

      if (kick_back_ > 8)
        move_forward(-4);
      else if (kick_back_ > 0)
        move_forward(1);
      else if (kick_back_ == 0)
        pos = Point(0, 0);
      kick_back_--;
      die_if_out_of_window();
    }
  };
}

class PillarArtillery: Artillery {
  double omega_;
  this(int deg, Registry reg, int fire_fast, int fire_rate, int fire_speed, Movable pillar, double change_deg = 1, int left_deg = 90, int right_deg = 90, double omega = 0) {
    super(degree_to_vector(33, false, deg - 90), deg, reg, fire_fast, fire_rate, fire_speed, change_deg, left_deg, right_deg, pillar, omega);
    omega_ = omega;
  }
  void run(Registry reg) {
    if (parent.dead())
      die();
    degree += omega_;
    pos = degree_to_vector(33);
    super.run(reg);
  }
};

class PillarBud: Bud {
  int bud_fire_rate_;
  double bud_fire_speed_;
  bool using_strip_;
  double omega_;
  this(int deg, int fire_rate, double fire_speed, Movable pillar, double omega = 0) {
    super(degree_to_vector(36, false, deg - 90), deg, pillar);
    bud_fire_rate_ = fire_rate;
    bud_fire_speed_ = fire_speed;
    omega_ = omega;
  }
  void run(Registry reg){
    if (parent.dead())
      die();
    degree += omega_;
    pos = degree_to_vector(36);
    if (count % bud_fire_rate_ == 0) {
      reg.add(new StripBullet(pos + degree_to_vector(15), bud_fire_speed_ * degree_to_vector(), true));
    }
  }
};


class Pillar: BaseEnemy {
private:
  int ct_;
public:
  double omega_;
  this(Point pos, Registry reg, int ct = 0) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.square.draw),
          400, Color.enemy);
    scale = 2.0;
    degree = 0;
    reg.add(new EnemyWall(this));
    set_destroyed_hook(&destroyed);
    ct_ = ct;
  }

  mixin DefineEnemy!(1000);
  mixin DefineRect!(pos, 40, 40, true);

  void run(Registry reg){
    scroll(reg);
    if (count > ct_)
      die_if_out_of_window();
  }
  static void destroyed(Registry reg, Enemy enemy) {
    reg.add(new BulletCreator(enemy.pos));
  }

  static class BulletCreator: BaseMovable {
  public:
    this(Point pos) {
      super(SubKind.NORMAL_BULLET_CREATOR, pos);
    }

    void run(Registry reg) {
      scroll(reg);
      fireToPlayer(reg, 32, pos, 8);
      if (count == 10)
        die();
    }
  };
};



void f01(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(OptionCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f02(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ReflectiveCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f11(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 180, 50, 200, 4));
}

void f12(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 180, 10000, 10000, 4));
}

void f13(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 0, 10000, 10000, 4));
}

void f14(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 90, 10000, 10000, 4));
}

void f15(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x, y), 270, 10000, 10000, 4));
}

void f21(double x, double y, Registry reg)
{
  reg.add(new Artillery(Point(x, y), 0, reg, 100, 200, 6));
}

void f22(double x, double y, Registry reg)
{
  reg.add(new Artillery(Point(x, y), 0, reg, 50, 200, 6, 1, 90, 0));
}

void f23(double x, double y, Registry reg)
{
  reg.add(new Artillery(Point(x, y), 180, reg, 50, 200, 6, 1, 0, 90));
}

void f31(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Bud {
    this(Point p) {
      super(p, 180);
    }
    void run(Registry reg){
      scroll(reg);
      if (count % 20 == 0)
        reg.add(new StripBullet(pos + degree_to_vector(15), degree_to_vector(6), true));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f32(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Bud {
    this(Point p) {
      super(p, 90);
    }
    void run(Registry reg){
      scroll(reg);
      if (count % 60 == 0)
        reg.add(new StripBullet(pos + degree_to_vector(15), degree_to_vector(6), true));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f33(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Bud {
    this(Point p) {
      super(p, 270);
    }
    void run(Registry reg){
      scroll(reg);
      if (count % 20 == 0)
        reg.add(new StripBullet(pos + degree_to_vector(15), degree_to_vector(6), true));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f34(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Bud {
    this(Point p) {
      super(p, 90);
    }
    void run(Registry reg){
      scroll(reg);
      if (count % 20 == 0)
        reg.add(new StripBullet(pos + degree_to_vector(15), degree_to_vector(6), true));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f35(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x, y)) Bud {
    this(Point p) {
      super(p, 0);
    }
    void run(Registry reg){
      scroll(reg);
      if (count % 60 == 0)
        reg.add(new StripBullet(pos + degree_to_vector(15), degree_to_vector(6), true));
      die_if_out_of_window();
    }
  };
  reg.add(enemy);
}

void f41(double x, double y, Registry reg)
{
  Pillar pillar = new Pillar(Point(x, y-20), reg);

  reg.add(pillar);
  reg.add(new PillarBud(90, 20, 6, pillar));
}

void f42(double x, double y, Registry reg)
{
  Pillar pillar = new Pillar(Point(x, y-20), reg);

  reg.add(pillar);
  reg.add(new PillarBud(180, 20, 6, pillar));
}

void f43(double x, double y, Registry reg)
{
  Pillar pillar = new Pillar(Point(x, y-20), reg);

  reg.add(pillar);
  reg.add(new PillarBud(270, 20, 6, pillar));
}

void f44(double x, double y, Registry reg)
{
  Pillar pillar = new Pillar(Point(x, y-20), reg);

  reg.add(pillar);
  reg.add(new PillarArtillery(0, reg, 50, 50, 6, pillar));
  reg.add(new PillarArtillery(90, reg, 50, 50, 6, pillar));
  reg.add(new PillarArtillery(180, reg, 50, 50, 6, pillar));
  reg.add(new PillarArtillery(270, reg, 50, 50, 6, pillar));
}

void f45(double x, double y, Registry reg)
{
  EnemyCreator pillar_creator = new class(Point(x,y-180)) EnemyCreator {
public:
    this(Point pos) {
      super(pos, 7, 120);
    }
  
    void run(Registry reg) {
      if (count % interval == 0) {
        Pillars pillar = new Pillars(pos, reg);
        reg.add(pillar);
      }
      super.run(reg);
    }
    
    static class Pillars : Pillar {
      double speed_ = 2;
      double move_deg_ = 0;
      this(Point p, Registry reg) {
        super(p, reg, 600);
        degree = 90;
        omega_ = -1;
      }
      void run(Registry reg) {
        if (count >= 60  && count < 60 + 9)
          move_deg_ -= 10;
        if (count >= 210 + 69 && count < 210 + 9 + 69)
          move_deg_ += 10;
        if (count >= 205 + 288 && count < 205 + 9 + 288)
          move_deg_ += 10;
        if (count >= 115 + 502 && count < 115 + 9 + 502)
          move_deg_ -= 10;
        move_forward(speed_, false, move_deg_);
        degree += omega_;
        super.run(reg);
      }
    }
  };
  reg.add(pillar_creator);
}

void f46(double x, double y, Registry reg)
{
  EnemyCreator pillar_creator = new class(Point(x,y-335)) EnemyCreator {
public:
    this(Point pos) {
      super(pos, 7, 120);
    }
    void run(Registry reg) {
      if (count % interval == 0) {
        Pillars pillar = new Pillars(pos, reg);
        reg.add(pillar);
      }
      super.run(reg);
    }
    
    static class Pillars : Pillar {
      double speed_ = 2;
      double move_deg_ = 180;
      this(Point p, Registry reg) {
        super(p, reg, 400);
        degree = 90;
        omega_ = 1;
      }
      void run(Registry reg) {
        if (count >= 100  && count < 100 + 9)
          move_deg_ += 10;
        if (count >= 110 + 109 && count < 110 + 9 + 109)
          move_deg_ -= 10;
        if (count >= 120 + 228 && count < 120 + 9 + 228)
          move_deg_ -= 10;
        move_forward(speed_, false, move_deg_);
        degree += omega_;
        super.run(reg);
      }
    }
  };
  reg.add(pillar_creator);
}

void f47(double x, double y, Registry reg)
{
  EnemyCreator pillar_creator = new class(Point(x,y-220)) EnemyCreator {
public:
    int number_ = 0;
    this(Point pos) {
      super(pos, 9, 60);
    }
    void run(Registry reg) {
      if (count % interval == 0) {
        Pillars pillar = new Pillars(pos, reg, number_);
        reg.add(pillar);
        reg.add(new PillarBud(270, 20, 6, pillar));
        number_++;
      }
      super.run(reg);
    }
    
    static class Pillars : Pillar {
      int number_;
      double distant_ = 0;
      double dy_ = 0;
      this(Point p, Registry reg, int number) {
        super(p, reg, 200);
        degree = 90;
        number_ = number;
      }
      void run(Registry reg) {
        dy_ += 0.04;
        distant_ += dy_;
        if (count < 20)
          move(Vector(-1, 0));
        if (distant_ <= 360 - (number_ * 40)) {
            move(Vector(-1, dy_));
        } else {
          distant_ -= dy_;
          dy_ = 0;
        }
        super.run(reg);
      }
      void collide_with_shot(Registry reg, Shot shot) {
          return;
      }
    };
  };
  reg.add(pillar_creator);
}

void f48(double x, double y, Registry reg)
{
  Pillar pillar = new class(Point(x, y-20), reg) Pillar {
    this(Point p, Registry reg) {
      super(p, reg);
      degree = 0;
      omega_ = 1;
    }
    void run(Registry reg) {
      degree += omega_;
      super.run(reg);
    }
  };

  reg.add(pillar);
  reg.add(new PillarArtillery(0, reg, 100, 50, 6, pillar, 5, 90, 90, pillar.omega_));
  reg.add(new PillarArtillery(90, reg, 100, 50, 6, pillar, 5, 90, 90, pillar.omega_));
  reg.add(new PillarArtillery(180, reg, 100, 50, 6, pillar, 5, 90, 90, pillar.omega_));
  reg.add(new PillarArtillery(270, reg, 100, 50, 6, pillar, 5, 90, 90, pillar.omega_));
}

void f49(double x, double y, Registry reg)
{
  Pillar pillar = new class(Point(x, y-20), reg) Pillar {
    this(Point p, Registry reg) {
      super(p, reg);
      degree = 0;
      omega_ = -1;
    }
    void run(Registry reg) {
      degree += omega_;
      super.run(reg);
    }
  };

  reg.add(pillar);
  reg.add(new PillarBud(0, 40, 6, pillar, pillar.omega_));
  reg.add(new PillarBud(90, 40, 6, pillar, pillar.omega_));
  reg.add(new PillarBud(180, 40, 6, pillar, pillar.omega_));
  reg.add(new PillarBud(270, 40, 6, pillar, pillar.omega_));
}

void f51(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 10, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 90;
      }
      void run(Registry reg) {
        if (count >= 65  && count < 65 + 9)
          degree -= 10;
        if (count >= 20 + 74 && count < 20 + 9 + 74)
          degree -= 10;
        if (count >= 25 + 103 && count < 25 + 18 + 103)
          degree += 5;
        scroll(reg);
        move_forward(speed_);
        if (count > 146)
          die_if_out_of_window();
      }
    };
  };
  reg.add(viper_creator);
}

void f52(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 10, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }

    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 270;
      }
      void run(Registry reg) {
        if (count >= 65  && count < 65 + 9)
          degree += 10;
        if (count >= 20 + 74 && count < 20 + 9 + 74)
          degree += 10;
        if (count >= 25 + 103 && count < 25 + 18 + 103)
          degree -= 5;
        scroll(reg);
        move_forward(speed_);
        if (count > 146)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f53(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y - 300)) EnemyCreator {
    this(Point p) {
      super(p, 40, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 90;
      }
      void run(Registry reg) {
        if (count >= 30  && count < 30 + 45)
          degree += 3;
        if (count >= 0 + 75 && count < 0 + 45 + 75)
          degree -= 1;
        if (count >= 0 + 120 && count < 0 + 45 + 120)
          degree -= 2;
        if (count >= 55 + 165 && count < 55 + 40 + 165)
          degree -= 3;
        if (count >= 0 + 260 && count < 0 + 40 + 260)
          degree += 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 300)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f54(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 70, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 90;
      }
      void run(Registry reg) {
        if (count >= 30  && count < 30 + 50)
          degree -= 3;
        if (count >= 0 + 80 && count < 0 + 20 + 80)
          degree += 3;
        if (count >= 125 + 100 && count < 125 + 30 + 100)
          degree += 3;
        if (count >= 50 + 255 && count < 50 + 60 + 255)
          degree -= 3;
        if (count >= 50 + 365 && count < 50 + 30 + 365)
          degree += 3;
        if (count >= 90 + 445 && count < 90 + 30 + 445)
          degree -= 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 565)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f55(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 40, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 270;
      }
      void run(Registry reg) {
        if (count >= 10  && count < 10 + 30)
          degree += 3;
        if (count >= 40 + 40 && count < 40 + 30 + 40)
          degree -= 3;
        if (count >= 40 + 110 && count < 40 + 70 + 110)
          degree += 3;
        if (count >= 10 + 220 && count < 10 + 10 + 220)
          degree -= 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 240)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f56(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 40, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 270;
      }
      void run(Registry reg) {
        if (count >= 60  && count < 60 + 10)
          degree += 3;
        if (count >= 0 + 70 && count < 0 + 10 + 70)
          degree -= 3;
        if (count >= 20 + 80 && count < 20 + 30 + 80)
          degree += 3;
        if (count >= 60 +130 && count < 60 + 30 + 130)
          degree += 3;
        if (count >= 10 +220 && count < 10 + 45 + 220)
          degree += 3;
        if (count >= 0 + 275 && count < 0 + 45 + 275)
          degree -= 3;
        if (count >= 10 + 320 && count < 10 + 30 + 320)
          degree -= 3;
        if (count >= 80 + 360 && count < 80 + 30 + 360)
          degree += 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 470)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f57(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x,y)) EnemyCreator {
    this(Point p) {
      super(p, 30, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 270;
      }
      void run(Registry reg) {
        if (count >= 80  && count < 80 + 70)
          degree -= 3;
        if (count >= 10 + 150  && count < 10 + 10 + 150)
          degree += 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 170)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f58(double x, double y, Registry reg)
{
  EnemyCreator viper_creator = new class(Point(x, y - 140)) EnemyCreator {
    this(Point p) {
      super(p, 30, 15);
    }

    void run(Registry reg) {
      if (count % interval == 0)
        reg.add(new Vipers(pos));
      super.run(reg);
    }
    
    static class Vipers : Viper {
      double speed_ = 4;
      this(Point p) {
        super(p);
        degree = 270;
      }
      void run(Registry reg) {
        if (count >= 80  && count < 80 + 70)
          degree += 3;
        if (count >= 10 + 150  && count < 10 + 10 + 150)
          degree -= 3;
        scroll(reg);
        move_forward(speed_);
        if (count > 170)
          die_if_out_of_window();
      }
    };

  };
  reg.add(viper_creator);
}

void f71(double x, double y, Registry reg)
{
  EnemyCreator cyclone_creator = new class(Point(x,y)) EnemyCreator {
public:
    this(Point pos) {
      super(pos, 5, 100);
    }
    void run(Registry reg) {
      int i;
      if (count % interval == 0) {
        for (i = 0; i < 10; i++) {
          reg.add(new Cyclone(reg, pos, 210 + (i * 25), 2));
        }
      }
      super.run(reg);
    }
    
    static class Cyclone : BaseEnemy {
      static const RADIUS = 50;
      Point center_;
      double omega_;
      this(Registry reg, Point pos, double deg, double omega) {
        super(SubKind.NORMAL_ENEMY, pos + degree_to_vector(RADIUS, false, deg),
              new Animation(&images.enemies.circle.draw),
              50, Color.enemy, deg);
        reg.add(new EnemyWall(this));
        omega_ = omega;
        center_ = pos;
      }
      mixin DefineEnemy!(200);
      mixin DefineRect!(pos, 16, 16, true);
      void run(Registry reg) {
        center_ += Vector(0, 1);
        if (count < 170)
          center_ += Vector(2, 0);
        else if (count < 130 + 170)
          center_ += Vector(0, -2);
        else if (count < 90 + 300)
          center_ += Vector(2, 0);
        else if (count < 100 + 390)
          center_ += Vector(0, -2);
        else if (count < 260 + 490)
          center_ += Vector(-2, 0);
        degree += omega_;
        pos = center_ + degree_to_vector(RADIUS, false, degree);
        if (count > 800)
          die();
      }
    };
  };
  reg.add(cyclone_creator);
}


class Boss6: BaseEnemy {
  double dy_ = 0;
  this(Point pos, Registry reg) {
    super(SubKind.BOSS_ENEMY, pos + Vector(0, -200),
          new Animation(&images.enemies.boss6.draw),
          800);
    scale = 10;
    reg.add(new Boss6.HitArea(Point(0,-25), this));
    set_destroyed_hook(&destroyed);
  }

  mixin DefineRect!(pos, 50, 80, true);
  mixin DefineEnemy!(20000);

  static const LOOP = 300;


  static void destroyed(Registry reg, Enemy e) {
    reg.add(new StageEnd(6, 7));
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
  
    //登場部分
    scroll(reg);
    if (count < 450) {
      return;
    }
    if (count < 600) {
      dy_ -= 0.1;
      move(Vector(0, dy_));
      return;
    }
    if (count == 600)
      pos = Point(pos.x, -140);
    if (count < 800) {
      return;
    }
    
    if (count == 800) {
      reg.add(new Boss6.Boss6Wall(Point(261, 120), 1, this));
      reg.add(new Boss6.Boss6Wall(Point(260, 120), -1, this));
    }
    
    if (count == 900) {
      reg.add(new Boss6PileBunker());
    }
    
    //時間切れ
    if (count >= 900 + LOOP*15) {
      if (count >= 920 + LOOP*15) {
        Wave.boss_destroyed.play();
        reg.add(new DestroyedEnemy(pos, true, rect().w));
        destroyed(reg, this);
        die();
      }
      return;
    }
    //ループ部分
    int ct = (count - 900) % LOOP;
    
    if (ct == 130 || ct == 160 || ct == 190) {
      reg.add(new NipperBullet(pos + Point(0, 35), Vector(0,0), 1));
      reg.add(new NipperBullet(pos + Point(0, 35), Vector(0,0), -1));
    }
    
    repeat(600, 2400, {
    });
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 900) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

  static class HitArea: ChildEnemy {
    this(Point pos, Movable boss) {
      super(SubKind.BOSS_ENEMY, pos, Animation.empty(),
            Color.enemy, 0, 0, boss); 
    }
    mixin DefineRect!(pos, 100, 30, true);
    mixin DefineEnemy!(0);
  }

  //壁関係

  static class Boss6PileBunker: MovableWall {
    double dy_ = 10;
    bool search_ = false;
    static const LOOP = 50;
    this() {
      super(Point(100, 20), 10, 1);
    }
    void run(Registry reg) {
      int ct = count % LOOP;
      auto pl = Player.search(reg);
      if (pl && ct == 0 && pl.pos.y < 100) {
        search_ = true;
        move(Vector(pl.pos.x - rect.p.x - 5, 0));
      }

      if (search_) {
        if (ct < 8)
          qmove(Vector(0, dy_));
        if (ct >= 20 && ct < 28)
          qmove(Vector(0, -dy_));
        if (ct == 29)
          search_ = false;
      }

      super.run(reg);
    }
  };
  
  static class Boss6Wall: MovableWall {
    double dx_ = 4;
    int sign_;
    Boss6 boss_;
    bool open_ = true;
    this(Point p, int sign, Boss6 boss) {
      super(Point(280 - (p.x * sign), p.y), 1, 20);
      sign_ = sign;
      boss_ = boss;
    }
    void run(Registry reg) {
      //壁登場部分
      if (count < 10)
        move(Vector(5 * sign_, 0));
      if (count < 30) {
        if (sign_ > 0)
          qmove(Vector(6 * sign_, 0));
        else
          pmove(Vector(6 * sign_, 0));
      }
      if (count == 30) {
        if (sign_ == 1)
          reg.add(new Boss6.Boss6SmallWall(Point(51, 120), sign_, boss_));
        else if (sign_ == -1)
          reg.add(new Boss6.Boss6SmallWall(Point(50, 120), sign_, boss_));
      }
      if (count < 100)
        return;
      //時間切れ
      if (count >= 100 + LOOP*15) {
        if (count <= 179 + LOOP*15)
          move(Vector(0, -1));
        return;
      }

      //ループ部分
      
      int ct = (count - 100) % LOOP;
      
      dx_ = 0;
      
      if (boss_.dead() && ct == 0) {
        open_ = false;
      }
      
      if (open_) {
        if (ct < 50) {
          dx_ = -1 * sign_;
        } else if (ct > 200 && ct <= 210) {
          dx_ = 5 * sign_;
        }
      }
      move(Vector(dx_, 0));

      super.run(reg);
    }
  };
  
  static class Boss6SmallWall: MovableWall {
    double dx_ = 4;
    int sign_;
    Boss6 boss_;
    bool open_ = true;
    this(Point p, int sign, Boss6 boss) {
      super(Point(280 - (p.x * sign), p.y), 1, 1);
      sign_ = sign;
      boss_ = boss;
    }
    void run(Registry reg) {
      //壁登場部分
      if (count < 5)
        qmove(Vector(0, 4));
      if (count < 10) {
        if (sign_ > 0)
          qmove(Vector(5 * sign_, 0));
        else
          pmove(Vector(5 * sign_, 0));
      }
      if (count < 70)
        return;
      //時間切れ
      if (count >= 70 + LOOP*15) {
        if (count <= 149 + LOOP*15)
          move(Vector(0, -1));
        return;
      }

      //ループ部分
      
      int ct = (count - 70) % LOOP;
      
      dx_ = 0;
      
      if (boss_.dead() && ct == 0) {
        open_ = false;
      }
      
      if (open_) {
        if (ct < 50) {
          dx_ = -1 * sign_;
        } else if (ct > 200 && ct <= 210) {
          dx_ = 5 * sign_;
        }
      }
      move(Vector(dx_, 0));

      super.run(reg);
    }
  };
}

class NipperBullet: LBullet
{
private:
  static void function()[] image = [ &images.enemies.saw.draw ];
  bool search_ = true;
  double dx_ = 0;
  double dy_ = 4;
  double omega_ = 5;
  int sign_;

public:
  this(Point pos, Point vect, int sign) {
    super(pos, vect, new Animation(image), false);
    sign_ = sign;
  }

  mixin DefineRect!(pos, 13, 13, true);

  void run(Registry reg) {
    if (search_) {
      if (count < 10) {
        dx_ = 0;
      } else if (count < 30) {
        dx_ += 0.4 * sign_;
      } else if (count < 50) {
        dx_ -= 0.4 * sign_;
      }
      auto pl = Player.search(reg);
      if (pl && count > 30) {
        if (abs(pl.pos.y - pos.y) < 16) {
          search_ = false;
          dx_ = -8 * sign_;
          dy_ = 0;
        }
      }
    }
    degree += omega_ * sign_;
    move(Vector(dx_, dy_));
    die_if_out_of_window();
  }

  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
}

class Boss6Left: BaseEnemy {
  double dx_ = 0;
  double dy_ = 0;
  Boss6 boss_;
  this(Boss6 boss, Registry reg) {
    super(SubKind.BOSS_ENEMY, boss.pos + Point(-90, 80),
          new Animation(&images.enemies.boss6_left.draw),
          10000, Color.enemy, 0, 10);
    boss_ = boss;
    reg.add(new Boss6Left.HitArea(Point(30,35), this));
    reg.add(new Boss6Left.HitArea(Point(30,-35), this));
  }

  mixin DefineRect!(pos, 25, 150, true);
  mixin DefineEnemy!(0);

  static const LOOP = 300;

  void run(Registry reg) {

    //登場部分
    scroll(reg);
    if (count < 450) {
      return;
    }
    if (count < 505) {
      dy_ -= 0.1;
      move(Vector(0, dy_));
    }
    if (count < 560 && count >= 505) {
      dy_ += 0.1;
      move(Vector(0, dy_));
    }
    if (count < 505) {
      dx_ -= 0.1;
      move(Vector(dx_, 0));
      return;
    }
    if (count == 505) {
      pos = Point(30, pos.y);
    }
    if (count < 840) {
      return;
    }
    
    //ループ部分
    int ct = (count - 900) % LOOP;

    if (boss_.dead()) {
      Wave.boss_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().h));
      die();
    }
    
    if (ct % 60 == 0 && (ct < 120 || ct >= 240))
      nway_fire(reg, pos + Point(35, 35), 1, 7);
    if ((ct % 60 == 30 || ct % 60 == -30) && (ct < 120 || ct >= 240))
      nway_fire(reg, pos + Point(35, -35), 5, 3);
  }

  void nway_fire(Registry reg, Point pos, double change_degree, int nway, double speed = 4) {
    auto pl = Player.search(reg);
    if (pl) {
      if (pl.pos.x < 100)
        return;
      double deg;
      Point diff = pl.pos - pos;
      deg = atan2(diff.y, diff.x) * 180 / PI;
      int i;
      for (i = 0; i < nway; i++) {
          reg.add(new NormalBullet(pos, degree_to_vector(speed, false, deg +  (i - (nway-1)/2.0 ) * change_degree)));
      }
    }
  }

  static class HitArea: ChildEnemy {
    this(Point pos, Movable boss) {
      super(SubKind.BOSS_ENEMY, pos, Animation.empty(),
            Color.enemy, 0, 0, boss); 
    }
    mixin DefineRect!(pos, 40, 40, true);
    mixin DefineEnemy!(0);
    void collide_with_shot(Registry reg, Shot shot) {
        return;
    }
  }

  void collide_with_shot(Registry reg, Shot shot) {
      return;
  }
}

class Boss6Right: BaseEnemy {
  double dx_ = 0;
  double dy_ = 0;
  Boss6 boss_;
  this(Boss6 boss, Registry reg) {
    super(SubKind.BOSS_ENEMY, boss.pos + Point(90, 80),
          new Animation(&images.enemies.boss6_right.draw),
          10000, Color.enemy, 0, 10);
    boss_ = boss;
    reg.add(new Boss6Right.HitArea(Point(-33,0), this));
  }

  mixin DefineRect!(pos, 30, 160, true);
  mixin DefineEnemy!(0);

  static const LOOP = 300;

  void run(Registry reg) {

    //登場部分
    scroll(reg);

    if (count < 450) {
      return;
    }
    if (count < 505) {
      dy_ -= 0.1;
      move(Vector(0, dy_));
    }
    if (count < 560 && count >= 505) {
      dy_ += 0.1;
      move(Vector(0, dy_));
    }
    if (count < 505) {
      dx_ += 0.1;
      move(Vector(dx_, 0));
      return;
    }
    if (count == 505) {
      pos = Point(529, pos.y);
    }
    if (count < 840) {
      return;
    }
    
    //ループ部分
    int ct = (count - 900) % LOOP;

    if (ct % 30 == 0 && (ct < 100 || ct >= 240)) {
      reg.add(new HomingBulletCreator(Point(-80,0), this));
    }
    
    if (boss_.dead()) {
      Wave.boss_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().h));
      die();
    }
    
  }
  void collide_with_shot(Registry reg, Shot shot) {
      return;
  }

  static class HomingBulletCreator: BaseMovable {
    static const HEAD_COUNT = 8;
    HomingBullet bullet;
    this(Point pos, Movable boss) {
      super(SubKind.NORMAL_BULLET_CREATOR, pos, boss);
      bullet = null;
    }
    
    void run(Registry reg) {
      if (count == HEAD_COUNT)
        die();
      bullet = new HomingBullet(pos, Vector(0,0), bullet);
      reg.add(bullet);
    }

  }

  static class HitArea: ChildEnemy {
    this(Point pos, Movable boss) {
      super(SubKind.BOSS_ENEMY, pos, Animation.empty(),
            Color.enemy, 0, 0, boss); 
    }
    mixin DefineRect!(pos, 80, 30, true);
    mixin DefineEnemy!(0);
    void collide_with_shot(Registry reg, Shot shot) {
        return;
    }
  }
}

class HomingBullet: LBullet
{
private:
  static void function()[] image = [ &images.enemies.circle.draw ];
  HomingBullet pre_bullet_;
  bool search_;
  bool head_;
  double dx_;
  double dy_;

public:
  Point old_pos_;
  this(Point pos, Point vect, HomingBullet bullet = null) {
    pre_bullet_ = bullet;
    if (!bullet) {
      super(pos, vect, new Animation(image), false);
      head_ = true;
      search_ = true;
      scale = 1.0;
    } else {
      super(bullet.old_pos_, vect, new Animation(image), false);
      head_ = false;
      search_ = false;
      scale = bullet.scale - 0.1;
    }
    dx_ = -6;
    dy_ = 0;
    old_pos_ = pos;
  }

  mixin DefineRect!(pos, 10, 10, true);

  void run(Registry reg) {
    if (!head_ && pre_bullet_.dead())
      die();
    auto pl = Player.search(reg);
    if (pl && search_) {
      if (abs(pl.pos.x - pos.x) < 10) {
        search_ = false;
        dy_ = -dx_ * sign(pl.pos.y - pos.y);
        dx_ = 0;
      }
    }

    old_pos_ = pos;
    if (head_) {
      move(Vector(dx_, dy_));
    } else {
      pos = pre_bullet_.pos;
    }
    die_if_out_of_window();
  }

  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
}


void f61(double x, double y, Registry reg)
{
  Boss6 boss = new Boss6(Point(x, y-100), reg);
  reg.add(boss);
  reg.add(new Boss6Left(boss, reg));
  reg.add(new Boss6Right(boss, reg));
  Music.boss[6].play();
}


void f62(double time, double x, double y, Registry reg)
{
  static class Brace: Pillar {
    this(Point p, Registry reg, int ct) {
      super(p, reg, ct);
    }
    void run(Registry reg) {
      if (count > 440 - pos.y / 30) {
        Wave.enemy_destroyed.play();
        reg.add(new DestroyedEnemy(pos, true, rect().w));
        die();
      }
      super.run(reg);
    }
  };

  reg.add(new Brace(Point(x, y + time), reg, 500));
}



class Events0{
const static Event[] events = [
  {1, &fun0},
  {1, &fun2},
  {30, &fun58},
  {30, &fun57},
  {65, &fun62},
  {65, &fun63},
  {65, &fun61},
  {65, &fun60},
  {65, &fun59},
  {65, &fun64},
  {80, &fun7},
  {80, &fun11},
  {80, &fun12},
  {80, &fun6},
  {80, &fun8},
  {113, &fun67},
  {113, &fun68},
  {113, &fun70},
  {113, &fun66},
  {113, &fun69},
  {113, &fun65},
  {360, &fun56},
  {360, &fun9},
  {400, &fun86},
  {440, &fun10},
  {540, &fun79},
  {580, &fun13},
  {580, &fun85},
  {620, &fun87},
  {700, &fun78},
  {740, &fun14},
  {745, &fun84},
  {745, &fun73},
  {745, &fun83},
  {760, &fun15},
  {793, &fun72},
  {793, &fun71},
  {796, &fun126},
  {920, &fun88},
  {1020, &fun80},
  {1060, &fun17},
  {1065, &fun121},
  {1065, &fun120},
  {1080, &fun18},
  {1080, &fun16},
  {1116, &fun123},
  {1116, &fun124},
  {1116, &fun125},
  {1140, &fun109},
  {1160, &fun115},
  {1200, &fun119},
  {1220, &fun114},
  {1260, &fun118},
  {1280, &fun91},
  {1280, &fun19},
  {1280, &fun113},
  {1300, &fun90},
  {1320, &fun117},
  {1340, &fun100},
  {1340, &fun112},
  {1360, &fun102},
  {1380, &fun116},
  {1400, &fun111},
  {1400, &fun101},
  {1420, &fun103},
  {1460, &fun110},
  {1460, &fun108},
  {1520, &fun104},
  {1565, &fun94},
  {1565, &fun122},
  {1565, &fun95},
  {1565, &fun89},
  {1565, &fun93},
  {1565, &fun96},
  {1580, &fun20},
  {1725, &fun92},
  {1740, &fun21},
  {1750, &fun107},
  {1773, &fun74},
  {1820, &fun82},
  {1820, &fun81},
  {1820, &fun99},
  {1868, &fun76},
  {1880, &fun22},
  {1890, &fun106},
  {1913, &fun75},
  {1960, &fun98},
  {2008, &fun77},
  {2020, &fun23},
  {2030, &fun105},
  {2056, &fun97},
  {2200, &fun24},
  {2200, &fun25},
  {2240, &fun169},
  {2240, &fun168},
  {2280, &fun175},
  {2280, &fun184},
  {2300, &fun167},
  {2300, &fun178},
  {2340, &fun174},
  {2340, &fun185},
  {2360, &fun166},
  {2360, &fun179},
  {2400, &fun173},
  {2400, &fun186},
  {2420, &fun180},
  {2420, &fun4},
  {2420, &fun165},
  {2460, &fun172},
  {2460, &fun187},
  {2467, &fun3},
  {2480, &fun164},
  {2480, &fun181},
  {2520, &fun177},
  {2520, &fun188},
  {2540, &fun171},
  {2540, &fun182},
  {2580, &fun189},
  {2580, &fun176},
  {2600, &fun170},
  {2600, &fun183},
  {2760, &fun26},
  {2760, &fun28},
  {2800, &fun156},
  {2840, &fun157},
  {2880, &fun158},
  {2960, &fun29},
  {2960, &fun30},
  {2960, &fun27},
  {2960, &fun54},
  {2996, &fun163},
  {2996, &fun196},
  {2996, &fun197},
  {2996, &fun198},
  {3000, &fun150},
  {3020, &fun149},
  {3040, &fun191},
  {3080, &fun192},
  {3100, &fun190},
  {3140, &fun193},
  {3160, &fun33},
  {3160, &fun32},
  {3160, &fun31},
  {3200, &fun194},
  {3260, &fun195},
  {3260, &fun34},
  {3340, &fun160},
  {3420, &fun36},
  {3460, &fun155},
  {3705, &fun162},
  {3705, &fun161},
  {3720, &fun38},
  {3720, &fun39},
  {3720, &fun40},
  {3720, &fun37},
  {3820, &fun153},
  {3900, &fun151},
  {4000, &fun159},
  {4060, &fun41},
  {4100, &fun152},
  {4140, &fun42},
  {4180, &fun44},
  {4180, &fun43},
  {4280, &fun154},
  {4320, &fun35},
  {4380, &fun52},
  {4380, &fun45},
  {4440, &fun128},
  {4487, &fun127},
  {4520, &fun53},
  {4520, &fun46},
  {4520, &fun47},
  {4520, &fun50},
  {4700, &fun55},
  {4720, &fun51},
  {4720, &fun5},
  {4720, &fun49},
  {4721, &fun134},
  {4721, &fun129},
  {4721, &fun130},
  {4721, &fun131},
  {4721, &fun132},
  {4721, &fun133},
  {4721, &fun148},
  {4721, &fun147},
  {4721, &fun146},
  {4721, &fun145},
  {4721, &fun144},
  {4721, &fun135},
  {4721, &fun142},
  {4721, &fun141},
  {4721, &fun140},
  {4721, &fun139},
  {4721, &fun138},
  {4721, &fun137},
  {4721, &fun136},
  {4721, &fun143},
  {5480, &fun48},
  {5520, &fun1},
];
static void fun0(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun2(Registry reg){
  set_scroll(reg, 1);
}
static void fun58(Registry reg){
  f52(601.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f51(-40.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f11(541.000000, -20.000000, reg);
}
static void fun63(Registry reg){
  f11(481.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f11(160.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f11(80.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f11(20.000000, -20.000000, reg);
}
static void fun64(Registry reg){
  f11(401.000000, -20.000000, reg);
}
static void fun7(Registry reg){
  f01(280.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(0, -560, 20, 540)));
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(540, -380, 20, 360)));
}
static void fun67(Registry reg){
  f21(180.000000, -20.000000, reg);
}
static void fun68(Registry reg){
  f21(381.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f21(461.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f21(100.000000, -20.000000, reg);
}
static void fun69(Registry reg){
  f21(521.000000, -20.000000, reg);
}
static void fun65(Registry reg){
  f21(40.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f53(-40.000000, -20.000000, reg);
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(200, -200, 20, 180)));
}
static void fun86(Registry reg){
  f41(300.000000, -20.000000, reg);
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(400, -360, 20, 340)));
}
static void fun79(Registry reg){
  f55(600.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(540, -380, 20, 360)));
}
static void fun85(Registry reg){
  f43(240.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  f42(360.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f54(-40.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, -1380, 20, 1360)));
}
static void fun84(Registry reg){
  f31(240.000000, -20.000000, reg);
}
static void fun73(Registry reg){
  f31(360.000000, -20.000000, reg);
}
static void fun83(Registry reg){
  f31(300.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 200, 20)));
}
static void fun72(Registry reg){
  f21(360.000000, -20.000000, reg);
}
static void fun71(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  f13(300.000000, -20.000000, reg);
}
static void fun88(Registry reg){
  f44(260.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  f56(600.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(540, -580, 20, 560)));
}
static void fun121(Registry reg){
  f12(360.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  f12(420.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(200, -340, 20, 320)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(200, -40, 260, 20)));
}
static void fun123(Registry reg){
  f13(280.000000, -20.000000, reg);
}
static void fun124(Registry reg){
  f13(340.000000, -20.000000, reg);
}
static void fun125(Registry reg){
  f13(400.000000, -20.000000, reg);
}
static void fun109(Registry reg){
  f32(235.000000, -20.000000, reg);
}
static void fun115(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  f14(235.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  f14(235.000000, -20.000000, reg);
}
static void fun91(Registry reg){
  f33(524.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(400, -200, 20, 180)));
}
static void fun113(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun90(Registry reg){
  f34(435.000000, -20.000000, reg);
}
static void fun117(Registry reg){
  f14(235.000000, -20.000000, reg);
}
static void fun100(Registry reg){
  f33(524.000000, -20.000000, reg);
}
static void fun112(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun102(Registry reg){
  f34(435.000000, -20.000000, reg);
}
static void fun116(Registry reg){
  f14(235.000000, -20.000000, reg);
}
static void fun111(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun101(Registry reg){
  f33(524.000000, -20.000000, reg);
}
static void fun103(Registry reg){
  f34(435.000000, -20.000000, reg);
}
static void fun110(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  f33(524.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f32(35.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  f11(180.000000, -20.000000, reg);
}
static void fun122(Registry reg){
  f11(360.000000, -20.000000, reg);
}
static void fun95(Registry reg){
  f11(240.000000, -20.000000, reg);
}
static void fun89(Registry reg){
  f31(60.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  f11(120.000000, -20.000000, reg);
}
static void fun96(Registry reg){
  f11(300.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 400, 20)));
}
static void fun92(Registry reg){
  f12(260.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 340, 20)));
}
static void fun107(Registry reg){
  f32(35.000000, -20.000000, reg);
}
static void fun74(Registry reg){
  f22(260.000000, -20.000000, reg);
}
static void fun82(Registry reg){
  f58(580.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  f57(580.000000, -20.000000, reg);
}
static void fun99(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun76(Registry reg){
  f23(260.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 340, 20)));
}
static void fun106(Registry reg){
  f32(35.000000, -20.000000, reg);
}
static void fun75(Registry reg){
  f22(260.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun77(Registry reg){
  f23(260.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 340, 20)));
}
static void fun105(Registry reg){
  f32(35.000000, -20.000000, reg);
}
static void fun97(Registry reg){
  f13(260.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(80, -460, 20, 440)));
}
static void fun25(Registry reg){
  reg.add(new NormalWall(Rect(460, -460, 20, 440)));
}
static void fun169(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun168(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun175(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun184(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun167(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun178(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun174(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun185(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun166(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun179(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun173(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun186(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun180(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun165(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun172(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun187(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun164(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun181(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun177(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun188(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun171(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun182(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun189(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun176(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun170(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun183(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  reg.add(new NormalWall(Rect(0, -400, 20, 380)));
}
static void fun28(Registry reg){
  reg.add(new NormalWall(Rect(340, -240, 20, 220)));
}
static void fun156(Registry reg){
  f49(80.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  f49(460.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  f48(180.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  reg.add(new NormalWall(Rect(540, -360, 20, 340)));
}
static void fun30(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun27(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun54(Registry reg){
  f02(280.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  f35(400.000000, -20.000000, reg);
}
static void fun196(Registry reg){
  f35(480.000000, -20.000000, reg);
}
static void fun197(Registry reg){
  f35(80.000000, -20.000000, reg);
}
static void fun198(Registry reg){
  f35(160.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  f46(620.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  f45(-60.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun192(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun190(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun193(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  reg.add(new NormalWall(Rect(340, -400, 20, 380)));
}
static void fun32(Registry reg){
  reg.add(new NormalWall(Rect(200, -40, 160, 20)));
}
static void fun31(Registry reg){
  reg.add(new NormalWall(Rect(200, -400, 20, 380)));
}
static void fun194(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun195(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  reg.add(new NormalWall(Rect(0, -700, 20, 680)));
}
static void fun160(Registry reg){
  f48(280.000000, -20.000000, reg);
}
static void fun36(Registry reg){
  reg.add(new NormalWall(Rect(540, -660, 20, 640)));
}
static void fun155(Registry reg){
  f49(280.000000, -20.000000, reg);
}
static void fun162(Registry reg){
  f31(440.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f31(100.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun39(Registry reg){
  reg.add(new NormalWall(Rect(200, -240, 20, 220)));
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(340, -240, 20, 220)));
}
static void fun37(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun153(Registry reg){
  f49(100.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  f47(580.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f71(-60.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(0, -360, 20, 340)));
}
static void fun152(Registry reg){
  f48(100.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(540, -1380, 20, 1360)));
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun154(Registry reg){
  f48(100.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 20, 80)));
}
static void fun52(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 100, 20)));
}
static void fun45(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 140, 20)));
}
static void fun128(Registry reg){
  set_scroll(reg, 1);
}
static void fun127(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun53(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 100, 20)));
}
static void fun46(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 20, 80)));
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 140, 20)));
}
static void fun50(Registry reg){
  reg.add(new NormalWall(Rect(0, -1000, 20, 980)));
}
static void fun55(Registry reg){
  f01(520.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun5(Registry reg){
  f61(280.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun134(Registry reg){
  f62(-280, 401.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f62(-160, 401.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  f62(-120, 401.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  f62(-80, 401.000000, -20.000000, reg);
}
static void fun132(Registry reg){
  f62(-40, 401.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f62(-320, 401.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f62(-360, 380.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f62(-360, 340.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f62(-360, 220.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f62(-200, 159.000000, -20.000000, reg);
}
static void fun144(Registry reg){
  f62(-240, 159.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f62(-240, 401.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f62(-320, 159.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f62(-40, 159.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f62(-80, 159.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f62(-120, 159.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f62(-160, 159.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f62(-360, 180.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f62(-200, 401.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f62(-280, 159.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 560, 20)));
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun20},
  {1, &fun3},
  {1, &fun4},
  {1, &fun173},
  {1, &fun6},
  {1, &fun8},
  {1, &fun9},
  {1, &fun10},
  {1, &fun11},
  {1, &fun12},
  {1, &fun13},
  {1, &fun14},
  {1, &fun15},
  {1, &fun16},
  {1, &fun17},
  {1, &fun18},
  {1, &fun19},
  {1, &fun165},
  {1, &fun21},
  {1, &fun22},
  {1, &fun23},
  {1, &fun24},
  {1, &fun25},
  {1, &fun172},
  {1, &fun187},
  {1, &fun186},
  {1, &fun164},
  {1, &fun181},
  {1, &fun180},
  {33, &fun188},
  {33, &fun177},
  {53, &fun182},
  {53, &fun171},
  {93, &fun189},
  {93, &fun176},
  {113, &fun170},
  {113, &fun183},
  {273, &fun28},
  {273, &fun26},
  {313, &fun156},
  {353, &fun157},
  {393, &fun158},
  {473, &fun30},
  {473, &fun27},
  {473, &fun54},
  {473, &fun29},
  {509, &fun198},
  {509, &fun196},
  {509, &fun197},
  {509, &fun163},
  {513, &fun150},
  {533, &fun149},
  {553, &fun191},
  {593, &fun192},
  {613, &fun190},
  {653, &fun193},
  {673, &fun32},
  {673, &fun31},
  {673, &fun33},
  {713, &fun194},
  {773, &fun34},
  {773, &fun195},
  {853, &fun160},
  {933, &fun36},
  {973, &fun155},
  {1218, &fun162},
  {1218, &fun161},
  {1233, &fun40},
  {1233, &fun37},
  {1233, &fun38},
  {1233, &fun39},
  {1333, &fun153},
  {1413, &fun151},
  {1513, &fun159},
  {1573, &fun41},
  {1613, &fun152},
  {1653, &fun42},
  {1693, &fun43},
  {1693, &fun44},
  {1793, &fun154},
  {1833, &fun35},
  {1893, &fun52},
  {1893, &fun45},
  {1953, &fun128},
  {2000, &fun127},
  {2033, &fun46},
  {2033, &fun47},
  {2033, &fun50},
  {2033, &fun53},
  {2213, &fun55},
  {2233, &fun51},
  {2233, &fun49},
  {2233, &fun5},
  {2234, &fun131},
  {2234, &fun132},
  {2234, &fun129},
  {2234, &fun133},
  {2234, &fun134},
  {2234, &fun135},
  {2234, &fun136},
  {2234, &fun137},
  {2234, &fun138},
  {2234, &fun139},
  {2234, &fun140},
  {2234, &fun141},
  {2234, &fun142},
  {2234, &fun143},
  {2234, &fun144},
  {2234, &fun145},
  {2234, &fun146},
  {2234, &fun147},
  {2234, &fun148},
  {2234, &fun130},
  {2993, &fun48},
  {3033, &fun1},
];
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(0, 867, 400, 20)));
}
static void fun3(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun4(Registry reg){
  set_scroll(reg, 1);
}
static void fun173(Registry reg){
  f14(495.000000, 67.000000, reg);
}
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(0, 1847, 20, 540)));
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(540, 2027, 20, 360)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(200, 1927, 20, 180)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(400, 1687, 20, 340)));
}
static void fun11(Registry reg){
  reg.add(new NormalWall(Rect(0, 2367, 220, 20)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(340, 2367, 220, 20)));
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(540, 1527, 20, 360)));
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, 367, 20, 1360)));
}
static void fun15(Registry reg){
  reg.add(new NormalWall(Rect(220, 1687, 200, 20)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(200, 1367, 260, 20)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(540, 847, 20, 560)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(200, 1067, 20, 320)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(400, 1007, 20, 180)));
}
static void fun165(Registry reg){
  f14(115.000000, 47.000000, reg);
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(220, 707, 340, 20)));
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(220, 567, 340, 20)));
}
static void fun23(Registry reg){
  reg.add(new NormalWall(Rect(220, 427, 340, 20)));
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(80, -173, 20, 440)));
}
static void fun25(Registry reg){
  reg.add(new NormalWall(Rect(460, -173, 20, 440)));
}
static void fun172(Registry reg){
  f14(495.000000, 7.000000, reg);
}
static void fun187(Registry reg){
  f15(64.000000, 7.000000, reg);
}
static void fun186(Registry reg){
  f15(64.000000, 67.000000, reg);
}
static void fun164(Registry reg){
  f14(115.000000, -13.000000, reg);
}
static void fun181(Registry reg){
  f15(444.000000, -13.000000, reg);
}
static void fun180(Registry reg){
  f15(444.000000, 47.000000, reg);
}
static void fun188(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun177(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun182(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun171(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun189(Registry reg){
  f15(64.000000, -20.000000, reg);
}
static void fun176(Registry reg){
  f14(495.000000, -20.000000, reg);
}
static void fun170(Registry reg){
  f14(115.000000, -20.000000, reg);
}
static void fun183(Registry reg){
  f15(444.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  reg.add(new NormalWall(Rect(340, -240, 20, 220)));
}
static void fun26(Registry reg){
  reg.add(new NormalWall(Rect(0, -400, 20, 380)));
}
static void fun156(Registry reg){
  f49(80.000000, -20.000000, reg);
}
static void fun157(Registry reg){
  f49(460.000000, -20.000000, reg);
}
static void fun158(Registry reg){
  f48(180.000000, -20.000000, reg);
}
static void fun30(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun27(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun54(Registry reg){
  f02(280.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  reg.add(new NormalWall(Rect(540, -360, 20, 340)));
}
static void fun198(Registry reg){
  f35(160.000000, -20.000000, reg);
}
static void fun196(Registry reg){
  f35(480.000000, -20.000000, reg);
}
static void fun197(Registry reg){
  f35(80.000000, -20.000000, reg);
}
static void fun163(Registry reg){
  f35(400.000000, -20.000000, reg);
}
static void fun150(Registry reg){
  f46(620.000000, -20.000000, reg);
}
static void fun149(Registry reg){
  f45(-60.000000, -20.000000, reg);
}
static void fun191(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun192(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun190(Registry reg){
  f14(35.000000, -20.000000, reg);
}
static void fun193(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun32(Registry reg){
  reg.add(new NormalWall(Rect(200, -40, 160, 20)));
}
static void fun31(Registry reg){
  reg.add(new NormalWall(Rect(200, -400, 20, 380)));
}
static void fun33(Registry reg){
  reg.add(new NormalWall(Rect(340, -400, 20, 380)));
}
static void fun194(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  reg.add(new NormalWall(Rect(0, -700, 20, 680)));
}
static void fun195(Registry reg){
  f15(524.000000, -20.000000, reg);
}
static void fun160(Registry reg){
  f48(280.000000, -20.000000, reg);
}
static void fun36(Registry reg){
  reg.add(new NormalWall(Rect(540, -660, 20, 640)));
}
static void fun155(Registry reg){
  f49(280.000000, -20.000000, reg);
}
static void fun162(Registry reg){
  f31(440.000000, -20.000000, reg);
}
static void fun161(Registry reg){
  f31(100.000000, -20.000000, reg);
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(340, -240, 20, 220)));
}
static void fun37(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun38(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun39(Registry reg){
  reg.add(new NormalWall(Rect(200, -240, 20, 220)));
}
static void fun153(Registry reg){
  f49(100.000000, -20.000000, reg);
}
static void fun151(Registry reg){
  f47(580.000000, -20.000000, reg);
}
static void fun159(Registry reg){
  f71(-60.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(0, -360, 20, 340)));
}
static void fun152(Registry reg){
  f48(100.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(540, -1380, 20, 1360)));
}
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun154(Registry reg){
  f48(100.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 20, 80)));
}
static void fun52(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 100, 20)));
}
static void fun45(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 140, 20)));
}
static void fun128(Registry reg){
  set_scroll(reg, 1);
}
static void fun127(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun46(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 20, 80)));
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 140, 20)));
}
static void fun50(Registry reg){
  reg.add(new NormalWall(Rect(0, -1000, 20, 980)));
}
static void fun53(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 100, 20)));
}
static void fun55(Registry reg){
  f01(520.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun49(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun5(Registry reg){
  f61(280.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  f62(-80, 401.000000, -20.000000, reg);
}
static void fun132(Registry reg){
  f62(-40, 401.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f62(-160, 401.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f62(-320, 401.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f62(-280, 401.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f62(-240, 401.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f62(-200, 401.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f62(-360, 180.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f62(-160, 159.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f62(-120, 159.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f62(-80, 159.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f62(-40, 159.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f62(-320, 159.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f62(-280, 159.000000, -20.000000, reg);
}
static void fun144(Registry reg){
  f62(-240, 159.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f62(-200, 159.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f62(-360, 220.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f62(-360, 340.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f62(-360, 380.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  f62(-120, 401.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 560, 20)));
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
class Events2{
const static Event[] events = [
  {1, &fun23},
  {1, &fun52},
  {1, &fun6},
  {1, &fun8},
  {1, &fun9},
  {1, &fun10},
  {1, &fun11},
  {1, &fun12},
  {1, &fun13},
  {1, &fun14},
  {1, &fun15},
  {1, &fun16},
  {1, &fun17},
  {1, &fun18},
  {1, &fun19},
  {1, &fun20},
  {1, &fun21},
  {1, &fun22},
  {1, &fun127},
  {1, &fun24},
  {1, &fun25},
  {1, &fun26},
  {1, &fun27},
  {1, &fun28},
  {1, &fun29},
  {1, &fun30},
  {1, &fun31},
  {1, &fun32},
  {1, &fun33},
  {1, &fun34},
  {1, &fun35},
  {1, &fun36},
  {1, &fun37},
  {1, &fun38},
  {1, &fun39},
  {1, &fun40},
  {1, &fun41},
  {1, &fun42},
  {1, &fun43},
  {1, &fun44},
  {1, &fun45},
  {1, &fun128},
  {13, &fun47},
  {13, &fun53},
  {13, &fun46},
  {13, &fun50},
  {193, &fun55},
  {213, &fun49},
  {213, &fun5},
  {213, &fun51},
  {214, &fun132},
  {214, &fun147},
  {214, &fun129},
  {214, &fun130},
  {214, &fun131},
  {214, &fun146},
  {214, &fun133},
  {214, &fun134},
  {214, &fun135},
  {214, &fun136},
  {214, &fun137},
  {214, &fun138},
  {214, &fun139},
  {214, &fun140},
  {214, &fun141},
  {214, &fun142},
  {214, &fun143},
  {214, &fun144},
  {214, &fun145},
  {214, &fun148},
  {973, &fun48},
  {1013, &fun1},
];
static void fun23(Registry reg){
  reg.add(new NormalWall(Rect(220, 2447, 340, 20)));
}
static void fun52(Registry reg){
  reg.add(new NormalWall(Rect(0, 87, 100, 20)));
}
static void fun6(Registry reg){
  reg.add(new NormalWall(Rect(0, 3867, 20, 540)));
}
static void fun8(Registry reg){
  reg.add(new NormalWall(Rect(540, 4047, 20, 360)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(200, 3947, 20, 180)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(400, 3707, 20, 340)));
}
static void fun11(Registry reg){
  reg.add(new NormalWall(Rect(0, 4387, 220, 20)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(340, 4387, 220, 20)));
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(540, 3547, 20, 360)));
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(0, 2387, 20, 1360)));
}
static void fun15(Registry reg){
  reg.add(new NormalWall(Rect(220, 3707, 200, 20)));
}
static void fun16(Registry reg){
  reg.add(new NormalWall(Rect(200, 3387, 260, 20)));
}
static void fun17(Registry reg){
  reg.add(new NormalWall(Rect(540, 2867, 20, 560)));
}
static void fun18(Registry reg){
  reg.add(new NormalWall(Rect(200, 3087, 20, 320)));
}
static void fun19(Registry reg){
  reg.add(new NormalWall(Rect(400, 3027, 20, 180)));
}
static void fun20(Registry reg){
  reg.add(new NormalWall(Rect(0, 2887, 400, 20)));
}
static void fun21(Registry reg){
  reg.add(new NormalWall(Rect(220, 2727, 340, 20)));
}
static void fun22(Registry reg){
  reg.add(new NormalWall(Rect(220, 2587, 340, 20)));
}
static void fun127(Registry reg){
  reg.root.set_restart_point(2);
}
static void fun24(Registry reg){
  reg.add(new NormalWall(Rect(80, 1847, 20, 440)));
}
static void fun25(Registry reg){
  reg.add(new NormalWall(Rect(460, 1847, 20, 440)));
}
static void fun26(Registry reg){
  reg.add(new NormalWall(Rect(0, 1347, 20, 380)));
}
static void fun27(Registry reg){
  reg.add(new NormalWall(Rect(0, 1507, 220, 20)));
}
static void fun28(Registry reg){
  reg.add(new NormalWall(Rect(340, 1507, 20, 220)));
}
static void fun29(Registry reg){
  reg.add(new NormalWall(Rect(540, 1187, 20, 340)));
}
static void fun30(Registry reg){
  reg.add(new NormalWall(Rect(340, 1507, 220, 20)));
}
static void fun31(Registry reg){
  reg.add(new NormalWall(Rect(200, 947, 20, 380)));
}
static void fun32(Registry reg){
  reg.add(new NormalWall(Rect(200, 1307, 160, 20)));
}
static void fun33(Registry reg){
  reg.add(new NormalWall(Rect(340, 947, 20, 380)));
}
static void fun34(Registry reg){
  reg.add(new NormalWall(Rect(0, 547, 20, 680)));
}
static void fun35(Registry reg){
  reg.add(new NormalWall(Rect(340, 87, 20, 80)));
}
static void fun36(Registry reg){
  reg.add(new NormalWall(Rect(540, 427, 20, 640)));
}
static void fun37(Registry reg){
  reg.add(new NormalWall(Rect(0, 747, 220, 20)));
}
static void fun38(Registry reg){
  reg.add(new NormalWall(Rect(340, 747, 220, 20)));
}
static void fun39(Registry reg){
  reg.add(new NormalWall(Rect(200, 547, 20, 220)));
}
static void fun40(Registry reg){
  reg.add(new NormalWall(Rect(340, 547, 20, 220)));
}
static void fun41(Registry reg){
  reg.add(new NormalWall(Rect(0, 87, 20, 340)));
}
static void fun42(Registry reg){
  reg.add(new NormalWall(Rect(540, -1013, 20, 1360)));
}
static void fun43(Registry reg){
  reg.add(new NormalWall(Rect(340, 287, 220, 20)));
}
static void fun44(Registry reg){
  reg.add(new NormalWall(Rect(0, 287, 220, 20)));
}
static void fun45(Registry reg){
  reg.add(new NormalWall(Rect(220, 87, 140, 20)));
}
static void fun128(Registry reg){
  set_scroll(reg, 1);
}
static void fun47(Registry reg){
  reg.add(new NormalWall(Rect(220, -40, 140, 20)));
}
static void fun53(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 100, 20)));
}
static void fun46(Registry reg){
  reg.add(new NormalWall(Rect(340, -100, 20, 80)));
}
static void fun50(Registry reg){
  reg.add(new NormalWall(Rect(0, -1000, 20, 980)));
}
static void fun55(Registry reg){
  f01(520.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  reg.add(new NormalWall(Rect(340, -40, 220, 20)));
}
static void fun5(Registry reg){
  f61(280.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 220, 20)));
}
static void fun132(Registry reg){
  f62(-40, 401.000000, -20.000000, reg);
}
static void fun147(Registry reg){
  f62(-360, 340.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f62(-160, 401.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  f62(-120, 401.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  f62(-80, 401.000000, -20.000000, reg);
}
static void fun146(Registry reg){
  f62(-360, 220.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f62(-320, 401.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f62(-280, 401.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f62(-240, 401.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f62(-200, 401.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f62(-360, 180.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f62(-160, 159.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f62(-120, 159.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f62(-80, 159.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f62(-40, 159.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f62(-320, 159.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f62(-280, 159.000000, -20.000000, reg);
}
static void fun144(Registry reg){
  f62(-240, 159.000000, -20.000000, reg);
}
static void fun145(Registry reg){
  f62(-200, 159.000000, -20.000000, reg);
}
static void fun148(Registry reg){
  f62(-360, 380.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  reg.add(new NormalWall(Rect(0, -40, 560, 20)));
}
static void fun1(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
  Events2.events,
];
