import kinds;
import util;
import base;
import images = images.src.images;
import opengl;
import glutils;
import system;
import enemy;
import ship;
import std.math;
import sounds;
import stage_common;

import rand;


class RushFlyer: Flyer {
private:
  Vector v_;
public:
  this(Point p, Registry reg, double speed = 8) {
    super(p);
    auto pl = Player.search(reg);
    if (pl) {
      Point diff = pl.pos - pos;
      degree = atan2(diff.y, diff.x) * 180 / PI + 90;
    } else {
      degree = 180;
    }
    set_repeating_fire(60, 3, 8);
    v_ = degree_to_vector(speed);
  }
  void run(Registry reg) {
    move(v_);
    die_if_out_of_window();
  }
  void collide_with_wall(Registry reg, Wall wall) {
    if (!dead()) {
      Wave.enemy_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().w));
      die();
    }
  }
}

void f01(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(OptionCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f02(double x, double y, Registry reg)
{
  Enemy enemy = new class(Point(x,y)) CapsuleCarrier {
    this(Point p) {
      super(p);
      set_capsule_drop!(ShortRangeCapsule)();
    }
    
    void run(Registry reg) { move(Vector(0, 2)); die_if_out_of_window();}
  };
  reg.add(enemy);
}

void f11(double x, double y, Registry reg)
{
  Shutter.add(Point(x,y), 420, 0, reg);
}

void f12(double x, double y, Registry reg)
{
  Shutter.add(Point(x,y), 140, 180, reg);
}


void f21(double x, double y, Registry reg)
{
  reg.add(new RushFlyer(Point(x,y), reg));
}

void f41(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x,y), 90, 100, 200, 4));
}

void f42(double x, double y, Registry reg)
{
  reg.add(new FisFlak(Point(x,y), 270, 100, 200, 4));
}

class Boss7_1: BaseEnemy {
  static const int INVINCIBLE = 300;
  static const int FIRST_FIRE = 600;
  static const int SECOND_FIRE = 360;
  static const int THIRD_FIRE_START = 700;
  static const int THIRD_FIRE = 940;
  static const int FORTH_FIRE_START = 1000;
  static const int FORTH_FIRE = 1600;
  static const int FIFTH_FIRE_START = 1700;
  static const int FIFTH_FIRE = 2130;
  static const int LOOP = 2160;
  
  static const int LIFE_MINI = 10000000;
  static const int DESTROYED_POINT = 5000;
  
  WallCreator wallcreator;
  this(Point pos, Registry reg) {
    super(SubKind.BOSS_ENEMY, pos + Vector(0, -90),
          new Animation(&images.enemies.circle.draw),
          10000 + LIFE_MINI);
    scale = 10;
    xdegree = 60;
    wallcreator = new WallCreator();
    reg.add(wallcreator);
    reg.add(new Boss7_1_OptionA(Point(-100,190), this, reg));
    reg.add(new Boss7_1_OptionB(Point(100,190), this, reg));
    reg.add(new Boss7_1_OptionC(Point(-220,140), -1, this, reg));
    reg.add(new Boss7_1_OptionC(Point(220,140), 1, this, reg));
    int i;
    for (i = 0; i < 6; i++)
      reg.add(new Boss7Wing(60*i, this, 1));
    for (i = 0; i < 3; i++)
      reg.add(new Boss7Ball(this, reg));
  }

  mixin DefineRect!(pos, 100, 100, true);
  mixin DefineEnemy!(DESTROYED_POINT);


  void to_boss7_2(Registry reg) {
    if (life <= LIFE_MINI)
      reg.root.add_score(DESTROYED_POINT);
    Wave.boss_destroyed.play();
    reg.add(new DestroyedEnemy(pos, true, rect().w));
    reg.add(new Boss7_1_to_2(reg, pos, count));
    die();
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
    //登場部分
    if (count < 250) {
      move(Vector(0,1));
      return;
    }

    //時間切れ
    if (count >= FIRST_FIRE + LOOP*3) {
      to_boss7_2(reg);
      die();
      return;
    }

    //ループ部分
    if (life <= LIFE_MINI) {
      to_boss7_2(reg);
      return;
    }
    int ct = (count - FIRST_FIRE) % LOOP;
    
    if (ct == THIRD_FIRE_START)
      reg.add(new Boss7_1_WallCreator(this));
    else if (ct < FIFTH_FIRE_START) {;}
    else if (ct < FIFTH_FIRE)
      beam(reg, ct);

    repeat(600, 2400, {
    });
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < INVINCIBLE) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

  //5波　ビーム攻撃
  void beam(Registry reg, int ct) {
    if (ct < FIFTH_FIRE_START + 40)
      move(Vector(0,2));
    else if (ct == FIFTH_FIRE_START +40)
      reg.add(new Boss7BeamCreator(pos, this));
    else if (ct < FIFTH_FIRE-40) {;}
    else if (ct < FIFTH_FIRE)
      move(Vector(0,-2));
  }


  //ボスのオプション関連
  static class Boss7_1_Option: ChildEnemy {
    RNG rng_;
    this(Point pos, Movable boss, Registry reg) {
      super(SubKind.BOSS_ENEMY, pos,
            new Animation(&images.enemies.boss7_1_option.draw),
            Color.enemy, 180, 0, boss); 
      rng_ = reg.root.rng;
    }
    mixin DefineRect!(pos, 80, 80, true);
    mixin DefineEnemy!(0);
    RNG rng(){return rng_;}
    void destroyed(Registry reg) {
      Wave.enemy_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().w));
    }
    
    void run(Registry reg) {
      if (parent.dead)
        destroyed(reg);
    }
    
    //1波　左側の攻撃
    void carriershot(Registry reg) {
      int ct = (count - INVINCIBLE);
      if (ct < 30) {
        move(degree_to_vector(2, false, -120));
        degree += 1;
      } else if (ct < 60) {;}
      else if (ct < 80) {
        move(degree_to_vector(4));
      } else if (ct < 130) {
        degree -= 1;
        move(degree_to_vector(3, false, degree + 180));
      } else if (ct < 150) {
        move(degree_to_vector(4, false, -160));
        degree += 1;
      }
      if (ct < 130 && ct > 80 && count % 3 == 0) {
        reg.add(new Boss7CapsuleCarrier(reg, pos + degree_to_vector(30), rng.irand(3, 5),
                                        degree - 90 + rng.irand(-20, 20), rng.irand(2, 8)));
      }
      if (ct < 130 && ct > 80 && count % 3 == 1) {
        reg.add(new Boss7CapsuleCarrier(reg, pos + degree_to_vector(30), rng.irand(3, 5),
                                        degree - 90 + rng.irand(-20, 20), rng.irand(2, 8), false));
      }
    }
    //1波　右側の攻撃
    void bomber1(Registry reg) {
      int ct = (count - INVINCIBLE);
      if (ct < 120)
        degree += 0.3;
      else if (ct < 300 && ct >= 180)
        degree -= 0.3;
      if (count % 50 == 0)
        reg.add(new Boss7Bomb(pos, 30, degree, 8));
    }
    //2波
    void random_fire(Registry reg, int ct, int fire_time) {
      int i;
      if (ct <= 300)
        degree = homing(degree, 1, reg);
      else if (ct > 300)
        degree = direction_to_target(degree, 180, 3);
      if (ct < 60)
        return;
      if (ct % 60 == fire_time && ct < 300)
        zone_fire(reg, pos + degree_to_vector(20), degree - 90, 5, 5, 3, 20);
    }
    //3波　壁が出る攻撃
    void wall_fire(Registry reg, int ct, int sign) {
      static const WALL_FIRE_LOOP = 120;
      ct = (ct + 30*(1-sign) - THIRD_FIRE_START) % WALL_FIRE_LOOP;
      if (ct < 10)
        move(Vector(5*sign, 1));
      else if (ct < 20) {;}
      else if (ct < 40) {
        move(Vector(-7*sign, 1));
        zone_fire(reg, pos + degree_to_vector(20), degree - 90, 10, 7, 6);
      } else if (ct < 60) {;}
      else if (ct < 90)
        move(Vector(3*sign, -1));
    }
    //4波　ボム2
    void bomber2(Registry reg, int ct, int sign) {
      degree = direction_to_target(degree, 180 + 30*sign, 1);
      if (sign == 1) {
        if (ct == FORTH_FIRE_START + 60 || ct == FORTH_FIRE_START + 510)
          reg.add(new Boss7Bomb(pos, 60, degree, 6));
      } else {
        if (ct == FORTH_FIRE_START + 300)
          reg.add(new Boss7Bomb(pos, 60, degree, 6));
      }
    }
    //5波　避ける
    void keep_away(int ct, int sign) {
      if (ct < FIFTH_FIRE_START + 30)
        move(Vector(4*sign, 2));
      else if (ct < FIFTH_FIRE - 30) {;}
      else if (ct < FIFTH_FIRE)
        move(Vector(-4*sign, -2));
    }

    //小さいオプションの攻撃
    //2波　歯車を吐き出す
    void gear(Registry reg, int ct, int sign) {
      if (ct % 120 == 60 + 45 * sign)
        reg.add(new Boss7Gear(pos, sign));
    }
    //3波　高速自機狙い連弾
    void rapid_fire(Registry reg, int ct) {
      if (ct % 30 < 20)
        degree = homing(degree, 3, reg);
      else
        reg.add(new StripBullet(pos, degree_to_vector(8)));
    }
    //4波　回転2WAY弾
    void roll_fire(Registry reg, int ct) {
      degree +=  3.0 / (FORTH_FIRE - FORTH_FIRE_START) * (ct - FORTH_FIRE_START);
      if (ct % 2 == 0) {
        reg.add(new NormalBullet(pos, degree_to_vector(6, false, degree - 90 + 20)));
        reg.add(new NormalBullet(pos, degree_to_vector(6, false, degree - 90 - 20)));
      }
    }
    //5波　近づく
    void move_to_boss(Registry reg, int ct, int sign) {
      if (ct < FIFTH_FIRE_START + 50)
        move(Vector(-2*sign, 0));
      else if (ct < FIFTH_FIRE - 50)
        {if (ct % 3 == 0) reg.add(new StripBullet(pos, degree_to_vector(8)));}
      else if (ct < FIFTH_FIRE)
        move(Vector(2*sign, 0));
    }
    
    //ゾーン狙い弾
    void zone_fire(Registry reg, Point pos, double deg, double zone, double min_speed, double rand_speed, int bullet_count = 1) {
      for (; bullet_count != 0; bullet_count--) {
        Vector v = (degree_to_vector(min_speed + reg.root.rng.irand(0, cast(int)(rand_speed * 100)) / 100.0,
                                     false, deg + reg.root.rng.irand(cast(int)(-zone*100), cast(int)(zone*100)) / 100.0));
        reg.add(new NormalBullet(pos, v));
      }
    }
    //向きを変える
    double direction_to_target(double pre_deg, double target_deg, double change_deg) {
      int deg_loop = cast(int)floor(pre_deg / 360);
      if (abs(pre_deg - target_deg - 360 * deg_loop) < change_deg+1)
        return target_deg;
      if (pre_deg - target_deg - 360 * deg_loop > 180)
        pre_deg += change_deg;
      else if (pre_deg - target_deg - 360 * deg_loop < -180)
        pre_deg -= change_deg;
      else if (pre_deg - target_deg - 360 * deg_loop > 0)
        pre_deg -= change_deg;
      else if (pre_deg - target_deg - 360 * deg_loop < 0)
        pre_deg += change_deg;
    return pre_deg;
    }
  };
  
  static class Boss7_1_OptionA: Boss7_1_Option {
    this(Point pos, Movable boss, Registry reg) {
      super(pos, boss, reg);
      scale = 4;
    }
    mixin DefineRect!(pos, 80, 80, true);
    void run(Registry reg) {
      super.run(reg);
      if (count < INVINCIBLE)
        return;
      if (count < FIRST_FIRE) {
        carriershot(reg);
        return;
      }
      int ct = (count - FIRST_FIRE) % LOOP;
      if (ct < SECOND_FIRE)
        random_fire(reg, ct, 0);
      else if (ct < THIRD_FIRE_START+60) {;}
      else if (ct < THIRD_FIRE+60)
        wall_fire(reg, ct, -1);
      else if (ct < FORTH_FIRE_START) {;}
      else if (ct < FORTH_FIRE)
        bomber2(reg, ct, -1);
      else if (ct < FIFTH_FIRE_START) {;}
      else if (ct < FIFTH_FIRE)
        keep_away(ct, -1);
    }
  };

  static class Boss7_1_OptionB: Boss7_1_Option {
    this(Point pos, Movable boss, Registry reg) {
      super(pos, boss, reg);
      scale = 4;
    }
    mixin DefineRect!(pos, 80, 80, true);
    void run(Registry reg) {
      super.run(reg);
      if (count < INVINCIBLE)
        return;
      if (count < FIRST_FIRE) {
        bomber1(reg);
        return;
      }
      int ct = (count - FIRST_FIRE) % LOOP;
      if (ct < SECOND_FIRE)
        random_fire(reg, ct, 30);
      else if (ct < THIRD_FIRE_START) {;}
      else if (ct < THIRD_FIRE)
        wall_fire(reg, ct, 1);
      else if (ct < FORTH_FIRE_START) {;}
      else if (ct < FORTH_FIRE)
        bomber2(reg, ct, 1);
      else if (ct < FIFTH_FIRE_START) {;}
      else if (ct < FIFTH_FIRE)
        keep_away(ct, 1);
    }
  };

  static class Boss7_1_OptionC: Boss7_1_Option {
    int sign_;
    this(Point pos, int sign, Movable boss, Registry reg) {
      super(pos, boss, reg);
      scale = 2;
      sign_ = sign;
    }
    mixin DefineRect!(pos, 40, 40, true);
    void run(Registry reg) {
      super.run(reg);
      if (count < FIRST_FIRE)
        return;
      int ct = (count - FIRST_FIRE) % LOOP;
      if (ct < SECOND_FIRE)
        gear(reg, ct, sign_);
      else if (ct < SECOND_FIRE + 150) {;}
      else if (ct < THIRD_FIRE)
        rapid_fire(reg, ct);
      else if (ct < FORTH_FIRE_START)
        degree = direction_to_target(degree, 90 * (1+sign_), 3);
      else if (ct < FORTH_FIRE)
        roll_fire(reg, ct);
      else if (ct < FIFTH_FIRE_START)
        degree = direction_to_target(degree, 180, 3);
      else if (ct < FIFTH_FIRE)
        move_to_boss(reg, ct, sign_);
    }
  };

  static class Boss7CapsuleCarrier: CapsuleCarrier {
    double speed_;
    double deg_;
    double omega_;
    this(Registry reg, Point p, double speed, double deg, double omega, bool option = true) {
      super(p);
      if (option)
        set_capsule_drop!(OptionCapsule)();
      else {
        int r;
        Capsule capsule;
        r = reg.root.rng.irand(1, 3);
        switch(r) {
          case 1:
            set_capsule_drop!(ShortRangeCapsule)();
            break;
          case 2:
            set_capsule_drop!(FiveWayCapsule)();
            break;
          case 3:
            set_capsule_drop!(BladeCapsule)();
            break;
          default:
            assert(false);
            break;
        }
      }
      speed_ = speed;
      deg_ = deg;
      omega_ = omega;
    }
    void run(Registry reg) {
      degree += omega_;
      move(degree_to_vector(speed_, false, deg_));
      die_if_out_of_window();
    }
    void collide_with_wall(Registry reg, Wall wall) {
      Wave.enemy_destroyed.play();
      reg.add(new DestroyedEnemy(pos, true, rect().w));
      die();
    }
  };

  static class Boss7_1_WallCreator: BaseWorker {
  private:
    int offset_;
    BaseEnemy boss_;
  public:
    this(BaseEnemy boss){
      super(SubKind.WALL_CREATOR);
      offset_ = 150;
      boss_ = boss;
    }
    void run(Registry reg) {
      if (count % 120 == 0)
        reg.add(new Boss7_1_WALL(offset_ + count, 1));
      if (count % 120 == 60)
        reg.add(new Boss7_1_WALL(offset_ + count, -1));
      if (count == 180 || boss_.dead)
        die();
    }
    static class Boss7_1_WALL: MovableWall {
      static const int WIDTH = 240;
      double dx_;
      double dy_ = 1;
      int sign_;
      this(double y, int sign) {
        super(Point(280 - WIDTH/2 + 430 * sign, y), WIDTH, 20);
        sign_ = sign;
        dx_ = -9 * sign;
      }
      void run(Registry reg) {
        if (count < 30)
          dx_ += -0.1 * sign_;
        else {
          dx_ = 0;
          dy_ += 0.3;
        }
        move(Vector(dx_, dy_));
        if(!rect.is_intersected(LARGE_WINDOW) && count > 30)
          die();
      }
    }
  };
}

class Boss7_1_to_2: BaseEnemy {
private:
  static const int LIFE_MINI = 10000000;
  int all_count_;
  double dy_ = 0;
public:
  this(Registry reg, Point pos, int ct) {
    super(SubKind.BOSS_ENEMY, pos,
          new Animation(&images.enemies.circle.draw),
          LIFE_MINI);
    scale = 10;
    xdegree = 60;
    all_count_ = ct;
    int i;
    for (i = 0; i < 6; i++)
      reg.add(new Boss7Wing(60*i + ct * 3, this, 1));
    for (i = 0; i < 3; i++)
      reg.add(new Boss7Ball(this, reg));
  }
  mixin DefineRect!(pos, 100, 100, true);
  mixin DefineEnemy!(0);

  void run(Registry reg) {
    all_count_++;
    dy_ += 0.05;
    move(Vector(0.5, dy_));
    if (count > 200) {
      reg.add(new Boss7_2(reg, all_count_));
      die();
    }
  }
}

class Boss7_2: BaseEnemy {
private:
  static const int INVINCIBLE = 250;
  static const int LOOP = 600;
  static const int LIFE_MINI = 10000000;
  static const int DESTROYED_POINT = 10000;
  int all_count_;
  Boss7Ball[3] ball_;
public:
  this(Registry reg, int ct) {
    super(SubKind.BOSS_ENEMY, Point(280, 700),
          new Animation(&images.enemies.circle.draw),
          3500 + LIFE_MINI);
    scale = 10;
    xdegree = 60;
    all_count_ = ct;
    int i;
    for (i = 0; i < 6; i++)
      reg.add(new Boss7Wing(60*i, this, -1));
    for (i = 0; i < 3; i++) {
      ball_[i] = new Boss7Ball(this, reg);
      reg.add(ball_[i]);
    }
  }
  mixin DefineRect!(pos, 100, 100, true);
  mixin DefineEnemy!(DESTROYED_POINT);

  void to_boss7_3(Registry reg) {
    if (life <= LIFE_MINI)
      reg.root.add_score(DESTROYED_POINT);
    Wave.boss_destroyed.play();
    reg.add(new DestroyedEnemy(pos, true, rect().w));
    reg.add(new Boss7_3(reg, pos, all_count_, ball_[]));
    die();
  }

  void run(Registry reg) {
    all_count_++;
    //登場部分
    if (count < INVINCIBLE) {
      move(Vector(0, -1));
      return;
    }

    //時間切れ
    if (count >= INVINCIBLE + LOOP*10) {
      to_boss7_3(reg);
      return;
    }
    
    //ループ部分
    if (life <= LIFE_MINI) {
      to_boss7_3(reg);
      return;
    }
    int loop_count = cast(int)floor(count / LOOP);
    int ct = (count - INVINCIBLE) % LOOP;
    
    if (ct == 80 || ct == 90 || ct == 100)
      add_missile(reg, ct);
    if (ct == 0)
      reg.add(new Boss7HomingBullet(pos, 2.1 + 0.1 * loop_count, 1.3 + 0.1 * loop_count, 1, this));
    

    repeat(600, 2400, {
    });
  }
  
  void add_missile(Registry reg, int ct) {
    reg.add(new Boss7Missile(pos + Vector(30, 10), 135));
    reg.add(new Boss7Missile(pos + Vector(-30, 10), 225));
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < INVINCIBLE) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

}

class Boss7_3: BaseEnemy {
private:
  static const int FIRST_FIRE_START = 700;
  static const int LOOP  = 300;
  static const int LIFE_MINI = 10000000;
  static const int DESTROYED_POINT = 15000;
  Boss7Ball[3] ball_;
  int all_count_;
public:
  this(Registry reg, Point pos, int all_count, Boss7Ball ball[]) {
    super(SubKind.BOSS_ENEMY, pos,
          Animation.empty(),
          1200 + LIFE_MINI);
    int i;
    for (i = 0; i < 3; i++) {
      ball_[i] = new Boss7Ball_3(this, reg, ball[i].pos_deg_a, ball[i].pos_deg_b, ball[i].direction_a, ball[i].direction_b);
      reg.add(ball_[i]);
    }
    all_count_ = all_count;
    WallCreator wall = cast(WallCreator)reg.search(SubKind.WALL_CREATOR);
    wall.boss7_3flag_on();
  }

  mixin DefineRect!(pos, 1, 1, true);
  mixin DefineEnemy!(0);



  void to_boss7explosion(Registry reg) {
    WallCreator wall = cast(WallCreator)reg.search(SubKind.WALL_CREATOR);
    wall.die();
    if (life > LIFE_MINI) {
      reg.add(new Reserve(SubKind.GAMEEND, 60, &reg.root.show_ending));
      die();
      return;
    }
    reg.add(new Boss7Explosion(reg, pos, DESTROYED_POINT, ball_[]));
    die();
  }
  
  void repeat(int offset, int period, void delegate() f) {
    if((count - offset)%period==0) 
      f();
  }
  
  void run(Registry reg) {
    all_count_++;
    //登場部分
    if (count < 350) {
      move(Vector(0, -1));
    }
    
    //時間切れ
    if (count >= 800 + LOOP*50) {
      move(Vector(0, -2));
      if (count >= 800 + 300 + LOOP*50) {
        to_boss7explosion(reg);
        return;
      }
    }

    if (life <= LIFE_MINI) {
      to_boss7explosion(reg);
      return;
    }
    //ループ部分
    int ct = (count - 900) % LOOP;
    
    repeat(600, 2400, {
    });
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 700) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

  static class Boss7Ball_3: Boss7Ball {
  private:
    void nop(Registry, int) {;};
    void delegate(Registry, int) fire_;
  public:
    this(Enemy boss, Registry reg, double pos_deg_a, double pos_deg_b, double direction_a, double direction_b) {
      super(boss, reg, 3);
      pos_deg_a_ = pos_deg_a;
      pos_deg_b_ = pos_deg_b;
      direction_a_ = direction_a;
      direction_b_ = direction_b;
      scale = normal_scale_ + sin(pos_deg_b_);
      fire_ = &nop;
    }
  
    void run(Registry reg) {
      super.run(reg);
      if (count == 400)
        nway_fire(reg, pos, 3, 120, 1);
      if (count < FIRST_FIRE_START)
        return;
      int ct = (count - FIRST_FIRE_START) % LOOP;
      if (ct == 0)
        set_fire(reg);
      fire_(reg, ct);
    }
  
    void set_fire(Registry reg) {
      int r = reg.root.rng.irand(1, 3);
      switch(r) {
        case 1:
          fire_ = &five_way_fire;
          break;
        case 2:
          fire_ = &fan_like_fire;
          break;
        case 3:
          fire_ = &all_round_fire;
          break;
        default:
          assert(false);
          break;
      }
    }
  
    void five_way_fire(Registry reg, int ct) {
      if (ct % 10 == 0) {
        nway_fire(reg, pos, 45, 5, 5, 8);
      }
    }
  
    void fan_like_fire(Registry reg, int ct) {
      if (ct % 5 == 0) { ;
        nway_fire(reg, pos, 2, 10, 10 + reg.root.rng.irand(-300, 300)/100.0);
      }
    }
  
    void all_round_fire(Registry reg, int ct) {
      if (ct % 3 == 0) { ;
        nway_fire(reg, pos, 36, 10, 10 + reg.root.rng.irand(-300, 300)/100.0);
      }
    }
  
    //nway弾
    void nway_fire(Registry reg, Point pos, double change_degree, int nway, double speed = 4, int loop = 1) {
      double deg = reg.root.rng.irand(1, 360);
      int i;
      int j;
      for (i = 0; i < nway; i++) {
        for (j = 0; j < loop; j++) {
          reg.add(new NormalBullet(pos, degree_to_vector(speed + j, false, deg +  (i - (nway-1)/2.0 ) * change_degree)));
        }
      }
    }
  };

}
class Boss7Explosion: BaseEnemy {
private:
  int LIFE_MINI = 10000000;
  int destroyed_point_;
  Boss7Ball[3] ball_;
public:
  this(Registry reg, Point pos, int destroyed_point, Boss7Ball ball[]) {
    super(SubKind.BOSS7_EXPLOSION, pos, Animation.empty(), LIFE_MINI);
    destroyed_point_ = destroyed_point;
    int i;
    for (i = 0; i < 3; i++) {
      ball_[i] = new Boss7BallExplosion(this, reg, ball[i].pos_deg_a, ball[i].pos_deg_b, ball[i].direction_a, ball[i].direction_b);
      reg.add(ball_[i]);
    }
  }
  mixin DefineRect!(pos, 0, 0, false);
  mixin DefineEnemy!(0);

  void run(Registry reg) {
    if (count == 1000) {
      reg.root.add_score(destroyed_point_);
      reg.add(new Reserve(SubKind.GAMEEND, 60, &reg.root.show_ending));
      die();
    }
  }

  void collide_with_shot(Registry reg, Shot shot) {;}


  static class Boss7BallExplosion: Boss7Ball {
  public:
    this(Animatable boss, Registry reg, double pos_deg_a, double pos_deg_b, double direction_a, double direction_b) {
      super(cast(Enemy)boss, reg, 3);
      pos_deg_a_ = pos_deg_a;
      pos_deg_b_ = pos_deg_b;
      direction_a_ = direction_a;
      direction_b_ = direction_b;
      scale = normal_scale_ + sin(pos_deg_b_);
    }

    void run(Registry reg) {
      super.run(reg);
      move(Vector(reg.root.rng.irand(-count/200 - 2, count/200 + 2), reg.root.rng.irand(-count/200 - 2, count/200 + 2)));
      if (count < 300 && count % 5 == 0) {
          reg.add(new Boss7BallChargeEffect(reg, this, -1));
      }
      if (count == 500) {
        int i;
        for (i = 0; i < 40; i++)
          reg.add(new Boss7BallChargeEffect(reg, this, 1));
        die();
      }
    }
    void collide_with_shot(Registry reg, Shot shot) {;}

    static class Boss7BallChargeEffect: Animatable {
    private:
      Vector v_;
      int ct_;
      int sign_;
      static void function()[] image = [ &images.enemies.circle.draw ];
    public:
      this(Registry reg, BaseMovable boss, int sign) {
        double deg = drand(0, 360) * PI / 180;
        v_ = drand(1, 8) * Vector(cos(deg), sin(deg));
        sign_ = sign;
        Point p;
        if (sign == -1) {
          ct_ = cast(int)drand(20, 60);
          p = ct_ * v_;
        } else {
          p = Point.zero;
          ct_ = cast(int)drand(100, 300);
        }
        super(SubKind.NORMAL_EFFECT, p,
              new Animation(image),
              Color.bullet, 0, 1, boss);
        scale = drand(3, 10) / 100.0;
      }

      void run(Registry reg) {
        move(sign_*v_);
        if (sign_ == 1)
          v_ += Vector (0, 0.02);
        if (count == ct_)
          die();
      }
    };
  };
}


class Boss7Wing: ChildEnemy {
private:
  double radius_a_ = 100;
  double radius_b_ = 60;
  int sign_;
  double omega_;
  double normal_scale_;
public:
  this(double pos_deg, Enemy boss, int sign, double omega = 3) {
    super(SubKind.BOSS_ENEMY, Vector(radius_a_ * cos(ydegree), radius_b_ * sin(ydegree)),
          new Animation(&images.enemies.boss7part.draw),
          Color.enemy, 90 * (1 - sign), 4, boss);
    ydegree = pos_deg;
    omega_ = omega * sign;
    sign_ = sign;
    ymirror = sign;
    normal_scale_ = 4;
  }

  mixin DefineRect!(pos, 40, 60, true);
  mixin DefineEnemy!(0);

  void run(Registry reg){
    ydegree += omega_;
    scale = normal_scale_ + sign_ * sin(ydegree*PI/180);
    pos = Vector(0, -5 + 55 * sign_) + Vector(radius_a_ * cos(ydegree*PI/180), sign_ * radius_b_ * sin(ydegree*PI/180));
  }
}

class Boss7Ball: ChildEnemy {
private:
  double radius_ = 40;
  double pos_deg_a_;
  double pos_deg_b_;
  double omega_;
  double direction_a_;
  double direction_b_;
  double normal_scale_ = 0.5;
public:
  this(Enemy boss, Registry reg, double omega = 3) {
    pos_deg_a_ = reg.root.rng.irand(0, 359);
    pos_deg_b_ = reg.root.rng.irand(0, 359);
    super(SubKind.BOSS_ENEMY, Vector(0, -20) + radius_*cos(pos_deg_b_*PI/180)*degree_to_vector(1, false, pos_deg_a_),
          new Animation(&images.enemies.circle.draw),
          Color.enemy, 0, normal_scale_ * (1.5 + sin(pos_deg_b_*PI/180)), boss);
    omega_ = omega;
    scale = normal_scale_ + sin(pos_deg_b_);
    direction_a_ = reg.root.rng.irand(1, 99) / 100.0;
    direction_b_ = (reg.root.rng.irand(0, 1) == 0) ? 1 : -1;
    direction_b_ *= 1 - direction_a_*direction_a_;
  }

  mixin DefineRect!(pos, 20, 20, true);
  mixin DefineEnemy!(0);
  
  double pos_deg_a() { return pos_deg_a_; }
  double pos_deg_b() { return pos_deg_b_; }
  double direction_a() { return direction_a_; }
  double direction_b() { return direction_b_; }
  
  void run(Registry reg){
    direction_a_ += reg.root.rng.irand(-3, 3) / 100.0;
    if (abs(direction_a_) > 1)
      direction_a_ = 99 / 100.0 * sign(direction_a_);
    direction_b_ = (1 - direction_a_*direction_a_) * sign(direction_b_);
    if (abs(direction_b_) < 3 / 100.0)
      direction_b_ = (reg.root.rng.irand(0, 1) == 0) ? 1 : -1;
    pos_deg_a_ += omega_ * direction_a_;
    pos_deg_b_ += omega_ * direction_b_;
    pos = Vector(0, -20) + radius_*cos(pos_deg_b_*PI/180)*degree_to_vector(1, false, pos_deg_a_);
    scale = normal_scale_ * (1.5 + sin(pos_deg_b_*PI/180));
  }

  void collide_with_shot(Registry reg, Shot shot) {
    if (count < 700) 
      return;
    else
      super.collide_with_shot(reg, shot);
  }

}

class Boss7Bomb: BaseEnemy {
public:
  int bomb_timer_;
  double speed_;
  this(Point pos, int time, double deg, double speed) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.bomb.draw),
          300, Color.enemy, deg, 2);
    speed_ = speed;
    bomb_timer_ = time;
  }

  mixin DefineEnemy!(0);
  mixin DefineRect!(pos, 40, 40, true);

  void run(Registry reg) {
    move_forward(speed_);
    if (count >= bomb_timer_)
      explosion(reg);
    die_if_out_of_window();
  }
  void collide_with_wall(Registry reg, Wall wall) {
    explosion(reg);
  }
  void explosion(Registry reg) {
    Wave.enemy_destroyed.play();
    reg.add(new DestroyedEnemy(pos, true, 100));
    reg.add(new Boss7BombExplosion(pos, reg));
    die();
  }
}

class Boss7BombExplosion: LBullet {
private:
  static void function()[] image = [ &images.enemies.circle.draw ];
public:
  this(Point pos, Registry reg) {
    super(pos, Vector.zero, new Animation(image), true);
    scale = 0;
    int i;
    for (i = 0; i < 16; i++) {
      reg.add(new Boss7BombExplosionHitArea(360/16.0*i, this));
    }
  }
  mixin DefineRect!(pos, 0, 0, true);
  void run(Registry reg) {
    scale += 0.7;
    if (count == 20)
      die();
    die_if_out_of_window();
  }
  
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
  void collide_with_wall(Registry, Wall) {
  }
}
class Boss7BombExplosionHitArea: BaseMovable, Bullet {
private:
  double pos_deg_;
public:
  this(double deg, Movable bullet) {
    super(SubKind.HIDDEN_BULLET, Vector.zero, bullet);
    pos_deg_ = deg;
  }
  mixin DefineRect!(pos, 30, 30, true);
  void run(Registry reg) {
    pos = (cast(Animatable)parent).scale * 7.5 * Vector(cos(pos_deg_/180*PI), sin(pos_deg_/180*PI));
    if (parent.dead)
      die();
  }
  void collide_with_wall(Registry, Wall) {
  }
  void collide_with_charge_blocker(Registry, ChargeBlocker) {
  }
}

class Boss7BeamCreator: BaseMovable {
private:
  BaseEnemy boss_;
public:
  this(Point pos, BaseEnemy boss) {
    super(SubKind.NORMAL_BULLET_CREATOR, Vector(0,40), boss);
    boss_ = boss;
  }

  void run(Registry reg) {
    if (count == 0)
      Wave.beam_charge.play();
    if (count < 70) {
      reg.add(new Boss7BeamChargeEffect(reg, this));
      reg.add(new Boss7BeamChargeEffect(reg, this));
    }
    if (count == 100) {
      Wave.beam.play();
      reg.add(new Boss7Beam(1, this));
      reg.add(new Boss7Beam(-1, this));
    }
    if (count == 400 || parent.dead)
      die();
  }
  static class Boss7BeamChargeEffect: Animatable {
  private:
    Vector v_;
    int ct_;
    static void function()[] image = [ &images.enemies.circle.draw ];
  public:
    this(Registry reg, BaseMovable bullet) {
      double deg = drand(30, 150) * PI / 180;
      v_ = drand(1, 8) * Vector(cos(deg), sin(deg));
      ct_ = cast(int)drand(5, 20);
      super(SubKind.NORMAL_EFFECT, ct_ * v_ ,
            new Animation(image),
            Color.bullet, 0, 1, bullet);
      scale = drand(3, 10) / 20.0;
    }
    mixin DefineRect!(pos, 30, 30, true);
    void run(Registry reg) {
      move(-1*v_);
      if (parent.dead || count == ct_)
        die();
    }
  };
  static class Boss7Beam: BaseMovable, Bullet {
  private:
    int sign_;
  public:
    this(int sign, BaseMovable creator) {
      super(SubKind.NORMAL_BULLET, Vector(0, 14), creator);
      sign_ = sign;
    }
    mixin DefineRect!(pos, 10, 2000, true);
    void run(Registry reg) {
      if (count < 90)
        move(Vector(1*sign_, -0.3));
      if (count == 5|| count == 20 || count == 50 || count == 80) {
        move(Vector(0, 1));
      }
      if (count >= 200 && count < 290)
        move(Vector(-1*sign_, 0.3));
      if (parent.dead || count == 300)
        die();
      int i;
      if (count < 180 && count % 3 == 0) {
        Point p = Point(reg.root.rng.irand(0, cast(int)abs(pos.x - parent.pos.x)) * sign_, reg.root.rng.irand(10, 50));
        reg.add(new Boss7BeamEffect(reg, p, parent));
      }
      if (count >= 180 && count < 260 && count % 3 == 0) {
        Point p = Point(reg.root.rng.irand(0, cast(int)abs(pos.x - parent.pos.x)) * sign_, reg.root.rng.irand(10, 50));
        reg.add(new Boss7BeamEffect(reg, p, parent));
      }
    }
    void draw(Registry reg) {
      glColor3da(Color.bullet);
      glPushMatrix();
      glTranslated(pos.x, pos.y, 0);
      glPushMatrix();
      glBegin(GL_LINES);
      glVertex2d(0,0);
      glVertex2d(0, 600);
      glEnd();
      glPopMatrix();
      glPopMatrix();
      super.draw(reg);
    }
    void collide_with_wall(Registry, Wall) {
    }
    void collide_with_charge_blocker(Registry, ChargeBlocker) {
    }
  };
  static class Boss7BeamEffect: BaseMovable {
  private:
    int length_;
    double dy_;
  public:
    this(Registry reg, Point pos, Movable creator) {
      super(SubKind.NORMAL_EFFECT, pos, creator);
      length_ = cast(int)drand(50, 200);
      dy_ = drand(10, 20);
    }
    void run(Registry reg) {
      move(Vector(0, dy_));
      if (parent.dead || count == 200 ||
          (abs(pos.x - parent.pos.x) + (cast(BaseWorker)parent).count - 390 > 0 && (cast(BaseWorker)parent).count >= 300))
        die();
    }
    void draw(Registry reg) {
      glColor3da(Color.bullet);
      glPushMatrix();
      glTranslated(pos.x, pos.y, 0);
      glPushMatrix();
      glBegin(GL_LINES);
      glVertex2d(0,0);
      glVertex2d(0, length_);
      glEnd();
      glPopMatrix();
      glPopMatrix();
      super.draw(reg);
    }
    void collide_with_wall(Registry, Wall) {
    }
    void collide_with_charge_blocker(Registry, ChargeBlocker) {
    }
  };
}

class Boss7HomingBullet: BaseEnemy {
private:
  BaseEnemy boss_;
  double add_max_speed_;
  double speed_axis_;
  double change_deg_;
public:
  this(Point pos, double add_max_speed, double speed_axis, double change_deg, BaseEnemy boss) {
    super(SubKind.NORMAL_ENEMY, pos,
          Animation.empty(),
          3000, Color.enemy, 0, 2);
    boss_ = boss;
    add_max_speed_ = add_max_speed;
    speed_axis_ = speed_axis;
    change_deg_ = change_deg;
  }
  mixin DefineRect!(pos, 30, 30, true);
  mixin DefineEnemy!(0);

  void run(Registry reg) {
    double speed = speed_axis_ + add_max_speed_ * sin(2*count*PI/180);
    if (count < 700 && !boss_.dead)
      degree = homing(degree, change_deg_, reg);
    else
      die_if_out_of_window();
    move_forward(speed);
    if (count >= 30 && count % 10 == 0)
      reg.add(new Boss7DelayBullet(pos, degree - 90));
  }
  class Boss7DelayBullet: SharpBullet {
  private:
    double speed_ = 0;
  public:
    this(Point pos, double deg) {
      super(pos, Vector.zero);
      degree = deg;
    }

    void run(Registry reg) {
      if (count > 500)
        speed_ += 0.05;
      move(degree_to_vector(speed_, false, degree));
      super.run(reg);
    }
  }
  
  void draw(Registry reg) {
    super.run(reg);
    draw_zigzag_circle(reg, pos, 20+10*sin(count*3 * PI/180), 2, 36);
    draw_zigzag_circle(reg, pos, 20+10*cos(count*3 * PI/180), 2, 36);
  }
  
  void draw_zigzag_circle(Registry reg, Point center, double radius, double offset, int n)
  {
    Point start;
    Point p, q;
    double deg;
    double change_deg = 360.0 / n;
    start = p = center + degree_to_vector(radius, false, 0) + degree_to_vector(offset, false, drand(0,359));
    int i;
    glBegin(GL_LINE_STRIP);
    for (i=1; i <= n; i++) {
      q = center + degree_to_vector(radius, false, i * change_deg) + degree_to_vector(offset, false, drand(0,359));
      (p, q).glVertex();
      p = q;
    }
    (q, start).glVertex();
    glEnd();
  }
}

class Boss7Missile: BaseEnemy {
private:
  double speed_ = 2;
public:
  this(Point pos, double deg) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.missile.draw),
          10, Color.bullet, deg, 1.3);
  }
  mixin DefineRect!(pos, 8, 8, true);
  mixin DefineEnemy!(0);

  void run(Registry reg) {
    if (count > 100)
      speed_ += 0.3;
    if (count < 105)
      degree = homing(degree, 2, reg);
    move_forward(speed_);
  }
}

class Boss7Gear: BaseEnemy {
private:
  double speed_;
  double move_deg_;
  double omega_;
  int sign_;
public:
  this(Point pos, int sign, double omega = 3) {
    super(SubKind.NORMAL_ENEMY, pos,
          new Animation(&images.enemies.gear.draw),
          80, Color.enemy, 0, 2);
    speed_ = 4;
    omega_ = omega;
    move_deg_ = 90;
    sign_ = sign;
  }

  mixin DefineRect!(pos, 30, 30, true);
  mixin DefineEnemy!(0);

  void run(Registry reg) {
    degree += omega_ * sign_;
      int i;
    if (count > 75 && count <= 75 + 25)
      move_deg_ += sign_*3;
    else if (count > 90 && count <= 90 + 20)
      speed_ -= 0.1;
    else if (count > 130 && count % 10 == 0) {
      for (i = 0; i < 10; i++) {
        reg.add(new NormalBullet(pos + degree_to_vector(10, false, count*sign_ + 36*i), degree_to_vector(5, false, count*sign_ + 36*i)));
      }
    }
    move_forward(speed_, false, move_deg_);
  }
}

class WallCreator: BaseWorker {
private:
  bool boss7_3flag_ = false;
  int side_wall_ = 0;
public:
  this() {
    super(SubKind.WALL_CREATOR);
  }
  void boss7_3flag_on() {boss7_3flag_ = true;}
  void boss7_die() {die();}
  void run(Registry reg) {
    if (count % 540 == 0 && side_wall_ == 0) {
      if (!boss7_3flag_) {
        reg.add(new Boss7Wall(Point(0, -100), 20, 500));
        reg.add(new Boss7Wall(Point(540, -100), 20, 500));
      } else {
        side_wall_ = 1;
      }
    }
    if (side_wall_ != 0)
      side_wall_++;
    if (side_wall_ >= 330 && side_wall_ % 100 == 0) {
        reg.add(new Boss7_3Wall(Point(-100, 200), 100, 10, 3));
        reg.add(new Boss7_3Wall(Point(640, 300), 100, 10, -2));
    }
  }

  static class Boss7Wall: MovableWall {
    this(Point pos, double width, double height) {
      super(pos, width, height);
    }
    void run(Registry reg) {
      if(!rect.is_intersected(LARGE_WINDOW) && count > 400)
        die();
      move(Vector(0,1));
    }
  };
  static class Boss7_3Wall: MovableWall {
    double speed_;
    this(Point pos, double width, double height, double speed) {
      super(pos, width, height);
      speed_ = speed;
    }
    void run(Registry reg) {
      if(!rect.is_intersected(LARGE_WINDOW) && count > 400)
        die();
      move(Vector(speed_, 0));
    }
  };
  
}

void f61(double x, double y, Registry reg)
{
  Boss7_1 boss = new Boss7_1(Point(x, y-250), reg);
  reg.add(boss);
  Music.boss[7].play();
}

class Events0{
const static Event[] events = [
  {1, &fun3},
  {1, &fun1},
  {80, &fun9},
  {80, &fun8},
  {80, &fun7},
  {100, &fun30},
  {100, &fun31},
  {110, &fun28},
  {110, &fun29},
  {150, &fun45},
  {150, &fun32},
  {180, &fun74},
  {190, &fun33},
  {190, &fun46},
  {230, &fun47},
  {230, &fun34},
  {260, &fun81},
  {270, &fun35},
  {270, &fun48},
  {310, &fun36},
  {310, &fun49},
  {320, &fun82},
  {350, &fun50},
  {350, &fun37},
  {390, &fun38},
  {390, &fun51},
  {400, &fun75},
  {480, &fun96},
  {520, &fun97},
  {560, &fun16},
  {560, &fun17},
  {580, &fun11},
  {580, &fun10},
  {580, &fun95},
  {610, &fun52},
  {610, &fun39},
  {670, &fun40},
  {670, &fun53},
  {680, &fun84},
  {700, &fun93},
  {730, &fun54},
  {730, &fun41},
  {740, &fun76},
  {790, &fun55},
  {790, &fun42},
  {800, &fun94},
  {840, &fun83},
  {850, &fun56},
  {850, &fun43},
  {900, &fun85},
  {910, &fun57},
  {910, &fun44},
  {960, &fun18},
  {960, &fun21},
  {980, &fun15},
  {980, &fun12},
  {1000, &fun103},
  {1000, &fun100},
  {1010, &fun58},
  {1010, &fun64},
  {1040, &fun99},
  {1040, &fun102},
  {1080, &fun101},
  {1080, &fun98},
  {1110, &fun59},
  {1110, &fun65},
  {1120, &fun92},
  {1120, &fun77},
  {1160, &fun104},
  {1160, &fun107},
  {1200, &fun89},
  {1200, &fun88},
  {1210, &fun66},
  {1210, &fun60},
  {1240, &fun106},
  {1240, &fun105},
  {1280, &fun87},
  {1280, &fun109},
  {1280, &fun108},
  {1300, &fun90},
  {1300, &fun91},
  {1310, &fun61},
  {1310, &fun67},
  {1320, &fun86},
  {1360, &fun78},
  {1360, &fun79},
  {1380, &fun111},
  {1380, &fun110},
  {1400, &fun80},
  {1410, &fun68},
  {1410, &fun62},
  {1510, &fun69},
  {1510, &fun63},
  {1520, &fun126},
  {1520, &fun132},
  {1520, &fun120},
  {1540, &fun141},
  {1540, &fun138},
  {1560, &fun0},
  {1560, &fun121},
  {1560, &fun133},
  {1560, &fun127},
  {1560, &fun19},
  {1580, &fun13},
  {1580, &fun115},
  {1580, &fun14},
  {1580, &fun112},
  {1600, &fun134},
  {1600, &fun128},
  {1600, &fun122},
  {1600, &fun113},
  {1600, &fun114},
  {1620, &fun139},
  {1620, &fun142},
  {1620, &fun119},
  {1620, &fun116},
  {1640, &fun135},
  {1640, &fun117},
  {1640, &fun118},
  {1640, &fun123},
  {1640, &fun129},
  {1650, &fun72},
  {1650, &fun70},
  {1680, &fun130},
  {1680, &fun124},
  {1680, &fun136},
  {1700, &fun143},
  {1700, &fun140},
  {1720, &fun137},
  {1720, &fun125},
  {1720, &fun131},
  {1750, &fun73},
  {1750, &fun71},
  {1820, &fun5},
  {1867, &fun4},
  {1980, &fun27},
  {2040, &fun22},
  {2040, &fun20},
  {2080, &fun6},
  {2080, &fun24},
  {2080, &fun23},
  {2120, &fun26},
  {2120, &fun25},
  {3060, &fun2},
];
static void fun3(Registry reg){
  set_scroll(reg, 1);
}
static void fun1(Registry reg){
  reg.root.set_restart_point(0);
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(540, -380, 20, 360)));
}
static void fun8(Registry reg){
  f02(280.000000, -20.000000, reg);
}
static void fun7(Registry reg){
  reg.add(new NormalWall(Rect(0, -380, 20, 360)));
}
static void fun30(Registry reg){
  f01(240.000000, -20.000000, reg);
}
static void fun31(Registry reg){
  f01(320.000000, -20.000000, reg);
}
static void fun28(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun29(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun45(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun32(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun74(Registry reg){
  f21(120.000000, -20.000000, reg);
}
static void fun33(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun46(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun47(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun34(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun81(Registry reg){
  f21(420.000000, -20.000000, reg);
}
static void fun35(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun48(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun36(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun49(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun82(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun50(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun37(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun38(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun51(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun75(Registry reg){
  f21(380.000000, -20.000000, reg);
}
static void fun96(Registry reg){
  f21(120.000000, -20.000000, reg);
}
static void fun97(Registry reg){
  f21(340.000000, -20.000000, reg);
}
static void fun16(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun17(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun11(Registry reg){
  reg.add(new NormalWall(Rect(0, -380, 20, 360)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(540, -380, 20, 360)));
}
static void fun95(Registry reg){
  f21(220.000000, -20.000000, reg);
}
static void fun52(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun39(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun40(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun53(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun84(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun93(Registry reg){
  f21(140.000000, -20.000000, reg);
}
static void fun54(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun41(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun76(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun55(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun42(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun94(Registry reg){
  f21(360.000000, -20.000000, reg);
}
static void fun83(Registry reg){
  f21(260.000000, -20.000000, reg);
}
static void fun56(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun43(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun85(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun57(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun44(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun18(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun21(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun15(Registry reg){
  reg.add(new NormalWall(Rect(0, -580, 20, 560)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(540, -580, 20, 560)));
}
static void fun103(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun100(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun58(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun64(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun99(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun102(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun101(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun98(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun59(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun65(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun92(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun77(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun104(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun107(Registry reg){
  f21(320.000000, -20.000000, reg);
}
static void fun89(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun88(Registry reg){
  f21(320.000000, -20.000000, reg);
}
static void fun66(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun60(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun106(Registry reg){
  f21(320.000000, -20.000000, reg);
}
static void fun105(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun87(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun109(Registry reg){
  f21(420.000000, -20.000000, reg);
}
static void fun108(Registry reg){
  f21(140.000000, -20.000000, reg);
}
static void fun90(Registry reg){
  f21(120.000000, -20.000000, reg);
}
static void fun91(Registry reg){
  f21(440.000000, -20.000000, reg);
}
static void fun61(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun67(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun86(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun78(Registry reg){
  f21(240.000000, -20.000000, reg);
}
static void fun79(Registry reg){
  f21(320.000000, -20.000000, reg);
}
static void fun111(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun110(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun80(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun68(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun62(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun69(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun63(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun126(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun132(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun120(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun141(Registry reg){
  f21(340.000000, -20.000000, reg);
}
static void fun138(Registry reg){
  f21(220.000000, -20.000000, reg);
}
static void fun0(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun121(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun133(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun127(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun19(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(0, -460, 20, 440)));
}
static void fun115(Registry reg){
  f21(500.000000, -20.000000, reg);
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(540, -460, 20, 440)));
}
static void fun112(Registry reg){
  f21(60.000000, -20.000000, reg);
}
static void fun134(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun128(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun122(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun113(Registry reg){
  f21(80.000000, -20.000000, reg);
}
static void fun114(Registry reg){
  f21(480.000000, -20.000000, reg);
}
static void fun139(Registry reg){
  f21(220.000000, -20.000000, reg);
}
static void fun142(Registry reg){
  f21(340.000000, -20.000000, reg);
}
static void fun119(Registry reg){
  f21(460.000000, -20.000000, reg);
}
static void fun116(Registry reg){
  f21(100.000000, -20.000000, reg);
}
static void fun135(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun117(Registry reg){
  f21(120.000000, -20.000000, reg);
}
static void fun118(Registry reg){
  f21(440.000000, -20.000000, reg);
}
static void fun123(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun129(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun72(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun70(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun130(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun124(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun136(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun143(Registry reg){
  f21(340.000000, -20.000000, reg);
}
static void fun140(Registry reg){
  f21(220.000000, -20.000000, reg);
}
static void fun137(Registry reg){
  f21(400.000000, -20.000000, reg);
}
static void fun125(Registry reg){
  f21(160.000000, -20.000000, reg);
}
static void fun131(Registry reg){
  f21(280.000000, -20.000000, reg);
}
static void fun73(Registry reg){
  f42(524.000000, -20.000000, reg);
}
static void fun71(Registry reg){
  f41(35.000000, -20.000000, reg);
}
static void fun5(Registry reg){
  set_scroll(reg, 1);
}
static void fun4(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun27(Registry reg){
  f02(280.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  f61(280.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun2(Registry reg){
  stop_scroll(reg);
}
}
class Events1{
const static Event[] events = [
  {1, &fun13},
  {1, &fun4},
  {1, &fun5},
  {1, &fun15},
  {1, &fun7},
  {1, &fun9},
  {1, &fun10},
  {1, &fun11},
  {1, &fun12},
  {1, &fun14},
  {93, &fun27},
  {153, &fun22},
  {153, &fun20},
  {193, &fun6},
  {193, &fun23},
  {193, &fun24},
  {233, &fun25},
  {233, &fun26},
  {1173, &fun2},
];
static void fun13(Registry reg){
  reg.add(new NormalWall(Rect(0, -153, 20, 440)));
}
static void fun4(Registry reg){
  reg.root.set_restart_point(1);
}
static void fun5(Registry reg){
  set_scroll(reg, 1);
}
static void fun15(Registry reg){
  reg.add(new NormalWall(Rect(0, 327, 20, 560)));
}
static void fun7(Registry reg){
  reg.add(new NormalWall(Rect(0, 1427, 20, 360)));
}
static void fun9(Registry reg){
  reg.add(new NormalWall(Rect(540, 1427, 20, 360)));
}
static void fun10(Registry reg){
  reg.add(new NormalWall(Rect(540, 927, 20, 360)));
}
static void fun11(Registry reg){
  reg.add(new NormalWall(Rect(0, 927, 20, 360)));
}
static void fun12(Registry reg){
  reg.add(new NormalWall(Rect(540, 327, 20, 560)));
}
static void fun14(Registry reg){
  reg.add(new NormalWall(Rect(540, -153, 20, 440)));
}
static void fun27(Registry reg){
  f02(280.000000, -20.000000, reg);
}
static void fun22(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun20(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun6(Registry reg){
  f61(280.000000, -20.000000, reg);
}
static void fun23(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun24(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun25(Registry reg){
  f11(600.000000, -20.000000, reg);
}
static void fun26(Registry reg){
  f12(-40.000000, -20.000000, reg);
}
static void fun2(Registry reg){
  stop_scroll(reg);
}
}
const Event[][] events = [
  Events0.events,
  Events1.events,
];
