import SDL;
import kinds;
import std.string;
import util;
import std.stdio;
import base;
import images = images.src.images;
import opengl;
import glutils;
import ship;
import std.typetuple;
import SDL_mixer;
import sounds;
static import version_str;

static const Rect WINDOW = {p:{0, 0}, q:{560, 474}};
static const Rect LARGE_WINDOW = {p:{-60, -60}, q:{620, 534}};
static const Rect INFO_WINDOW = {p:{564, 0}, q:{640, 480}};
template CollisionDetector(S, T, Kind s_kind, Kind t_kind, SubKind subkind)
{
  this(){ super(subkind); }
  
  void run(Registry reg) {
    foreach (Worker s; reg.workers(s_kind)) {
      foreach (Worker t; reg.workers(t_kind)) {
        if (check(cast(S)s, cast(T)t))
          collide(cast(S)s, cast(T)t, reg);
      }
    }
  }

  bool check(S s, T t) {
    return s.rect.is_intersected(t.rect);
  }
}

class Reserve: BaseWorker
{
private:
  int count_;
  void delegate() dg_;
  
public:
  this(SubKind subkind, int count, void delegate() dg) {
    super(subkind);
    count_ = count;
    dg_ = dg;
  }

  void run(Registry) {
    count_--;
    if (count_ <= 0){
      dg_();
      die();
    }
  }
}

class KeyConfig {
  int shot, charge, option_change, speed_change;

  this(int s, int c, int o, int sp) {
    shot = s; charge = c;
    option_change = o; speed_change = sp;
  }
  
  bool validate(SDL_Joystick* joy) {
    int num = SDL_JoystickNumButtons(joy);
    return 0 <= shot && shot < num && 0 <= charge && charge < num &&
      0 <= option_change && option_change < num &&
      0 <= speed_change && speed_change < num;
  }

  
  static KeyConfig default_config(){
    return new KeyConfig(0,1,2,3);
  };
}

class GameConfig {
public:
  enum GameMode { NORMAL=0, TRAINING=1, DEBUG=2 };
  
private:
  GameMode mode_;
  bool invincible_, draw_bounding_box_;
  int start_life_, start_num_options_, start_magazine_;
  int start_stage_, start_point_;
  bool sound_, fullscreen_;
  char[] replayfile_;
  
  void set_normal_mode_params() {
    mode_ = GameMode.NORMAL;
    start_life_ = 2;
    start_magazine_ = 0;
    start_num_options_ = 0;
    start_stage_ = 1;
    start_point_ = 0;
    invincible_ = false;
    draw_bounding_box_ = false;
  }
  
  void set_training_mode_params() {
    mode_ = GameMode.TRAINING;
    start_life_ = 2;
    start_magazine_ = 0;
    start_num_options_ = 2;
    start_stage_ = 1;
    start_point_ = 0;
    invincible_ = false;
    draw_bounding_box_ = false;
  }
  
public:
  this(GameMode mode, bool sound, bool fullscreen,
       bool invincible, bool draw_bounding_box,
       int life, int magazine, int nopt,
       int stage, int point, char[] replayfile) {
    switch (mode) {
    case GameMode.DEBUG:
      mode_ = GameMode.DEBUG;
      start_life_ = life;
      start_magazine_ = magazine;
      start_num_options_ = nopt;
      start_stage_ = stage;
      start_point_ = point;
      invincible_ = invincible;
      draw_bounding_box_ = draw_bounding_box;
      break;
    case GameMode.NORMAL:
      set_normal_mode_params();
      break;
    case GameMode.TRAINING:
      set_training_mode_params();
      break;
    }
    sound_ = sound;
    fullscreen_ = fullscreen;
    replayfile_ = replayfile;
  }

  GameMode mode(){ return mode_; }
  void mode(GameMode mode){ mode_ = mode; }
  bool invincible() { return invincible_; }
  bool draw_bounding_box() { return draw_bounding_box_; }
  int start_life() { return start_life_; }
  int start_num_options() { return start_num_options_; }
  int start_magazine() { return start_magazine_; }
  int start_stage() { return start_stage_; }
  int start_point() { return start_point_; }
  bool sound() { return sound_; }
  bool fullscreen() { return fullscreen_; }

  bool debug_mode() { return mode_ == GameMode.DEBUG; }

  char[] replayfile() { return replayfile_; }
  void update_replayfilename(){ replayfile_ = null; }
  
  char[] config_string() {
    switch (mode_) {
    case GameMode.NORMAL:
      return "NORMAL MODE";
    case GameMode.TRAINING:
      return "TRAINING MODE";
    case GameMode.DEBUG:
      return "DEBUG MODE";
    }
  }
}

abstract class Input: BaseWorker
{
  this(SubKind kind) {
    super(kind);
  }
  
  bool left();
  bool right();
  bool up();
  bool down();
  bool shot();
  bool speed_change();
  bool option_change();
  int charge();
  bool charging(){ return charge() > 20; }
  ushort encode_to_ushort();

  
  static Input search(Registry reg) {
    return reg.search!(Input)(SubKind.INPUT);
  }
}

static import recorder;

class RealInput: Input
{
private:
  bool left_, right_, up_, down_;
  bool shot_, speed_change_, option_change_;
  bool speed_change_pressed_ = false;
  bool option_change_pressed_ = false;
  static int threshold = 6000;
  SDL_Joystick* joy_=null;
  int shot_count_=0;
  int charge_ = 0;
  KeyConfig config_;
  
  static const int SHOT_WAIT = 10;
public:
  this(KeyConfig config) {
    config_ = config;
    super(SubKind.INPUT);
    if (SDL_NumJoysticks() > 0) {
      joy_ = SDL_JoystickOpen(0);
      config_.validate(joy_);
    }
  }

  bool pressed(Uint8* keystate, Uint8 key, int joybutton) {
    return (keystate[key] == SDL_PRESSED) ||
      (joy_ && SDL_JoystickGetButton(joy_, joybutton));
  }
  
  void run(Registry reg) {
    Uint8* keystate = SDL_GetKeyState(null);

    left_ = (keystate[SDLK_LEFT] == SDL_PRESSED);
    right_ = keystate[SDLK_RIGHT] == SDL_PRESSED;
    up_ = keystate[SDLK_UP] == SDL_PRESSED;
    down_ = keystate[SDLK_DOWN] == SDL_PRESSED;

    if (joy_) {
      left_ = left_ || (SDL_JoystickGetAxis(joy_, 0) < -threshold);
      right_ = right_ || (SDL_JoystickGetAxis(joy_, 0) > threshold);
      up_ = up_ || (SDL_JoystickGetAxis(joy_, 1) < -threshold);
      down_ = down_ || (SDL_JoystickGetAxis(joy_, 1) > threshold);
    }
    
    if(shot_count_>0)shot_count_--;
    if (pressed(keystate, SDLK_z, config_.shot) && shot_count_ <= 0) {
      shot_count_ = SHOT_WAIT;
      shot_ = true;
    } else {
      shot_ = false;
    }

    option_change_ = pressed(keystate, SDLK_c, config_.option_change)
      && !option_change_pressed_;
    option_change_pressed_ = pressed(keystate, SDLK_c, config_.option_change);

    speed_change_ =
      pressed(keystate, SDLK_v, config_.speed_change) &&
      !speed_change_pressed_;
    
    if (speed_change_)
      reg.root.speed_change();

    speed_change_pressed_ = pressed(keystate, SDLK_v, config_.speed_change);

    
    if (pressed(keystate, SDLK_x, config_.charge)) {
      charge_++;
    } else {
      charge_ = 0;
    }
  }

  bool left(){ return left_; }
  bool right(){ return right_; }
  bool up(){ return up_; }
  bool down(){ return down_; }
  bool shot(){ return shot_; }
  bool speed_change(){ return speed_change_; }
  bool option_change(){ return option_change_; }
  int charge(){ return charge_; }
  

  ushort encode_to_ushort() {
    return recorder.encode_input_to_ushort(left_, right_,
                                           up_, down_,
                                           shot_, speed_change_,
                                           option_change_,
                                           charge_);
  }

  static Input search(Registry reg) {
    return reg.search!(Input)(SubKind.INPUT);
  }
}

class DrawWindow: BaseWorker
{
  this() {
    super(SubKind.DRAW_WINDOW);
  }

  void draw(Registry) {
    glColor3da(Color.white);
    WINDOW.draw();
    glTranslated(-2.5, -2.5, 0);
    glColor3da(Color.black);
    INFO_WINDOW.filled_draw();
  }
}

class ShiftScreen: BaseWorker {
  this() {
    super(SubKind.SHIFT_SCREEN);
  }

  void draw(Registry) {
    glTranslated(2.5, 2.5, 0);
  }
}

class SystemInfo: BaseMovable
{
  this() {
    super(SubKind.SYSTEMINFO, Point(620, 0));
  }

  void draw(Registry reg) {
    glPushMatrix();
    glTranslated(pos.x, pos.y, 0);
    glRotated(90, 0, 0, 1);
    glColor3da(Color.white);
    glScaled(0.5, 0.5, 1);
    draw_text(std.string.format("FPS:%3.1f  WORKERS:%3d  SKIP:%2d/S",
                                reg.root.observed_fps,
                                reg.num_workers(),
                                reg.root.observed_skip));
    glPopMatrix();
  }
}

class GameInfo: BaseMovable
{
  this() {
    super(SubKind.GAMEINFO, Point(600, 0));
  }

  void draw(Registry reg) {
    glPushMatrix();
    glTranslated(pos.x, pos.y, 0);
    glRotated(90, 0, 0, 1);
    glColor3da(Color.white);
    draw_text(std.string.format("REST:%1d       SCORE:%10d     CONTINUE:%2d",
                                reg.root.rest_players,
                                reg.root.score,
                                reg.root.continue_count));
    glPopMatrix();
    glPushMatrix();
    glTranslated(pos.x+24, pos.y+270, 0);
    glRotated(90, 0, 0, 1);
    glColor3da(Color.white);
    draw_text(std.string.format("HISCORE:%10d",
                                reg.root.hiscore));
    glPopMatrix();
    glPushMatrix();
    glTranslated(pos.x - 24, pos.y+10, 0);
    glRotated(90, 0, 0, 1);
    glColor3da(Color.white);
    glScaled(2, 2, 1);
    for (int i=0; i<=reg.root.speed; i++) {
      images.triangle.draw();
      glTranslated(6, 0, 0);
    }
    glPopMatrix();
  }
}

struct Event{
  int time;
  void function(Registry) fun;
}

class Stage: BaseWorker
{
private:
  Event[] events_;
  int count_;
  
public:
  this(Event[] events) {
    super(SubKind.STAGE);
    events_ = events;
    count_ = 0;
  }

  void skip(int count) {
    foreach (int idx, Event ev; events_) {
      if (count < ev.time ) {
        events_ = events_[idx..$];
        return;
      }
    }
    events_ = null;
  }
  
  void run(Registry reg) {
    while (events_ != null && events_[0].time == count_) {
      events_[0].fun(reg);
      if (events_.length > 1)
        events_ = events_[1..$];
      else
        events_ = null;
    }
    
    count_++;
  }
}


class ShotWallCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Wall, Shot, Kind.WALL, Kind.SHOT,
                           SubKind.SHOT_WALL_COLLISION_DETECTOR);

  bool check(Wall w, Shot s) {
    return (s.pos in w.rect);
  }
  
  void collide(Wall wall, Shot shot, Registry reg) {
    shot.collide_with_wall(reg, wall);
  }
}

class BulletWallCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Wall, Bullet, Kind.WALL, Kind.BULLET,
                           SubKind.BULLET_WALL_COLLISION_DETECTOR);

  void collide(Wall wall, Bullet bullet, Registry reg) {
    bullet.collide_with_wall(reg, wall);
  }
}

class BulletChargeCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(ChargeBlocker, Bullet,
                           Kind.CHARGE_BLOCKER, Kind.BULLET,
                           SubKind.BULLET_CHARGE_COLLISION_DETECTOR);

  void collide(ChargeBlocker blocker, Bullet bullet,
               Registry reg) {
    bullet.collide_with_charge_blocker(reg, blocker);
  }
}
class ShotEnemyCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Shot, Enemy, Kind.SHOT, Kind.ENEMY,
                           SubKind.SHOT_ENEMY_COLLISION_DETECTOR);
  
  void collide(Shot shot, Enemy enemy, Registry reg) {
    if (shot.power != 0) {
      enemy.collide_with_shot(reg, shot);
      shot.collide_with_enemy(reg, enemy);
    }
  }
}

class WallEnemyCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Wall, Enemy, Kind.WALL, Kind.ENEMY,
                           SubKind.WALL_ENEMY_COLLISION_DETECTOR);

  void collide(Wall wall, Enemy enemy, Registry reg) {
    enemy.collide_with_wall(reg, wall);
  }
}

class PlayerWallCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Player, Wall, Kind.PLAYER, Kind.WALL,
                           SubKind.PLAYER_WALL_COLLISION_DETECTOR);
  
  bool check(Player player, Wall wall) {
    return player.pos() in wall.rect;
  }
  
  void collide(Player player, Wall, Registry reg) {
    player.destroy(reg);
  }
}

class PlayerEnemyCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Player, Enemy, Kind.PLAYER, Kind.ENEMY,
                           SubKind.PLAYER_ENEMY_COLLISION_DETECTOR);
  
  void collide(Player player, Enemy, Registry reg) {
    player.destroy(reg);
  }
}

class PlayerBulletCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Player, Bullet, Kind.PLAYER, Kind.BULLET,
                           SubKind.PLAYER_BULLET_COLLISION_DETECTOR);

  void collide(Player player, Bullet, Registry reg) {
    player.destroy(reg);
  }
}

class PlayerCapsuleCollisionDetector: BaseWorker
{
  mixin CollisionDetector!(Player, Capsule, Kind.PLAYER, Kind.CAPSULE,
                           SubKind.PLAYER_CAPSULE_COLLISION_DETECTOR);

  void collide(Player player, Capsule capsule, Registry reg) {
    capsule.collide_with_player(reg, player);
  }
}

class StageEnd: BaseWorker
{
private:
  int currentstage_, nextstage_;

public:
  this(int currentstage, int nextstage) {
    super(SubKind.STAGEEND);
    currentstage_ = currentstage;
    nextstage_ = nextstage;
  }

  void run(Registry reg) {
    if (180 < count) {
      reg.root.hold_replaydata();
      reg.root.reserve_start_stage(nextstage_, 0);
      die();
    }
  }

  void draw(Registry) {
    if (count > 60) {
      glPushMatrix();
      glTranslated(60, 200, 0);
      glScaled(1.5, 1.5, 0);
      draw_text(format("STAGE %d CLEAR! GOTO NEXT STAGE", currentstage_));
      
      glPopMatrix();
    }
  }
}

class Title: BaseWorker
{
private:
  enum MenuItem { START=0, QUIT };
  MenuItem selected_;
  int wait_;
  static const int WAIT = 10;
  
public:
  this() {
    super(SubKind.TITLE);
    selected_ = MenuItem.START;
    wait_ = 0;
  }
  
  void run(Registry reg) {
    auto input = Input.search(reg);

    if (wait_ > 0)
      --wait_;
    
    if (wait_ == 0) {
      if (input.up || input.down) {
        selected_ = cast(MenuItem)((selected_ + 1) % 2);
        wait_ = WAIT;
      }
    }
    
    if (input.shot) {
      switch (selected_) {
      case MenuItem.START:
        reg.root.start_next_game();
        break;
      case MenuItem.QUIT:
        reg.root.quit();
      default:
        break;
      }
    }
  }
  
  void draw(Registry reg) {
    translated_scaled(100, 150, 2, {
      draw_text("PROJECT PENCIL-BULLET");
    });
    if (reg.root.load_error_flag)
      translated_scaled(140, 200, 1.5, {
        draw_text("LOAD ERROR");
      });
    translated_scaled(130, 260, 1.5, {
      draw_text("START");
    });
    translated_scaled(280, 260, 1.5, {
      draw_text(reg.root.game_config.config_string);
    });
    translated_scaled(130, 290, 1.5, {
      draw_text("QUIT");
    });
    translated_scaled(20, 450, 1, {
      draw_text(version_str.version_str);
    });
    translated_scaled(110, 272+selected_*30, 3, {
      images.triangle.draw();
    });
  }
}

class GameEnd: BaseWorker
{
public:
  this() {
    super(SubKind.GAMEEND);
  }

  void run(Registry reg) {
    if (count > 300)
      reg.root.show_game_over();
  }

  void draw(Registry reg) {
    glPushMatrix();
    glTranslated(160, 160, 0);
    glScaled(2, 2, 1);
    draw_text("CONGRATULATION");
    glPopMatrix();
    glPushMatrix();
    glTranslated(150, 200, 0);
    glScaled(2, 2, 1);
    draw_text("CLEAR ALL STAGES");
    glPopMatrix();
    glPushMatrix();
    glTranslated(120, 280, 0);
    glScaled(2, 2, 1);
    draw_text(format("SCORE: %10d", reg.root.score));
    glPopMatrix();
  }
}

class Continue: BaseWorker
{
private:
  int wait_count_;
  bool select_yes_;
  
  static Rect YES_BOX = {p:{198, 238}, q:{262, 272}};
  static Rect NO_BOX = {p:{198, 278}, q:{262, 312}};
  
public:
  this() {
    super(SubKind.CONTINUE);
    wait_count_ = 0;
    select_yes_ = true;
  }

  void run(Registry reg) {
    Input input = Input.search(reg);
    
    if (wait_count_ > 0) {
      wait_count_--;
      return;
    }

    if (input.up || input.down) {
      wait_count_ = 10;
      select_yes_ = !select_yes_;
      return;
    }

    if (input.shot) {
      if (select_yes_) {
        reg.root.continue_game();
      } else {
        reg.root.show_game_over();
      }
    }
  }

  void draw(Registry reg) {
    glPushMatrix();
    glTranslated(160, 200, 0);
    glScaled(2, 2, 1);
    draw_text("CONTINUE?");
    glPopMatrix();
    glPushMatrix();
    glTranslated(200, 240, 0);
    glScaled(2, 2, 1);
    draw_text("YES");
    glPopMatrix();
    glPushMatrix();
    glTranslated(210, 280, 0);
    glScaled(2, 2, 1);
    draw_text("NO");
    glPopMatrix();
    glPushMatrix();
    glTranslated(120, 320, 0);
    glScaled(2, 2, 1);
    draw_text(format("SCORE: %10d", reg.root.score));
    glPopMatrix();

    if (wait_count_ == 0) {
      if (select_yes_)
        YES_BOX.draw();
      else
        NO_BOX.draw();
    }
  }
}

class GameOver: BaseWorker
{
  this() {
    super(SubKind.GAMEOVER);
  }

  void run(Registry reg) {
    if (count == 0) {
      Music.gameover.play();
    }
    
    if (count > 240) {
      Music.halt();
      reg.root.show_title();
    }
  }
  
  void draw(Registry reg) {
    glPushMatrix();
    glTranslated(200, 200, 0);
    glScaled(2, 2, 1);
    draw_text("GAME OVER");
    glPopMatrix();
    glPushMatrix();
    glTranslated(120, 240, 0);
    glScaled(2, 2, 1);
    draw_text(format("SCORE: %10d", reg.root.score));
    glPopMatrix();
  }
}

class BGLine: BaseWorker {
private:
  Point p_, q_;
public:
  this(Point p, Point q) {
    super(SubKind.BG_LINE);
    p_ = p;
    q_ = q;
  }

  void run(Registry reg) {
    Vector v = Scroll.search(reg).v;
    p_ += v; q_ += v;
    if (p_ in LARGE_WINDOW) return;
    if (q_ in LARGE_WINDOW) return;
    die();
  }
  
  void draw(Registry) {
    glColor3da(Color.wall);
    glBegin(GL_LINES);
    p_.glVertex();
    q_.glVertex();
    glEnd();
  }
}


class ReplayTitle: BaseWorker {
public:
  this() {
    super(SubKind.TITLE);
  }

  void draw(Registry reg) {
    translated_scaled(70, 100, 2, {
      draw_text("PROJECT PENCIL-BULLET");
    });
    translated_scaled(140, 140, 2, {
      draw_text("SHOW REPLAY");
    });
    
    translated_scaled(80, 230, 1.5, {
      draw_text("PRESS RIGHT TO START REPLAY");
    });
    translated_scaled(80, 260, 1.5, {
      draw_text("PRESS ESCAPE TO QUIT");
    });
    translated_scaled(80, 290, 1.5, {
      draw_text("PRESS S KEY TO RETURN HERE");
    });
    translated_scaled(80, 340, 1.5, {
      draw_text("RIGHT:SEEK NEXT");
    });
    translated_scaled(80, 370, 1.5, {
      draw_text("LEFT:SEEK PREVIOUS");
    });

    translated_scaled(280, 415, 1.5, {
      draw_text(format("%17s", reg.root.game_config.config_string));
    });
    translated_scaled(280, 440, 1.5, {
      draw_text(format("SCORE: %10d", reg.root.hiscore));
    });
  }
}

class ReplayEnd: BaseWorker {
public:
  this() {
    super(SubKind.TITLE);
  }

  void draw(Registry reg) {
    translated_scaled(70, 100, 2, {
      draw_text("PROJECT PENCIL-BULLET");
    });
    translated_scaled(140, 140, 2, {
      draw_text("REPLAY MODE");
    });
    
    translated_scaled(80, 250, 1.5, {
      draw_text("PRESS S KEY TO GOTO TITLE");
    });
    translated_scaled(80, 280, 1.5, {
      draw_text("PRESS ESCAPE TO QUIT");
    });

    translated_scaled(280, 440, 1.5, {
      draw_text(format("SCORE: %10d", reg.root.hiscore));
    });
  }
}
