import std.stdio;

class VArray(T)
{
  private T[] ary;
  private size_t nitems;
  
  this(int n=100) {
    ary = new T[n];
    nitems=0;
  }

  void clear() {
    nitems = 0;
  }

  void extend() {
    auto newary = new T[ary.length*2+5];
    newary[0..ary.length] = ary[0..ary.length];
    ary = newary;
  }
  
  void add(T item) {
    if (nitems>=ary.length) {
      extend();
    }
    ary[nitems] = item;
    nitems++;
  }

  void insert_by(T item, int delegate(T) dg) {
    if (nitems>=ary.length)
      extend();
    
    int p = dg(item);
    for (int j=0; j<nitems; j++) {
      if (p <= dg(ary[j])) {
        for (int k=nitems-1; j<=k; k--) {
          ary[k+1] = ary[k];
        }
        ary[j] = item;
        nitems++;
        return;
      }
    }

    add(item);
  }
  
  T[] to_ary() {
    return ary[0..nitems];
  }

  
  T opIndex(size_t j) {
    if (j>=nitems)
      throw new Error("Out of Range");
    return ary[j];
  }

  int length() { return nitems; }
  
  int opApply(int delegate(int, inout T) dg) {
    for (int j=0; j<nitems; j++) {
      int r = dg(j, ary[j]);
      if (r)
        return r;
    }
    return 0;
  }

    
  int opApply(int delegate(inout T) dg) {
    for (int j=0; j<nitems; j++) {
      int r = dg(ary[j]);
      if (r)
        return r;
    }
    return 0;
  }

  int opApplyReverse(int delegate(inout T) dg) {
    for (int j=nitems-1; j>=0; j--) {
      int r = dg(ary[j]);
      if (r)
        return r;
    }
    return 0;
  }

  void delete_if(bool delegate(T) dg) {
    int i, j;
    for (i=0, j=0; i<nitems; i++) {
      ary[j] = ary[i];
      if (!dg(ary[i]))
        j++;
    }
    nitems = j;
  }
}

debug (VAry) {
  int main(char[][] args)
    {
      int i;
      auto x = new VArray!(int)(10);
      
      for(i=0; i<100; i++) {
        x.add(i*2);
      }
      
      foreach (int n ;x.to_ary) {
        writef("%d ", n);
      }
      writefln("");

      foreach (int n ;x) {
        writef("%d ", n);
      }
      writefln("");

      writefln("%d", x[12]);

      x.insert_by(5, delegate(int n){return n;});
      x.insert_by(5, delegate(int n){return n;});
      x.insert_by(-1, delegate(int n){return n;});
      x.insert_by(2004, delegate(int n){return n;});
      foreach (int n ;x) {
        writef("%d ", n);
      }
      writefln("");

      x.clear();
      for(i=0; i<1000; i++) {
        x.add(i);
      }

      x.delete_if(delegate(int i){ return i%3 == 0; });
      foreach (int n ;x) {
        writef("%d ", n);
      }
      writefln("");
      
      return 0;
    }
}  
